/* Automatically @generated by tree-sitter v0.25.10 */

#include "tree_sitter/parser.h"

#if defined(__GNUC__) || defined(__clang__)
#pragma GCC diagnostic ignored "-Wmissing-field-initializers"
#endif

#ifdef _MSC_VER
#pragma optimize("", off)
#elif defined(__clang__)
#pragma clang optimize off
#elif defined(__GNUC__)
#pragma GCC optimize ("O0")
#endif

#define LANGUAGE_VERSION 15
#define STATE_COUNT 886
#define LARGE_STATE_COUNT 4
#define SYMBOL_COUNT 248
#define ALIAS_COUNT 2
#define TOKEN_COUNT 104
#define EXTERNAL_TOKEN_COUNT 8
#define FIELD_COUNT 35
#define MAX_ALIAS_SEQUENCE_LENGTH 14
#define MAX_RESERVED_WORD_SET_SIZE 0
#define PRODUCTION_ID_COUNT 112
#define SUPERTYPE_COUNT 8

enum ts_symbol_identifiers {
  sym_line_comment = 1,
  anon_sym_PERCENT_STAR_BANG = 2,
  aux_sym_doc_comment_token1 = 3,
  anon_sym_STAR_PERCENT = 4,
  anon_sym_COMMA = 5,
  aux_sym_doc_predicate_token1 = 6,
  anon_sym_RPAREN = 7,
  sym_doc_var = 8,
  sym_doc_fragment_emph = 9,
  sym_doc_fragment_bold = 10,
  sym_doc_fragment_italic = 11,
  sym_doc_fragment_code = 12,
  anon_sym_COLON = 13,
  sym_identifier = 14,
  anon_sym_DQUOTE = 15,
  anon_sym_f_DQUOTE = 16,
  sym__string_fragment = 17,
  sym__fstring_fragment = 18,
  sym__string_escape = 19,
  sym__fstring_escape = 20,
  aux_sym_fstring_accessor_token1 = 21,
  aux_sym_fstring_accessor_token2 = 22,
  aux_sym_fstring_accessor_token3 = 23,
  aux_sym_fstring_accessor_token4 = 24,
  aux_sym_fstring_conversion_token1 = 25,
  aux_sym_fstring_conversion_token2 = 26,
  sym_fstring_align = 27,
  sym_fstring_sign = 28,
  sym_fstring_alternate = 29,
  sym_fstring_grouping = 30,
  sym_fstring_type = 31,
  aux_sym_fstring_spec_token1 = 32,
  anon_sym_LBRACE = 33,
  anon_sym_RBRACE = 34,
  sym_supremum = 35,
  sym_infimum = 36,
  sym_number = 37,
  sym_anonymous = 38,
  sym_variable = 39,
  anon_sym_CARET = 40,
  anon_sym_QMARK = 41,
  anon_sym_AMP = 42,
  anon_sym_PLUS = 43,
  anon_sym_DASH = 44,
  anon_sym_STAR = 45,
  anon_sym_SLASH = 46,
  anon_sym_BSLASH = 47,
  anon_sym_STAR_STAR = 48,
  anon_sym_TILDE = 49,
  anon_sym_PIPE = 50,
  anon_sym_COMMA2 = 51,
  anon_sym_LPAREN = 52,
  anon_sym_RPAREN2 = 53,
  anon_sym_DOT_DOT = 54,
  anon_sym_SEMI = 55,
  anon_sym_AT = 56,
  sym_theory_operator = 57,
  anon_sym_LBRACK = 58,
  anon_sym_RBRACK = 59,
  anon_sym_RBRACE2 = 60,
  sym_boolean_constant = 61,
  sym_relation = 62,
  sym_default_negation = 63,
  sym_double_default_negation = 64,
  sym_aggregate_function = 65,
  anon_sym_DOT = 66,
  anon_sym_COLON_DASH = 67,
  anon_sym_COLON_TILDE = 68,
  anon_sym_POUNDmaximize = 69,
  anon_sym_POUNDmaximise = 70,
  anon_sym_POUNDminimize = 71,
  anon_sym_POUNDminimise = 72,
  anon_sym_POUNDshow = 73,
  anon_sym_true = 74,
  anon_sym_false = 75,
  anon_sym_POUNDdefined = 76,
  anon_sym_POUNDproject = 77,
  anon_sym_POUNDprogram = 78,
  sym_code = 79,
  anon_sym_POUNDscript = 80,
  anon_sym_POUNDend = 81,
  sym_const_type = 82,
  anon_sym_POUNDconst = 83,
  anon_sym_EQ = 84,
  anon_sym_POUNDedge = 85,
  anon_sym_POUNDheuristic = 86,
  anon_sym_POUNDinclude = 87,
  anon_sym_LT = 88,
  anon_sym_GT = 89,
  anon_sym_POUNDexternal = 90,
  sym_theory_operator_arity = 91,
  anon_sym_binary = 92,
  sym_theory_operator_associativity = 93,
  sym_theory_atom_type = 94,
  anon_sym_POUNDtheory = 95,
  sym_empty_pool_item_first = 96,
  sym_empty_pool_item = 97,
  sym_colon = 98,
  sym_block_comment = 99,
  sym_doc_fragment_string = 100,
  sym__doc_token_args = 101,
  sym__doc_token_paren = 102,
  sym__doc_token_minus = 103,
  sym_source_file = 104,
  sym_doc_comment = 105,
  sym_variables = 106,
  sym_doc_predicate = 107,
  sym_doc_desc = 108,
  sym_doc_args = 109,
  sym_doc_arg = 110,
  sym_string = 111,
  sym_fstring = 112,
  sym_fstring_accessor = 113,
  sym_fstring_conversion = 114,
  sym_fstring_width = 115,
  sym_fstring_spec = 116,
  sym_fstring_field = 117,
  sym__const_term = 118,
  sym_const_binary_operation = 119,
  sym_const_unary_operation = 120,
  sym_const_abs = 121,
  sym_const_terms = 122,
  sym__const_arg_pool = 123,
  sym_const_function = 124,
  sym__const_term_comma = 125,
  sym__const_terms_trail = 126,
  sym_const_tuple = 127,
  sym_binary_operation = 128,
  sym_unary_operation = 129,
  sym_abs = 130,
  sym_terms = 131,
  sym_projection = 132,
  sym__term_or_projection = 133,
  sym__terms_or_projection = 134,
  sym__arg_pool_n = 135,
  sym__arg_pool = 136,
  sym_function = 137,
  sym_external_function = 138,
  sym__term_comma = 139,
  sym__terms_trail = 140,
  sym_tuple_pool_item = 141,
  sym__tuple_pool_n = 142,
  sym_tuple = 143,
  sym_term = 144,
  sym_theory_operators = 145,
  sym_theory_terms = 146,
  sym__theory_arguments = 147,
  sym_theory_function = 148,
  sym__theory_terms_trail = 149,
  sym_theory_tuple = 150,
  sym_theory_list = 151,
  sym_theory_set = 152,
  sym_theory_unparsed_term = 153,
  sym_theory_term = 154,
  sym_theory_root_term = 155,
  sym_symbolic_atom = 156,
  sym_comparison = 157,
  sym_simple_atom = 158,
  sym_literal_sign = 159,
  sym_literal = 160,
  sym_condition = 161,
  sym_upper = 162,
  sym_lower = 163,
  sym_set_aggregate_element = 164,
  sym_set_aggregate_elements = 165,
  sym_set_aggregate = 166,
  sym_theory_element = 167,
  sym_theory_elements = 168,
  sym_theory_atom_upper = 169,
  sym_theory_atom = 170,
  sym_body_aggregate_element = 171,
  sym_body_aggregate_elements = 172,
  sym_body_aggregate = 173,
  sym_body_literal = 174,
  sym_body = 175,
  sym_head_aggregate_element = 176,
  sym_head_aggregate_elements = 177,
  sym_head_aggregate = 178,
  sym__conditional_literal_n = 179,
  sym__conditional_literal_0 = 180,
  sym_conditional_literal = 181,
  sym_disjunction = 182,
  sym_head = 183,
  sym_rule = 184,
  sym_integrity_constraint = 185,
  sym_weight = 186,
  sym_optimize_element = 187,
  sym_optimize_elements = 188,
  sym_weak_constraint = 189,
  sym_maximize = 190,
  sym_minimize = 191,
  sym_signature = 192,
  sym_show = 193,
  sym_show_term = 194,
  sym_boolean = 195,
  sym_show_signature = 196,
  sym_defined = 197,
  sym_project_signature = 198,
  sym_project_atom = 199,
  sym_parameters = 200,
  sym_program = 201,
  sym_script = 202,
  sym_const = 203,
  sym_edge_pair = 204,
  sym_edge = 205,
  sym_heuristic = 206,
  sym_include = 207,
  sym_external = 208,
  sym__theory_operator_arity_binary = 209,
  sym_theory_operator_definition = 210,
  sym_theory_operator_definitions = 211,
  sym_theory_term_definition = 212,
  sym__theory_operators_sep = 213,
  sym_theory_atom_definition = 214,
  sym_theory = 215,
  sym_statement = 216,
  aux_sym_source_file_repeat1 = 217,
  aux_sym_variables_repeat1 = 218,
  aux_sym_doc_desc_repeat1 = 219,
  aux_sym_doc_args_repeat1 = 220,
  aux_sym_string_repeat1 = 221,
  aux_sym_fstring_repeat1 = 222,
  aux_sym_fstring_accessor_repeat1 = 223,
  aux_sym_const_terms_repeat1 = 224,
  aux_sym_abs_repeat1 = 225,
  aux_sym_terms_repeat1 = 226,
  aux_sym__terms_or_projection_repeat1 = 227,
  aux_sym__arg_pool_n_repeat1 = 228,
  aux_sym__tuple_pool_n_repeat1 = 229,
  aux_sym_theory_operators_repeat1 = 230,
  aux_sym_theory_terms_repeat1 = 231,
  aux_sym_theory_unparsed_term_repeat1 = 232,
  aux_sym_comparison_repeat1 = 233,
  aux_sym_condition_repeat1 = 234,
  aux_sym_set_aggregate_elements_repeat1 = 235,
  aux_sym_theory_elements_repeat1 = 236,
  aux_sym_body_aggregate_elements_repeat1 = 237,
  aux_sym_body_repeat1 = 238,
  aux_sym_head_aggregate_elements_repeat1 = 239,
  aux_sym_disjunction_repeat1 = 240,
  aux_sym_optimize_elements_repeat1 = 241,
  aux_sym_parameters_repeat1 = 242,
  aux_sym_edge_repeat1 = 243,
  aux_sym_theory_operator_definitions_repeat1 = 244,
  aux_sym__theory_operators_sep_repeat1 = 245,
  aux_sym__theory_definitions_repeat1 = 246,
  aux_sym__theory_definitions_repeat2 = 247,
  alias_sym_classical_negation = 248,
  alias_sym_lone_comma = 249,
};

static const char * const ts_symbol_names[] = {
  [ts_builtin_sym_end] = "end",
  [sym_line_comment] = "line_comment",
  [anon_sym_PERCENT_STAR_BANG] = "%*!",
  [aux_sym_doc_comment_token1] = "doc_comment_token1",
  [anon_sym_STAR_PERCENT] = "*%",
  [anon_sym_COMMA] = ",",
  [aux_sym_doc_predicate_token1] = "identifier",
  [anon_sym_RPAREN] = ")",
  [sym_doc_var] = "variable",
  [sym_doc_fragment_emph] = "doc_fragment_emph",
  [sym_doc_fragment_bold] = "doc_fragment_bold",
  [sym_doc_fragment_italic] = "doc_fragment_italic",
  [sym_doc_fragment_code] = "doc_fragment_code",
  [anon_sym_COLON] = ":",
  [sym_identifier] = "identifier",
  [anon_sym_DQUOTE] = "\"",
  [anon_sym_f_DQUOTE] = "f\"",
  [sym__string_fragment] = "string_fragment",
  [sym__fstring_fragment] = "string_fragment",
  [sym__string_escape] = "escape_sequence",
  [sym__fstring_escape] = "escape_sequence",
  [aux_sym_fstring_accessor_token1] = "identifier",
  [aux_sym_fstring_accessor_token2] = "fstring_accessor_token2",
  [aux_sym_fstring_accessor_token3] = "fstring_accessor_token3",
  [aux_sym_fstring_accessor_token4] = "fstring_accessor_token4",
  [aux_sym_fstring_conversion_token1] = "fstring_conversion_token1",
  [aux_sym_fstring_conversion_token2] = "fstring_conversion_token2",
  [sym_fstring_align] = "fstring_align",
  [sym_fstring_sign] = "fstring_sign",
  [sym_fstring_alternate] = "fstring_alternate",
  [sym_fstring_grouping] = "fstring_grouping",
  [sym_fstring_type] = "fstring_type",
  [aux_sym_fstring_spec_token1] = "fstring_spec_token1",
  [anon_sym_LBRACE] = "{",
  [anon_sym_RBRACE] = "}",
  [sym_supremum] = "supremum",
  [sym_infimum] = "infimum",
  [sym_number] = "number",
  [sym_anonymous] = "anonymous",
  [sym_variable] = "variable",
  [anon_sym_CARET] = "^",
  [anon_sym_QMARK] = "\?",
  [anon_sym_AMP] = "&",
  [anon_sym_PLUS] = "+",
  [anon_sym_DASH] = "-",
  [anon_sym_STAR] = "*",
  [anon_sym_SLASH] = "/",
  [anon_sym_BSLASH] = "\\",
  [anon_sym_STAR_STAR] = "**",
  [anon_sym_TILDE] = "~",
  [anon_sym_PIPE] = "|",
  [anon_sym_COMMA2] = ",",
  [anon_sym_LPAREN] = "(",
  [anon_sym_RPAREN2] = ")",
  [anon_sym_DOT_DOT] = "..",
  [anon_sym_SEMI] = ";",
  [anon_sym_AT] = "@",
  [sym_theory_operator] = "theory_operator",
  [anon_sym_LBRACK] = "[",
  [anon_sym_RBRACK] = "]",
  [anon_sym_RBRACE2] = "}",
  [sym_boolean_constant] = "boolean_constant",
  [sym_relation] = "relation",
  [sym_default_negation] = "default_negation",
  [sym_double_default_negation] = "double_default_negation",
  [sym_aggregate_function] = "aggregate_function",
  [anon_sym_DOT] = ".",
  [anon_sym_COLON_DASH] = ":-",
  [anon_sym_COLON_TILDE] = ":~",
  [anon_sym_POUNDmaximize] = "#maximize",
  [anon_sym_POUNDmaximise] = "#maximise",
  [anon_sym_POUNDminimize] = "#minimize",
  [anon_sym_POUNDminimise] = "#minimise",
  [anon_sym_POUNDshow] = "#show",
  [anon_sym_true] = "true",
  [anon_sym_false] = "false",
  [anon_sym_POUNDdefined] = "#defined",
  [anon_sym_POUNDproject] = "#project",
  [anon_sym_POUNDprogram] = "#program",
  [sym_code] = "code",
  [anon_sym_POUNDscript] = "#script",
  [anon_sym_POUNDend] = "#end",
  [sym_const_type] = "const_type",
  [anon_sym_POUNDconst] = "#const",
  [anon_sym_EQ] = "=",
  [anon_sym_POUNDedge] = "#edge",
  [anon_sym_POUNDheuristic] = "#heuristic",
  [anon_sym_POUNDinclude] = "#include",
  [anon_sym_LT] = "<",
  [anon_sym_GT] = ">",
  [anon_sym_POUNDexternal] = "#external",
  [sym_theory_operator_arity] = "theory_operator_arity",
  [anon_sym_binary] = "binary",
  [sym_theory_operator_associativity] = "theory_operator_associativity",
  [sym_theory_atom_type] = "theory_atom_type",
  [anon_sym_POUNDtheory] = "#theory",
  [sym_empty_pool_item_first] = "empty_pool_item",
  [sym_empty_pool_item] = "empty_pool_item",
  [sym_colon] = ":",
  [sym_block_comment] = "block_comment",
  [sym_doc_fragment_string] = "doc_fragment_string",
  [sym__doc_token_args] = "Args:",
  [sym__doc_token_paren] = "(",
  [sym__doc_token_minus] = "-",
  [sym_source_file] = "source_file",
  [sym_doc_comment] = "doc_comment",
  [sym_variables] = "variables",
  [sym_doc_predicate] = "doc_predicate",
  [sym_doc_desc] = "doc_desc",
  [sym_doc_args] = "doc_args",
  [sym_doc_arg] = "doc_arg",
  [sym_string] = "string",
  [sym_fstring] = "fstring",
  [sym_fstring_accessor] = "fstring_accessor",
  [sym_fstring_conversion] = "fstring_conversion",
  [sym_fstring_width] = "fstring_width",
  [sym_fstring_spec] = "fstring_spec",
  [sym_fstring_field] = "fstring_field",
  [sym__const_term] = "_const_term",
  [sym_const_binary_operation] = "binary_operation",
  [sym_const_unary_operation] = "unary_operation",
  [sym_const_abs] = "abs",
  [sym_const_terms] = "terms",
  [sym__const_arg_pool] = "_const_arg_pool",
  [sym_const_function] = "function",
  [sym__const_term_comma] = "terms",
  [sym__const_terms_trail] = "terms",
  [sym_const_tuple] = "tuple",
  [sym_binary_operation] = "binary_operation",
  [sym_unary_operation] = "unary_operation",
  [sym_abs] = "abs",
  [sym_terms] = "terms",
  [sym_projection] = "projection",
  [sym__term_or_projection] = "_term_or_projection",
  [sym__terms_or_projection] = "terms",
  [sym__arg_pool_n] = "_arg_pool_n",
  [sym__arg_pool] = "_arg_pool",
  [sym_function] = "function",
  [sym_external_function] = "external_function",
  [sym__term_comma] = "terms",
  [sym__terms_trail] = "terms",
  [sym_tuple_pool_item] = "tuple_pool_item",
  [sym__tuple_pool_n] = "_tuple_pool_n",
  [sym_tuple] = "tuple",
  [sym_term] = "term",
  [sym_theory_operators] = "theory_operators",
  [sym_theory_terms] = "theory_terms",
  [sym__theory_arguments] = "_theory_arguments",
  [sym_theory_function] = "theory_function",
  [sym__theory_terms_trail] = "theory_terms",
  [sym_theory_tuple] = "theory_tuple",
  [sym_theory_list] = "theory_list",
  [sym_theory_set] = "theory_set",
  [sym_theory_unparsed_term] = "theory_unparsed_term",
  [sym_theory_term] = "theory_term",
  [sym_theory_root_term] = "theory_root_term",
  [sym_symbolic_atom] = "symbolic_atom",
  [sym_comparison] = "comparison",
  [sym_simple_atom] = "simple_atom",
  [sym_literal_sign] = "literal_sign",
  [sym_literal] = "literal",
  [sym_condition] = "condition",
  [sym_upper] = "upper",
  [sym_lower] = "lower",
  [sym_set_aggregate_element] = "set_aggregate_element",
  [sym_set_aggregate_elements] = "set_aggregate_elements",
  [sym_set_aggregate] = "set_aggregate",
  [sym_theory_element] = "theory_element",
  [sym_theory_elements] = "theory_elements",
  [sym_theory_atom_upper] = "theory_atom_upper",
  [sym_theory_atom] = "theory_atom",
  [sym_body_aggregate_element] = "body_aggregate_element",
  [sym_body_aggregate_elements] = "body_aggregate_elements",
  [sym_body_aggregate] = "body_aggregate",
  [sym_body_literal] = "body_literal",
  [sym_body] = "body",
  [sym_head_aggregate_element] = "head_aggregate_element",
  [sym_head_aggregate_elements] = "head_aggregate_elements",
  [sym_head_aggregate] = "head_aggregate",
  [sym__conditional_literal_n] = "_conditional_literal_n",
  [sym__conditional_literal_0] = "_conditional_literal_0",
  [sym_conditional_literal] = "conditional_literal",
  [sym_disjunction] = "disjunction",
  [sym_head] = "head",
  [sym_rule] = "rule",
  [sym_integrity_constraint] = "integrity_constraint",
  [sym_weight] = "weight",
  [sym_optimize_element] = "optimize_element",
  [sym_optimize_elements] = "optimize_elements",
  [sym_weak_constraint] = "weak_constraint",
  [sym_maximize] = "maximize",
  [sym_minimize] = "minimize",
  [sym_signature] = "signature",
  [sym_show] = "show",
  [sym_show_term] = "show_term",
  [sym_boolean] = "boolean",
  [sym_show_signature] = "show_signature",
  [sym_defined] = "defined",
  [sym_project_signature] = "project_signature",
  [sym_project_atom] = "project_atom",
  [sym_parameters] = "parameters",
  [sym_program] = "program",
  [sym_script] = "script",
  [sym_const] = "const",
  [sym_edge_pair] = "edge_pair",
  [sym_edge] = "edge",
  [sym_heuristic] = "heuristic",
  [sym_include] = "include",
  [sym_external] = "external",
  [sym__theory_operator_arity_binary] = "theory_operator_arity",
  [sym_theory_operator_definition] = "theory_operator_definition",
  [sym_theory_operator_definitions] = "theory_operator_definitions",
  [sym_theory_term_definition] = "theory_term_definition",
  [sym__theory_operators_sep] = "theory_operators",
  [sym_theory_atom_definition] = "theory_atom_definition",
  [sym_theory] = "theory",
  [sym_statement] = "statement",
  [aux_sym_source_file_repeat1] = "source_file_repeat1",
  [aux_sym_variables_repeat1] = "variables_repeat1",
  [aux_sym_doc_desc_repeat1] = "doc_desc_repeat1",
  [aux_sym_doc_args_repeat1] = "doc_args_repeat1",
  [aux_sym_string_repeat1] = "string_repeat1",
  [aux_sym_fstring_repeat1] = "fstring_repeat1",
  [aux_sym_fstring_accessor_repeat1] = "fstring_accessor_repeat1",
  [aux_sym_const_terms_repeat1] = "const_terms_repeat1",
  [aux_sym_abs_repeat1] = "abs_repeat1",
  [aux_sym_terms_repeat1] = "terms_repeat1",
  [aux_sym__terms_or_projection_repeat1] = "_terms_or_projection_repeat1",
  [aux_sym__arg_pool_n_repeat1] = "_arg_pool_n_repeat1",
  [aux_sym__tuple_pool_n_repeat1] = "_tuple_pool_n_repeat1",
  [aux_sym_theory_operators_repeat1] = "theory_operators_repeat1",
  [aux_sym_theory_terms_repeat1] = "theory_terms_repeat1",
  [aux_sym_theory_unparsed_term_repeat1] = "theory_unparsed_term_repeat1",
  [aux_sym_comparison_repeat1] = "comparison_repeat1",
  [aux_sym_condition_repeat1] = "condition_repeat1",
  [aux_sym_set_aggregate_elements_repeat1] = "set_aggregate_elements_repeat1",
  [aux_sym_theory_elements_repeat1] = "theory_elements_repeat1",
  [aux_sym_body_aggregate_elements_repeat1] = "body_aggregate_elements_repeat1",
  [aux_sym_body_repeat1] = "body_repeat1",
  [aux_sym_head_aggregate_elements_repeat1] = "head_aggregate_elements_repeat1",
  [aux_sym_disjunction_repeat1] = "disjunction_repeat1",
  [aux_sym_optimize_elements_repeat1] = "optimize_elements_repeat1",
  [aux_sym_parameters_repeat1] = "parameters_repeat1",
  [aux_sym_edge_repeat1] = "edge_repeat1",
  [aux_sym_theory_operator_definitions_repeat1] = "theory_operator_definitions_repeat1",
  [aux_sym__theory_operators_sep_repeat1] = "_theory_operators_sep_repeat1",
  [aux_sym__theory_definitions_repeat1] = "_theory_definitions_repeat1",
  [aux_sym__theory_definitions_repeat2] = "_theory_definitions_repeat2",
  [alias_sym_classical_negation] = "classical_negation",
  [alias_sym_lone_comma] = "lone_comma",
};

static const TSSymbol ts_symbol_map[] = {
  [ts_builtin_sym_end] = ts_builtin_sym_end,
  [sym_line_comment] = sym_line_comment,
  [anon_sym_PERCENT_STAR_BANG] = anon_sym_PERCENT_STAR_BANG,
  [aux_sym_doc_comment_token1] = aux_sym_doc_comment_token1,
  [anon_sym_STAR_PERCENT] = anon_sym_STAR_PERCENT,
  [anon_sym_COMMA] = anon_sym_COMMA,
  [aux_sym_doc_predicate_token1] = sym_identifier,
  [anon_sym_RPAREN] = anon_sym_RPAREN,
  [sym_doc_var] = sym_variable,
  [sym_doc_fragment_emph] = sym_doc_fragment_emph,
  [sym_doc_fragment_bold] = sym_doc_fragment_bold,
  [sym_doc_fragment_italic] = sym_doc_fragment_italic,
  [sym_doc_fragment_code] = sym_doc_fragment_code,
  [anon_sym_COLON] = anon_sym_COLON,
  [sym_identifier] = sym_identifier,
  [anon_sym_DQUOTE] = anon_sym_DQUOTE,
  [anon_sym_f_DQUOTE] = anon_sym_f_DQUOTE,
  [sym__string_fragment] = sym__string_fragment,
  [sym__fstring_fragment] = sym__string_fragment,
  [sym__string_escape] = sym__string_escape,
  [sym__fstring_escape] = sym__string_escape,
  [aux_sym_fstring_accessor_token1] = sym_identifier,
  [aux_sym_fstring_accessor_token2] = aux_sym_fstring_accessor_token2,
  [aux_sym_fstring_accessor_token3] = aux_sym_fstring_accessor_token3,
  [aux_sym_fstring_accessor_token4] = aux_sym_fstring_accessor_token4,
  [aux_sym_fstring_conversion_token1] = aux_sym_fstring_conversion_token1,
  [aux_sym_fstring_conversion_token2] = aux_sym_fstring_conversion_token2,
  [sym_fstring_align] = sym_fstring_align,
  [sym_fstring_sign] = sym_fstring_sign,
  [sym_fstring_alternate] = sym_fstring_alternate,
  [sym_fstring_grouping] = sym_fstring_grouping,
  [sym_fstring_type] = sym_fstring_type,
  [aux_sym_fstring_spec_token1] = aux_sym_fstring_spec_token1,
  [anon_sym_LBRACE] = anon_sym_LBRACE,
  [anon_sym_RBRACE] = anon_sym_RBRACE,
  [sym_supremum] = sym_supremum,
  [sym_infimum] = sym_infimum,
  [sym_number] = sym_number,
  [sym_anonymous] = sym_anonymous,
  [sym_variable] = sym_variable,
  [anon_sym_CARET] = anon_sym_CARET,
  [anon_sym_QMARK] = anon_sym_QMARK,
  [anon_sym_AMP] = anon_sym_AMP,
  [anon_sym_PLUS] = anon_sym_PLUS,
  [anon_sym_DASH] = anon_sym_DASH,
  [anon_sym_STAR] = anon_sym_STAR,
  [anon_sym_SLASH] = anon_sym_SLASH,
  [anon_sym_BSLASH] = anon_sym_BSLASH,
  [anon_sym_STAR_STAR] = anon_sym_STAR_STAR,
  [anon_sym_TILDE] = anon_sym_TILDE,
  [anon_sym_PIPE] = anon_sym_PIPE,
  [anon_sym_COMMA2] = anon_sym_COMMA,
  [anon_sym_LPAREN] = anon_sym_LPAREN,
  [anon_sym_RPAREN2] = anon_sym_RPAREN,
  [anon_sym_DOT_DOT] = anon_sym_DOT_DOT,
  [anon_sym_SEMI] = anon_sym_SEMI,
  [anon_sym_AT] = anon_sym_AT,
  [sym_theory_operator] = sym_theory_operator,
  [anon_sym_LBRACK] = anon_sym_LBRACK,
  [anon_sym_RBRACK] = anon_sym_RBRACK,
  [anon_sym_RBRACE2] = anon_sym_RBRACE,
  [sym_boolean_constant] = sym_boolean_constant,
  [sym_relation] = sym_relation,
  [sym_default_negation] = sym_default_negation,
  [sym_double_default_negation] = sym_double_default_negation,
  [sym_aggregate_function] = sym_aggregate_function,
  [anon_sym_DOT] = anon_sym_DOT,
  [anon_sym_COLON_DASH] = anon_sym_COLON_DASH,
  [anon_sym_COLON_TILDE] = anon_sym_COLON_TILDE,
  [anon_sym_POUNDmaximize] = anon_sym_POUNDmaximize,
  [anon_sym_POUNDmaximise] = anon_sym_POUNDmaximise,
  [anon_sym_POUNDminimize] = anon_sym_POUNDminimize,
  [anon_sym_POUNDminimise] = anon_sym_POUNDminimise,
  [anon_sym_POUNDshow] = anon_sym_POUNDshow,
  [anon_sym_true] = anon_sym_true,
  [anon_sym_false] = anon_sym_false,
  [anon_sym_POUNDdefined] = anon_sym_POUNDdefined,
  [anon_sym_POUNDproject] = anon_sym_POUNDproject,
  [anon_sym_POUNDprogram] = anon_sym_POUNDprogram,
  [sym_code] = sym_code,
  [anon_sym_POUNDscript] = anon_sym_POUNDscript,
  [anon_sym_POUNDend] = anon_sym_POUNDend,
  [sym_const_type] = sym_const_type,
  [anon_sym_POUNDconst] = anon_sym_POUNDconst,
  [anon_sym_EQ] = anon_sym_EQ,
  [anon_sym_POUNDedge] = anon_sym_POUNDedge,
  [anon_sym_POUNDheuristic] = anon_sym_POUNDheuristic,
  [anon_sym_POUNDinclude] = anon_sym_POUNDinclude,
  [anon_sym_LT] = anon_sym_LT,
  [anon_sym_GT] = anon_sym_GT,
  [anon_sym_POUNDexternal] = anon_sym_POUNDexternal,
  [sym_theory_operator_arity] = sym_theory_operator_arity,
  [anon_sym_binary] = anon_sym_binary,
  [sym_theory_operator_associativity] = sym_theory_operator_associativity,
  [sym_theory_atom_type] = sym_theory_atom_type,
  [anon_sym_POUNDtheory] = anon_sym_POUNDtheory,
  [sym_empty_pool_item_first] = sym_empty_pool_item,
  [sym_empty_pool_item] = sym_empty_pool_item,
  [sym_colon] = anon_sym_COLON,
  [sym_block_comment] = sym_block_comment,
  [sym_doc_fragment_string] = sym_doc_fragment_string,
  [sym__doc_token_args] = sym__doc_token_args,
  [sym__doc_token_paren] = anon_sym_LPAREN,
  [sym__doc_token_minus] = anon_sym_DASH,
  [sym_source_file] = sym_source_file,
  [sym_doc_comment] = sym_doc_comment,
  [sym_variables] = sym_variables,
  [sym_doc_predicate] = sym_doc_predicate,
  [sym_doc_desc] = sym_doc_desc,
  [sym_doc_args] = sym_doc_args,
  [sym_doc_arg] = sym_doc_arg,
  [sym_string] = sym_string,
  [sym_fstring] = sym_fstring,
  [sym_fstring_accessor] = sym_fstring_accessor,
  [sym_fstring_conversion] = sym_fstring_conversion,
  [sym_fstring_width] = sym_fstring_width,
  [sym_fstring_spec] = sym_fstring_spec,
  [sym_fstring_field] = sym_fstring_field,
  [sym__const_term] = sym__const_term,
  [sym_const_binary_operation] = sym_binary_operation,
  [sym_const_unary_operation] = sym_unary_operation,
  [sym_const_abs] = sym_abs,
  [sym_const_terms] = sym_terms,
  [sym__const_arg_pool] = sym__const_arg_pool,
  [sym_const_function] = sym_function,
  [sym__const_term_comma] = sym_terms,
  [sym__const_terms_trail] = sym_terms,
  [sym_const_tuple] = sym_tuple,
  [sym_binary_operation] = sym_binary_operation,
  [sym_unary_operation] = sym_unary_operation,
  [sym_abs] = sym_abs,
  [sym_terms] = sym_terms,
  [sym_projection] = sym_projection,
  [sym__term_or_projection] = sym__term_or_projection,
  [sym__terms_or_projection] = sym_terms,
  [sym__arg_pool_n] = sym__arg_pool_n,
  [sym__arg_pool] = sym__arg_pool,
  [sym_function] = sym_function,
  [sym_external_function] = sym_external_function,
  [sym__term_comma] = sym_terms,
  [sym__terms_trail] = sym_terms,
  [sym_tuple_pool_item] = sym_tuple_pool_item,
  [sym__tuple_pool_n] = sym__tuple_pool_n,
  [sym_tuple] = sym_tuple,
  [sym_term] = sym_term,
  [sym_theory_operators] = sym_theory_operators,
  [sym_theory_terms] = sym_theory_terms,
  [sym__theory_arguments] = sym__theory_arguments,
  [sym_theory_function] = sym_theory_function,
  [sym__theory_terms_trail] = sym_theory_terms,
  [sym_theory_tuple] = sym_theory_tuple,
  [sym_theory_list] = sym_theory_list,
  [sym_theory_set] = sym_theory_set,
  [sym_theory_unparsed_term] = sym_theory_unparsed_term,
  [sym_theory_term] = sym_theory_term,
  [sym_theory_root_term] = sym_theory_root_term,
  [sym_symbolic_atom] = sym_symbolic_atom,
  [sym_comparison] = sym_comparison,
  [sym_simple_atom] = sym_simple_atom,
  [sym_literal_sign] = sym_literal_sign,
  [sym_literal] = sym_literal,
  [sym_condition] = sym_condition,
  [sym_upper] = sym_upper,
  [sym_lower] = sym_lower,
  [sym_set_aggregate_element] = sym_set_aggregate_element,
  [sym_set_aggregate_elements] = sym_set_aggregate_elements,
  [sym_set_aggregate] = sym_set_aggregate,
  [sym_theory_element] = sym_theory_element,
  [sym_theory_elements] = sym_theory_elements,
  [sym_theory_atom_upper] = sym_theory_atom_upper,
  [sym_theory_atom] = sym_theory_atom,
  [sym_body_aggregate_element] = sym_body_aggregate_element,
  [sym_body_aggregate_elements] = sym_body_aggregate_elements,
  [sym_body_aggregate] = sym_body_aggregate,
  [sym_body_literal] = sym_body_literal,
  [sym_body] = sym_body,
  [sym_head_aggregate_element] = sym_head_aggregate_element,
  [sym_head_aggregate_elements] = sym_head_aggregate_elements,
  [sym_head_aggregate] = sym_head_aggregate,
  [sym__conditional_literal_n] = sym__conditional_literal_n,
  [sym__conditional_literal_0] = sym__conditional_literal_0,
  [sym_conditional_literal] = sym_conditional_literal,
  [sym_disjunction] = sym_disjunction,
  [sym_head] = sym_head,
  [sym_rule] = sym_rule,
  [sym_integrity_constraint] = sym_integrity_constraint,
  [sym_weight] = sym_weight,
  [sym_optimize_element] = sym_optimize_element,
  [sym_optimize_elements] = sym_optimize_elements,
  [sym_weak_constraint] = sym_weak_constraint,
  [sym_maximize] = sym_maximize,
  [sym_minimize] = sym_minimize,
  [sym_signature] = sym_signature,
  [sym_show] = sym_show,
  [sym_show_term] = sym_show_term,
  [sym_boolean] = sym_boolean,
  [sym_show_signature] = sym_show_signature,
  [sym_defined] = sym_defined,
  [sym_project_signature] = sym_project_signature,
  [sym_project_atom] = sym_project_atom,
  [sym_parameters] = sym_parameters,
  [sym_program] = sym_program,
  [sym_script] = sym_script,
  [sym_const] = sym_const,
  [sym_edge_pair] = sym_edge_pair,
  [sym_edge] = sym_edge,
  [sym_heuristic] = sym_heuristic,
  [sym_include] = sym_include,
  [sym_external] = sym_external,
  [sym__theory_operator_arity_binary] = sym_theory_operator_arity,
  [sym_theory_operator_definition] = sym_theory_operator_definition,
  [sym_theory_operator_definitions] = sym_theory_operator_definitions,
  [sym_theory_term_definition] = sym_theory_term_definition,
  [sym__theory_operators_sep] = sym_theory_operators,
  [sym_theory_atom_definition] = sym_theory_atom_definition,
  [sym_theory] = sym_theory,
  [sym_statement] = sym_statement,
  [aux_sym_source_file_repeat1] = aux_sym_source_file_repeat1,
  [aux_sym_variables_repeat1] = aux_sym_variables_repeat1,
  [aux_sym_doc_desc_repeat1] = aux_sym_doc_desc_repeat1,
  [aux_sym_doc_args_repeat1] = aux_sym_doc_args_repeat1,
  [aux_sym_string_repeat1] = aux_sym_string_repeat1,
  [aux_sym_fstring_repeat1] = aux_sym_fstring_repeat1,
  [aux_sym_fstring_accessor_repeat1] = aux_sym_fstring_accessor_repeat1,
  [aux_sym_const_terms_repeat1] = aux_sym_const_terms_repeat1,
  [aux_sym_abs_repeat1] = aux_sym_abs_repeat1,
  [aux_sym_terms_repeat1] = aux_sym_terms_repeat1,
  [aux_sym__terms_or_projection_repeat1] = aux_sym__terms_or_projection_repeat1,
  [aux_sym__arg_pool_n_repeat1] = aux_sym__arg_pool_n_repeat1,
  [aux_sym__tuple_pool_n_repeat1] = aux_sym__tuple_pool_n_repeat1,
  [aux_sym_theory_operators_repeat1] = aux_sym_theory_operators_repeat1,
  [aux_sym_theory_terms_repeat1] = aux_sym_theory_terms_repeat1,
  [aux_sym_theory_unparsed_term_repeat1] = aux_sym_theory_unparsed_term_repeat1,
  [aux_sym_comparison_repeat1] = aux_sym_comparison_repeat1,
  [aux_sym_condition_repeat1] = aux_sym_condition_repeat1,
  [aux_sym_set_aggregate_elements_repeat1] = aux_sym_set_aggregate_elements_repeat1,
  [aux_sym_theory_elements_repeat1] = aux_sym_theory_elements_repeat1,
  [aux_sym_body_aggregate_elements_repeat1] = aux_sym_body_aggregate_elements_repeat1,
  [aux_sym_body_repeat1] = aux_sym_body_repeat1,
  [aux_sym_head_aggregate_elements_repeat1] = aux_sym_head_aggregate_elements_repeat1,
  [aux_sym_disjunction_repeat1] = aux_sym_disjunction_repeat1,
  [aux_sym_optimize_elements_repeat1] = aux_sym_optimize_elements_repeat1,
  [aux_sym_parameters_repeat1] = aux_sym_parameters_repeat1,
  [aux_sym_edge_repeat1] = aux_sym_edge_repeat1,
  [aux_sym_theory_operator_definitions_repeat1] = aux_sym_theory_operator_definitions_repeat1,
  [aux_sym__theory_operators_sep_repeat1] = aux_sym__theory_operators_sep_repeat1,
  [aux_sym__theory_definitions_repeat1] = aux_sym__theory_definitions_repeat1,
  [aux_sym__theory_definitions_repeat2] = aux_sym__theory_definitions_repeat2,
  [alias_sym_classical_negation] = alias_sym_classical_negation,
  [alias_sym_lone_comma] = alias_sym_lone_comma,
};

static const TSSymbolMetadata ts_symbol_metadata[] = {
  [ts_builtin_sym_end] = {
    .visible = false,
    .named = true,
  },
  [sym_line_comment] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_PERCENT_STAR_BANG] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_doc_comment_token1] = {
    .visible = false,
    .named = false,
  },
  [anon_sym_STAR_PERCENT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_COMMA] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_doc_predicate_token1] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_RPAREN] = {
    .visible = true,
    .named = false,
  },
  [sym_doc_var] = {
    .visible = true,
    .named = true,
  },
  [sym_doc_fragment_emph] = {
    .visible = true,
    .named = true,
  },
  [sym_doc_fragment_bold] = {
    .visible = true,
    .named = true,
  },
  [sym_doc_fragment_italic] = {
    .visible = true,
    .named = true,
  },
  [sym_doc_fragment_code] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_COLON] = {
    .visible = true,
    .named = false,
  },
  [sym_identifier] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_DQUOTE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_f_DQUOTE] = {
    .visible = true,
    .named = false,
  },
  [sym__string_fragment] = {
    .visible = true,
    .named = true,
  },
  [sym__fstring_fragment] = {
    .visible = true,
    .named = true,
  },
  [sym__string_escape] = {
    .visible = true,
    .named = true,
  },
  [sym__fstring_escape] = {
    .visible = true,
    .named = true,
  },
  [aux_sym_fstring_accessor_token1] = {
    .visible = true,
    .named = true,
  },
  [aux_sym_fstring_accessor_token2] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_fstring_accessor_token3] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_fstring_accessor_token4] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_fstring_conversion_token1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_fstring_conversion_token2] = {
    .visible = false,
    .named = false,
  },
  [sym_fstring_align] = {
    .visible = true,
    .named = true,
  },
  [sym_fstring_sign] = {
    .visible = true,
    .named = true,
  },
  [sym_fstring_alternate] = {
    .visible = true,
    .named = true,
  },
  [sym_fstring_grouping] = {
    .visible = true,
    .named = true,
  },
  [sym_fstring_type] = {
    .visible = true,
    .named = true,
  },
  [aux_sym_fstring_spec_token1] = {
    .visible = false,
    .named = false,
  },
  [anon_sym_LBRACE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RBRACE] = {
    .visible = true,
    .named = false,
  },
  [sym_supremum] = {
    .visible = true,
    .named = true,
  },
  [sym_infimum] = {
    .visible = true,
    .named = true,
  },
  [sym_number] = {
    .visible = true,
    .named = true,
  },
  [sym_anonymous] = {
    .visible = true,
    .named = true,
  },
  [sym_variable] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_CARET] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_QMARK] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AMP] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PLUS] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DASH] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STAR] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SLASH] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_BSLASH] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STAR_STAR] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_TILDE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PIPE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_COMMA2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LPAREN] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RPAREN2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DOT_DOT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SEMI] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AT] = {
    .visible = true,
    .named = false,
  },
  [sym_theory_operator] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_LBRACK] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RBRACK] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RBRACE2] = {
    .visible = true,
    .named = false,
  },
  [sym_boolean_constant] = {
    .visible = true,
    .named = true,
  },
  [sym_relation] = {
    .visible = true,
    .named = true,
  },
  [sym_default_negation] = {
    .visible = true,
    .named = true,
  },
  [sym_double_default_negation] = {
    .visible = true,
    .named = true,
  },
  [sym_aggregate_function] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_DOT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_COLON_DASH] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_COLON_TILDE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_POUNDmaximize] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_POUNDmaximise] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_POUNDminimize] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_POUNDminimise] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_POUNDshow] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_true] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_false] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_POUNDdefined] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_POUNDproject] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_POUNDprogram] = {
    .visible = true,
    .named = false,
  },
  [sym_code] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_POUNDscript] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_POUNDend] = {
    .visible = true,
    .named = false,
  },
  [sym_const_type] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_POUNDconst] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_POUNDedge] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_POUNDheuristic] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_POUNDinclude] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_POUNDexternal] = {
    .visible = true,
    .named = false,
  },
  [sym_theory_operator_arity] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_binary] = {
    .visible = true,
    .named = false,
  },
  [sym_theory_operator_associativity] = {
    .visible = true,
    .named = true,
  },
  [sym_theory_atom_type] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_POUNDtheory] = {
    .visible = true,
    .named = false,
  },
  [sym_empty_pool_item_first] = {
    .visible = true,
    .named = true,
  },
  [sym_empty_pool_item] = {
    .visible = true,
    .named = true,
  },
  [sym_colon] = {
    .visible = true,
    .named = false,
  },
  [sym_block_comment] = {
    .visible = true,
    .named = true,
  },
  [sym_doc_fragment_string] = {
    .visible = true,
    .named = true,
  },
  [sym__doc_token_args] = {
    .visible = true,
    .named = false,
  },
  [sym__doc_token_paren] = {
    .visible = true,
    .named = false,
  },
  [sym__doc_token_minus] = {
    .visible = true,
    .named = false,
  },
  [sym_source_file] = {
    .visible = true,
    .named = true,
  },
  [sym_doc_comment] = {
    .visible = true,
    .named = true,
  },
  [sym_variables] = {
    .visible = true,
    .named = true,
  },
  [sym_doc_predicate] = {
    .visible = true,
    .named = true,
  },
  [sym_doc_desc] = {
    .visible = true,
    .named = true,
  },
  [sym_doc_args] = {
    .visible = true,
    .named = true,
  },
  [sym_doc_arg] = {
    .visible = true,
    .named = true,
  },
  [sym_string] = {
    .visible = true,
    .named = true,
  },
  [sym_fstring] = {
    .visible = true,
    .named = true,
  },
  [sym_fstring_accessor] = {
    .visible = true,
    .named = true,
  },
  [sym_fstring_conversion] = {
    .visible = true,
    .named = true,
  },
  [sym_fstring_width] = {
    .visible = true,
    .named = true,
  },
  [sym_fstring_spec] = {
    .visible = true,
    .named = true,
  },
  [sym_fstring_field] = {
    .visible = true,
    .named = true,
  },
  [sym__const_term] = {
    .visible = false,
    .named = true,
  },
  [sym_const_binary_operation] = {
    .visible = true,
    .named = true,
  },
  [sym_const_unary_operation] = {
    .visible = true,
    .named = true,
  },
  [sym_const_abs] = {
    .visible = true,
    .named = true,
  },
  [sym_const_terms] = {
    .visible = true,
    .named = true,
  },
  [sym__const_arg_pool] = {
    .visible = false,
    .named = true,
  },
  [sym_const_function] = {
    .visible = true,
    .named = true,
  },
  [sym__const_term_comma] = {
    .visible = true,
    .named = true,
  },
  [sym__const_terms_trail] = {
    .visible = true,
    .named = true,
  },
  [sym_const_tuple] = {
    .visible = true,
    .named = true,
  },
  [sym_binary_operation] = {
    .visible = true,
    .named = true,
  },
  [sym_unary_operation] = {
    .visible = true,
    .named = true,
  },
  [sym_abs] = {
    .visible = true,
    .named = true,
  },
  [sym_terms] = {
    .visible = true,
    .named = true,
  },
  [sym_projection] = {
    .visible = true,
    .named = true,
  },
  [sym__term_or_projection] = {
    .visible = false,
    .named = true,
  },
  [sym__terms_or_projection] = {
    .visible = true,
    .named = true,
  },
  [sym__arg_pool_n] = {
    .visible = false,
    .named = true,
  },
  [sym__arg_pool] = {
    .visible = false,
    .named = true,
  },
  [sym_function] = {
    .visible = true,
    .named = true,
  },
  [sym_external_function] = {
    .visible = true,
    .named = true,
  },
  [sym__term_comma] = {
    .visible = true,
    .named = true,
  },
  [sym__terms_trail] = {
    .visible = true,
    .named = true,
  },
  [sym_tuple_pool_item] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym__tuple_pool_n] = {
    .visible = false,
    .named = true,
  },
  [sym_tuple] = {
    .visible = true,
    .named = true,
  },
  [sym_term] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym_theory_operators] = {
    .visible = true,
    .named = true,
  },
  [sym_theory_terms] = {
    .visible = true,
    .named = true,
  },
  [sym__theory_arguments] = {
    .visible = false,
    .named = true,
  },
  [sym_theory_function] = {
    .visible = true,
    .named = true,
  },
  [sym__theory_terms_trail] = {
    .visible = true,
    .named = true,
  },
  [sym_theory_tuple] = {
    .visible = true,
    .named = true,
  },
  [sym_theory_list] = {
    .visible = true,
    .named = true,
  },
  [sym_theory_set] = {
    .visible = true,
    .named = true,
  },
  [sym_theory_unparsed_term] = {
    .visible = true,
    .named = true,
  },
  [sym_theory_term] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym_theory_root_term] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym_symbolic_atom] = {
    .visible = true,
    .named = true,
  },
  [sym_comparison] = {
    .visible = true,
    .named = true,
  },
  [sym_simple_atom] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym_literal_sign] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym_literal] = {
    .visible = true,
    .named = true,
  },
  [sym_condition] = {
    .visible = true,
    .named = true,
  },
  [sym_upper] = {
    .visible = true,
    .named = true,
  },
  [sym_lower] = {
    .visible = true,
    .named = true,
  },
  [sym_set_aggregate_element] = {
    .visible = true,
    .named = true,
  },
  [sym_set_aggregate_elements] = {
    .visible = true,
    .named = true,
  },
  [sym_set_aggregate] = {
    .visible = true,
    .named = true,
  },
  [sym_theory_element] = {
    .visible = true,
    .named = true,
  },
  [sym_theory_elements] = {
    .visible = true,
    .named = true,
  },
  [sym_theory_atom_upper] = {
    .visible = true,
    .named = true,
  },
  [sym_theory_atom] = {
    .visible = true,
    .named = true,
  },
  [sym_body_aggregate_element] = {
    .visible = true,
    .named = true,
  },
  [sym_body_aggregate_elements] = {
    .visible = true,
    .named = true,
  },
  [sym_body_aggregate] = {
    .visible = true,
    .named = true,
  },
  [sym_body_literal] = {
    .visible = true,
    .named = true,
  },
  [sym_body] = {
    .visible = true,
    .named = true,
  },
  [sym_head_aggregate_element] = {
    .visible = true,
    .named = true,
  },
  [sym_head_aggregate_elements] = {
    .visible = true,
    .named = true,
  },
  [sym_head_aggregate] = {
    .visible = true,
    .named = true,
  },
  [sym__conditional_literal_n] = {
    .visible = false,
    .named = true,
  },
  [sym__conditional_literal_0] = {
    .visible = false,
    .named = true,
  },
  [sym_conditional_literal] = {
    .visible = true,
    .named = true,
  },
  [sym_disjunction] = {
    .visible = true,
    .named = true,
  },
  [sym_head] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym_rule] = {
    .visible = true,
    .named = true,
  },
  [sym_integrity_constraint] = {
    .visible = true,
    .named = true,
  },
  [sym_weight] = {
    .visible = true,
    .named = true,
  },
  [sym_optimize_element] = {
    .visible = true,
    .named = true,
  },
  [sym_optimize_elements] = {
    .visible = true,
    .named = true,
  },
  [sym_weak_constraint] = {
    .visible = true,
    .named = true,
  },
  [sym_maximize] = {
    .visible = true,
    .named = true,
  },
  [sym_minimize] = {
    .visible = true,
    .named = true,
  },
  [sym_signature] = {
    .visible = true,
    .named = true,
  },
  [sym_show] = {
    .visible = true,
    .named = true,
  },
  [sym_show_term] = {
    .visible = true,
    .named = true,
  },
  [sym_boolean] = {
    .visible = true,
    .named = true,
  },
  [sym_show_signature] = {
    .visible = true,
    .named = true,
  },
  [sym_defined] = {
    .visible = true,
    .named = true,
  },
  [sym_project_signature] = {
    .visible = true,
    .named = true,
  },
  [sym_project_atom] = {
    .visible = true,
    .named = true,
  },
  [sym_parameters] = {
    .visible = true,
    .named = true,
  },
  [sym_program] = {
    .visible = true,
    .named = true,
  },
  [sym_script] = {
    .visible = true,
    .named = true,
  },
  [sym_const] = {
    .visible = true,
    .named = true,
  },
  [sym_edge_pair] = {
    .visible = true,
    .named = true,
  },
  [sym_edge] = {
    .visible = true,
    .named = true,
  },
  [sym_heuristic] = {
    .visible = true,
    .named = true,
  },
  [sym_include] = {
    .visible = true,
    .named = true,
  },
  [sym_external] = {
    .visible = true,
    .named = true,
  },
  [sym__theory_operator_arity_binary] = {
    .visible = true,
    .named = true,
  },
  [sym_theory_operator_definition] = {
    .visible = true,
    .named = true,
  },
  [sym_theory_operator_definitions] = {
    .visible = true,
    .named = true,
  },
  [sym_theory_term_definition] = {
    .visible = true,
    .named = true,
  },
  [sym__theory_operators_sep] = {
    .visible = true,
    .named = true,
  },
  [sym_theory_atom_definition] = {
    .visible = true,
    .named = true,
  },
  [sym_theory] = {
    .visible = true,
    .named = true,
  },
  [sym_statement] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [aux_sym_source_file_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_variables_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_doc_desc_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_doc_args_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_string_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_fstring_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_fstring_accessor_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_const_terms_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_abs_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_terms_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__terms_or_projection_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__arg_pool_n_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__tuple_pool_n_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_theory_operators_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_theory_terms_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_theory_unparsed_term_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_comparison_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_condition_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_set_aggregate_elements_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_theory_elements_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_body_aggregate_elements_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_body_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_head_aggregate_elements_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_disjunction_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_optimize_elements_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_parameters_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_edge_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_theory_operator_definitions_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__theory_operators_sep_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__theory_definitions_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__theory_definitions_repeat2] = {
    .visible = false,
    .named = false,
  },
  [alias_sym_classical_negation] = {
    .visible = true,
    .named = true,
  },
  [alias_sym_lone_comma] = {
    .visible = true,
    .named = true,
  },
};

enum ts_field_identifiers {
  field_arguments = 1,
  field_arity = 2,
  field_associativity = 3,
  field_atom = 4,
  field_atom_type = 5,
  field_body = 6,
  field_code = 7,
  field_condition = 8,
  field_description = 9,
  field_edge_pair = 10,
  field_elements = 11,
  field_function = 12,
  field_guard = 13,
  field_head = 14,
  field_language = 15,
  field_left = 16,
  field_literal = 17,
  field_name = 18,
  field_operator = 19,
  field_operators = 20,
  field_parameters = 21,
  field_predicate = 22,
  field_priority = 23,
  field_right = 24,
  field_sign = 25,
  field_signature = 26,
  field_term = 27,
  field_terms = 28,
  field_theory_term_name = 29,
  field_theory_terms = 30,
  field_type = 31,
  field_value = 32,
  field_variable = 33,
  field_variables = 34,
  field_weight = 35,
};

static const char * const ts_field_names[] = {
  [0] = NULL,
  [field_arguments] = "arguments",
  [field_arity] = "arity",
  [field_associativity] = "associativity",
  [field_atom] = "atom",
  [field_atom_type] = "atom_type",
  [field_body] = "body",
  [field_code] = "code",
  [field_condition] = "condition",
  [field_description] = "description",
  [field_edge_pair] = "edge_pair",
  [field_elements] = "elements",
  [field_function] = "function",
  [field_guard] = "guard",
  [field_head] = "head",
  [field_language] = "language",
  [field_left] = "left",
  [field_literal] = "literal",
  [field_name] = "name",
  [field_operator] = "operator",
  [field_operators] = "operators",
  [field_parameters] = "parameters",
  [field_predicate] = "predicate",
  [field_priority] = "priority",
  [field_right] = "right",
  [field_sign] = "sign",
  [field_signature] = "signature",
  [field_term] = "term",
  [field_terms] = "terms",
  [field_theory_term_name] = "theory_term_name",
  [field_theory_terms] = "theory_terms",
  [field_type] = "type",
  [field_value] = "value",
  [field_variable] = "variable",
  [field_variables] = "variables",
  [field_weight] = "weight",
};

static const TSMapSlice ts_field_map_slices[PRODUCTION_ID_COUNT] = {
  [1] = {.index = 0, .length = 1},
  [2] = {.index = 1, .length = 1},
  [3] = {.index = 2, .length = 2},
  [4] = {.index = 4, .length = 1},
  [5] = {.index = 5, .length = 2},
  [6] = {.index = 7, .length = 1},
  [7] = {.index = 8, .length = 1},
  [8] = {.index = 9, .length = 2},
  [9] = {.index = 11, .length = 2},
  [11] = {.index = 13, .length = 2},
  [12] = {.index = 2, .length = 2},
  [13] = {.index = 4, .length = 1},
  [14] = {.index = 15, .length = 1},
  [15] = {.index = 16, .length = 1},
  [16] = {.index = 17, .length = 1},
  [17] = {.index = 18, .length = 3},
  [18] = {.index = 21, .length = 2},
  [19] = {.index = 23, .length = 3},
  [20] = {.index = 26, .length = 2},
  [21] = {.index = 28, .length = 1},
  [22] = {.index = 29, .length = 1},
  [23] = {.index = 30, .length = 1},
  [24] = {.index = 31, .length = 1},
  [25] = {.index = 32, .length = 1},
  [26] = {.index = 33, .length = 1},
  [27] = {.index = 34, .length = 1},
  [28] = {.index = 35, .length = 3},
  [29] = {.index = 38, .length = 2},
  [30] = {.index = 40, .length = 1},
  [31] = {.index = 41, .length = 1},
  [32] = {.index = 42, .length = 2},
  [33] = {.index = 44, .length = 1},
  [34] = {.index = 45, .length = 1},
  [35] = {.index = 46, .length = 2},
  [37] = {.index = 48, .length = 1},
  [38] = {.index = 49, .length = 4},
  [39] = {.index = 53, .length = 1},
  [40] = {.index = 54, .length = 2},
  [41] = {.index = 56, .length = 2},
  [42] = {.index = 58, .length = 1},
  [43] = {.index = 59, .length = 2},
  [44] = {.index = 61, .length = 2},
  [45] = {.index = 63, .length = 2},
  [46] = {.index = 65, .length = 2},
  [47] = {.index = 67, .length = 2},
  [48] = {.index = 69, .length = 2},
  [49] = {.index = 71, .length = 2},
  [50] = {.index = 73, .length = 2},
  [51] = {.index = 75, .length = 1},
  [52] = {.index = 76, .length = 2},
  [53] = {.index = 78, .length = 2},
  [54] = {.index = 80, .length = 2},
  [55] = {.index = 82, .length = 3},
  [56] = {.index = 85, .length = 1},
  [57] = {.index = 86, .length = 2},
  [58] = {.index = 88, .length = 2},
  [59] = {.index = 90, .length = 3},
  [60] = {.index = 93, .length = 1},
  [61] = {.index = 94, .length = 3},
  [62] = {.index = 97, .length = 2},
  [63] = {.index = 99, .length = 2},
  [64] = {.index = 101, .length = 2},
  [65] = {.index = 103, .length = 1},
  [66] = {.index = 104, .length = 1},
  [67] = {.index = 105, .length = 2},
  [68] = {.index = 107, .length = 3},
  [69] = {.index = 110, .length = 3},
  [70] = {.index = 113, .length = 3},
  [71] = {.index = 116, .length = 3},
  [73] = {.index = 119, .length = 2},
  [74] = {.index = 121, .length = 3},
  [75] = {.index = 124, .length = 4},
  [76] = {.index = 128, .length = 4},
  [77] = {.index = 132, .length = 2},
  [78] = {.index = 134, .length = 2},
  [79] = {.index = 136, .length = 2},
  [80] = {.index = 138, .length = 2},
  [81] = {.index = 140, .length = 2},
  [83] = {.index = 142, .length = 2},
  [84] = {.index = 144, .length = 2},
  [85] = {.index = 146, .length = 4},
  [86] = {.index = 150, .length = 5},
  [87] = {.index = 155, .length = 3},
  [88] = {.index = 158, .length = 3},
  [89] = {.index = 161, .length = 4},
  [90] = {.index = 165, .length = 2},
  [91] = {.index = 167, .length = 2},
  [92] = {.index = 169, .length = 2},
  [93] = {.index = 171, .length = 2},
  [94] = {.index = 173, .length = 4},
  [95] = {.index = 177, .length = 3},
  [96] = {.index = 180, .length = 3},
  [97] = {.index = 183, .length = 3},
  [98] = {.index = 186, .length = 3},
  [99] = {.index = 189, .length = 2},
  [100] = {.index = 191, .length = 3},
  [101] = {.index = 194, .length = 4},
  [102] = {.index = 198, .length = 3},
  [103] = {.index = 198, .length = 3},
  [104] = {.index = 201, .length = 1},
  [105] = {.index = 202, .length = 4},
  [106] = {.index = 206, .length = 2},
  [107] = {.index = 208, .length = 4},
  [108] = {.index = 208, .length = 4},
  [110] = {.index = 212, .length = 5},
  [111] = {.index = 217, .length = 6},
};

static const TSFieldMapEntry ts_field_map_entries[] = {
  [0] =
    {field_name, 0},
  [1] =
    {field_atom, 0},
  [2] =
    {field_condition, 0, .inherited = true},
    {field_literal, 0, .inherited = true},
  [4] =
    {field_literal, 0, .inherited = true},
  [5] =
    {field_arguments, 1, .inherited = true},
    {field_name, 0},
  [7] =
    {field_literal, 0},
  [8] =
    {field_name, 1},
  [9] =
    {field_name, 1},
    {field_sign, 0},
  [11] =
    {field_operator, 0},
    {field_right, 1},
  [13] =
    {field_atom, 1},
    {field_sign, 0},
  [15] =
    {field_head, 0},
  [16] =
    {field_right, 2},
  [17] =
    {field_elements, 1},
  [18] =
    {field_arguments, 2},
    {field_arguments, 2, .inherited = true},
    {field_name, 1},
  [21] =
    {field_name, 1},
    {field_right, 2},
  [23] =
    {field_arguments, 2, .inherited = true},
    {field_name, 1},
    {field_sign, 0},
  [26] =
    {field_arguments, 2, .inherited = true},
    {field_name, 1},
  [28] =
    {field_function, 0},
  [29] =
    {field_body, 1},
  [30] =
    {field_term, 0},
  [31] =
    {field_weight, 0},
  [32] =
    {field_term, 1},
  [33] =
    {field_signature, 1},
  [34] =
    {field_atom, 1},
  [35] =
    {field_left, 0},
    {field_operator, 1},
    {field_right, 2},
  [38] =
    {field_condition, 2},
    {field_literal, 0},
  [40] =
    {field_left, 0},
  [41] =
    {field_predicate, 2},
  [42] =
    {field_arguments, 0},
    {field_arguments, 1, .inherited = true},
  [44] =
    {field_arguments, 1},
  [45] =
    {field_arguments, 1, .inherited = true},
  [46] =
    {field_elements, 1},
    {field_right, 3},
  [48] =
    {field_theory_terms, 0},
  [49] =
    {field_arguments, 2},
    {field_arguments, 2, .inherited = true},
    {field_name, 1},
    {field_right, 3},
  [53] =
    {field_literal, 1},
  [54] =
    {field_function, 0},
    {field_right, 3},
  [56] =
    {field_elements, 2},
    {field_function, 0},
  [58] =
    {field_terms, 0},
  [59] =
    {field_arity, 2},
    {field_name, 0},
  [61] =
    {field_left, 0},
    {field_right, 3},
  [63] =
    {field_elements, 2},
    {field_left, 0},
  [65] =
    {field_function, 1},
    {field_left, 0},
  [67] =
    {field_body, 2},
    {field_head, 0},
  [69] =
    {field_description, 3},
    {field_predicate, 2},
  [71] =
    {field_arguments, 3},
    {field_predicate, 2},
  [73] =
    {field_arguments, 0, .inherited = true},
    {field_arguments, 1, .inherited = true},
  [75] =
    {field_condition, 1},
  [76] =
    {field_name, 1},
    {field_right, 4},
  [78] =
    {field_elements, 3},
    {field_name, 1},
  [80] =
    {field_literal, 2},
    {field_terms, 0},
  [82] =
    {field_elements, 2},
    {field_function, 0},
    {field_right, 4},
  [85] =
    {field_weight, 3},
  [86] =
    {field_priority, 2},
    {field_term, 0},
  [88] =
    {field_condition, 2},
    {field_weight, 0},
  [90] =
    {field_terms, 1},
    {field_terms, 2},
    {field_weight, 0},
  [93] =
    {field_elements, 2},
  [94] =
    {field_arity, 3},
    {field_name, 1},
    {field_sign, 0},
  [97] =
    {field_body, 3},
    {field_term, 1},
  [99] =
    {field_atom, 1},
    {field_body, 3},
  [101] =
    {field_name, 1},
    {field_value, 3},
  [103] =
    {field_edge_pair, 2},
  [104] =
    {field_edge_pair, 1},
  [105] =
    {field_edge_pair, 0, .inherited = true},
    {field_edge_pair, 1, .inherited = true},
  [107] =
    {field_elements, 2},
    {field_left, 0},
    {field_right, 4},
  [110] =
    {field_function, 1},
    {field_left, 0},
    {field_right, 4},
  [113] =
    {field_elements, 3},
    {field_function, 1},
    {field_left, 0},
  [116] =
    {field_arguments, 4},
    {field_description, 3},
    {field_predicate, 2},
  [119] =
    {field_condition, 2},
    {field_theory_terms, 0},
  [121] =
    {field_elements, 3},
    {field_name, 1},
    {field_right, 5},
  [124] =
    {field_arguments, 2},
    {field_arguments, 2, .inherited = true},
    {field_name, 1},
    {field_right, 5},
  [128] =
    {field_arguments, 2},
    {field_arguments, 2, .inherited = true},
    {field_elements, 4},
    {field_name, 1},
  [132] =
    {field_condition, 3},
    {field_literal, 1},
  [134] =
    {field_condition, 2},
    {field_terms, 0},
  [136] =
    {field_body, 1},
    {field_weight, 4},
  [138] =
    {field_signature, 1},
    {field_value, 4},
  [140] =
    {field_name, 1},
    {field_parameters, 3},
  [142] =
    {field_edge_pair, 2},
    {field_edge_pair, 3, .inherited = true},
  [144] =
    {field_atom, 1},
    {field_type, 4},
  [146] =
    {field_elements, 3},
    {field_function, 1},
    {field_left, 0},
    {field_right, 5},
  [150] =
    {field_arguments, 2},
    {field_arguments, 2, .inherited = true},
    {field_elements, 4},
    {field_name, 1},
    {field_right, 6},
  [155] =
    {field_condition, 4},
    {field_literal, 2},
    {field_terms, 0},
  [158] =
    {field_terms, 4},
    {field_terms, 5},
    {field_weight, 3},
  [161] =
    {field_condition, 4},
    {field_terms, 1},
    {field_terms, 2},
    {field_weight, 0},
  [165] =
    {field_code, 4},
    {field_language, 2},
  [167] =
    {field_body, 5},
    {field_edge_pair, 2},
  [169] =
    {field_atom, 1},
    {field_type, 5},
  [171] =
    {field_name, 0},
    {field_operators, 2},
  [173] =
    {field_body, 1},
    {field_terms, 5},
    {field_terms, 6},
    {field_weight, 4},
  [177] =
    {field_name, 1},
    {field_type, 6},
    {field_value, 3},
  [180] =
    {field_body, 6},
    {field_edge_pair, 2},
    {field_edge_pair, 3, .inherited = true},
  [183] =
    {field_atom, 1},
    {field_type, 6},
    {field_weight, 4},
  [186] =
    {field_atom, 1},
    {field_body, 3},
    {field_type, 6},
  [189] =
    {field_name, 0},
    {field_variables, 4},
  [191] =
    {field_atom, 1},
    {field_type, 7},
    {field_weight, 5},
  [194] =
    {field_atom, 1},
    {field_body, 3},
    {field_type, 8},
    {field_weight, 6},
  [198] =
    {field_arity, 4},
    {field_operator, 0},
    {field_priority, 2},
  [201] =
    {field_variable, 2},
  [202] =
    {field_arity, 3},
    {field_atom_type, 7},
    {field_name, 1},
    {field_theory_term_name, 5},
  [206] =
    {field_description, 6},
    {field_variable, 2},
  [208] =
    {field_arity, 4},
    {field_associativity, 6},
    {field_operator, 0},
    {field_priority, 2},
  [212] =
    {field_arity, 3},
    {field_atom_type, 12},
    {field_guard, 10},
    {field_name, 1},
    {field_theory_term_name, 5},
  [217] =
    {field_arity, 3},
    {field_atom_type, 13},
    {field_guard, 11},
    {field_name, 1},
    {field_operators, 8},
    {field_theory_term_name, 5},
};

static const TSSymbol ts_alias_sequences[PRODUCTION_ID_COUNT][MAX_ALIAS_SEQUENCE_LENGTH] = {
  [0] = {0},
  [8] = {
    [0] = alias_sym_classical_negation,
  },
  [10] = {
    [0] = alias_sym_lone_comma,
  },
  [12] = {
    [0] = sym_conditional_literal,
  },
  [13] = {
    [0] = sym_conditional_literal,
  },
  [19] = {
    [0] = alias_sym_classical_negation,
  },
  [36] = {
    [0] = sym_theory_operator,
  },
  [61] = {
    [0] = alias_sym_classical_negation,
  },
  [72] = {
    [1] = sym_number,
  },
  [82] = {
    [1] = alias_sym_lone_comma,
  },
  [103] = {
    [0] = sym_theory_operator,
  },
  [108] = {
    [0] = sym_theory_operator,
  },
  [109] = {
    [1] = sym_theory_operator,
  },
};

static const uint16_t ts_non_terminal_alias_map[] = {
  sym__conditional_literal_n, 2,
    sym__conditional_literal_n,
    sym_conditional_literal,
  sym__conditional_literal_0, 2,
    sym__conditional_literal_0,
    sym_conditional_literal,
  0,
};

static const TSStateId ts_primary_state_ids[STATE_COUNT] = {
  [0] = 0,
  [1] = 1,
  [2] = 2,
  [3] = 3,
  [4] = 4,
  [5] = 5,
  [6] = 6,
  [7] = 7,
  [8] = 8,
  [9] = 9,
  [10] = 10,
  [11] = 11,
  [12] = 12,
  [13] = 13,
  [14] = 14,
  [15] = 15,
  [16] = 16,
  [17] = 17,
  [18] = 18,
  [19] = 19,
  [20] = 20,
  [21] = 21,
  [22] = 22,
  [23] = 23,
  [24] = 24,
  [25] = 25,
  [26] = 26,
  [27] = 27,
  [28] = 28,
  [29] = 29,
  [30] = 30,
  [31] = 31,
  [32] = 32,
  [33] = 33,
  [34] = 34,
  [35] = 35,
  [36] = 36,
  [37] = 37,
  [38] = 38,
  [39] = 39,
  [40] = 40,
  [41] = 41,
  [42] = 42,
  [43] = 43,
  [44] = 44,
  [45] = 45,
  [46] = 46,
  [47] = 47,
  [48] = 48,
  [49] = 49,
  [50] = 50,
  [51] = 51,
  [52] = 52,
  [53] = 53,
  [54] = 54,
  [55] = 55,
  [56] = 56,
  [57] = 57,
  [58] = 58,
  [59] = 59,
  [60] = 60,
  [61] = 61,
  [62] = 62,
  [63] = 63,
  [64] = 64,
  [65] = 65,
  [66] = 66,
  [67] = 67,
  [68] = 68,
  [69] = 69,
  [70] = 70,
  [71] = 71,
  [72] = 72,
  [73] = 73,
  [74] = 74,
  [75] = 75,
  [76] = 76,
  [77] = 77,
  [78] = 75,
  [79] = 79,
  [80] = 80,
  [81] = 81,
  [82] = 82,
  [83] = 83,
  [84] = 84,
  [85] = 85,
  [86] = 86,
  [87] = 87,
  [88] = 77,
  [89] = 87,
  [90] = 90,
  [91] = 90,
  [92] = 90,
  [93] = 73,
  [94] = 94,
  [95] = 95,
  [96] = 96,
  [97] = 97,
  [98] = 98,
  [99] = 96,
  [100] = 100,
  [101] = 100,
  [102] = 100,
  [103] = 100,
  [104] = 104,
  [105] = 105,
  [106] = 106,
  [107] = 107,
  [108] = 108,
  [109] = 109,
  [110] = 110,
  [111] = 111,
  [112] = 112,
  [113] = 113,
  [114] = 114,
  [115] = 115,
  [116] = 116,
  [117] = 117,
  [118] = 110,
  [119] = 113,
  [120] = 120,
  [121] = 121,
  [122] = 122,
  [123] = 123,
  [124] = 123,
  [125] = 125,
  [126] = 126,
  [127] = 127,
  [128] = 128,
  [129] = 129,
  [130] = 130,
  [131] = 131,
  [132] = 123,
  [133] = 107,
  [134] = 104,
  [135] = 108,
  [136] = 112,
  [137] = 105,
  [138] = 121,
  [139] = 122,
  [140] = 120,
  [141] = 141,
  [142] = 142,
  [143] = 130,
  [144] = 144,
  [145] = 145,
  [146] = 146,
  [147] = 147,
  [148] = 148,
  [149] = 149,
  [150] = 150,
  [151] = 145,
  [152] = 149,
  [153] = 148,
  [154] = 154,
  [155] = 155,
  [156] = 156,
  [157] = 157,
  [158] = 158,
  [159] = 159,
  [160] = 160,
  [161] = 161,
  [162] = 162,
  [163] = 163,
  [164] = 164,
  [165] = 165,
  [166] = 166,
  [167] = 167,
  [168] = 168,
  [169] = 169,
  [170] = 170,
  [171] = 171,
  [172] = 172,
  [173] = 173,
  [174] = 174,
  [175] = 175,
  [176] = 176,
  [177] = 177,
  [178] = 178,
  [179] = 179,
  [180] = 180,
  [181] = 181,
  [182] = 173,
  [183] = 183,
  [184] = 184,
  [185] = 185,
  [186] = 186,
  [187] = 187,
  [188] = 188,
  [189] = 189,
  [190] = 190,
  [191] = 191,
  [192] = 192,
  [193] = 193,
  [194] = 194,
  [195] = 195,
  [196] = 196,
  [197] = 197,
  [198] = 198,
  [199] = 199,
  [200] = 200,
  [201] = 201,
  [202] = 198,
  [203] = 203,
  [204] = 200,
  [205] = 203,
  [206] = 206,
  [207] = 207,
  [208] = 185,
  [209] = 189,
  [210] = 193,
  [211] = 206,
  [212] = 212,
  [213] = 187,
  [214] = 196,
  [215] = 207,
  [216] = 185,
  [217] = 198,
  [218] = 218,
  [219] = 200,
  [220] = 203,
  [221] = 206,
  [222] = 207,
  [223] = 185,
  [224] = 224,
  [225] = 225,
  [226] = 226,
  [227] = 198,
  [228] = 200,
  [229] = 203,
  [230] = 206,
  [231] = 207,
  [232] = 193,
  [233] = 226,
  [234] = 196,
  [235] = 212,
  [236] = 236,
  [237] = 237,
  [238] = 199,
  [239] = 226,
  [240] = 199,
  [241] = 167,
  [242] = 165,
  [243] = 129,
  [244] = 125,
  [245] = 164,
  [246] = 246,
  [247] = 247,
  [248] = 248,
  [249] = 249,
  [250] = 250,
  [251] = 163,
  [252] = 158,
  [253] = 218,
  [254] = 129,
  [255] = 186,
  [256] = 125,
  [257] = 250,
  [258] = 258,
  [259] = 166,
  [260] = 260,
  [261] = 225,
  [262] = 170,
  [263] = 249,
  [264] = 264,
  [265] = 265,
  [266] = 266,
  [267] = 188,
  [268] = 169,
  [269] = 269,
  [270] = 266,
  [271] = 156,
  [272] = 155,
  [273] = 159,
  [274] = 161,
  [275] = 275,
  [276] = 264,
  [277] = 265,
  [278] = 269,
  [279] = 260,
  [280] = 280,
  [281] = 281,
  [282] = 166,
  [283] = 169,
  [284] = 156,
  [285] = 155,
  [286] = 159,
  [287] = 161,
  [288] = 249,
  [289] = 264,
  [290] = 265,
  [291] = 269,
  [292] = 266,
  [293] = 163,
  [294] = 164,
  [295] = 167,
  [296] = 165,
  [297] = 297,
  [298] = 281,
  [299] = 158,
  [300] = 246,
  [301] = 170,
  [302] = 250,
  [303] = 303,
  [304] = 247,
  [305] = 305,
  [306] = 306,
  [307] = 307,
  [308] = 308,
  [309] = 309,
  [310] = 310,
  [311] = 311,
  [312] = 312,
  [313] = 313,
  [314] = 314,
  [315] = 315,
  [316] = 316,
  [317] = 307,
  [318] = 318,
  [319] = 218,
  [320] = 186,
  [321] = 321,
  [322] = 269,
  [323] = 266,
  [324] = 188,
  [325] = 325,
  [326] = 225,
  [327] = 327,
  [328] = 328,
  [329] = 249,
  [330] = 260,
  [331] = 250,
  [332] = 265,
  [333] = 264,
  [334] = 334,
  [335] = 335,
  [336] = 336,
  [337] = 281,
  [338] = 338,
  [339] = 339,
  [340] = 340,
  [341] = 341,
  [342] = 342,
  [343] = 343,
  [344] = 344,
  [345] = 345,
  [346] = 346,
  [347] = 347,
  [348] = 348,
  [349] = 349,
  [350] = 350,
  [351] = 351,
  [352] = 352,
  [353] = 353,
  [354] = 354,
  [355] = 355,
  [356] = 356,
  [357] = 325,
  [358] = 328,
  [359] = 359,
  [360] = 342,
  [361] = 361,
  [362] = 342,
  [363] = 363,
  [364] = 364,
  [365] = 340,
  [366] = 366,
  [367] = 367,
  [368] = 368,
  [369] = 369,
  [370] = 338,
  [371] = 336,
  [372] = 372,
  [373] = 373,
  [374] = 374,
  [375] = 375,
  [376] = 334,
  [377] = 377,
  [378] = 378,
  [379] = 379,
  [380] = 380,
  [381] = 381,
  [382] = 382,
  [383] = 383,
  [384] = 382,
  [385] = 385,
  [386] = 386,
  [387] = 387,
  [388] = 382,
  [389] = 389,
  [390] = 390,
  [391] = 391,
  [392] = 392,
  [393] = 366,
  [394] = 394,
  [395] = 395,
  [396] = 396,
  [397] = 397,
  [398] = 398,
  [399] = 399,
  [400] = 400,
  [401] = 368,
  [402] = 402,
  [403] = 403,
  [404] = 404,
  [405] = 405,
  [406] = 406,
  [407] = 369,
  [408] = 367,
  [409] = 225,
  [410] = 188,
  [411] = 411,
  [412] = 412,
  [413] = 413,
  [414] = 414,
  [415] = 412,
  [416] = 386,
  [417] = 394,
  [418] = 413,
  [419] = 419,
  [420] = 419,
  [421] = 421,
  [422] = 422,
  [423] = 423,
  [424] = 424,
  [425] = 425,
  [426] = 426,
  [427] = 427,
  [428] = 428,
  [429] = 429,
  [430] = 430,
  [431] = 431,
  [432] = 165,
  [433] = 433,
  [434] = 434,
  [435] = 435,
  [436] = 164,
  [437] = 437,
  [438] = 167,
  [439] = 439,
  [440] = 440,
  [441] = 398,
  [442] = 399,
  [443] = 443,
  [444] = 400,
  [445] = 445,
  [446] = 402,
  [447] = 426,
  [448] = 225,
  [449] = 435,
  [450] = 406,
  [451] = 403,
  [452] = 188,
  [453] = 433,
  [454] = 454,
  [455] = 404,
  [456] = 405,
  [457] = 457,
  [458] = 454,
  [459] = 459,
  [460] = 460,
  [461] = 461,
  [462] = 462,
  [463] = 459,
  [464] = 454,
  [465] = 465,
  [466] = 459,
  [467] = 395,
  [468] = 396,
  [469] = 397,
  [470] = 470,
  [471] = 471,
  [472] = 472,
  [473] = 473,
  [474] = 473,
  [475] = 475,
  [476] = 476,
  [477] = 475,
  [478] = 478,
  [479] = 479,
  [480] = 480,
  [481] = 412,
  [482] = 394,
  [483] = 483,
  [484] = 484,
  [485] = 485,
  [486] = 486,
  [487] = 478,
  [488] = 488,
  [489] = 489,
  [490] = 490,
  [491] = 491,
  [492] = 492,
  [493] = 493,
  [494] = 494,
  [495] = 495,
  [496] = 496,
  [497] = 424,
  [498] = 491,
  [499] = 499,
  [500] = 500,
  [501] = 501,
  [502] = 502,
  [503] = 503,
  [504] = 504,
  [505] = 505,
  [506] = 506,
  [507] = 507,
  [508] = 508,
  [509] = 509,
  [510] = 510,
  [511] = 511,
  [512] = 512,
  [513] = 513,
  [514] = 514,
  [515] = 515,
  [516] = 515,
  [517] = 517,
  [518] = 518,
  [519] = 519,
  [520] = 520,
  [521] = 521,
  [522] = 518,
  [523] = 523,
  [524] = 524,
  [525] = 525,
  [526] = 427,
  [527] = 439,
  [528] = 515,
  [529] = 529,
  [530] = 530,
  [531] = 518,
  [532] = 532,
  [533] = 437,
  [534] = 515,
  [535] = 518,
  [536] = 515,
  [537] = 518,
  [538] = 538,
  [539] = 539,
  [540] = 540,
  [541] = 541,
  [542] = 542,
  [543] = 543,
  [544] = 544,
  [545] = 545,
  [546] = 546,
  [547] = 547,
  [548] = 548,
  [549] = 549,
  [550] = 550,
  [551] = 551,
  [552] = 552,
  [553] = 553,
  [554] = 554,
  [555] = 555,
  [556] = 556,
  [557] = 557,
  [558] = 558,
  [559] = 559,
  [560] = 560,
  [561] = 561,
  [562] = 562,
  [563] = 563,
  [564] = 564,
  [565] = 565,
  [566] = 566,
  [567] = 567,
  [568] = 568,
  [569] = 569,
  [570] = 570,
  [571] = 571,
  [572] = 572,
  [573] = 573,
  [574] = 574,
  [575] = 575,
  [576] = 576,
  [577] = 577,
  [578] = 578,
  [579] = 579,
  [580] = 580,
  [581] = 581,
  [582] = 582,
  [583] = 583,
  [584] = 584,
  [585] = 480,
  [586] = 586,
  [587] = 587,
  [588] = 588,
  [589] = 589,
  [590] = 590,
  [591] = 591,
  [592] = 483,
  [593] = 593,
  [594] = 594,
  [595] = 595,
  [596] = 596,
  [597] = 597,
  [598] = 598,
  [599] = 599,
  [600] = 600,
  [601] = 601,
  [602] = 602,
  [603] = 603,
  [604] = 604,
  [605] = 605,
  [606] = 606,
  [607] = 607,
  [608] = 608,
  [609] = 609,
  [610] = 610,
  [611] = 611,
  [612] = 612,
  [613] = 613,
  [614] = 614,
  [615] = 615,
  [616] = 616,
  [617] = 551,
  [618] = 618,
  [619] = 619,
  [620] = 620,
  [621] = 621,
  [622] = 557,
  [623] = 623,
  [624] = 624,
  [625] = 625,
  [626] = 626,
  [627] = 627,
  [628] = 628,
  [629] = 629,
  [630] = 551,
  [631] = 621,
  [632] = 557,
  [633] = 621,
  [634] = 634,
  [635] = 621,
  [636] = 636,
  [637] = 637,
  [638] = 638,
  [639] = 639,
  [640] = 640,
  [641] = 641,
  [642] = 642,
  [643] = 643,
  [644] = 644,
  [645] = 645,
  [646] = 646,
  [647] = 647,
  [648] = 648,
  [649] = 649,
  [650] = 650,
  [651] = 651,
  [652] = 652,
  [653] = 653,
  [654] = 654,
  [655] = 655,
  [656] = 656,
  [657] = 657,
  [658] = 658,
  [659] = 659,
  [660] = 660,
  [661] = 661,
  [662] = 662,
  [663] = 663,
  [664] = 664,
  [665] = 665,
  [666] = 666,
  [667] = 667,
  [668] = 668,
  [669] = 669,
  [670] = 670,
  [671] = 671,
  [672] = 672,
  [673] = 673,
  [674] = 674,
  [675] = 675,
  [676] = 676,
  [677] = 677,
  [678] = 678,
  [679] = 679,
  [680] = 680,
  [681] = 681,
  [682] = 682,
  [683] = 683,
  [684] = 684,
  [685] = 685,
  [686] = 686,
  [687] = 687,
  [688] = 688,
  [689] = 689,
  [690] = 690,
  [691] = 691,
  [692] = 692,
  [693] = 693,
  [694] = 694,
  [695] = 695,
  [696] = 696,
  [697] = 697,
  [698] = 698,
  [699] = 699,
  [700] = 700,
  [701] = 701,
  [702] = 702,
  [703] = 703,
  [704] = 704,
  [705] = 705,
  [706] = 706,
  [707] = 707,
  [708] = 708,
  [709] = 709,
  [710] = 710,
  [711] = 666,
  [712] = 712,
  [713] = 713,
  [714] = 714,
  [715] = 715,
  [716] = 716,
  [717] = 717,
  [718] = 718,
  [719] = 719,
  [720] = 720,
  [721] = 721,
  [722] = 722,
  [723] = 723,
  [724] = 724,
  [725] = 725,
  [726] = 726,
  [727] = 727,
  [728] = 728,
  [729] = 729,
  [730] = 730,
  [731] = 731,
  [732] = 732,
  [733] = 733,
  [734] = 734,
  [735] = 735,
  [736] = 736,
  [737] = 737,
  [738] = 738,
  [739] = 739,
  [740] = 740,
  [741] = 741,
  [742] = 742,
  [743] = 743,
  [744] = 744,
  [745] = 745,
  [746] = 746,
  [747] = 747,
  [748] = 748,
  [749] = 749,
  [750] = 750,
  [751] = 751,
  [752] = 752,
  [753] = 753,
  [754] = 754,
  [755] = 755,
  [756] = 756,
  [757] = 757,
  [758] = 758,
  [759] = 759,
  [760] = 760,
  [761] = 761,
  [762] = 762,
  [763] = 763,
  [764] = 764,
  [765] = 765,
  [766] = 766,
  [767] = 767,
  [768] = 768,
  [769] = 769,
  [770] = 770,
  [771] = 771,
  [772] = 772,
  [773] = 773,
  [774] = 774,
  [775] = 775,
  [776] = 776,
  [777] = 777,
  [778] = 778,
  [779] = 779,
  [780] = 780,
  [781] = 781,
  [782] = 782,
  [783] = 783,
  [784] = 784,
  [785] = 785,
  [786] = 786,
  [787] = 787,
  [788] = 788,
  [789] = 789,
  [790] = 790,
  [791] = 791,
  [792] = 792,
  [793] = 793,
  [794] = 794,
  [795] = 795,
  [796] = 796,
  [797] = 797,
  [798] = 798,
  [799] = 799,
  [800] = 800,
  [801] = 801,
  [802] = 802,
  [803] = 803,
  [804] = 804,
  [805] = 805,
  [806] = 806,
  [807] = 807,
  [808] = 808,
  [809] = 775,
  [810] = 810,
  [811] = 811,
  [812] = 812,
  [813] = 813,
  [814] = 814,
  [815] = 815,
  [816] = 816,
  [817] = 760,
  [818] = 818,
  [819] = 819,
  [820] = 820,
  [821] = 821,
  [822] = 822,
  [823] = 823,
  [824] = 824,
  [825] = 825,
  [826] = 826,
  [827] = 827,
  [828] = 828,
  [829] = 829,
  [830] = 830,
  [831] = 831,
  [832] = 832,
  [833] = 833,
  [834] = 834,
  [835] = 835,
  [836] = 783,
  [837] = 837,
  [838] = 838,
  [839] = 839,
  [840] = 840,
  [841] = 841,
  [842] = 842,
  [843] = 843,
  [844] = 844,
  [845] = 845,
  [846] = 735,
  [847] = 743,
  [848] = 753,
  [849] = 723,
  [850] = 850,
  [851] = 764,
  [852] = 852,
  [853] = 853,
  [854] = 854,
  [855] = 814,
  [856] = 775,
  [857] = 857,
  [858] = 858,
  [859] = 859,
  [860] = 860,
  [861] = 861,
  [862] = 760,
  [863] = 863,
  [864] = 827,
  [865] = 865,
  [866] = 866,
  [867] = 867,
  [868] = 868,
  [869] = 869,
  [870] = 827,
  [871] = 871,
  [872] = 872,
  [873] = 873,
  [874] = 874,
  [875] = 875,
  [876] = 818,
  [877] = 725,
  [878] = 878,
  [879] = 724,
  [880] = 880,
  [881] = 881,
  [882] = 882,
  [883] = 827,
  [884] = 884,
  [885] = 840,
};

static const TSSymbol ts_supertype_symbols[SUPERTYPE_COUNT] = {
  sym_head,
  sym_literal_sign,
  sym_simple_atom,
  sym_statement,
  sym_term,
  sym_theory_root_term,
  sym_theory_term,
  sym_tuple_pool_item,
};

static const TSMapSlice ts_supertype_map_slices[] = {
  [sym_head] = {.index = 0, .length = 5},
  [sym_literal_sign] = {.index = 5, .length = 2},
  [sym_simple_atom] = {.index = 7, .length = 3},
  [sym_statement] = {.index = 10, .length = 20},
  [sym_term] = {.index = 30, .length = 13},
  [sym_theory_root_term] = {.index = 43, .length = 10},
  [sym_theory_term] = {.index = 53, .length = 2},
  [sym_tuple_pool_item] = {.index = 55, .length = 9},
};

static const TSSymbol ts_supertype_map_entries[] = {
  [0] =
    sym_disjunction,
    sym_head_aggregate,
    sym_literal,
    sym_set_aggregate,
    sym_theory_atom,
  [5] =
    sym_default_negation,
    sym_double_default_negation,
  [7] =
    sym_boolean_constant,
    sym_comparison,
    sym_symbolic_atom,
  [10] =
    sym_const,
    sym_defined,
    sym_doc_comment,
    sym_edge,
    sym_external,
    sym_heuristic,
    sym_include,
    sym_integrity_constraint,
    sym_maximize,
    sym_minimize,
    sym_program,
    sym_project_atom,
    sym_project_signature,
    sym_rule,
    sym_script,
    sym_show,
    sym_show_signature,
    sym_show_term,
    sym_theory,
    sym_weak_constraint,
  [30] =
    sym_abs,
    sym_anonymous,
    sym_binary_operation,
    sym_external_function,
    sym_fstring,
    sym_function,
    sym_infimum,
    sym_number,
    sym_string,
    sym_supremum,
    sym_tuple,
    sym_unary_operation,
    sym_variable,
  [43] =
    sym_anonymous,
    sym_infimum,
    sym_number,
    sym_string,
    sym_supremum,
    sym_theory_function,
    sym_theory_list,
    sym_theory_set,
    sym_theory_tuple,
    sym_variable,
  [53] =
    sym_theory_root_term,
    sym_theory_unparsed_term,
  [55] =
    alias_sym_lone_comma,
    sym__const_term_comma,
    sym__const_terms_trail,
    sym__term_comma,
    sym__terms_or_projection,
    sym__terms_trail,
    sym_const_terms,
    sym_term,
    sym_terms,
};

static const TSCharacterRange sym_theory_operator_character_set_1[] = {
  {'!', '!'}, {'&', '&'}, {'*', '+'}, {'-', '/'}, {':', '@'}, {'\\', '\\'}, {'^', '^'}, {'|', '|'},
  {'~', '~'},
};

static bool ts_lex(TSLexer *lexer, TSStateId state) {
  START_LEXER();
  eof = lexer->eof(lexer);
  switch (state) {
    case 0:
      if (eof) ADVANCE(218);
      ADVANCE_MAP(
        '!', 267,
        '"', 247,
        '#', 276,
        '%', 222,
        '&', 306,
        '\'', 198,
        '(', 317,
        ')', 318,
        '*', 310,
        '+', 307,
        ',', 316,
        '-', 308,
        '.', 342,
        '/', 311,
        '0', 261,
        ':', 286,
        ';', 320,
        '<', 368,
        '=', 363,
        '>', 370,
        '?', 305,
        '@', 322,
        'X', 235,
        '[', 326,
        '\\', 312,
        ']', 327,
        '^', 304,
        '_', 300,
        '`', 211,
        'a', 120,
        'b', 282,
        'd', 281,
        'f', 13,
        'h', 74,
        'l', 75,
        'n', 283,
        'o', 284,
        'r', 93,
        't', 145,
        'u', 125,
        '{', 287,
        '|', 315,
        '}', 328,
        '~', 314,
        'c', 280,
        's', 280,
        'x', 280,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(215);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(264);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(235);
      END_STATE();
    case 1:
      if (lookahead == '\n') SKIP(18);
      if (lookahead == ' ') ADVANCE(273);
      if (lookahead == '#') ADVANCE(277);
      if (lookahead == '%') ADVANCE(226);
      if (lookahead == '0') ADVANCE(262);
      if (lookahead == '}') ADVANCE(290);
      if (lookahead == '+' ||
          lookahead == '-') ADVANCE(274);
      if (lookahead == ',' ||
          lookahead == '_') ADVANCE(279);
      if (('\t' <= lookahead && lookahead <= '\r')) ADVANCE(17);
      if (('<' <= lookahead && lookahead <= '>') ||
          lookahead == '^') ADVANCE(271);
      if (lookahead == 'X' ||
          ('b' <= lookahead && lookahead <= 'd') ||
          lookahead == 'n' ||
          lookahead == 'o' ||
          lookahead == 's' ||
          lookahead == 'x') ADVANCE(285);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(263);
      if (lookahead != 0) ADVANCE(205);
      END_STATE();
    case 2:
      ADVANCE_MAP(
        ' ', 273,
        '#', 275,
        '%', 227,
        '0', 261,
        '}', 289,
        '+', 273,
        '-', 273,
        ',', 278,
        '_', 278,
      );
      if (('\t' <= lookahead && lookahead <= '\r')) SKIP(18);
      if (lookahead == 'X' ||
          ('b' <= lookahead && lookahead <= 'd') ||
          lookahead == 'n' ||
          lookahead == 'o' ||
          lookahead == 's' ||
          lookahead == 'x') ADVANCE(280);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(264);
      END_STATE();
    case 3:
      ADVANCE_MAP(
        '!', 267,
        '#', 49,
        '%', 227,
        '&', 306,
        '(', 317,
        ')', 318,
        '*', 310,
        '+', 307,
        ',', 316,
        '-', 308,
        '.', 27,
        '/', 311,
        '0', 261,
        ':', 286,
        ';', 320,
        '<', 331,
        '=', 330,
        '>', 331,
        '?', 305,
        '@', 322,
        '[', 260,
        '\\', 312,
        ']', 327,
        '^', 304,
        'a', 120,
        'b', 104,
        'd', 60,
        'f', 31,
        'h', 74,
        'l', 75,
        'o', 187,
        'r', 93,
        't', 145,
        'u', 125,
        '{', 287,
        '|', 315,
        '}', 289,
        '\'', 198,
        '_', 198,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(4);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(264);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(235);
      END_STATE();
    case 4:
      ADVANCE_MAP(
        '!', 267,
        '#', 49,
        '%', 227,
        '&', 306,
        '(', 317,
        ')', 318,
        '*', 310,
        '+', 307,
        ',', 316,
        '-', 308,
        '.', 27,
        '/', 311,
        ':', 286,
        ';', 320,
        '<', 331,
        '=', 330,
        '>', 331,
        '?', 305,
        '@', 322,
        '[', 260,
        '\\', 312,
        ']', 327,
        '^', 304,
        'a', 120,
        'b', 104,
        'd', 60,
        'f', 31,
        'h', 74,
        'l', 75,
        'o', 187,
        'r', 93,
        't', 145,
        'u', 125,
        '{', 287,
        '|', 315,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(4);
      END_STATE();
    case 5:
      if (lookahead == '!') ADVANCE(229);
      END_STATE();
    case 6:
      ADVANCE_MAP(
        '!', 28,
        '"', 247,
        '#', 48,
        '%', 227,
        '&', 306,
        '\'', 199,
        '(', 317,
        ')', 318,
        '*', 309,
        ',', 316,
        '-', 308,
        '.', 341,
        '0', 295,
        ':', 23,
        ';', 320,
        '<', 331,
        '=', 330,
        '>', 331,
        '@', 322,
        '[', 326,
        ']', 265,
        '_', 301,
        'f', 241,
        '{', 287,
        '|', 315,
        '}', 328,
        '~', 314,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(7);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(298);
      if (('a' <= lookahead && lookahead <= 'z')) ADVANCE(246);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(303);
      END_STATE();
    case 7:
      ADVANCE_MAP(
        '!', 28,
        '"', 247,
        '#', 48,
        '%', 227,
        '&', 306,
        '\'', 199,
        '(', 317,
        ')', 318,
        '*', 309,
        ',', 316,
        '-', 308,
        '.', 341,
        '0', 295,
        ':', 23,
        ';', 320,
        '<', 331,
        '=', 330,
        '>', 331,
        '@', 322,
        '[', 326,
        '_', 301,
        'f', 241,
        '{', 287,
        '|', 315,
        '}', 328,
        '~', 314,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(7);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(298);
      if (('a' <= lookahead && lookahead <= 'z')) ADVANCE(246);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(303);
      END_STATE();
    case 8:
      ADVANCE_MAP(
        '!', 266,
        '"', 247,
        '#', 275,
        '%', 227,
        '&', 306,
        ')', 234,
        '*', 310,
        '+', 307,
        '-', 308,
        '.', 27,
        '/', 311,
        '0', 261,
        ':', 286,
        '<', 367,
        '=', 363,
        '>', 369,
        '?', 305,
        '[', 260,
        '\\', 312,
        '^', 304,
        '}', 289,
        ',', 278,
        '_', 278,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(9);
      if (lookahead == 'X' ||
          ('b' <= lookahead && lookahead <= 'd') ||
          lookahead == 'n' ||
          lookahead == 'o' ||
          lookahead == 's' ||
          lookahead == 'x') ADVANCE(280);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(264);
      END_STATE();
    case 9:
      ADVANCE_MAP(
        '!', 266,
        '"', 247,
        '%', 227,
        '&', 306,
        '*', 310,
        '+', 307,
        '-', 308,
        '.', 27,
        '/', 311,
        ':', 286,
        '<', 367,
        '=', 363,
        '>', 369,
        '?', 305,
        '[', 260,
        '\\', 312,
        '^', 304,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(9);
      END_STATE();
    case 10:
      ADVANCE_MAP(
        '"', 247,
        '#', 100,
        '%', 227,
        '\'', 199,
        '(', 317,
        ')', 318,
        ',', 316,
        '0', 295,
        ':', 26,
        '[', 326,
        ']', 327,
        '_', 301,
        'n', 243,
        '{', 287,
        '}', 328,
        '.', 207,
        ';', 207,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(10);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(298);
      if (lookahead == '!' ||
          ('&' <= lookahead && lookahead <= '@') ||
          ('\\' <= lookahead && lookahead <= '^') ||
          ('|' <= lookahead && lookahead <= '~')) ADVANCE(324);
      if (('a' <= lookahead && lookahead <= 'z')) ADVANCE(246);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(303);
      END_STATE();
    case 11:
      if (lookahead == '"') ADVANCE(247);
      if (lookahead == '%') ADVANCE(255);
      if (lookahead == '\\') ADVANCE(203);
      if (lookahead == '{') ADVANCE(288);
      if (lookahead == '}') ADVANCE(197);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(253);
      if (lookahead != 0) ADVANCE(256);
      END_STATE();
    case 12:
      if (lookahead == '"') ADVANCE(247);
      if (lookahead == '%') ADVANCE(251);
      if (lookahead == '\\') ADVANCE(204);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(249);
      if (lookahead != 0) ADVANCE(252);
      END_STATE();
    case 13:
      if (lookahead == '"') ADVANCE(248);
      if (lookahead == 'a') ADVANCE(106);
      END_STATE();
    case 14:
      ADVANCE_MAP(
        '%', 227,
        '(', 317,
        ')', 318,
        ',', 316,
        '.', 343,
        ':', 25,
        ';', 321,
        ']', 327,
        'n', 142,
        '{', 287,
        '}', 328,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(14);
      if (lookahead == '!' ||
          lookahead == '&' ||
          ('*' <= lookahead && lookahead <= '/') ||
          ('<' <= lookahead && lookahead <= '@') ||
          ('\\' <= lookahead && lookahead <= '^') ||
          ('|' <= lookahead && lookahead <= '~')) ADVANCE(324);
      END_STATE();
    case 15:
      if (lookahead == '%') ADVANCE(227);
      if (lookahead == ')') ADVANCE(234);
      if (lookahead == '*') ADVANCE(19);
      if (lookahead == ',') ADVANCE(232);
      if (lookahead == ':') ADVANCE(240);
      if (lookahead == '_') ADVANCE(212);
      if (lookahead == '`') ADVANCE(211);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(18);
      END_STATE();
    case 16:
      if (lookahead == '%') ADVANCE(227);
      if (lookahead == ')') ADVANCE(234);
      if (lookahead == '\'' ||
          lookahead == '_') ADVANCE(201);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(18);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(235);
      if (('a' <= lookahead && lookahead <= 'z')) ADVANCE(233);
      END_STATE();
    case 17:
      if (lookahead == '%') ADVANCE(227);
      if (('<' <= lookahead && lookahead <= '>') ||
          lookahead == '^') ADVANCE(270);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(18);
      END_STATE();
    case 18:
      if (lookahead == '%') ADVANCE(227);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(18);
      END_STATE();
    case 19:
      if (lookahead == '%') ADVANCE(231);
      if (lookahead == '*') ADVANCE(210);
      if (lookahead != 0 &&
          lookahead != '\n' &&
          lookahead != '\r' &&
          lookahead != '_' &&
          lookahead != '`') ADVANCE(20);
      END_STATE();
    case 20:
      if (lookahead == '*') ADVANCE(236);
      if (lookahead != 0 &&
          lookahead != '\n' &&
          lookahead != '\r' &&
          lookahead != '_' &&
          lookahead != '`') ADVANCE(20);
      END_STATE();
    case 21:
      if (lookahead == '*') ADVANCE(22);
      if (lookahead != 0 &&
          lookahead != '\n' &&
          lookahead != '\r' &&
          lookahead != '_' &&
          lookahead != '`') ADVANCE(21);
      END_STATE();
    case 22:
      if (lookahead == '*') ADVANCE(237);
      END_STATE();
    case 23:
      if (lookahead == '-') ADVANCE(344);
      END_STATE();
    case 24:
      if (lookahead == '-') ADVANCE(344);
      if (lookahead == '~') ADVANCE(346);
      END_STATE();
    case 25:
      if (lookahead == '-') ADVANCE(345);
      if (set_contains(sym_theory_operator_character_set_1, 9, lookahead)) ADVANCE(325);
      END_STATE();
    case 26:
      if (lookahead == '-') ADVANCE(208);
      if (set_contains(sym_theory_operator_character_set_1, 9, lookahead)) ADVANCE(325);
      END_STATE();
    case 27:
      if (lookahead == '.') ADVANCE(319);
      if (('a' <= lookahead && lookahead <= 'z')) ADVANCE(259);
      END_STATE();
    case 28:
      if (lookahead == '=') ADVANCE(330);
      END_STATE();
    case 29:
      if (lookahead == '_') ADVANCE(238);
      if (lookahead != 0 &&
          lookahead != '\n' &&
          lookahead != '\r' &&
          lookahead != '*' &&
          lookahead != '_' &&
          lookahead != '`') ADVANCE(29);
      END_STATE();
    case 30:
      if (lookahead == '`') ADVANCE(239);
      if (lookahead != 0 &&
          lookahead != '\n' &&
          lookahead != '\r' &&
          lookahead != '*' &&
          lookahead != '_' &&
          lookahead != '`') ADVANCE(30);
      END_STATE();
    case 31:
      if (lookahead == 'a') ADVANCE(106);
      END_STATE();
    case 32:
      if (lookahead == 'a') ADVANCE(192);
      if (lookahead == 'i') ADVANCE(134);
      END_STATE();
    case 33:
      if (lookahead == 'a') ADVANCE(190);
      if (lookahead == 'i') ADVANCE(133);
      END_STATE();
    case 34:
      if (lookahead == 'a') ADVANCE(52);
      END_STATE();
    case 35:
      if (lookahead == 'a') ADVANCE(178);
      END_STATE();
    case 36:
      if (lookahead == 'a') ADVANCE(191);
      if (lookahead == 'i') ADVANCE(122);
      END_STATE();
    case 37:
      if (lookahead == 'a') ADVANCE(107);
      END_STATE();
    case 38:
      if (lookahead == 'a') ADVANCE(112);
      END_STATE();
    case 39:
      if (lookahead == 'a') ADVANCE(146);
      END_STATE();
    case 40:
      if (lookahead == 'a') ADVANCE(149);
      END_STATE();
    case 41:
      if (lookahead == 'a') ADVANCE(110);
      END_STATE();
    case 42:
      if (lookahead == 'c') ADVANCE(365);
      END_STATE();
    case 43:
      if (lookahead == 'c') ADVANCE(108);
      if (lookahead == 'f') ADVANCE(294);
      END_STATE();
    case 44:
      if (lookahead == 'c') ADVANCE(173);
      END_STATE();
    case 45:
      ADVANCE_MAP(
        'c', 137,
        'd', 77,
        'e', 56,
        'f', 41,
        'h', 76,
        'i', 121,
        'm', 33,
        'p', 147,
        's', 50,
        't', 92,
      );
      END_STATE();
    case 46:
      if (lookahead == 'c') ADVANCE(168);
      END_STATE();
    case 47:
      if (lookahead == 'c') ADVANCE(140);
      if (lookahead == 'e') ADVANCE(124);
      if (lookahead == 'f') ADVANCE(41);
      if (lookahead == 'i') ADVANCE(126);
      if (lookahead == 'm') ADVANCE(36);
      if (lookahead == 's') ADVANCE(176);
      if (lookahead == 't') ADVANCE(157);
      END_STATE();
    case 48:
      if (lookahead == 'c') ADVANCE(140);
      if (lookahead == 'f') ADVANCE(41);
      if (lookahead == 'i') ADVANCE(126);
      if (lookahead == 'm') ADVANCE(36);
      if (lookahead == 's') ADVANCE(176);
      if (lookahead == 't') ADVANCE(157);
      END_STATE();
    case 49:
      if (lookahead == 'c') ADVANCE(140);
      if (lookahead == 'm') ADVANCE(36);
      if (lookahead == 's') ADVANCE(183);
      END_STATE();
    case 50:
      if (lookahead == 'c') ADVANCE(156);
      if (lookahead == 'h') ADVANCE(138);
      if (lookahead == 'u') ADVANCE(114);
      END_STATE();
    case 51:
      if (lookahead == 'd') ADVANCE(193);
      END_STATE();
    case 52:
      if (lookahead == 'd') ADVANCE(375);
      END_STATE();
    case 53:
      if (lookahead == 'd') ADVANCE(360);
      END_STATE();
    case 54:
      if (lookahead == 'd') ADVANCE(354);
      END_STATE();
    case 55:
      if (lookahead == 'd') ADVANCE(89);
      if (lookahead == 'n') ADVANCE(53);
      if (lookahead == 'x') ADVANCE(175);
      END_STATE();
    case 56:
      if (lookahead == 'd') ADVANCE(89);
      if (lookahead == 'x') ADVANCE(175);
      END_STATE();
    case 57:
      if (lookahead == 'd') ADVANCE(68);
      END_STATE();
    case 58:
      if (lookahead == 'd') ADVANCE(67);
      END_STATE();
    case 59:
      if (lookahead == 'e') ADVANCE(55);
      if (lookahead == 'f') ADVANCE(41);
      if (lookahead == 'h') ADVANCE(76);
      if (lookahead == 'i') ADVANCE(121);
      if (lookahead == 'm') ADVANCE(32);
      if (lookahead == 'p') ADVANCE(147);
      if (lookahead == 't') ADVANCE(92);
      END_STATE();
    case 60:
      if (lookahead == 'e') ADVANCE(86);
      if (lookahead == 'i') ADVANCE(154);
      END_STATE();
    case 61:
      if (lookahead == 'e') ADVANCE(375);
      END_STATE();
    case 62:
      if (lookahead == 'e') ADVANCE(44);
      END_STATE();
    case 63:
      if (lookahead == 'e') ADVANCE(352);
      END_STATE();
    case 64:
      if (lookahead == 'e') ADVANCE(364);
      END_STATE();
    case 65:
      if (lookahead == 'e') ADVANCE(329);
      END_STATE();
    case 66:
      if (lookahead == 'e') ADVANCE(353);
      END_STATE();
    case 67:
      if (lookahead == 'e') ADVANCE(361);
      END_STATE();
    case 68:
      if (lookahead == 'e') ADVANCE(366);
      END_STATE();
    case 69:
      if (lookahead == 'e') ADVANCE(348);
      END_STATE();
    case 70:
      if (lookahead == 'e') ADVANCE(347);
      END_STATE();
    case 71:
      if (lookahead == 'e') ADVANCE(350);
      END_STATE();
    case 72:
      if (lookahead == 'e') ADVANCE(349);
      END_STATE();
    case 73:
      if (lookahead == 'e') ADVANCE(123);
      if (lookahead != 0) ADVANCE(213);
      END_STATE();
    case 74:
      if (lookahead == 'e') ADVANCE(34);
      END_STATE();
    case 75:
      if (lookahead == 'e') ADVANCE(85);
      END_STATE();
    case 76:
      if (lookahead == 'e') ADVANCE(182);
      END_STATE();
    case 77:
      if (lookahead == 'e') ADVANCE(87);
      END_STATE();
    case 78:
      if (lookahead == 'e') ADVANCE(141);
      END_STATE();
    case 79:
      if (lookahead == 'e') ADVANCE(155);
      END_STATE();
    case 80:
      if (lookahead == 'e') ADVANCE(54);
      END_STATE();
    case 81:
      if (lookahead == 'e') ADVANCE(153);
      END_STATE();
    case 82:
      if (lookahead == 'e') ADVANCE(46);
      END_STATE();
    case 83:
      if (lookahead == 'e') ADVANCE(118);
      END_STATE();
    case 84:
      if (lookahead == 'f') ADVANCE(294);
      END_STATE();
    case 85:
      if (lookahead == 'f') ADVANCE(165);
      END_STATE();
    case 86:
      if (lookahead == 'f') ADVANCE(35);
      END_STATE();
    case 87:
      if (lookahead == 'f') ADVANCE(99);
      END_STATE();
    case 88:
      if (lookahead == 'g') ADVANCE(91);
      END_STATE();
    case 89:
      if (lookahead == 'g') ADVANCE(64);
      END_STATE();
    case 90:
      if (lookahead == 'g') ADVANCE(152);
      if (lookahead == 'j') ADVANCE(82);
      END_STATE();
    case 91:
      if (lookahead == 'h') ADVANCE(165);
      END_STATE();
    case 92:
      if (lookahead == 'h') ADVANCE(78);
      if (lookahead == 'r') ADVANCE(186);
      END_STATE();
    case 93:
      if (lookahead == 'i') ADVANCE(88);
      END_STATE();
    case 94:
      if (lookahead == 'i') ADVANCE(161);
      END_STATE();
    case 95:
      if (lookahead == 'i') ADVANCE(144);
      END_STATE();
    case 96:
      if (lookahead == 'i') ADVANCE(163);
      END_STATE();
    case 97:
      if (lookahead == 'i') ADVANCE(42);
      END_STATE();
    case 98:
      if (lookahead == 'i') ADVANCE(116);
      END_STATE();
    case 99:
      if (lookahead == 'i') ADVANCE(129);
      END_STATE();
    case 100:
      if (lookahead == 'i') ADVANCE(126);
      if (lookahead == 's') ADVANCE(177);
      END_STATE();
    case 101:
      if (lookahead == 'i') ADVANCE(188);
      END_STATE();
    case 102:
      if (lookahead == 'i') ADVANCE(162);
      END_STATE();
    case 103:
      if (lookahead == 'i') ADVANCE(58);
      END_STATE();
    case 104:
      if (lookahead == 'i') ADVANCE(132);
      if (lookahead == 'o') ADVANCE(51);
      END_STATE();
    case 105:
      if (lookahead == 'i') ADVANCE(119);
      END_STATE();
    case 106:
      if (lookahead == 'l') ADVANCE(160);
      END_STATE();
    case 107:
      if (lookahead == 'l') ADVANCE(371);
      END_STATE();
    case 108:
      if (lookahead == 'l') ADVANCE(180);
      END_STATE();
    case 109:
      if (lookahead == 'l') ADVANCE(166);
      END_STATE();
    case 110:
      if (lookahead == 'l') ADVANCE(159);
      END_STATE();
    case 111:
      if (lookahead == 'm') ADVANCE(293);
      END_STATE();
    case 112:
      if (lookahead == 'm') ADVANCE(356);
      END_STATE();
    case 113:
      if (lookahead == 'm') ADVANCE(338);
      END_STATE();
    case 114:
      if (lookahead == 'm') ADVANCE(338);
      if (lookahead == 'p') ADVANCE(292);
      END_STATE();
    case 115:
      if (lookahead == 'm') ADVANCE(291);
      END_STATE();
    case 116:
      if (lookahead == 'm') ADVANCE(94);
      END_STATE();
    case 117:
      if (lookahead == 'm') ADVANCE(179);
      END_STATE();
    case 118:
      if (lookahead == 'm') ADVANCE(181);
      END_STATE();
    case 119:
      if (lookahead == 'm') ADVANCE(102);
      END_STATE();
    case 120:
      if (lookahead == 'n') ADVANCE(193);
      END_STATE();
    case 121:
      if (lookahead == 'n') ADVANCE(43);
      END_STATE();
    case 122:
      if (lookahead == 'n') ADVANCE(337);
      END_STATE();
    case 123:
      if (lookahead == 'n') ADVANCE(214);
      if (lookahead != 0) ADVANCE(213);
      END_STATE();
    case 124:
      if (lookahead == 'n') ADVANCE(53);
      END_STATE();
    case 125:
      if (lookahead == 'n') ADVANCE(39);
      END_STATE();
    case 126:
      if (lookahead == 'n') ADVANCE(84);
      END_STATE();
    case 127:
      if (lookahead == 'n') ADVANCE(139);
      END_STATE();
    case 128:
      if (lookahead == 'n') ADVANCE(169);
      END_STATE();
    case 129:
      if (lookahead == 'n') ADVANCE(80);
      END_STATE();
    case 130:
      if (lookahead == 'n') ADVANCE(37);
      END_STATE();
    case 131:
      if (lookahead == 'n') ADVANCE(158);
      if (lookahead == 'u') ADVANCE(128);
      END_STATE();
    case 132:
      if (lookahead == 'n') ADVANCE(40);
      END_STATE();
    case 133:
      if (lookahead == 'n') ADVANCE(340);
      END_STATE();
    case 134:
      if (lookahead == 'n') ADVANCE(105);
      END_STATE();
    case 135:
      if (lookahead == 'o') ADVANCE(164);
      END_STATE();
    case 136:
      if (lookahead == 'o') ADVANCE(90);
      END_STATE();
    case 137:
      if (lookahead == 'o') ADVANCE(131);
      END_STATE();
    case 138:
      if (lookahead == 'o') ADVANCE(189);
      END_STATE();
    case 139:
      if (lookahead == 'o') ADVANCE(167);
      END_STATE();
    case 140:
      if (lookahead == 'o') ADVANCE(185);
      END_STATE();
    case 141:
      if (lookahead == 'o') ADVANCE(151);
      END_STATE();
    case 142:
      if (lookahead == 'o') ADVANCE(172);
      END_STATE();
    case 143:
      if (lookahead == 'p') ADVANCE(292);
      END_STATE();
    case 144:
      if (lookahead == 'p') ADVANCE(171);
      END_STATE();
    case 145:
      if (lookahead == 'r') ADVANCE(184);
      END_STATE();
    case 146:
      if (lookahead == 'r') ADVANCE(194);
      END_STATE();
    case 147:
      if (lookahead == 'r') ADVANCE(136);
      END_STATE();
    case 148:
      if (lookahead == 'r') ADVANCE(96);
      END_STATE();
    case 149:
      if (lookahead == 'r') ADVANCE(195);
      END_STATE();
    case 150:
      if (lookahead == 'r') ADVANCE(103);
      END_STATE();
    case 151:
      if (lookahead == 'r') ADVANCE(196);
      END_STATE();
    case 152:
      if (lookahead == 'r') ADVANCE(38);
      END_STATE();
    case 153:
      if (lookahead == 'r') ADVANCE(130);
      END_STATE();
    case 154:
      if (lookahead == 'r') ADVANCE(62);
      END_STATE();
    case 155:
      if (lookahead == 'r') ADVANCE(150);
      END_STATE();
    case 156:
      if (lookahead == 'r') ADVANCE(95);
      END_STATE();
    case 157:
      if (lookahead == 'r') ADVANCE(186);
      END_STATE();
    case 158:
      if (lookahead == 's') ADVANCE(170);
      END_STATE();
    case 159:
      if (lookahead == 's') ADVANCE(65);
      END_STATE();
    case 160:
      if (lookahead == 's') ADVANCE(66);
      END_STATE();
    case 161:
      if (lookahead == 's') ADVANCE(69);
      if (lookahead == 'z') ADVANCE(70);
      END_STATE();
    case 162:
      if (lookahead == 's') ADVANCE(71);
      if (lookahead == 'z') ADVANCE(72);
      END_STATE();
    case 163:
      if (lookahead == 's') ADVANCE(174);
      END_STATE();
    case 164:
      if (lookahead == 't') ADVANCE(333);
      END_STATE();
    case 165:
      if (lookahead == 't') ADVANCE(374);
      END_STATE();
    case 166:
      if (lookahead == 't') ADVANCE(361);
      END_STATE();
    case 167:
      if (lookahead == 't') ADVANCE(336);
      END_STATE();
    case 168:
      if (lookahead == 't') ADVANCE(355);
      END_STATE();
    case 169:
      if (lookahead == 't') ADVANCE(337);
      END_STATE();
    case 170:
      if (lookahead == 't') ADVANCE(362);
      END_STATE();
    case 171:
      if (lookahead == 't') ADVANCE(359);
      END_STATE();
    case 172:
      if (lookahead == 't') ADVANCE(332);
      END_STATE();
    case 173:
      if (lookahead == 't') ADVANCE(101);
      END_STATE();
    case 174:
      if (lookahead == 't') ADVANCE(97);
      END_STATE();
    case 175:
      if (lookahead == 't') ADVANCE(81);
      END_STATE();
    case 176:
      if (lookahead == 'u') ADVANCE(114);
      END_STATE();
    case 177:
      if (lookahead == 'u') ADVANCE(143);
      END_STATE();
    case 178:
      if (lookahead == 'u') ADVANCE(109);
      END_STATE();
    case 179:
      if (lookahead == 'u') ADVANCE(111);
      END_STATE();
    case 180:
      if (lookahead == 'u') ADVANCE(57);
      END_STATE();
    case 181:
      if (lookahead == 'u') ADVANCE(115);
      END_STATE();
    case 182:
      if (lookahead == 'u') ADVANCE(148);
      END_STATE();
    case 183:
      if (lookahead == 'u') ADVANCE(113);
      END_STATE();
    case 184:
      if (lookahead == 'u') ADVANCE(63);
      END_STATE();
    case 185:
      if (lookahead == 'u') ADVANCE(128);
      END_STATE();
    case 186:
      if (lookahead == 'u') ADVANCE(65);
      END_STATE();
    case 187:
      if (lookahead == 'v') ADVANCE(79);
      END_STATE();
    case 188:
      if (lookahead == 'v') ADVANCE(61);
      END_STATE();
    case 189:
      if (lookahead == 'w') ADVANCE(351);
      END_STATE();
    case 190:
      if (lookahead == 'x') ADVANCE(339);
      END_STATE();
    case 191:
      if (lookahead == 'x') ADVANCE(337);
      END_STATE();
    case 192:
      if (lookahead == 'x') ADVANCE(98);
      END_STATE();
    case 193:
      if (lookahead == 'y') ADVANCE(375);
      END_STATE();
    case 194:
      if (lookahead == 'y') ADVANCE(372);
      END_STATE();
    case 195:
      if (lookahead == 'y') ADVANCE(373);
      END_STATE();
    case 196:
      if (lookahead == 'y') ADVANCE(376);
      END_STATE();
    case 197:
      if (lookahead == '}') ADVANCE(258);
      END_STATE();
    case 198:
      if (lookahead == '\'' ||
          lookahead == '_') ADVANCE(198);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(235);
      END_STATE();
    case 199:
      if (lookahead == '\'' ||
          lookahead == '_') ADVANCE(199);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(303);
      if (('a' <= lookahead && lookahead <= 'z')) ADVANCE(246);
      END_STATE();
    case 200:
      if (lookahead == '\'' ||
          lookahead == '_') ADVANCE(200);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(303);
      END_STATE();
    case 201:
      if (lookahead == '\'' ||
          lookahead == '_') ADVANCE(201);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(235);
      if (('a' <= lookahead && lookahead <= 'z')) ADVANCE(233);
      END_STATE();
    case 202:
      if (lookahead == '0' ||
          lookahead == '1') ADVANCE(296);
      END_STATE();
    case 203:
      if (lookahead == '"' ||
          lookahead == '\\' ||
          lookahead == 'n') ADVANCE(258);
      END_STATE();
    case 204:
      if (lookahead == '"' ||
          lookahead == '\\' ||
          lookahead == 'n') ADVANCE(257);
      END_STATE();
    case 205:
      if (('<' <= lookahead && lookahead <= '>') ||
          lookahead == '^') ADVANCE(270);
      END_STATE();
    case 206:
      if (('1' <= lookahead && lookahead <= '7')) ADVANCE(297);
      END_STATE();
    case 207:
      if (set_contains(sym_theory_operator_character_set_1, 9, lookahead)) ADVANCE(324);
      END_STATE();
    case 208:
      if (set_contains(sym_theory_operator_character_set_1, 9, lookahead)) ADVANCE(323);
      END_STATE();
    case 209:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(299);
      END_STATE();
    case 210:
      if (lookahead != 0 &&
          lookahead != '\n' &&
          lookahead != '\r' &&
          lookahead != '%' &&
          lookahead != '*' &&
          lookahead != '_' &&
          lookahead != '`') ADVANCE(21);
      END_STATE();
    case 211:
      if (lookahead != 0 &&
          lookahead != '\n' &&
          lookahead != '\r' &&
          lookahead != '*' &&
          lookahead != '_' &&
          lookahead != '`') ADVANCE(30);
      END_STATE();
    case 212:
      if (lookahead != 0 &&
          lookahead != '\n' &&
          lookahead != '\r' &&
          lookahead != '*' &&
          lookahead != '_' &&
          lookahead != '`') ADVANCE(29);
      END_STATE();
    case 213:
      if (lookahead != 0 &&
          lookahead != '#') ADVANCE(358);
      END_STATE();
    case 214:
      if (lookahead != 0 &&
          lookahead != 'd') ADVANCE(213);
      END_STATE();
    case 215:
      if (eof) ADVANCE(218);
      ADVANCE_MAP(
        '!', 267,
        '"', 247,
        '#', 59,
        '%', 222,
        '&', 306,
        '\'', 200,
        '(', 317,
        ')', 318,
        '*', 310,
        '+', 307,
        ',', 316,
        '-', 308,
        '.', 342,
        '/', 311,
        ':', 286,
        ';', 320,
        '<', 368,
        '=', 363,
        '>', 370,
        '?', 305,
        '@', 322,
        '[', 326,
        '\\', 312,
        ']', 327,
        '^', 304,
        '_', 302,
        'a', 120,
        'b', 104,
        'd', 60,
        'f', 13,
        'h', 74,
        'l', 75,
        'n', 135,
        'o', 187,
        'r', 93,
        't', 145,
        'u', 125,
        '{', 287,
        '|', 315,
        '}', 328,
        '~', 314,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(215);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(303);
      END_STATE();
    case 216:
      if (eof) ADVANCE(218);
      ADVANCE_MAP(
        '!', 28,
        '"', 247,
        '#', 47,
        '%', 227,
        '&', 306,
        '\'', 199,
        '(', 317,
        ')', 318,
        '*', 310,
        '+', 307,
        ',', 316,
        '-', 308,
        '.', 342,
        '/', 311,
        '0', 295,
        ':', 23,
        ';', 320,
        '<', 331,
        '=', 330,
        '>', 331,
        '?', 305,
        '@', 322,
        '[', 326,
        '\\', 312,
        ']', 327,
        '^', 304,
        '_', 301,
        'f', 241,
        'n', 242,
        '{', 287,
        '|', 315,
        '}', 328,
        '~', 314,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(216);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(298);
      if (('a' <= lookahead && lookahead <= 'z')) ADVANCE(246);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(303);
      END_STATE();
    case 217:
      if (eof) ADVANCE(218);
      ADVANCE_MAP(
        '"', 247,
        '#', 45,
        '%', 222,
        '&', 306,
        '\'', 199,
        '(', 317,
        '-', 308,
        '0', 295,
        ':', 24,
        '@', 322,
        '[', 326,
        '_', 301,
        'f', 241,
        'n', 242,
        '{', 287,
        '|', 315,
        '~', 314,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(217);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(298);
      if (('a' <= lookahead && lookahead <= 'z')) ADVANCE(246);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(303);
      END_STATE();
    case 218:
      ACCEPT_TOKEN(ts_builtin_sym_end);
      END_STATE();
    case 219:
      ACCEPT_TOKEN(sym_line_comment);
      if (lookahead == '#') ADVANCE(228);
      if (lookahead == '\n' ||
          lookahead == '\r') ADVANCE(358);
      if (lookahead != 0) ADVANCE(220);
      END_STATE();
    case 220:
      ACCEPT_TOKEN(sym_line_comment);
      if (lookahead == '#') ADVANCE(224);
      if (lookahead == '\n' ||
          lookahead == '\r') ADVANCE(358);
      if (lookahead != 0) ADVANCE(220);
      END_STATE();
    case 221:
      ACCEPT_TOKEN(sym_line_comment);
      if (lookahead == '#') ADVANCE(224);
      if (lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == '*') ADVANCE(358);
      if (lookahead != 0) ADVANCE(220);
      END_STATE();
    case 222:
      ACCEPT_TOKEN(sym_line_comment);
      if (lookahead == '*') ADVANCE(5);
      if (lookahead != 0 &&
          lookahead != '\n' &&
          lookahead != '\r') ADVANCE(228);
      END_STATE();
    case 223:
      ACCEPT_TOKEN(sym_line_comment);
      if (lookahead == 'd') ADVANCE(228);
      if (lookahead == '\n' ||
          lookahead == '\r') ADVANCE(213);
      if (lookahead != 0) ADVANCE(219);
      END_STATE();
    case 224:
      ACCEPT_TOKEN(sym_line_comment);
      if (lookahead == 'e') ADVANCE(225);
      if (lookahead == '\n' ||
          lookahead == '\r') ADVANCE(213);
      if (lookahead != 0) ADVANCE(219);
      END_STATE();
    case 225:
      ACCEPT_TOKEN(sym_line_comment);
      if (lookahead == 'n') ADVANCE(223);
      if (lookahead == '\n' ||
          lookahead == '\r') ADVANCE(213);
      if (lookahead != 0) ADVANCE(219);
      END_STATE();
    case 226:
      ACCEPT_TOKEN(sym_line_comment);
      if (('<' <= lookahead && lookahead <= '>') ||
          lookahead == '^') ADVANCE(272);
      if (lookahead != 0 &&
          lookahead != '\n' &&
          lookahead != '\r' &&
          lookahead != '*') ADVANCE(228);
      END_STATE();
    case 227:
      ACCEPT_TOKEN(sym_line_comment);
      if (lookahead != 0 &&
          lookahead != '\n' &&
          lookahead != '\r' &&
          lookahead != '*') ADVANCE(228);
      END_STATE();
    case 228:
      ACCEPT_TOKEN(sym_line_comment);
      if (lookahead != 0 &&
          lookahead != '\n' &&
          lookahead != '\r') ADVANCE(228);
      END_STATE();
    case 229:
      ACCEPT_TOKEN(anon_sym_PERCENT_STAR_BANG);
      END_STATE();
    case 230:
      ACCEPT_TOKEN(aux_sym_doc_comment_token1);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(230);
      END_STATE();
    case 231:
      ACCEPT_TOKEN(anon_sym_STAR_PERCENT);
      END_STATE();
    case 232:
      ACCEPT_TOKEN(anon_sym_COMMA);
      END_STATE();
    case 233:
      ACCEPT_TOKEN(aux_sym_doc_predicate_token1);
      if (lookahead == '\'' ||
          ('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(233);
      END_STATE();
    case 234:
      ACCEPT_TOKEN(anon_sym_RPAREN);
      END_STATE();
    case 235:
      ACCEPT_TOKEN(sym_doc_var);
      if (lookahead == '\'' ||
          ('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(235);
      END_STATE();
    case 236:
      ACCEPT_TOKEN(sym_doc_fragment_emph);
      END_STATE();
    case 237:
      ACCEPT_TOKEN(sym_doc_fragment_bold);
      END_STATE();
    case 238:
      ACCEPT_TOKEN(sym_doc_fragment_italic);
      END_STATE();
    case 239:
      ACCEPT_TOKEN(sym_doc_fragment_code);
      END_STATE();
    case 240:
      ACCEPT_TOKEN(anon_sym_COLON);
      END_STATE();
    case 241:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '"') ADVANCE(248);
      if (lookahead == '\'' ||
          ('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(246);
      END_STATE();
    case 242:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'o') ADVANCE(244);
      if (lookahead == '\'' ||
          ('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(246);
      END_STATE();
    case 243:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 'o') ADVANCE(245);
      if (lookahead == '\'' ||
          ('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(246);
      END_STATE();
    case 244:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 't') ADVANCE(334);
      if (lookahead == '\'' ||
          ('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(246);
      END_STATE();
    case 245:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == 't') ADVANCE(335);
      if (lookahead == '\'' ||
          ('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(246);
      END_STATE();
    case 246:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '\'' ||
          ('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(246);
      END_STATE();
    case 247:
      ACCEPT_TOKEN(anon_sym_DQUOTE);
      END_STATE();
    case 248:
      ACCEPT_TOKEN(anon_sym_f_DQUOTE);
      END_STATE();
    case 249:
      ACCEPT_TOKEN(sym__string_fragment);
      if (lookahead == '%') ADVANCE(251);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(249);
      if (lookahead != 0 &&
          lookahead != '"' &&
          lookahead != '\\') ADVANCE(252);
      END_STATE();
    case 250:
      ACCEPT_TOKEN(sym__string_fragment);
      if (lookahead == '\n' ||
          lookahead == '\r') ADVANCE(252);
      if (lookahead != 0 &&
          lookahead != '"' &&
          lookahead != '\\') ADVANCE(250);
      END_STATE();
    case 251:
      ACCEPT_TOKEN(sym__string_fragment);
      if (lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == '*') ADVANCE(252);
      if (lookahead != 0 &&
          lookahead != '"' &&
          lookahead != '\\') ADVANCE(250);
      END_STATE();
    case 252:
      ACCEPT_TOKEN(sym__string_fragment);
      if (lookahead != 0 &&
          lookahead != '"' &&
          lookahead != '\\') ADVANCE(252);
      END_STATE();
    case 253:
      ACCEPT_TOKEN(sym__fstring_fragment);
      if (lookahead == '%') ADVANCE(255);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(253);
      if (lookahead != 0 &&
          lookahead != '"' &&
          lookahead != '\\' &&
          lookahead != '{' &&
          lookahead != '}') ADVANCE(256);
      END_STATE();
    case 254:
      ACCEPT_TOKEN(sym__fstring_fragment);
      if (lookahead == '\n' ||
          lookahead == '\r') ADVANCE(256);
      if (lookahead != 0 &&
          lookahead != '"' &&
          lookahead != '\\' &&
          lookahead != '{' &&
          lookahead != '}') ADVANCE(254);
      END_STATE();
    case 255:
      ACCEPT_TOKEN(sym__fstring_fragment);
      if (lookahead == '\n' ||
          lookahead == '\r' ||
          lookahead == '*') ADVANCE(256);
      if (lookahead != 0 &&
          lookahead != '"' &&
          lookahead != '\\' &&
          lookahead != '{' &&
          lookahead != '}') ADVANCE(254);
      END_STATE();
    case 256:
      ACCEPT_TOKEN(sym__fstring_fragment);
      if (lookahead != 0 &&
          lookahead != '"' &&
          lookahead != '\\' &&
          lookahead != '{' &&
          lookahead != '}') ADVANCE(256);
      END_STATE();
    case 257:
      ACCEPT_TOKEN(sym__string_escape);
      END_STATE();
    case 258:
      ACCEPT_TOKEN(sym__fstring_escape);
      END_STATE();
    case 259:
      ACCEPT_TOKEN(aux_sym_fstring_accessor_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(259);
      END_STATE();
    case 260:
      ACCEPT_TOKEN(aux_sym_fstring_accessor_token2);
      END_STATE();
    case 261:
      ACCEPT_TOKEN(aux_sym_fstring_accessor_token3);
      END_STATE();
    case 262:
      ACCEPT_TOKEN(aux_sym_fstring_accessor_token3);
      if (('<' <= lookahead && lookahead <= '>') ||
          lookahead == '^') ADVANCE(270);
      END_STATE();
    case 263:
      ACCEPT_TOKEN(aux_sym_fstring_accessor_token3);
      if (('<' <= lookahead && lookahead <= '>') ||
          lookahead == '^') ADVANCE(270);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(264);
      END_STATE();
    case 264:
      ACCEPT_TOKEN(aux_sym_fstring_accessor_token3);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(264);
      END_STATE();
    case 265:
      ACCEPT_TOKEN(aux_sym_fstring_accessor_token4);
      END_STATE();
    case 266:
      ACCEPT_TOKEN(aux_sym_fstring_conversion_token1);
      END_STATE();
    case 267:
      ACCEPT_TOKEN(aux_sym_fstring_conversion_token1);
      if (lookahead == '=') ADVANCE(330);
      END_STATE();
    case 268:
      ACCEPT_TOKEN(aux_sym_fstring_conversion_token2);
      END_STATE();
    case 269:
      ACCEPT_TOKEN(aux_sym_fstring_conversion_token2);
      if (lookahead == 'r' ||
          lookahead == 's') ADVANCE(268);
      END_STATE();
    case 270:
      ACCEPT_TOKEN(sym_fstring_align);
      END_STATE();
    case 271:
      ACCEPT_TOKEN(sym_fstring_align);
      if (('<' <= lookahead && lookahead <= '>') ||
          lookahead == '^') ADVANCE(270);
      END_STATE();
    case 272:
      ACCEPT_TOKEN(sym_fstring_align);
      if (lookahead != 0 &&
          lookahead != '\n' &&
          lookahead != '\r') ADVANCE(228);
      END_STATE();
    case 273:
      ACCEPT_TOKEN(sym_fstring_sign);
      END_STATE();
    case 274:
      ACCEPT_TOKEN(sym_fstring_sign);
      if (('<' <= lookahead && lookahead <= '>') ||
          lookahead == '^') ADVANCE(270);
      END_STATE();
    case 275:
      ACCEPT_TOKEN(sym_fstring_alternate);
      END_STATE();
    case 276:
      ACCEPT_TOKEN(sym_fstring_alternate);
      if (lookahead == 'e') ADVANCE(55);
      if (lookahead == 'f') ADVANCE(41);
      if (lookahead == 'h') ADVANCE(76);
      if (lookahead == 'i') ADVANCE(121);
      if (lookahead == 'm') ADVANCE(32);
      if (lookahead == 'p') ADVANCE(147);
      if (lookahead == 't') ADVANCE(92);
      END_STATE();
    case 277:
      ACCEPT_TOKEN(sym_fstring_alternate);
      if (('<' <= lookahead && lookahead <= '>') ||
          lookahead == '^') ADVANCE(270);
      END_STATE();
    case 278:
      ACCEPT_TOKEN(sym_fstring_grouping);
      END_STATE();
    case 279:
      ACCEPT_TOKEN(sym_fstring_grouping);
      if (('<' <= lookahead && lookahead <= '>') ||
          lookahead == '^') ADVANCE(270);
      END_STATE();
    case 280:
      ACCEPT_TOKEN(sym_fstring_type);
      END_STATE();
    case 281:
      ACCEPT_TOKEN(sym_fstring_type);
      if (lookahead == 'e') ADVANCE(86);
      if (lookahead == 'i') ADVANCE(154);
      END_STATE();
    case 282:
      ACCEPT_TOKEN(sym_fstring_type);
      if (lookahead == 'i') ADVANCE(132);
      if (lookahead == 'o') ADVANCE(51);
      END_STATE();
    case 283:
      ACCEPT_TOKEN(sym_fstring_type);
      if (lookahead == 'o') ADVANCE(164);
      END_STATE();
    case 284:
      ACCEPT_TOKEN(sym_fstring_type);
      if (lookahead == 'v') ADVANCE(79);
      END_STATE();
    case 285:
      ACCEPT_TOKEN(sym_fstring_type);
      if (('<' <= lookahead && lookahead <= '>') ||
          lookahead == '^') ADVANCE(270);
      END_STATE();
    case 286:
      ACCEPT_TOKEN(aux_sym_fstring_spec_token1);
      END_STATE();
    case 287:
      ACCEPT_TOKEN(anon_sym_LBRACE);
      END_STATE();
    case 288:
      ACCEPT_TOKEN(anon_sym_LBRACE);
      if (lookahead == '{') ADVANCE(258);
      END_STATE();
    case 289:
      ACCEPT_TOKEN(anon_sym_RBRACE);
      END_STATE();
    case 290:
      ACCEPT_TOKEN(anon_sym_RBRACE);
      if (('<' <= lookahead && lookahead <= '>') ||
          lookahead == '^') ADVANCE(270);
      END_STATE();
    case 291:
      ACCEPT_TOKEN(sym_supremum);
      END_STATE();
    case 292:
      ACCEPT_TOKEN(sym_supremum);
      if (lookahead == 'r') ADVANCE(83);
      END_STATE();
    case 293:
      ACCEPT_TOKEN(sym_infimum);
      END_STATE();
    case 294:
      ACCEPT_TOKEN(sym_infimum);
      if (lookahead == 'i') ADVANCE(117);
      END_STATE();
    case 295:
      ACCEPT_TOKEN(sym_number);
      if (lookahead == 'b') ADVANCE(202);
      if (lookahead == 'o') ADVANCE(206);
      if (lookahead == 'x') ADVANCE(209);
      END_STATE();
    case 296:
      ACCEPT_TOKEN(sym_number);
      if (lookahead == '0' ||
          lookahead == '1') ADVANCE(296);
      END_STATE();
    case 297:
      ACCEPT_TOKEN(sym_number);
      if (('1' <= lookahead && lookahead <= '7')) ADVANCE(297);
      END_STATE();
    case 298:
      ACCEPT_TOKEN(sym_number);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(298);
      END_STATE();
    case 299:
      ACCEPT_TOKEN(sym_number);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(299);
      END_STATE();
    case 300:
      ACCEPT_TOKEN(sym_anonymous);
      if (lookahead == '\'' ||
          lookahead == '_') ADVANCE(198);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(235);
      END_STATE();
    case 301:
      ACCEPT_TOKEN(sym_anonymous);
      if (lookahead == '\'' ||
          lookahead == '_') ADVANCE(199);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(303);
      if (('a' <= lookahead && lookahead <= 'z')) ADVANCE(246);
      END_STATE();
    case 302:
      ACCEPT_TOKEN(sym_anonymous);
      if (lookahead == '\'' ||
          lookahead == '_') ADVANCE(200);
      if (('A' <= lookahead && lookahead <= 'Z')) ADVANCE(303);
      END_STATE();
    case 303:
      ACCEPT_TOKEN(sym_variable);
      if (lookahead == '\'' ||
          ('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(303);
      END_STATE();
    case 304:
      ACCEPT_TOKEN(anon_sym_CARET);
      END_STATE();
    case 305:
      ACCEPT_TOKEN(anon_sym_QMARK);
      END_STATE();
    case 306:
      ACCEPT_TOKEN(anon_sym_AMP);
      END_STATE();
    case 307:
      ACCEPT_TOKEN(anon_sym_PLUS);
      END_STATE();
    case 308:
      ACCEPT_TOKEN(anon_sym_DASH);
      END_STATE();
    case 309:
      ACCEPT_TOKEN(anon_sym_STAR);
      END_STATE();
    case 310:
      ACCEPT_TOKEN(anon_sym_STAR);
      if (lookahead == '*') ADVANCE(313);
      END_STATE();
    case 311:
      ACCEPT_TOKEN(anon_sym_SLASH);
      END_STATE();
    case 312:
      ACCEPT_TOKEN(anon_sym_BSLASH);
      END_STATE();
    case 313:
      ACCEPT_TOKEN(anon_sym_STAR_STAR);
      END_STATE();
    case 314:
      ACCEPT_TOKEN(anon_sym_TILDE);
      END_STATE();
    case 315:
      ACCEPT_TOKEN(anon_sym_PIPE);
      END_STATE();
    case 316:
      ACCEPT_TOKEN(anon_sym_COMMA2);
      END_STATE();
    case 317:
      ACCEPT_TOKEN(anon_sym_LPAREN);
      END_STATE();
    case 318:
      ACCEPT_TOKEN(anon_sym_RPAREN2);
      END_STATE();
    case 319:
      ACCEPT_TOKEN(anon_sym_DOT_DOT);
      END_STATE();
    case 320:
      ACCEPT_TOKEN(anon_sym_SEMI);
      END_STATE();
    case 321:
      ACCEPT_TOKEN(anon_sym_SEMI);
      if (set_contains(sym_theory_operator_character_set_1, 9, lookahead)) ADVANCE(324);
      END_STATE();
    case 322:
      ACCEPT_TOKEN(anon_sym_AT);
      END_STATE();
    case 323:
      ACCEPT_TOKEN(sym_theory_operator);
      END_STATE();
    case 324:
      ACCEPT_TOKEN(sym_theory_operator);
      if (set_contains(sym_theory_operator_character_set_1, 9, lookahead)) ADVANCE(324);
      END_STATE();
    case 325:
      ACCEPT_TOKEN(sym_theory_operator);
      if (set_contains(sym_theory_operator_character_set_1, 9, lookahead)) ADVANCE(323);
      END_STATE();
    case 326:
      ACCEPT_TOKEN(anon_sym_LBRACK);
      END_STATE();
    case 327:
      ACCEPT_TOKEN(anon_sym_RBRACK);
      END_STATE();
    case 328:
      ACCEPT_TOKEN(anon_sym_RBRACE2);
      END_STATE();
    case 329:
      ACCEPT_TOKEN(sym_boolean_constant);
      END_STATE();
    case 330:
      ACCEPT_TOKEN(sym_relation);
      END_STATE();
    case 331:
      ACCEPT_TOKEN(sym_relation);
      if (lookahead == '=') ADVANCE(330);
      END_STATE();
    case 332:
      ACCEPT_TOKEN(sym_default_negation);
      END_STATE();
    case 333:
      ACCEPT_TOKEN(sym_default_negation);
      if (lookahead == ' ') ADVANCE(127);
      END_STATE();
    case 334:
      ACCEPT_TOKEN(sym_default_negation);
      if (lookahead == ' ') ADVANCE(127);
      if (lookahead == '\'' ||
          ('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(246);
      END_STATE();
    case 335:
      ACCEPT_TOKEN(sym_default_negation);
      if (lookahead == '\'' ||
          ('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(246);
      END_STATE();
    case 336:
      ACCEPT_TOKEN(sym_double_default_negation);
      END_STATE();
    case 337:
      ACCEPT_TOKEN(sym_aggregate_function);
      END_STATE();
    case 338:
      ACCEPT_TOKEN(sym_aggregate_function);
      if (lookahead == '+') ADVANCE(337);
      END_STATE();
    case 339:
      ACCEPT_TOKEN(sym_aggregate_function);
      if (lookahead == 'i') ADVANCE(116);
      END_STATE();
    case 340:
      ACCEPT_TOKEN(sym_aggregate_function);
      if (lookahead == 'i') ADVANCE(119);
      END_STATE();
    case 341:
      ACCEPT_TOKEN(anon_sym_DOT);
      END_STATE();
    case 342:
      ACCEPT_TOKEN(anon_sym_DOT);
      if (lookahead == '.') ADVANCE(319);
      END_STATE();
    case 343:
      ACCEPT_TOKEN(anon_sym_DOT);
      if (set_contains(sym_theory_operator_character_set_1, 9, lookahead)) ADVANCE(324);
      END_STATE();
    case 344:
      ACCEPT_TOKEN(anon_sym_COLON_DASH);
      END_STATE();
    case 345:
      ACCEPT_TOKEN(anon_sym_COLON_DASH);
      if (set_contains(sym_theory_operator_character_set_1, 9, lookahead)) ADVANCE(323);
      END_STATE();
    case 346:
      ACCEPT_TOKEN(anon_sym_COLON_TILDE);
      END_STATE();
    case 347:
      ACCEPT_TOKEN(anon_sym_POUNDmaximize);
      END_STATE();
    case 348:
      ACCEPT_TOKEN(anon_sym_POUNDmaximise);
      END_STATE();
    case 349:
      ACCEPT_TOKEN(anon_sym_POUNDminimize);
      END_STATE();
    case 350:
      ACCEPT_TOKEN(anon_sym_POUNDminimise);
      END_STATE();
    case 351:
      ACCEPT_TOKEN(anon_sym_POUNDshow);
      END_STATE();
    case 352:
      ACCEPT_TOKEN(anon_sym_true);
      END_STATE();
    case 353:
      ACCEPT_TOKEN(anon_sym_false);
      END_STATE();
    case 354:
      ACCEPT_TOKEN(anon_sym_POUNDdefined);
      END_STATE();
    case 355:
      ACCEPT_TOKEN(anon_sym_POUNDproject);
      END_STATE();
    case 356:
      ACCEPT_TOKEN(anon_sym_POUNDprogram);
      END_STATE();
    case 357:
      ACCEPT_TOKEN(sym_code);
      if (lookahead == '#') ADVANCE(73);
      if (lookahead == '%') ADVANCE(221);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(357);
      if (lookahead != 0) ADVANCE(358);
      END_STATE();
    case 358:
      ACCEPT_TOKEN(sym_code);
      if (lookahead == '#') ADVANCE(73);
      if (lookahead != 0) ADVANCE(358);
      END_STATE();
    case 359:
      ACCEPT_TOKEN(anon_sym_POUNDscript);
      END_STATE();
    case 360:
      ACCEPT_TOKEN(anon_sym_POUNDend);
      END_STATE();
    case 361:
      ACCEPT_TOKEN(sym_const_type);
      END_STATE();
    case 362:
      ACCEPT_TOKEN(anon_sym_POUNDconst);
      END_STATE();
    case 363:
      ACCEPT_TOKEN(anon_sym_EQ);
      END_STATE();
    case 364:
      ACCEPT_TOKEN(anon_sym_POUNDedge);
      END_STATE();
    case 365:
      ACCEPT_TOKEN(anon_sym_POUNDheuristic);
      END_STATE();
    case 366:
      ACCEPT_TOKEN(anon_sym_POUNDinclude);
      END_STATE();
    case 367:
      ACCEPT_TOKEN(anon_sym_LT);
      END_STATE();
    case 368:
      ACCEPT_TOKEN(anon_sym_LT);
      if (lookahead == '=') ADVANCE(330);
      END_STATE();
    case 369:
      ACCEPT_TOKEN(anon_sym_GT);
      END_STATE();
    case 370:
      ACCEPT_TOKEN(anon_sym_GT);
      if (lookahead == '=') ADVANCE(330);
      END_STATE();
    case 371:
      ACCEPT_TOKEN(anon_sym_POUNDexternal);
      END_STATE();
    case 372:
      ACCEPT_TOKEN(sym_theory_operator_arity);
      END_STATE();
    case 373:
      ACCEPT_TOKEN(anon_sym_binary);
      END_STATE();
    case 374:
      ACCEPT_TOKEN(sym_theory_operator_associativity);
      END_STATE();
    case 375:
      ACCEPT_TOKEN(sym_theory_atom_type);
      END_STATE();
    case 376:
      ACCEPT_TOKEN(anon_sym_POUNDtheory);
      END_STATE();
    default:
      return false;
  }
}

static const TSLexerMode ts_lex_modes[STATE_COUNT] = {
  [0] = {.lex_state = 0, .external_lex_state = 1},
  [1] = {.lex_state = 217, .external_lex_state = 2},
  [2] = {.lex_state = 217, .external_lex_state = 2},
  [3] = {.lex_state = 217, .external_lex_state = 2},
  [4] = {.lex_state = 216, .external_lex_state = 2},
  [5] = {.lex_state = 216, .external_lex_state = 2},
  [6] = {.lex_state = 216, .external_lex_state = 2},
  [7] = {.lex_state = 216, .external_lex_state = 2},
  [8] = {.lex_state = 216, .external_lex_state = 2},
  [9] = {.lex_state = 216, .external_lex_state = 2},
  [10] = {.lex_state = 216, .external_lex_state = 2},
  [11] = {.lex_state = 216, .external_lex_state = 2},
  [12] = {.lex_state = 216, .external_lex_state = 2},
  [13] = {.lex_state = 216, .external_lex_state = 2},
  [14] = {.lex_state = 216, .external_lex_state = 2},
  [15] = {.lex_state = 217, .external_lex_state = 2},
  [16] = {.lex_state = 217, .external_lex_state = 2},
  [17] = {.lex_state = 217, .external_lex_state = 2},
  [18] = {.lex_state = 217, .external_lex_state = 2},
  [19] = {.lex_state = 217, .external_lex_state = 2},
  [20] = {.lex_state = 217, .external_lex_state = 2},
  [21] = {.lex_state = 217, .external_lex_state = 2},
  [22] = {.lex_state = 217, .external_lex_state = 2},
  [23] = {.lex_state = 217, .external_lex_state = 2},
  [24] = {.lex_state = 217, .external_lex_state = 2},
  [25] = {.lex_state = 217, .external_lex_state = 2},
  [26] = {.lex_state = 217, .external_lex_state = 2},
  [27] = {.lex_state = 217, .external_lex_state = 2},
  [28] = {.lex_state = 217, .external_lex_state = 2},
  [29] = {.lex_state = 217, .external_lex_state = 2},
  [30] = {.lex_state = 217, .external_lex_state = 2},
  [31] = {.lex_state = 217, .external_lex_state = 2},
  [32] = {.lex_state = 217, .external_lex_state = 2},
  [33] = {.lex_state = 217, .external_lex_state = 2},
  [34] = {.lex_state = 217, .external_lex_state = 2},
  [35] = {.lex_state = 217, .external_lex_state = 2},
  [36] = {.lex_state = 217, .external_lex_state = 2},
  [37] = {.lex_state = 217, .external_lex_state = 2},
  [38] = {.lex_state = 217, .external_lex_state = 2},
  [39] = {.lex_state = 217, .external_lex_state = 2},
  [40] = {.lex_state = 217, .external_lex_state = 2},
  [41] = {.lex_state = 217, .external_lex_state = 2},
  [42] = {.lex_state = 217, .external_lex_state = 2},
  [43] = {.lex_state = 217, .external_lex_state = 2},
  [44] = {.lex_state = 217, .external_lex_state = 2},
  [45] = {.lex_state = 217, .external_lex_state = 2},
  [46] = {.lex_state = 217, .external_lex_state = 2},
  [47] = {.lex_state = 217, .external_lex_state = 2},
  [48] = {.lex_state = 217, .external_lex_state = 2},
  [49] = {.lex_state = 217, .external_lex_state = 2},
  [50] = {.lex_state = 217, .external_lex_state = 2},
  [51] = {.lex_state = 217, .external_lex_state = 2},
  [52] = {.lex_state = 217, .external_lex_state = 2},
  [53] = {.lex_state = 217, .external_lex_state = 2},
  [54] = {.lex_state = 217, .external_lex_state = 2},
  [55] = {.lex_state = 217, .external_lex_state = 2},
  [56] = {.lex_state = 217, .external_lex_state = 2},
  [57] = {.lex_state = 217, .external_lex_state = 2},
  [58] = {.lex_state = 217, .external_lex_state = 2},
  [59] = {.lex_state = 217, .external_lex_state = 2},
  [60] = {.lex_state = 217, .external_lex_state = 2},
  [61] = {.lex_state = 217, .external_lex_state = 2},
  [62] = {.lex_state = 217, .external_lex_state = 2},
  [63] = {.lex_state = 217, .external_lex_state = 2},
  [64] = {.lex_state = 217, .external_lex_state = 2},
  [65] = {.lex_state = 217, .external_lex_state = 2},
  [66] = {.lex_state = 217, .external_lex_state = 2},
  [67] = {.lex_state = 217, .external_lex_state = 2},
  [68] = {.lex_state = 217, .external_lex_state = 2},
  [69] = {.lex_state = 217, .external_lex_state = 2},
  [70] = {.lex_state = 217, .external_lex_state = 2},
  [71] = {.lex_state = 217, .external_lex_state = 2},
  [72] = {.lex_state = 216, .external_lex_state = 2},
  [73] = {.lex_state = 216, .external_lex_state = 2},
  [74] = {.lex_state = 216, .external_lex_state = 2},
  [75] = {.lex_state = 216, .external_lex_state = 2},
  [76] = {.lex_state = 216, .external_lex_state = 2},
  [77] = {.lex_state = 6, .external_lex_state = 2},
  [78] = {.lex_state = 216, .external_lex_state = 2},
  [79] = {.lex_state = 216, .external_lex_state = 2},
  [80] = {.lex_state = 216, .external_lex_state = 2},
  [81] = {.lex_state = 216, .external_lex_state = 2},
  [82] = {.lex_state = 216, .external_lex_state = 2},
  [83] = {.lex_state = 216, .external_lex_state = 2},
  [84] = {.lex_state = 216, .external_lex_state = 2},
  [85] = {.lex_state = 216, .external_lex_state = 2},
  [86] = {.lex_state = 216, .external_lex_state = 2},
  [87] = {.lex_state = 216, .external_lex_state = 2},
  [88] = {.lex_state = 6, .external_lex_state = 2},
  [89] = {.lex_state = 216, .external_lex_state = 2},
  [90] = {.lex_state = 6, .external_lex_state = 3},
  [91] = {.lex_state = 6, .external_lex_state = 3},
  [92] = {.lex_state = 6, .external_lex_state = 3},
  [93] = {.lex_state = 216, .external_lex_state = 2},
  [94] = {.lex_state = 216, .external_lex_state = 2},
  [95] = {.lex_state = 216, .external_lex_state = 2},
  [96] = {.lex_state = 216, .external_lex_state = 2},
  [97] = {.lex_state = 216, .external_lex_state = 2},
  [98] = {.lex_state = 6, .external_lex_state = 4},
  [99] = {.lex_state = 216, .external_lex_state = 2},
  [100] = {.lex_state = 6, .external_lex_state = 3},
  [101] = {.lex_state = 6, .external_lex_state = 3},
  [102] = {.lex_state = 6, .external_lex_state = 3},
  [103] = {.lex_state = 6, .external_lex_state = 3},
  [104] = {.lex_state = 6, .external_lex_state = 2},
  [105] = {.lex_state = 6, .external_lex_state = 2},
  [106] = {.lex_state = 10, .external_lex_state = 5},
  [107] = {.lex_state = 6, .external_lex_state = 2},
  [108] = {.lex_state = 6, .external_lex_state = 2},
  [109] = {.lex_state = 10, .external_lex_state = 5},
  [110] = {.lex_state = 6, .external_lex_state = 5},
  [111] = {.lex_state = 6, .external_lex_state = 4},
  [112] = {.lex_state = 6, .external_lex_state = 2},
  [113] = {.lex_state = 6, .external_lex_state = 5},
  [114] = {.lex_state = 6, .external_lex_state = 5},
  [115] = {.lex_state = 6, .external_lex_state = 2},
  [116] = {.lex_state = 6, .external_lex_state = 5},
  [117] = {.lex_state = 6, .external_lex_state = 2},
  [118] = {.lex_state = 6, .external_lex_state = 5},
  [119] = {.lex_state = 6, .external_lex_state = 5},
  [120] = {.lex_state = 6, .external_lex_state = 2},
  [121] = {.lex_state = 6, .external_lex_state = 2},
  [122] = {.lex_state = 6, .external_lex_state = 2},
  [123] = {.lex_state = 6, .external_lex_state = 2},
  [124] = {.lex_state = 6, .external_lex_state = 2},
  [125] = {.lex_state = 3, .external_lex_state = 2},
  [126] = {.lex_state = 6, .external_lex_state = 2},
  [127] = {.lex_state = 6, .external_lex_state = 2},
  [128] = {.lex_state = 10, .external_lex_state = 5},
  [129] = {.lex_state = 3, .external_lex_state = 2},
  [130] = {.lex_state = 10, .external_lex_state = 2},
  [131] = {.lex_state = 6, .external_lex_state = 2},
  [132] = {.lex_state = 6, .external_lex_state = 2},
  [133] = {.lex_state = 6, .external_lex_state = 2},
  [134] = {.lex_state = 6, .external_lex_state = 2},
  [135] = {.lex_state = 6, .external_lex_state = 2},
  [136] = {.lex_state = 6, .external_lex_state = 2},
  [137] = {.lex_state = 6, .external_lex_state = 2},
  [138] = {.lex_state = 6, .external_lex_state = 2},
  [139] = {.lex_state = 6, .external_lex_state = 2},
  [140] = {.lex_state = 6, .external_lex_state = 2},
  [141] = {.lex_state = 6, .external_lex_state = 2},
  [142] = {.lex_state = 6, .external_lex_state = 2},
  [143] = {.lex_state = 10, .external_lex_state = 2},
  [144] = {.lex_state = 6, .external_lex_state = 2},
  [145] = {.lex_state = 10, .external_lex_state = 2},
  [146] = {.lex_state = 6, .external_lex_state = 2},
  [147] = {.lex_state = 6, .external_lex_state = 5},
  [148] = {.lex_state = 10, .external_lex_state = 2},
  [149] = {.lex_state = 10, .external_lex_state = 2},
  [150] = {.lex_state = 6, .external_lex_state = 5},
  [151] = {.lex_state = 10, .external_lex_state = 2},
  [152] = {.lex_state = 10, .external_lex_state = 2},
  [153] = {.lex_state = 10, .external_lex_state = 2},
  [154] = {.lex_state = 6, .external_lex_state = 2},
  [155] = {.lex_state = 3, .external_lex_state = 2},
  [156] = {.lex_state = 3, .external_lex_state = 2},
  [157] = {.lex_state = 10, .external_lex_state = 2},
  [158] = {.lex_state = 3, .external_lex_state = 2},
  [159] = {.lex_state = 3, .external_lex_state = 2},
  [160] = {.lex_state = 6, .external_lex_state = 2},
  [161] = {.lex_state = 3, .external_lex_state = 2},
  [162] = {.lex_state = 10, .external_lex_state = 2},
  [163] = {.lex_state = 3, .external_lex_state = 2},
  [164] = {.lex_state = 3, .external_lex_state = 2},
  [165] = {.lex_state = 3, .external_lex_state = 2},
  [166] = {.lex_state = 3, .external_lex_state = 2},
  [167] = {.lex_state = 3, .external_lex_state = 2},
  [168] = {.lex_state = 6, .external_lex_state = 2},
  [169] = {.lex_state = 3, .external_lex_state = 2},
  [170] = {.lex_state = 3, .external_lex_state = 2},
  [171] = {.lex_state = 6, .external_lex_state = 2},
  [172] = {.lex_state = 6, .external_lex_state = 2},
  [173] = {.lex_state = 10, .external_lex_state = 2},
  [174] = {.lex_state = 6, .external_lex_state = 2},
  [175] = {.lex_state = 6, .external_lex_state = 2},
  [176] = {.lex_state = 6, .external_lex_state = 2},
  [177] = {.lex_state = 6, .external_lex_state = 2},
  [178] = {.lex_state = 6, .external_lex_state = 2},
  [179] = {.lex_state = 6, .external_lex_state = 2},
  [180] = {.lex_state = 6, .external_lex_state = 2},
  [181] = {.lex_state = 6, .external_lex_state = 2},
  [182] = {.lex_state = 10, .external_lex_state = 2},
  [183] = {.lex_state = 10, .external_lex_state = 2},
  [184] = {.lex_state = 10, .external_lex_state = 2},
  [185] = {.lex_state = 6, .external_lex_state = 2},
  [186] = {.lex_state = 216, .external_lex_state = 5},
  [187] = {.lex_state = 6, .external_lex_state = 2},
  [188] = {.lex_state = 216, .external_lex_state = 2},
  [189] = {.lex_state = 6, .external_lex_state = 2},
  [190] = {.lex_state = 6, .external_lex_state = 2},
  [191] = {.lex_state = 6, .external_lex_state = 2},
  [192] = {.lex_state = 6, .external_lex_state = 2},
  [193] = {.lex_state = 6, .external_lex_state = 2},
  [194] = {.lex_state = 6, .external_lex_state = 2},
  [195] = {.lex_state = 6, .external_lex_state = 2},
  [196] = {.lex_state = 6, .external_lex_state = 2},
  [197] = {.lex_state = 6, .external_lex_state = 2},
  [198] = {.lex_state = 6, .external_lex_state = 2},
  [199] = {.lex_state = 6, .external_lex_state = 2},
  [200] = {.lex_state = 6, .external_lex_state = 2},
  [201] = {.lex_state = 6, .external_lex_state = 2},
  [202] = {.lex_state = 6, .external_lex_state = 2},
  [203] = {.lex_state = 6, .external_lex_state = 2},
  [204] = {.lex_state = 6, .external_lex_state = 2},
  [205] = {.lex_state = 6, .external_lex_state = 2},
  [206] = {.lex_state = 6, .external_lex_state = 2},
  [207] = {.lex_state = 6, .external_lex_state = 2},
  [208] = {.lex_state = 6, .external_lex_state = 2},
  [209] = {.lex_state = 6, .external_lex_state = 2},
  [210] = {.lex_state = 6, .external_lex_state = 2},
  [211] = {.lex_state = 6, .external_lex_state = 2},
  [212] = {.lex_state = 6, .external_lex_state = 2},
  [213] = {.lex_state = 6, .external_lex_state = 2},
  [214] = {.lex_state = 6, .external_lex_state = 2},
  [215] = {.lex_state = 6, .external_lex_state = 2},
  [216] = {.lex_state = 6, .external_lex_state = 2},
  [217] = {.lex_state = 6, .external_lex_state = 2},
  [218] = {.lex_state = 216, .external_lex_state = 5},
  [219] = {.lex_state = 6, .external_lex_state = 2},
  [220] = {.lex_state = 6, .external_lex_state = 2},
  [221] = {.lex_state = 6, .external_lex_state = 2},
  [222] = {.lex_state = 6, .external_lex_state = 2},
  [223] = {.lex_state = 6, .external_lex_state = 2},
  [224] = {.lex_state = 6, .external_lex_state = 2},
  [225] = {.lex_state = 216, .external_lex_state = 2},
  [226] = {.lex_state = 6, .external_lex_state = 2},
  [227] = {.lex_state = 6, .external_lex_state = 2},
  [228] = {.lex_state = 6, .external_lex_state = 2},
  [229] = {.lex_state = 6, .external_lex_state = 2},
  [230] = {.lex_state = 6, .external_lex_state = 2},
  [231] = {.lex_state = 6, .external_lex_state = 2},
  [232] = {.lex_state = 6, .external_lex_state = 2},
  [233] = {.lex_state = 6, .external_lex_state = 2},
  [234] = {.lex_state = 6, .external_lex_state = 2},
  [235] = {.lex_state = 6, .external_lex_state = 2},
  [236] = {.lex_state = 6, .external_lex_state = 2},
  [237] = {.lex_state = 6, .external_lex_state = 2},
  [238] = {.lex_state = 6, .external_lex_state = 2},
  [239] = {.lex_state = 6, .external_lex_state = 2},
  [240] = {.lex_state = 6, .external_lex_state = 2},
  [241] = {.lex_state = 216, .external_lex_state = 5},
  [242] = {.lex_state = 216, .external_lex_state = 5},
  [243] = {.lex_state = 216, .external_lex_state = 5},
  [244] = {.lex_state = 216, .external_lex_state = 5},
  [245] = {.lex_state = 216, .external_lex_state = 5},
  [246] = {.lex_state = 216, .external_lex_state = 5},
  [247] = {.lex_state = 216, .external_lex_state = 5},
  [248] = {.lex_state = 6, .external_lex_state = 2},
  [249] = {.lex_state = 216, .external_lex_state = 5},
  [250] = {.lex_state = 216, .external_lex_state = 2},
  [251] = {.lex_state = 216, .external_lex_state = 5},
  [252] = {.lex_state = 216, .external_lex_state = 5},
  [253] = {.lex_state = 216, .external_lex_state = 2},
  [254] = {.lex_state = 216, .external_lex_state = 2},
  [255] = {.lex_state = 216, .external_lex_state = 2},
  [256] = {.lex_state = 216, .external_lex_state = 2},
  [257] = {.lex_state = 216, .external_lex_state = 5},
  [258] = {.lex_state = 8, .external_lex_state = 2},
  [259] = {.lex_state = 216, .external_lex_state = 5},
  [260] = {.lex_state = 216, .external_lex_state = 5},
  [261] = {.lex_state = 216, .external_lex_state = 5},
  [262] = {.lex_state = 216, .external_lex_state = 5},
  [263] = {.lex_state = 216, .external_lex_state = 2},
  [264] = {.lex_state = 216, .external_lex_state = 2},
  [265] = {.lex_state = 216, .external_lex_state = 2},
  [266] = {.lex_state = 216, .external_lex_state = 5},
  [267] = {.lex_state = 216, .external_lex_state = 5},
  [268] = {.lex_state = 216, .external_lex_state = 5},
  [269] = {.lex_state = 216, .external_lex_state = 2},
  [270] = {.lex_state = 216, .external_lex_state = 2},
  [271] = {.lex_state = 216, .external_lex_state = 5},
  [272] = {.lex_state = 216, .external_lex_state = 5},
  [273] = {.lex_state = 216, .external_lex_state = 5},
  [274] = {.lex_state = 216, .external_lex_state = 5},
  [275] = {.lex_state = 216, .external_lex_state = 2},
  [276] = {.lex_state = 216, .external_lex_state = 5},
  [277] = {.lex_state = 216, .external_lex_state = 5},
  [278] = {.lex_state = 216, .external_lex_state = 5},
  [279] = {.lex_state = 216, .external_lex_state = 2},
  [280] = {.lex_state = 6, .external_lex_state = 2},
  [281] = {.lex_state = 216, .external_lex_state = 5},
  [282] = {.lex_state = 216, .external_lex_state = 2},
  [283] = {.lex_state = 216, .external_lex_state = 2},
  [284] = {.lex_state = 216, .external_lex_state = 2},
  [285] = {.lex_state = 216, .external_lex_state = 2},
  [286] = {.lex_state = 216, .external_lex_state = 2},
  [287] = {.lex_state = 216, .external_lex_state = 2},
  [288] = {.lex_state = 216, .external_lex_state = 2},
  [289] = {.lex_state = 216, .external_lex_state = 2},
  [290] = {.lex_state = 216, .external_lex_state = 2},
  [291] = {.lex_state = 216, .external_lex_state = 2},
  [292] = {.lex_state = 216, .external_lex_state = 2},
  [293] = {.lex_state = 216, .external_lex_state = 2},
  [294] = {.lex_state = 216, .external_lex_state = 2},
  [295] = {.lex_state = 216, .external_lex_state = 2},
  [296] = {.lex_state = 216, .external_lex_state = 2},
  [297] = {.lex_state = 6, .external_lex_state = 2},
  [298] = {.lex_state = 216, .external_lex_state = 2},
  [299] = {.lex_state = 216, .external_lex_state = 2},
  [300] = {.lex_state = 216, .external_lex_state = 2},
  [301] = {.lex_state = 216, .external_lex_state = 2},
  [302] = {.lex_state = 216, .external_lex_state = 2},
  [303] = {.lex_state = 6, .external_lex_state = 2},
  [304] = {.lex_state = 216, .external_lex_state = 2},
  [305] = {.lex_state = 6, .external_lex_state = 2},
  [306] = {.lex_state = 216, .external_lex_state = 2},
  [307] = {.lex_state = 6, .external_lex_state = 2},
  [308] = {.lex_state = 6, .external_lex_state = 2},
  [309] = {.lex_state = 6, .external_lex_state = 2},
  [310] = {.lex_state = 6, .external_lex_state = 2},
  [311] = {.lex_state = 6, .external_lex_state = 2},
  [312] = {.lex_state = 6, .external_lex_state = 2},
  [313] = {.lex_state = 216, .external_lex_state = 2},
  [314] = {.lex_state = 216, .external_lex_state = 2},
  [315] = {.lex_state = 6, .external_lex_state = 2},
  [316] = {.lex_state = 216, .external_lex_state = 2},
  [317] = {.lex_state = 6, .external_lex_state = 2},
  [318] = {.lex_state = 6, .external_lex_state = 2},
  [319] = {.lex_state = 216, .external_lex_state = 2},
  [320] = {.lex_state = 216, .external_lex_state = 2},
  [321] = {.lex_state = 6, .external_lex_state = 2},
  [322] = {.lex_state = 8, .external_lex_state = 2},
  [323] = {.lex_state = 8, .external_lex_state = 2},
  [324] = {.lex_state = 8, .external_lex_state = 2},
  [325] = {.lex_state = 216, .external_lex_state = 5},
  [326] = {.lex_state = 8, .external_lex_state = 2},
  [327] = {.lex_state = 216, .external_lex_state = 2},
  [328] = {.lex_state = 216, .external_lex_state = 5},
  [329] = {.lex_state = 8, .external_lex_state = 2},
  [330] = {.lex_state = 216, .external_lex_state = 2},
  [331] = {.lex_state = 8, .external_lex_state = 2},
  [332] = {.lex_state = 8, .external_lex_state = 2},
  [333] = {.lex_state = 8, .external_lex_state = 2},
  [334] = {.lex_state = 216, .external_lex_state = 2},
  [335] = {.lex_state = 216, .external_lex_state = 5},
  [336] = {.lex_state = 216, .external_lex_state = 5},
  [337] = {.lex_state = 216, .external_lex_state = 2},
  [338] = {.lex_state = 216, .external_lex_state = 5},
  [339] = {.lex_state = 216, .external_lex_state = 5},
  [340] = {.lex_state = 216, .external_lex_state = 2},
  [341] = {.lex_state = 216, .external_lex_state = 2},
  [342] = {.lex_state = 216, .external_lex_state = 2},
  [343] = {.lex_state = 10, .external_lex_state = 2},
  [344] = {.lex_state = 216, .external_lex_state = 2},
  [345] = {.lex_state = 216, .external_lex_state = 2},
  [346] = {.lex_state = 216, .external_lex_state = 2},
  [347] = {.lex_state = 216, .external_lex_state = 2},
  [348] = {.lex_state = 216, .external_lex_state = 2},
  [349] = {.lex_state = 216, .external_lex_state = 2},
  [350] = {.lex_state = 216, .external_lex_state = 2},
  [351] = {.lex_state = 216, .external_lex_state = 2},
  [352] = {.lex_state = 216, .external_lex_state = 2},
  [353] = {.lex_state = 216, .external_lex_state = 2},
  [354] = {.lex_state = 216, .external_lex_state = 2},
  [355] = {.lex_state = 216, .external_lex_state = 2},
  [356] = {.lex_state = 216, .external_lex_state = 2},
  [357] = {.lex_state = 216, .external_lex_state = 2},
  [358] = {.lex_state = 216, .external_lex_state = 2},
  [359] = {.lex_state = 10, .external_lex_state = 2},
  [360] = {.lex_state = 216, .external_lex_state = 2},
  [361] = {.lex_state = 216, .external_lex_state = 2},
  [362] = {.lex_state = 216, .external_lex_state = 2},
  [363] = {.lex_state = 216, .external_lex_state = 2},
  [364] = {.lex_state = 216, .external_lex_state = 2},
  [365] = {.lex_state = 216, .external_lex_state = 2},
  [366] = {.lex_state = 14, .external_lex_state = 2},
  [367] = {.lex_state = 14, .external_lex_state = 2},
  [368] = {.lex_state = 14, .external_lex_state = 2},
  [369] = {.lex_state = 14, .external_lex_state = 2},
  [370] = {.lex_state = 216, .external_lex_state = 2},
  [371] = {.lex_state = 216, .external_lex_state = 2},
  [372] = {.lex_state = 10, .external_lex_state = 2},
  [373] = {.lex_state = 216, .external_lex_state = 2},
  [374] = {.lex_state = 216, .external_lex_state = 2},
  [375] = {.lex_state = 216, .external_lex_state = 5},
  [376] = {.lex_state = 216, .external_lex_state = 2},
  [377] = {.lex_state = 216, .external_lex_state = 2},
  [378] = {.lex_state = 216, .external_lex_state = 2},
  [379] = {.lex_state = 216, .external_lex_state = 2},
  [380] = {.lex_state = 216, .external_lex_state = 2},
  [381] = {.lex_state = 216, .external_lex_state = 2},
  [382] = {.lex_state = 216, .external_lex_state = 2},
  [383] = {.lex_state = 216, .external_lex_state = 2},
  [384] = {.lex_state = 216, .external_lex_state = 2},
  [385] = {.lex_state = 216, .external_lex_state = 2},
  [386] = {.lex_state = 14, .external_lex_state = 2},
  [387] = {.lex_state = 216, .external_lex_state = 2},
  [388] = {.lex_state = 216, .external_lex_state = 2},
  [389] = {.lex_state = 14, .external_lex_state = 2},
  [390] = {.lex_state = 216, .external_lex_state = 2},
  [391] = {.lex_state = 216, .external_lex_state = 2},
  [392] = {.lex_state = 15, .external_lex_state = 6},
  [393] = {.lex_state = 14, .external_lex_state = 5},
  [394] = {.lex_state = 216, .external_lex_state = 5},
  [395] = {.lex_state = 14, .external_lex_state = 2},
  [396] = {.lex_state = 14, .external_lex_state = 2},
  [397] = {.lex_state = 14, .external_lex_state = 2},
  [398] = {.lex_state = 14, .external_lex_state = 2},
  [399] = {.lex_state = 14, .external_lex_state = 2},
  [400] = {.lex_state = 14, .external_lex_state = 2},
  [401] = {.lex_state = 14, .external_lex_state = 5},
  [402] = {.lex_state = 14, .external_lex_state = 2},
  [403] = {.lex_state = 14, .external_lex_state = 2},
  [404] = {.lex_state = 14, .external_lex_state = 2},
  [405] = {.lex_state = 14, .external_lex_state = 2},
  [406] = {.lex_state = 14, .external_lex_state = 2},
  [407] = {.lex_state = 14, .external_lex_state = 5},
  [408] = {.lex_state = 14, .external_lex_state = 5},
  [409] = {.lex_state = 14, .external_lex_state = 2},
  [410] = {.lex_state = 14, .external_lex_state = 2},
  [411] = {.lex_state = 15, .external_lex_state = 7},
  [412] = {.lex_state = 216, .external_lex_state = 5},
  [413] = {.lex_state = 15, .external_lex_state = 6},
  [414] = {.lex_state = 1, .external_lex_state = 2},
  [415] = {.lex_state = 216, .external_lex_state = 2},
  [416] = {.lex_state = 14, .external_lex_state = 5},
  [417] = {.lex_state = 216, .external_lex_state = 2},
  [418] = {.lex_state = 15, .external_lex_state = 7},
  [419] = {.lex_state = 15, .external_lex_state = 7},
  [420] = {.lex_state = 15, .external_lex_state = 6},
  [421] = {.lex_state = 14, .external_lex_state = 2},
  [422] = {.lex_state = 15, .external_lex_state = 8},
  [423] = {.lex_state = 14, .external_lex_state = 2},
  [424] = {.lex_state = 216, .external_lex_state = 2},
  [425] = {.lex_state = 2, .external_lex_state = 2},
  [426] = {.lex_state = 216, .external_lex_state = 5},
  [427] = {.lex_state = 216, .external_lex_state = 2},
  [428] = {.lex_state = 14, .external_lex_state = 2},
  [429] = {.lex_state = 15, .external_lex_state = 6},
  [430] = {.lex_state = 14, .external_lex_state = 2},
  [431] = {.lex_state = 15, .external_lex_state = 6},
  [432] = {.lex_state = 14, .external_lex_state = 2},
  [433] = {.lex_state = 216, .external_lex_state = 5},
  [434] = {.lex_state = 14, .external_lex_state = 2},
  [435] = {.lex_state = 216, .external_lex_state = 5},
  [436] = {.lex_state = 14, .external_lex_state = 2},
  [437] = {.lex_state = 216, .external_lex_state = 2},
  [438] = {.lex_state = 14, .external_lex_state = 2},
  [439] = {.lex_state = 216, .external_lex_state = 2},
  [440] = {.lex_state = 216, .external_lex_state = 5},
  [441] = {.lex_state = 14, .external_lex_state = 5},
  [442] = {.lex_state = 14, .external_lex_state = 5},
  [443] = {.lex_state = 216, .external_lex_state = 2},
  [444] = {.lex_state = 14, .external_lex_state = 5},
  [445] = {.lex_state = 8, .external_lex_state = 2},
  [446] = {.lex_state = 14, .external_lex_state = 5},
  [447] = {.lex_state = 216, .external_lex_state = 2},
  [448] = {.lex_state = 14, .external_lex_state = 5},
  [449] = {.lex_state = 216, .external_lex_state = 2},
  [450] = {.lex_state = 14, .external_lex_state = 5},
  [451] = {.lex_state = 14, .external_lex_state = 5},
  [452] = {.lex_state = 14, .external_lex_state = 5},
  [453] = {.lex_state = 216, .external_lex_state = 2},
  [454] = {.lex_state = 11, .external_lex_state = 2},
  [455] = {.lex_state = 14, .external_lex_state = 5},
  [456] = {.lex_state = 14, .external_lex_state = 5},
  [457] = {.lex_state = 11, .external_lex_state = 2},
  [458] = {.lex_state = 11, .external_lex_state = 2},
  [459] = {.lex_state = 11, .external_lex_state = 2},
  [460] = {.lex_state = 8, .external_lex_state = 2},
  [461] = {.lex_state = 216, .external_lex_state = 5},
  [462] = {.lex_state = 8, .external_lex_state = 2},
  [463] = {.lex_state = 11, .external_lex_state = 2},
  [464] = {.lex_state = 11, .external_lex_state = 2},
  [465] = {.lex_state = 8, .external_lex_state = 2},
  [466] = {.lex_state = 11, .external_lex_state = 2},
  [467] = {.lex_state = 14, .external_lex_state = 5},
  [468] = {.lex_state = 14, .external_lex_state = 5},
  [469] = {.lex_state = 14, .external_lex_state = 5},
  [470] = {.lex_state = 216, .external_lex_state = 5},
  [471] = {.lex_state = 8, .external_lex_state = 2},
  [472] = {.lex_state = 8, .external_lex_state = 2},
  [473] = {.lex_state = 216, .external_lex_state = 5},
  [474] = {.lex_state = 216, .external_lex_state = 2},
  [475] = {.lex_state = 216, .external_lex_state = 5},
  [476] = {.lex_state = 6, .external_lex_state = 2},
  [477] = {.lex_state = 216, .external_lex_state = 2},
  [478] = {.lex_state = 216, .external_lex_state = 5},
  [479] = {.lex_state = 14, .external_lex_state = 2},
  [480] = {.lex_state = 216, .external_lex_state = 5},
  [481] = {.lex_state = 216, .external_lex_state = 2},
  [482] = {.lex_state = 216, .external_lex_state = 2},
  [483] = {.lex_state = 216, .external_lex_state = 5},
  [484] = {.lex_state = 8, .external_lex_state = 2},
  [485] = {.lex_state = 8, .external_lex_state = 2},
  [486] = {.lex_state = 216, .external_lex_state = 5},
  [487] = {.lex_state = 216, .external_lex_state = 2},
  [488] = {.lex_state = 8, .external_lex_state = 2},
  [489] = {.lex_state = 216, .external_lex_state = 2},
  [490] = {.lex_state = 216, .external_lex_state = 5},
  [491] = {.lex_state = 216, .external_lex_state = 2},
  [492] = {.lex_state = 216, .external_lex_state = 5},
  [493] = {.lex_state = 216, .external_lex_state = 2},
  [494] = {.lex_state = 216, .external_lex_state = 5},
  [495] = {.lex_state = 216, .external_lex_state = 2},
  [496] = {.lex_state = 216, .external_lex_state = 5},
  [497] = {.lex_state = 216, .external_lex_state = 5},
  [498] = {.lex_state = 216, .external_lex_state = 5},
  [499] = {.lex_state = 12, .external_lex_state = 2},
  [500] = {.lex_state = 216, .external_lex_state = 2},
  [501] = {.lex_state = 6, .external_lex_state = 2},
  [502] = {.lex_state = 216, .external_lex_state = 2},
  [503] = {.lex_state = 216, .external_lex_state = 2},
  [504] = {.lex_state = 216, .external_lex_state = 5},
  [505] = {.lex_state = 216, .external_lex_state = 2},
  [506] = {.lex_state = 216, .external_lex_state = 2},
  [507] = {.lex_state = 216, .external_lex_state = 2},
  [508] = {.lex_state = 216, .external_lex_state = 2},
  [509] = {.lex_state = 216, .external_lex_state = 2},
  [510] = {.lex_state = 11, .external_lex_state = 2},
  [511] = {.lex_state = 216, .external_lex_state = 2},
  [512] = {.lex_state = 11, .external_lex_state = 2},
  [513] = {.lex_state = 216, .external_lex_state = 2},
  [514] = {.lex_state = 11, .external_lex_state = 2},
  [515] = {.lex_state = 12, .external_lex_state = 2},
  [516] = {.lex_state = 12, .external_lex_state = 2},
  [517] = {.lex_state = 15, .external_lex_state = 9},
  [518] = {.lex_state = 12, .external_lex_state = 2},
  [519] = {.lex_state = 15, .external_lex_state = 9},
  [520] = {.lex_state = 216, .external_lex_state = 2},
  [521] = {.lex_state = 11, .external_lex_state = 2},
  [522] = {.lex_state = 12, .external_lex_state = 2},
  [523] = {.lex_state = 6, .external_lex_state = 2},
  [524] = {.lex_state = 216, .external_lex_state = 2},
  [525] = {.lex_state = 14, .external_lex_state = 2},
  [526] = {.lex_state = 216, .external_lex_state = 2},
  [527] = {.lex_state = 216, .external_lex_state = 2},
  [528] = {.lex_state = 12, .external_lex_state = 2},
  [529] = {.lex_state = 216, .external_lex_state = 2},
  [530] = {.lex_state = 216, .external_lex_state = 2},
  [531] = {.lex_state = 12, .external_lex_state = 2},
  [532] = {.lex_state = 15, .external_lex_state = 9},
  [533] = {.lex_state = 216, .external_lex_state = 2},
  [534] = {.lex_state = 12, .external_lex_state = 2},
  [535] = {.lex_state = 12, .external_lex_state = 2},
  [536] = {.lex_state = 12, .external_lex_state = 2},
  [537] = {.lex_state = 12, .external_lex_state = 2},
  [538] = {.lex_state = 216, .external_lex_state = 2},
  [539] = {.lex_state = 216, .external_lex_state = 5},
  [540] = {.lex_state = 216, .external_lex_state = 2},
  [541] = {.lex_state = 216, .external_lex_state = 2},
  [542] = {.lex_state = 3, .external_lex_state = 2},
  [543] = {.lex_state = 216, .external_lex_state = 2},
  [544] = {.lex_state = 216, .external_lex_state = 2},
  [545] = {.lex_state = 216, .external_lex_state = 2},
  [546] = {.lex_state = 216, .external_lex_state = 2},
  [547] = {.lex_state = 216, .external_lex_state = 2},
  [548] = {.lex_state = 8, .external_lex_state = 2},
  [549] = {.lex_state = 6, .external_lex_state = 2},
  [550] = {.lex_state = 216, .external_lex_state = 2},
  [551] = {.lex_state = 216, .external_lex_state = 2},
  [552] = {.lex_state = 216, .external_lex_state = 2},
  [553] = {.lex_state = 216, .external_lex_state = 5},
  [554] = {.lex_state = 6, .external_lex_state = 2},
  [555] = {.lex_state = 216, .external_lex_state = 2},
  [556] = {.lex_state = 216, .external_lex_state = 2},
  [557] = {.lex_state = 216, .external_lex_state = 2},
  [558] = {.lex_state = 14, .external_lex_state = 2},
  [559] = {.lex_state = 216, .external_lex_state = 2},
  [560] = {.lex_state = 216, .external_lex_state = 2},
  [561] = {.lex_state = 216, .external_lex_state = 2},
  [562] = {.lex_state = 15, .external_lex_state = 2},
  [563] = {.lex_state = 8, .external_lex_state = 2},
  [564] = {.lex_state = 216, .external_lex_state = 2},
  [565] = {.lex_state = 15, .external_lex_state = 2},
  [566] = {.lex_state = 216, .external_lex_state = 2},
  [567] = {.lex_state = 3, .external_lex_state = 2},
  [568] = {.lex_state = 3, .external_lex_state = 2},
  [569] = {.lex_state = 15, .external_lex_state = 2},
  [570] = {.lex_state = 216, .external_lex_state = 2},
  [571] = {.lex_state = 216, .external_lex_state = 2},
  [572] = {.lex_state = 216, .external_lex_state = 2},
  [573] = {.lex_state = 216, .external_lex_state = 2},
  [574] = {.lex_state = 216, .external_lex_state = 2},
  [575] = {.lex_state = 8, .external_lex_state = 2},
  [576] = {.lex_state = 6, .external_lex_state = 2},
  [577] = {.lex_state = 8, .external_lex_state = 2},
  [578] = {.lex_state = 15, .external_lex_state = 10},
  [579] = {.lex_state = 216, .external_lex_state = 2},
  [580] = {.lex_state = 216, .external_lex_state = 2},
  [581] = {.lex_state = 216, .external_lex_state = 2},
  [582] = {.lex_state = 3, .external_lex_state = 2},
  [583] = {.lex_state = 216, .external_lex_state = 2},
  [584] = {.lex_state = 216, .external_lex_state = 2},
  [585] = {.lex_state = 216, .external_lex_state = 2},
  [586] = {.lex_state = 216, .external_lex_state = 2},
  [587] = {.lex_state = 216, .external_lex_state = 2},
  [588] = {.lex_state = 216, .external_lex_state = 2},
  [589] = {.lex_state = 216, .external_lex_state = 2},
  [590] = {.lex_state = 216, .external_lex_state = 2},
  [591] = {.lex_state = 16, .external_lex_state = 2},
  [592] = {.lex_state = 216, .external_lex_state = 2},
  [593] = {.lex_state = 216, .external_lex_state = 2},
  [594] = {.lex_state = 216, .external_lex_state = 5},
  [595] = {.lex_state = 8, .external_lex_state = 2},
  [596] = {.lex_state = 216, .external_lex_state = 2},
  [597] = {.lex_state = 216, .external_lex_state = 2},
  [598] = {.lex_state = 216, .external_lex_state = 5},
  [599] = {.lex_state = 216, .external_lex_state = 2},
  [600] = {.lex_state = 216, .external_lex_state = 2},
  [601] = {.lex_state = 216, .external_lex_state = 2},
  [602] = {.lex_state = 216, .external_lex_state = 2},
  [603] = {.lex_state = 216, .external_lex_state = 2},
  [604] = {.lex_state = 216, .external_lex_state = 2},
  [605] = {.lex_state = 216, .external_lex_state = 2},
  [606] = {.lex_state = 216, .external_lex_state = 2},
  [607] = {.lex_state = 216, .external_lex_state = 2},
  [608] = {.lex_state = 8, .external_lex_state = 2},
  [609] = {.lex_state = 216, .external_lex_state = 5},
  [610] = {.lex_state = 216, .external_lex_state = 2},
  [611] = {.lex_state = 216, .external_lex_state = 2},
  [612] = {.lex_state = 216, .external_lex_state = 2},
  [613] = {.lex_state = 216, .external_lex_state = 2},
  [614] = {.lex_state = 216, .external_lex_state = 5},
  [615] = {.lex_state = 216, .external_lex_state = 2},
  [616] = {.lex_state = 216, .external_lex_state = 2},
  [617] = {.lex_state = 216, .external_lex_state = 2},
  [618] = {.lex_state = 6, .external_lex_state = 2},
  [619] = {.lex_state = 216, .external_lex_state = 2},
  [620] = {.lex_state = 216, .external_lex_state = 2},
  [621] = {.lex_state = 216, .external_lex_state = 2},
  [622] = {.lex_state = 216, .external_lex_state = 2},
  [623] = {.lex_state = 3, .external_lex_state = 2},
  [624] = {.lex_state = 216, .external_lex_state = 2},
  [625] = {.lex_state = 216, .external_lex_state = 5},
  [626] = {.lex_state = 216, .external_lex_state = 2},
  [627] = {.lex_state = 216, .external_lex_state = 2},
  [628] = {.lex_state = 216, .external_lex_state = 2},
  [629] = {.lex_state = 216, .external_lex_state = 2},
  [630] = {.lex_state = 216, .external_lex_state = 2},
  [631] = {.lex_state = 216, .external_lex_state = 2},
  [632] = {.lex_state = 216, .external_lex_state = 2},
  [633] = {.lex_state = 216, .external_lex_state = 2},
  [634] = {.lex_state = 216, .external_lex_state = 2},
  [635] = {.lex_state = 216, .external_lex_state = 2},
  [636] = {.lex_state = 216, .external_lex_state = 2},
  [637] = {.lex_state = 216, .external_lex_state = 2},
  [638] = {.lex_state = 8, .external_lex_state = 2},
  [639] = {.lex_state = 216, .external_lex_state = 2},
  [640] = {.lex_state = 216, .external_lex_state = 2},
  [641] = {.lex_state = 3, .external_lex_state = 2},
  [642] = {.lex_state = 216, .external_lex_state = 2},
  [643] = {.lex_state = 15, .external_lex_state = 2},
  [644] = {.lex_state = 15, .external_lex_state = 9},
  [645] = {.lex_state = 216, .external_lex_state = 2},
  [646] = {.lex_state = 216, .external_lex_state = 2},
  [647] = {.lex_state = 14, .external_lex_state = 2},
  [648] = {.lex_state = 216, .external_lex_state = 2},
  [649] = {.lex_state = 216, .external_lex_state = 2},
  [650] = {.lex_state = 216, .external_lex_state = 2},
  [651] = {.lex_state = 216, .external_lex_state = 2},
  [652] = {.lex_state = 216, .external_lex_state = 2},
  [653] = {.lex_state = 216, .external_lex_state = 2},
  [654] = {.lex_state = 216, .external_lex_state = 2},
  [655] = {.lex_state = 216, .external_lex_state = 5},
  [656] = {.lex_state = 216, .external_lex_state = 2},
  [657] = {.lex_state = 216, .external_lex_state = 2},
  [658] = {.lex_state = 216, .external_lex_state = 2},
  [659] = {.lex_state = 216, .external_lex_state = 2},
  [660] = {.lex_state = 216, .external_lex_state = 2},
  [661] = {.lex_state = 216, .external_lex_state = 5},
  [662] = {.lex_state = 216, .external_lex_state = 2},
  [663] = {.lex_state = 216, .external_lex_state = 2},
  [664] = {.lex_state = 216, .external_lex_state = 5},
  [665] = {.lex_state = 216, .external_lex_state = 2},
  [666] = {.lex_state = 216, .external_lex_state = 2},
  [667] = {.lex_state = 216, .external_lex_state = 2},
  [668] = {.lex_state = 216, .external_lex_state = 2},
  [669] = {.lex_state = 216, .external_lex_state = 2},
  [670] = {.lex_state = 216, .external_lex_state = 2},
  [671] = {.lex_state = 216, .external_lex_state = 2},
  [672] = {.lex_state = 216, .external_lex_state = 5},
  [673] = {.lex_state = 216, .external_lex_state = 2},
  [674] = {.lex_state = 216, .external_lex_state = 2},
  [675] = {.lex_state = 216, .external_lex_state = 2},
  [676] = {.lex_state = 216, .external_lex_state = 5},
  [677] = {.lex_state = 216, .external_lex_state = 5},
  [678] = {.lex_state = 216, .external_lex_state = 2},
  [679] = {.lex_state = 216, .external_lex_state = 2},
  [680] = {.lex_state = 216, .external_lex_state = 2},
  [681] = {.lex_state = 216, .external_lex_state = 2},
  [682] = {.lex_state = 216, .external_lex_state = 2},
  [683] = {.lex_state = 216, .external_lex_state = 2},
  [684] = {.lex_state = 3, .external_lex_state = 2},
  [685] = {.lex_state = 216, .external_lex_state = 2},
  [686] = {.lex_state = 6, .external_lex_state = 2},
  [687] = {.lex_state = 216, .external_lex_state = 2},
  [688] = {.lex_state = 216, .external_lex_state = 2},
  [689] = {.lex_state = 216, .external_lex_state = 2},
  [690] = {.lex_state = 216, .external_lex_state = 2},
  [691] = {.lex_state = 216, .external_lex_state = 2},
  [692] = {.lex_state = 216, .external_lex_state = 2},
  [693] = {.lex_state = 8, .external_lex_state = 2},
  [694] = {.lex_state = 16, .external_lex_state = 2},
  [695] = {.lex_state = 216, .external_lex_state = 2},
  [696] = {.lex_state = 216, .external_lex_state = 2},
  [697] = {.lex_state = 216, .external_lex_state = 2},
  [698] = {.lex_state = 216, .external_lex_state = 2},
  [699] = {.lex_state = 216, .external_lex_state = 2},
  [700] = {.lex_state = 8, .external_lex_state = 2},
  [701] = {.lex_state = 216, .external_lex_state = 2},
  [702] = {.lex_state = 216, .external_lex_state = 2},
  [703] = {.lex_state = 216, .external_lex_state = 5},
  [704] = {.lex_state = 216, .external_lex_state = 2},
  [705] = {.lex_state = 216, .external_lex_state = 2},
  [706] = {.lex_state = 8, .external_lex_state = 2},
  [707] = {.lex_state = 216, .external_lex_state = 2},
  [708] = {.lex_state = 216, .external_lex_state = 2},
  [709] = {.lex_state = 216, .external_lex_state = 2},
  [710] = {.lex_state = 216, .external_lex_state = 2},
  [711] = {.lex_state = 216, .external_lex_state = 2},
  [712] = {.lex_state = 216, .external_lex_state = 2},
  [713] = {.lex_state = 8, .external_lex_state = 2},
  [714] = {.lex_state = 216, .external_lex_state = 2},
  [715] = {.lex_state = 216, .external_lex_state = 5},
  [716] = {.lex_state = 6, .external_lex_state = 2},
  [717] = {.lex_state = 216, .external_lex_state = 5},
  [718] = {.lex_state = 3, .external_lex_state = 2},
  [719] = {.lex_state = 216, .external_lex_state = 2},
  [720] = {.lex_state = 6, .external_lex_state = 2},
  [721] = {.lex_state = 216, .external_lex_state = 2},
  [722] = {.lex_state = 216, .external_lex_state = 2},
  [723] = {.lex_state = 216, .external_lex_state = 2},
  [724] = {.lex_state = 216, .external_lex_state = 2},
  [725] = {.lex_state = 216, .external_lex_state = 5},
  [726] = {.lex_state = 6, .external_lex_state = 2},
  [727] = {.lex_state = 216, .external_lex_state = 2},
  [728] = {.lex_state = 216, .external_lex_state = 2},
  [729] = {.lex_state = 216, .external_lex_state = 2},
  [730] = {.lex_state = 216, .external_lex_state = 2},
  [731] = {.lex_state = 230, .external_lex_state = 2},
  [732] = {.lex_state = 216, .external_lex_state = 2},
  [733] = {.lex_state = 216, .external_lex_state = 2},
  [734] = {.lex_state = 216, .external_lex_state = 2},
  [735] = {.lex_state = 216, .external_lex_state = 2},
  [736] = {.lex_state = 216, .external_lex_state = 2},
  [737] = {.lex_state = 216, .external_lex_state = 2},
  [738] = {.lex_state = 3, .external_lex_state = 2},
  [739] = {.lex_state = 216, .external_lex_state = 2},
  [740] = {.lex_state = 216, .external_lex_state = 2},
  [741] = {.lex_state = 216, .external_lex_state = 2},
  [742] = {.lex_state = 230, .external_lex_state = 2},
  [743] = {.lex_state = 216, .external_lex_state = 2},
  [744] = {.lex_state = 6, .external_lex_state = 2},
  [745] = {.lex_state = 3, .external_lex_state = 2},
  [746] = {.lex_state = 216, .external_lex_state = 2},
  [747] = {.lex_state = 216, .external_lex_state = 2},
  [748] = {.lex_state = 3, .external_lex_state = 2},
  [749] = {.lex_state = 216, .external_lex_state = 2},
  [750] = {.lex_state = 3, .external_lex_state = 2},
  [751] = {.lex_state = 230, .external_lex_state = 2},
  [752] = {.lex_state = 216, .external_lex_state = 2},
  [753] = {.lex_state = 216, .external_lex_state = 2},
  [754] = {.lex_state = 216, .external_lex_state = 2},
  [755] = {.lex_state = 216, .external_lex_state = 2},
  [756] = {.lex_state = 15, .external_lex_state = 2},
  [757] = {.lex_state = 216, .external_lex_state = 2},
  [758] = {.lex_state = 216, .external_lex_state = 2},
  [759] = {.lex_state = 3, .external_lex_state = 2},
  [760] = {.lex_state = 216, .external_lex_state = 2},
  [761] = {.lex_state = 216, .external_lex_state = 2},
  [762] = {.lex_state = 3, .external_lex_state = 2},
  [763] = {.lex_state = 216, .external_lex_state = 2},
  [764] = {.lex_state = 216, .external_lex_state = 2},
  [765] = {.lex_state = 216, .external_lex_state = 2},
  [766] = {.lex_state = 216, .external_lex_state = 2},
  [767] = {.lex_state = 216, .external_lex_state = 2},
  [768] = {.lex_state = 230, .external_lex_state = 2},
  [769] = {.lex_state = 216, .external_lex_state = 2},
  [770] = {.lex_state = 6, .external_lex_state = 2},
  [771] = {.lex_state = 216, .external_lex_state = 2},
  [772] = {.lex_state = 216, .external_lex_state = 2},
  [773] = {.lex_state = 216, .external_lex_state = 5},
  [774] = {.lex_state = 216, .external_lex_state = 2},
  [775] = {.lex_state = 6, .external_lex_state = 2},
  [776] = {.lex_state = 6, .external_lex_state = 2},
  [777] = {.lex_state = 216, .external_lex_state = 2},
  [778] = {.lex_state = 216, .external_lex_state = 2},
  [779] = {.lex_state = 216, .external_lex_state = 2},
  [780] = {.lex_state = 216, .external_lex_state = 2},
  [781] = {.lex_state = 216, .external_lex_state = 2},
  [782] = {.lex_state = 3, .external_lex_state = 2},
  [783] = {.lex_state = 216, .external_lex_state = 2},
  [784] = {.lex_state = 216, .external_lex_state = 2},
  [785] = {.lex_state = 6, .external_lex_state = 2},
  [786] = {.lex_state = 230, .external_lex_state = 2},
  [787] = {.lex_state = 216, .external_lex_state = 2},
  [788] = {.lex_state = 230, .external_lex_state = 2},
  [789] = {.lex_state = 216, .external_lex_state = 2},
  [790] = {.lex_state = 216, .external_lex_state = 2},
  [791] = {.lex_state = 216, .external_lex_state = 2},
  [792] = {.lex_state = 216, .external_lex_state = 2},
  [793] = {.lex_state = 216, .external_lex_state = 2},
  [794] = {.lex_state = 230, .external_lex_state = 2},
  [795] = {.lex_state = 216, .external_lex_state = 2},
  [796] = {.lex_state = 3, .external_lex_state = 2},
  [797] = {.lex_state = 3, .external_lex_state = 2},
  [798] = {.lex_state = 230, .external_lex_state = 2},
  [799] = {.lex_state = 216, .external_lex_state = 2},
  [800] = {.lex_state = 230, .external_lex_state = 2},
  [801] = {.lex_state = 216, .external_lex_state = 2},
  [802] = {.lex_state = 8, .external_lex_state = 2},
  [803] = {.lex_state = 216, .external_lex_state = 2},
  [804] = {.lex_state = 216, .external_lex_state = 2},
  [805] = {.lex_state = 216, .external_lex_state = 2},
  [806] = {.lex_state = 3, .external_lex_state = 2},
  [807] = {.lex_state = 216, .external_lex_state = 2},
  [808] = {.lex_state = 216, .external_lex_state = 2},
  [809] = {.lex_state = 6, .external_lex_state = 2},
  [810] = {.lex_state = 216, .external_lex_state = 2},
  [811] = {.lex_state = 216, .external_lex_state = 2},
  [812] = {.lex_state = 216, .external_lex_state = 2},
  [813] = {.lex_state = 216, .external_lex_state = 2},
  [814] = {.lex_state = 216, .external_lex_state = 2},
  [815] = {.lex_state = 216, .external_lex_state = 2},
  [816] = {.lex_state = 8, .external_lex_state = 2},
  [817] = {.lex_state = 216, .external_lex_state = 2},
  [818] = {.lex_state = 216, .external_lex_state = 2},
  [819] = {.lex_state = 216, .external_lex_state = 2},
  [820] = {.lex_state = 6, .external_lex_state = 2},
  [821] = {.lex_state = 15, .external_lex_state = 2},
  [822] = {.lex_state = 6, .external_lex_state = 2},
  [823] = {.lex_state = 216, .external_lex_state = 2},
  [824] = {.lex_state = 216, .external_lex_state = 2},
  [825] = {.lex_state = 216, .external_lex_state = 2},
  [826] = {.lex_state = 216, .external_lex_state = 2},
  [827] = {.lex_state = 216, .external_lex_state = 2},
  [828] = {.lex_state = 216, .external_lex_state = 2},
  [829] = {.lex_state = 3, .external_lex_state = 2},
  [830] = {.lex_state = 216, .external_lex_state = 5},
  [831] = {.lex_state = 216, .external_lex_state = 2},
  [832] = {.lex_state = 216, .external_lex_state = 2},
  [833] = {.lex_state = 216, .external_lex_state = 2},
  [834] = {.lex_state = 216, .external_lex_state = 2},
  [835] = {.lex_state = 216, .external_lex_state = 2},
  [836] = {.lex_state = 216, .external_lex_state = 2},
  [837] = {.lex_state = 230, .external_lex_state = 2},
  [838] = {.lex_state = 216, .external_lex_state = 2},
  [839] = {.lex_state = 230, .external_lex_state = 2},
  [840] = {.lex_state = 216, .external_lex_state = 2},
  [841] = {.lex_state = 216, .external_lex_state = 2},
  [842] = {.lex_state = 6, .external_lex_state = 2},
  [843] = {.lex_state = 216, .external_lex_state = 2},
  [844] = {.lex_state = 216, .external_lex_state = 2},
  [845] = {.lex_state = 216, .external_lex_state = 2},
  [846] = {.lex_state = 216, .external_lex_state = 2},
  [847] = {.lex_state = 216, .external_lex_state = 2},
  [848] = {.lex_state = 216, .external_lex_state = 2},
  [849] = {.lex_state = 216, .external_lex_state = 2},
  [850] = {.lex_state = 230, .external_lex_state = 2},
  [851] = {.lex_state = 216, .external_lex_state = 2},
  [852] = {.lex_state = 357, .external_lex_state = 2},
  [853] = {.lex_state = 216, .external_lex_state = 2},
  [854] = {.lex_state = 8, .external_lex_state = 2},
  [855] = {.lex_state = 216, .external_lex_state = 2},
  [856] = {.lex_state = 6, .external_lex_state = 2},
  [857] = {.lex_state = 216, .external_lex_state = 2},
  [858] = {.lex_state = 216, .external_lex_state = 2},
  [859] = {.lex_state = 6, .external_lex_state = 2},
  [860] = {.lex_state = 15, .external_lex_state = 2},
  [861] = {.lex_state = 3, .external_lex_state = 2},
  [862] = {.lex_state = 216, .external_lex_state = 2},
  [863] = {.lex_state = 6, .external_lex_state = 2},
  [864] = {.lex_state = 216, .external_lex_state = 2},
  [865] = {.lex_state = 3, .external_lex_state = 2},
  [866] = {.lex_state = 216, .external_lex_state = 2},
  [867] = {.lex_state = 6, .external_lex_state = 2},
  [868] = {.lex_state = 216, .external_lex_state = 2},
  [869] = {.lex_state = 3, .external_lex_state = 2},
  [870] = {.lex_state = 216, .external_lex_state = 2},
  [871] = {.lex_state = 216, .external_lex_state = 2},
  [872] = {.lex_state = 3, .external_lex_state = 2},
  [873] = {.lex_state = 3, .external_lex_state = 2},
  [874] = {.lex_state = 216, .external_lex_state = 2},
  [875] = {.lex_state = 6, .external_lex_state = 2},
  [876] = {.lex_state = 216, .external_lex_state = 2},
  [877] = {.lex_state = 216, .external_lex_state = 5},
  [878] = {.lex_state = 216, .external_lex_state = 2},
  [879] = {.lex_state = 216, .external_lex_state = 2},
  [880] = {.lex_state = 269, .external_lex_state = 2},
  [881] = {.lex_state = 6, .external_lex_state = 2},
  [882] = {.lex_state = 216, .external_lex_state = 2},
  [883] = {.lex_state = 216, .external_lex_state = 2},
  [884] = {.lex_state = 216, .external_lex_state = 2},
  [885] = {.lex_state = 216, .external_lex_state = 2},
};

static const uint16_t ts_parse_table[LARGE_STATE_COUNT][SYMBOL_COUNT] = {
  [STATE(0)] = {
    [ts_builtin_sym_end] = ACTIONS(1),
    [sym_line_comment] = ACTIONS(3),
    [anon_sym_PERCENT_STAR_BANG] = ACTIONS(1),
    [sym_doc_var] = ACTIONS(1),
    [sym_doc_fragment_code] = ACTIONS(1),
    [anon_sym_DQUOTE] = ACTIONS(1),
    [anon_sym_f_DQUOTE] = ACTIONS(1),
    [aux_sym_fstring_accessor_token2] = ACTIONS(1),
    [aux_sym_fstring_accessor_token3] = ACTIONS(1),
    [aux_sym_fstring_accessor_token4] = ACTIONS(1),
    [aux_sym_fstring_conversion_token1] = ACTIONS(1),
    [sym_fstring_alternate] = ACTIONS(1),
    [sym_fstring_grouping] = ACTIONS(1),
    [sym_fstring_type] = ACTIONS(1),
    [aux_sym_fstring_spec_token1] = ACTIONS(1),
    [anon_sym_LBRACE] = ACTIONS(1),
    [sym_infimum] = ACTIONS(1),
    [sym_anonymous] = ACTIONS(1),
    [sym_variable] = ACTIONS(1),
    [anon_sym_CARET] = ACTIONS(1),
    [anon_sym_QMARK] = ACTIONS(1),
    [anon_sym_AMP] = ACTIONS(1),
    [anon_sym_PLUS] = ACTIONS(1),
    [anon_sym_DASH] = ACTIONS(1),
    [anon_sym_STAR] = ACTIONS(1),
    [anon_sym_SLASH] = ACTIONS(1),
    [anon_sym_BSLASH] = ACTIONS(1),
    [anon_sym_STAR_STAR] = ACTIONS(1),
    [anon_sym_TILDE] = ACTIONS(1),
    [anon_sym_PIPE] = ACTIONS(1),
    [anon_sym_COMMA2] = ACTIONS(1),
    [anon_sym_LPAREN] = ACTIONS(1),
    [anon_sym_RPAREN2] = ACTIONS(1),
    [anon_sym_DOT_DOT] = ACTIONS(1),
    [anon_sym_SEMI] = ACTIONS(1),
    [anon_sym_AT] = ACTIONS(1),
    [anon_sym_LBRACK] = ACTIONS(1),
    [anon_sym_RBRACK] = ACTIONS(1),
    [anon_sym_RBRACE2] = ACTIONS(1),
    [sym_boolean_constant] = ACTIONS(1),
    [sym_relation] = ACTIONS(1),
    [sym_default_negation] = ACTIONS(1),
    [sym_double_default_negation] = ACTIONS(1),
    [anon_sym_DOT] = ACTIONS(1),
    [anon_sym_POUNDmaximize] = ACTIONS(1),
    [anon_sym_POUNDmaximise] = ACTIONS(1),
    [anon_sym_POUNDminimize] = ACTIONS(1),
    [anon_sym_POUNDminimise] = ACTIONS(1),
    [anon_sym_true] = ACTIONS(1),
    [anon_sym_false] = ACTIONS(1),
    [anon_sym_POUNDproject] = ACTIONS(1),
    [anon_sym_POUNDprogram] = ACTIONS(1),
    [anon_sym_POUNDend] = ACTIONS(1),
    [sym_const_type] = ACTIONS(1),
    [anon_sym_EQ] = ACTIONS(1),
    [anon_sym_POUNDedge] = ACTIONS(1),
    [anon_sym_POUNDheuristic] = ACTIONS(1),
    [anon_sym_POUNDinclude] = ACTIONS(1),
    [anon_sym_LT] = ACTIONS(1),
    [anon_sym_GT] = ACTIONS(1),
    [anon_sym_POUNDexternal] = ACTIONS(1),
    [sym_theory_operator_arity] = ACTIONS(1),
    [anon_sym_binary] = ACTIONS(1),
    [sym_theory_operator_associativity] = ACTIONS(1),
    [sym_theory_atom_type] = ACTIONS(1),
    [anon_sym_POUNDtheory] = ACTIONS(1),
    [sym_empty_pool_item_first] = ACTIONS(1),
    [sym_empty_pool_item] = ACTIONS(1),
    [sym_colon] = ACTIONS(1),
    [sym_block_comment] = ACTIONS(5),
    [sym_doc_fragment_string] = ACTIONS(1),
    [sym__doc_token_args] = ACTIONS(1),
    [sym__doc_token_paren] = ACTIONS(1),
    [sym__doc_token_minus] = ACTIONS(1),
  },
  [STATE(1)] = {
    [sym_source_file] = STATE(882),
    [sym_doc_comment] = STATE(23),
    [sym_string] = STATE(158),
    [sym_fstring] = STATE(158),
    [sym_binary_operation] = STATE(158),
    [sym_unary_operation] = STATE(158),
    [sym_abs] = STATE(158),
    [sym_function] = STATE(158),
    [sym_external_function] = STATE(158),
    [sym_tuple] = STATE(158),
    [sym_term] = STATE(355),
    [sym_symbolic_atom] = STATE(426),
    [sym_comparison] = STATE(426),
    [sym_simple_atom] = STATE(435),
    [sym_literal_sign] = STATE(123),
    [sym_literal] = STATE(461),
    [sym_lower] = STATE(695),
    [sym_set_aggregate] = STATE(658),
    [sym_theory_atom] = STATE(658),
    [sym_head_aggregate] = STATE(658),
    [sym__conditional_literal_n] = STATE(520),
    [sym__conditional_literal_0] = STATE(597),
    [sym_conditional_literal] = STATE(682),
    [sym_disjunction] = STATE(658),
    [sym_head] = STATE(685),
    [sym_rule] = STATE(23),
    [sym_integrity_constraint] = STATE(23),
    [sym_weak_constraint] = STATE(23),
    [sym_maximize] = STATE(23),
    [sym_minimize] = STATE(23),
    [sym_show] = STATE(23),
    [sym_show_term] = STATE(53),
    [sym_show_signature] = STATE(21),
    [sym_defined] = STATE(23),
    [sym_project_signature] = STATE(23),
    [sym_project_atom] = STATE(23),
    [sym_program] = STATE(23),
    [sym_script] = STATE(23),
    [sym_const] = STATE(23),
    [sym_edge] = STATE(23),
    [sym_heuristic] = STATE(23),
    [sym_include] = STATE(23),
    [sym_external] = STATE(23),
    [sym_theory] = STATE(23),
    [sym_statement] = STATE(2),
    [aux_sym_source_file_repeat1] = STATE(2),
    [aux_sym_disjunction_repeat1] = STATE(72),
    [ts_builtin_sym_end] = ACTIONS(7),
    [sym_line_comment] = ACTIONS(3),
    [anon_sym_PERCENT_STAR_BANG] = ACTIONS(9),
    [sym_identifier] = ACTIONS(11),
    [anon_sym_DQUOTE] = ACTIONS(13),
    [anon_sym_f_DQUOTE] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [sym_supremum] = ACTIONS(19),
    [sym_infimum] = ACTIONS(19),
    [sym_number] = ACTIONS(19),
    [sym_anonymous] = ACTIONS(21),
    [sym_variable] = ACTIONS(19),
    [anon_sym_AMP] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(25),
    [anon_sym_TILDE] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LPAREN] = ACTIONS(31),
    [anon_sym_AT] = ACTIONS(33),
    [sym_boolean_constant] = ACTIONS(35),
    [sym_default_negation] = ACTIONS(37),
    [sym_double_default_negation] = ACTIONS(39),
    [sym_aggregate_function] = ACTIONS(41),
    [anon_sym_COLON_DASH] = ACTIONS(43),
    [anon_sym_COLON_TILDE] = ACTIONS(45),
    [anon_sym_POUNDmaximize] = ACTIONS(47),
    [anon_sym_POUNDmaximise] = ACTIONS(47),
    [anon_sym_POUNDminimize] = ACTIONS(49),
    [anon_sym_POUNDminimise] = ACTIONS(49),
    [anon_sym_POUNDshow] = ACTIONS(51),
    [anon_sym_POUNDdefined] = ACTIONS(53),
    [anon_sym_POUNDproject] = ACTIONS(55),
    [anon_sym_POUNDprogram] = ACTIONS(57),
    [anon_sym_POUNDscript] = ACTIONS(59),
    [anon_sym_POUNDconst] = ACTIONS(61),
    [anon_sym_POUNDedge] = ACTIONS(63),
    [anon_sym_POUNDheuristic] = ACTIONS(65),
    [anon_sym_POUNDinclude] = ACTIONS(67),
    [anon_sym_POUNDexternal] = ACTIONS(69),
    [anon_sym_POUNDtheory] = ACTIONS(71),
    [sym_block_comment] = ACTIONS(5),
  },
  [STATE(2)] = {
    [sym_doc_comment] = STATE(23),
    [sym_string] = STATE(158),
    [sym_fstring] = STATE(158),
    [sym_binary_operation] = STATE(158),
    [sym_unary_operation] = STATE(158),
    [sym_abs] = STATE(158),
    [sym_function] = STATE(158),
    [sym_external_function] = STATE(158),
    [sym_tuple] = STATE(158),
    [sym_term] = STATE(355),
    [sym_symbolic_atom] = STATE(426),
    [sym_comparison] = STATE(426),
    [sym_simple_atom] = STATE(435),
    [sym_literal_sign] = STATE(123),
    [sym_literal] = STATE(461),
    [sym_lower] = STATE(695),
    [sym_set_aggregate] = STATE(658),
    [sym_theory_atom] = STATE(658),
    [sym_head_aggregate] = STATE(658),
    [sym__conditional_literal_n] = STATE(520),
    [sym__conditional_literal_0] = STATE(597),
    [sym_conditional_literal] = STATE(682),
    [sym_disjunction] = STATE(658),
    [sym_head] = STATE(685),
    [sym_rule] = STATE(23),
    [sym_integrity_constraint] = STATE(23),
    [sym_weak_constraint] = STATE(23),
    [sym_maximize] = STATE(23),
    [sym_minimize] = STATE(23),
    [sym_show] = STATE(23),
    [sym_show_term] = STATE(53),
    [sym_show_signature] = STATE(21),
    [sym_defined] = STATE(23),
    [sym_project_signature] = STATE(23),
    [sym_project_atom] = STATE(23),
    [sym_program] = STATE(23),
    [sym_script] = STATE(23),
    [sym_const] = STATE(23),
    [sym_edge] = STATE(23),
    [sym_heuristic] = STATE(23),
    [sym_include] = STATE(23),
    [sym_external] = STATE(23),
    [sym_theory] = STATE(23),
    [sym_statement] = STATE(3),
    [aux_sym_source_file_repeat1] = STATE(3),
    [aux_sym_disjunction_repeat1] = STATE(72),
    [ts_builtin_sym_end] = ACTIONS(73),
    [sym_line_comment] = ACTIONS(3),
    [anon_sym_PERCENT_STAR_BANG] = ACTIONS(9),
    [sym_identifier] = ACTIONS(11),
    [anon_sym_DQUOTE] = ACTIONS(13),
    [anon_sym_f_DQUOTE] = ACTIONS(15),
    [anon_sym_LBRACE] = ACTIONS(17),
    [sym_supremum] = ACTIONS(19),
    [sym_infimum] = ACTIONS(19),
    [sym_number] = ACTIONS(19),
    [sym_anonymous] = ACTIONS(21),
    [sym_variable] = ACTIONS(19),
    [anon_sym_AMP] = ACTIONS(23),
    [anon_sym_DASH] = ACTIONS(25),
    [anon_sym_TILDE] = ACTIONS(27),
    [anon_sym_PIPE] = ACTIONS(29),
    [anon_sym_LPAREN] = ACTIONS(31),
    [anon_sym_AT] = ACTIONS(33),
    [sym_boolean_constant] = ACTIONS(35),
    [sym_default_negation] = ACTIONS(37),
    [sym_double_default_negation] = ACTIONS(39),
    [sym_aggregate_function] = ACTIONS(41),
    [anon_sym_COLON_DASH] = ACTIONS(43),
    [anon_sym_COLON_TILDE] = ACTIONS(45),
    [anon_sym_POUNDmaximize] = ACTIONS(47),
    [anon_sym_POUNDmaximise] = ACTIONS(47),
    [anon_sym_POUNDminimize] = ACTIONS(49),
    [anon_sym_POUNDminimise] = ACTIONS(49),
    [anon_sym_POUNDshow] = ACTIONS(51),
    [anon_sym_POUNDdefined] = ACTIONS(53),
    [anon_sym_POUNDproject] = ACTIONS(55),
    [anon_sym_POUNDprogram] = ACTIONS(57),
    [anon_sym_POUNDscript] = ACTIONS(59),
    [anon_sym_POUNDconst] = ACTIONS(61),
    [anon_sym_POUNDedge] = ACTIONS(63),
    [anon_sym_POUNDheuristic] = ACTIONS(65),
    [anon_sym_POUNDinclude] = ACTIONS(67),
    [anon_sym_POUNDexternal] = ACTIONS(69),
    [anon_sym_POUNDtheory] = ACTIONS(71),
    [sym_block_comment] = ACTIONS(5),
  },
  [STATE(3)] = {
    [sym_doc_comment] = STATE(23),
    [sym_string] = STATE(158),
    [sym_fstring] = STATE(158),
    [sym_binary_operation] = STATE(158),
    [sym_unary_operation] = STATE(158),
    [sym_abs] = STATE(158),
    [sym_function] = STATE(158),
    [sym_external_function] = STATE(158),
    [sym_tuple] = STATE(158),
    [sym_term] = STATE(355),
    [sym_symbolic_atom] = STATE(426),
    [sym_comparison] = STATE(426),
    [sym_simple_atom] = STATE(435),
    [sym_literal_sign] = STATE(123),
    [sym_literal] = STATE(461),
    [sym_lower] = STATE(695),
    [sym_set_aggregate] = STATE(658),
    [sym_theory_atom] = STATE(658),
    [sym_head_aggregate] = STATE(658),
    [sym__conditional_literal_n] = STATE(520),
    [sym__conditional_literal_0] = STATE(597),
    [sym_conditional_literal] = STATE(682),
    [sym_disjunction] = STATE(658),
    [sym_head] = STATE(685),
    [sym_rule] = STATE(23),
    [sym_integrity_constraint] = STATE(23),
    [sym_weak_constraint] = STATE(23),
    [sym_maximize] = STATE(23),
    [sym_minimize] = STATE(23),
    [sym_show] = STATE(23),
    [sym_show_term] = STATE(53),
    [sym_show_signature] = STATE(21),
    [sym_defined] = STATE(23),
    [sym_project_signature] = STATE(23),
    [sym_project_atom] = STATE(23),
    [sym_program] = STATE(23),
    [sym_script] = STATE(23),
    [sym_const] = STATE(23),
    [sym_edge] = STATE(23),
    [sym_heuristic] = STATE(23),
    [sym_include] = STATE(23),
    [sym_external] = STATE(23),
    [sym_theory] = STATE(23),
    [sym_statement] = STATE(3),
    [aux_sym_source_file_repeat1] = STATE(3),
    [aux_sym_disjunction_repeat1] = STATE(72),
    [ts_builtin_sym_end] = ACTIONS(75),
    [sym_line_comment] = ACTIONS(3),
    [anon_sym_PERCENT_STAR_BANG] = ACTIONS(77),
    [sym_identifier] = ACTIONS(80),
    [anon_sym_DQUOTE] = ACTIONS(83),
    [anon_sym_f_DQUOTE] = ACTIONS(86),
    [anon_sym_LBRACE] = ACTIONS(89),
    [sym_supremum] = ACTIONS(92),
    [sym_infimum] = ACTIONS(92),
    [sym_number] = ACTIONS(92),
    [sym_anonymous] = ACTIONS(95),
    [sym_variable] = ACTIONS(92),
    [anon_sym_AMP] = ACTIONS(98),
    [anon_sym_DASH] = ACTIONS(101),
    [anon_sym_TILDE] = ACTIONS(104),
    [anon_sym_PIPE] = ACTIONS(107),
    [anon_sym_LPAREN] = ACTIONS(110),
    [anon_sym_AT] = ACTIONS(113),
    [sym_boolean_constant] = ACTIONS(116),
    [sym_default_negation] = ACTIONS(119),
    [sym_double_default_negation] = ACTIONS(122),
    [sym_aggregate_function] = ACTIONS(125),
    [anon_sym_COLON_DASH] = ACTIONS(128),
    [anon_sym_COLON_TILDE] = ACTIONS(131),
    [anon_sym_POUNDmaximize] = ACTIONS(134),
    [anon_sym_POUNDmaximise] = ACTIONS(134),
    [anon_sym_POUNDminimize] = ACTIONS(137),
    [anon_sym_POUNDminimise] = ACTIONS(137),
    [anon_sym_POUNDshow] = ACTIONS(140),
    [anon_sym_POUNDdefined] = ACTIONS(143),
    [anon_sym_POUNDproject] = ACTIONS(146),
    [anon_sym_POUNDprogram] = ACTIONS(149),
    [anon_sym_POUNDscript] = ACTIONS(152),
    [anon_sym_POUNDconst] = ACTIONS(155),
    [anon_sym_POUNDedge] = ACTIONS(158),
    [anon_sym_POUNDheuristic] = ACTIONS(161),
    [anon_sym_POUNDinclude] = ACTIONS(164),
    [anon_sym_POUNDexternal] = ACTIONS(167),
    [anon_sym_POUNDtheory] = ACTIONS(170),
    [sym_block_comment] = ACTIONS(5),
  },
};

static const uint16_t ts_small_parse_table[] = {
  [0] = 32,
    ACTIONS(11), 1,
      sym_identifier,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(17), 1,
      anon_sym_LBRACE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(23), 1,
      anon_sym_AMP,
    ACTIONS(25), 1,
      anon_sym_DASH,
    ACTIONS(27), 1,
      anon_sym_TILDE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(35), 1,
      sym_boolean_constant,
    ACTIONS(37), 1,
      sym_default_negation,
    ACTIONS(39), 1,
      sym_double_default_negation,
    ACTIONS(173), 1,
      sym_aggregate_function,
    ACTIONS(175), 1,
      anon_sym_DOT,
    STATE(14), 1,
      aux_sym_body_repeat1,
    STATE(77), 1,
      sym_literal_sign,
    STATE(355), 1,
      sym_term,
    STATE(490), 1,
      sym_simple_atom,
    STATE(599), 1,
      sym_body_literal,
    STATE(666), 1,
      sym_lower,
    STATE(679), 1,
      sym_conditional_literal,
    STATE(710), 1,
      sym__conditional_literal_0,
    STATE(712), 1,
      sym__conditional_literal_n,
    STATE(725), 1,
      sym_literal,
    STATE(811), 1,
      sym_body,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    STATE(426), 2,
      sym_symbolic_atom,
      sym_comparison,
    STATE(593), 3,
      sym_set_aggregate,
      sym_theory_atom,
      sym_body_aggregate,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [111] = 32,
    ACTIONS(11), 1,
      sym_identifier,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(17), 1,
      anon_sym_LBRACE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(23), 1,
      anon_sym_AMP,
    ACTIONS(25), 1,
      anon_sym_DASH,
    ACTIONS(27), 1,
      anon_sym_TILDE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(35), 1,
      sym_boolean_constant,
    ACTIONS(37), 1,
      sym_default_negation,
    ACTIONS(39), 1,
      sym_double_default_negation,
    ACTIONS(173), 1,
      sym_aggregate_function,
    ACTIONS(177), 1,
      anon_sym_DOT,
    STATE(14), 1,
      aux_sym_body_repeat1,
    STATE(77), 1,
      sym_literal_sign,
    STATE(355), 1,
      sym_term,
    STATE(490), 1,
      sym_simple_atom,
    STATE(599), 1,
      sym_body_literal,
    STATE(666), 1,
      sym_lower,
    STATE(679), 1,
      sym_conditional_literal,
    STATE(710), 1,
      sym__conditional_literal_0,
    STATE(712), 1,
      sym__conditional_literal_n,
    STATE(725), 1,
      sym_literal,
    STATE(808), 1,
      sym_body,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    STATE(426), 2,
      sym_symbolic_atom,
      sym_comparison,
    STATE(593), 3,
      sym_set_aggregate,
      sym_theory_atom,
      sym_body_aggregate,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [222] = 32,
    ACTIONS(11), 1,
      sym_identifier,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(17), 1,
      anon_sym_LBRACE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(23), 1,
      anon_sym_AMP,
    ACTIONS(25), 1,
      anon_sym_DASH,
    ACTIONS(27), 1,
      anon_sym_TILDE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(35), 1,
      sym_boolean_constant,
    ACTIONS(37), 1,
      sym_default_negation,
    ACTIONS(39), 1,
      sym_double_default_negation,
    ACTIONS(173), 1,
      sym_aggregate_function,
    ACTIONS(179), 1,
      anon_sym_DOT,
    STATE(14), 1,
      aux_sym_body_repeat1,
    STATE(77), 1,
      sym_literal_sign,
    STATE(355), 1,
      sym_term,
    STATE(490), 1,
      sym_simple_atom,
    STATE(599), 1,
      sym_body_literal,
    STATE(666), 1,
      sym_lower,
    STATE(679), 1,
      sym_conditional_literal,
    STATE(710), 1,
      sym__conditional_literal_0,
    STATE(712), 1,
      sym__conditional_literal_n,
    STATE(725), 1,
      sym_literal,
    STATE(835), 1,
      sym_body,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    STATE(426), 2,
      sym_symbolic_atom,
      sym_comparison,
    STATE(593), 3,
      sym_set_aggregate,
      sym_theory_atom,
      sym_body_aggregate,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [333] = 32,
    ACTIONS(11), 1,
      sym_identifier,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(17), 1,
      anon_sym_LBRACE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(23), 1,
      anon_sym_AMP,
    ACTIONS(25), 1,
      anon_sym_DASH,
    ACTIONS(27), 1,
      anon_sym_TILDE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(35), 1,
      sym_boolean_constant,
    ACTIONS(37), 1,
      sym_default_negation,
    ACTIONS(39), 1,
      sym_double_default_negation,
    ACTIONS(173), 1,
      sym_aggregate_function,
    ACTIONS(181), 1,
      anon_sym_DOT,
    STATE(14), 1,
      aux_sym_body_repeat1,
    STATE(77), 1,
      sym_literal_sign,
    STATE(355), 1,
      sym_term,
    STATE(490), 1,
      sym_simple_atom,
    STATE(599), 1,
      sym_body_literal,
    STATE(666), 1,
      sym_lower,
    STATE(679), 1,
      sym_conditional_literal,
    STATE(710), 1,
      sym__conditional_literal_0,
    STATE(712), 1,
      sym__conditional_literal_n,
    STATE(725), 1,
      sym_literal,
    STATE(771), 1,
      sym_body,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    STATE(426), 2,
      sym_symbolic_atom,
      sym_comparison,
    STATE(593), 3,
      sym_set_aggregate,
      sym_theory_atom,
      sym_body_aggregate,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [444] = 32,
    ACTIONS(11), 1,
      sym_identifier,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(17), 1,
      anon_sym_LBRACE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(23), 1,
      anon_sym_AMP,
    ACTIONS(25), 1,
      anon_sym_DASH,
    ACTIONS(27), 1,
      anon_sym_TILDE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(35), 1,
      sym_boolean_constant,
    ACTIONS(37), 1,
      sym_default_negation,
    ACTIONS(39), 1,
      sym_double_default_negation,
    ACTIONS(173), 1,
      sym_aggregate_function,
    ACTIONS(183), 1,
      anon_sym_DOT,
    STATE(14), 1,
      aux_sym_body_repeat1,
    STATE(77), 1,
      sym_literal_sign,
    STATE(355), 1,
      sym_term,
    STATE(490), 1,
      sym_simple_atom,
    STATE(599), 1,
      sym_body_literal,
    STATE(666), 1,
      sym_lower,
    STATE(679), 1,
      sym_conditional_literal,
    STATE(710), 1,
      sym__conditional_literal_0,
    STATE(712), 1,
      sym__conditional_literal_n,
    STATE(725), 1,
      sym_literal,
    STATE(841), 1,
      sym_body,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    STATE(426), 2,
      sym_symbolic_atom,
      sym_comparison,
    STATE(593), 3,
      sym_set_aggregate,
      sym_theory_atom,
      sym_body_aggregate,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [555] = 32,
    ACTIONS(11), 1,
      sym_identifier,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(17), 1,
      anon_sym_LBRACE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(23), 1,
      anon_sym_AMP,
    ACTIONS(25), 1,
      anon_sym_DASH,
    ACTIONS(27), 1,
      anon_sym_TILDE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(35), 1,
      sym_boolean_constant,
    ACTIONS(37), 1,
      sym_default_negation,
    ACTIONS(39), 1,
      sym_double_default_negation,
    ACTIONS(173), 1,
      sym_aggregate_function,
    ACTIONS(185), 1,
      anon_sym_DOT,
    STATE(14), 1,
      aux_sym_body_repeat1,
    STATE(77), 1,
      sym_literal_sign,
    STATE(355), 1,
      sym_term,
    STATE(490), 1,
      sym_simple_atom,
    STATE(599), 1,
      sym_body_literal,
    STATE(666), 1,
      sym_lower,
    STATE(679), 1,
      sym_conditional_literal,
    STATE(710), 1,
      sym__conditional_literal_0,
    STATE(712), 1,
      sym__conditional_literal_n,
    STATE(714), 1,
      sym_body,
    STATE(725), 1,
      sym_literal,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    STATE(426), 2,
      sym_symbolic_atom,
      sym_comparison,
    STATE(593), 3,
      sym_set_aggregate,
      sym_theory_atom,
      sym_body_aggregate,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [666] = 32,
    ACTIONS(11), 1,
      sym_identifier,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(17), 1,
      anon_sym_LBRACE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(23), 1,
      anon_sym_AMP,
    ACTIONS(25), 1,
      anon_sym_DASH,
    ACTIONS(27), 1,
      anon_sym_TILDE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(35), 1,
      sym_boolean_constant,
    ACTIONS(37), 1,
      sym_default_negation,
    ACTIONS(39), 1,
      sym_double_default_negation,
    ACTIONS(173), 1,
      sym_aggregate_function,
    ACTIONS(187), 1,
      anon_sym_DOT,
    STATE(14), 1,
      aux_sym_body_repeat1,
    STATE(77), 1,
      sym_literal_sign,
    STATE(355), 1,
      sym_term,
    STATE(490), 1,
      sym_simple_atom,
    STATE(599), 1,
      sym_body_literal,
    STATE(666), 1,
      sym_lower,
    STATE(679), 1,
      sym_conditional_literal,
    STATE(710), 1,
      sym__conditional_literal_0,
    STATE(712), 1,
      sym__conditional_literal_n,
    STATE(725), 1,
      sym_literal,
    STATE(728), 1,
      sym_body,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    STATE(426), 2,
      sym_symbolic_atom,
      sym_comparison,
    STATE(593), 3,
      sym_set_aggregate,
      sym_theory_atom,
      sym_body_aggregate,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [777] = 32,
    ACTIONS(11), 1,
      sym_identifier,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(17), 1,
      anon_sym_LBRACE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(23), 1,
      anon_sym_AMP,
    ACTIONS(25), 1,
      anon_sym_DASH,
    ACTIONS(27), 1,
      anon_sym_TILDE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(35), 1,
      sym_boolean_constant,
    ACTIONS(37), 1,
      sym_default_negation,
    ACTIONS(39), 1,
      sym_double_default_negation,
    ACTIONS(173), 1,
      sym_aggregate_function,
    ACTIONS(189), 1,
      anon_sym_DOT,
    STATE(14), 1,
      aux_sym_body_repeat1,
    STATE(77), 1,
      sym_literal_sign,
    STATE(355), 1,
      sym_term,
    STATE(490), 1,
      sym_simple_atom,
    STATE(599), 1,
      sym_body_literal,
    STATE(666), 1,
      sym_lower,
    STATE(679), 1,
      sym_conditional_literal,
    STATE(710), 1,
      sym__conditional_literal_0,
    STATE(712), 1,
      sym__conditional_literal_n,
    STATE(725), 1,
      sym_literal,
    STATE(749), 1,
      sym_body,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    STATE(426), 2,
      sym_symbolic_atom,
      sym_comparison,
    STATE(593), 3,
      sym_set_aggregate,
      sym_theory_atom,
      sym_body_aggregate,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [888] = 31,
    ACTIONS(11), 1,
      sym_identifier,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(17), 1,
      anon_sym_LBRACE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(23), 1,
      anon_sym_AMP,
    ACTIONS(25), 1,
      anon_sym_DASH,
    ACTIONS(27), 1,
      anon_sym_TILDE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(35), 1,
      sym_boolean_constant,
    ACTIONS(37), 1,
      sym_default_negation,
    ACTIONS(39), 1,
      sym_double_default_negation,
    ACTIONS(173), 1,
      sym_aggregate_function,
    STATE(14), 1,
      aux_sym_body_repeat1,
    STATE(77), 1,
      sym_literal_sign,
    STATE(355), 1,
      sym_term,
    STATE(490), 1,
      sym_simple_atom,
    STATE(599), 1,
      sym_body_literal,
    STATE(666), 1,
      sym_lower,
    STATE(679), 1,
      sym_conditional_literal,
    STATE(710), 1,
      sym__conditional_literal_0,
    STATE(712), 1,
      sym__conditional_literal_n,
    STATE(725), 1,
      sym_literal,
    STATE(789), 1,
      sym_body,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    STATE(426), 2,
      sym_symbolic_atom,
      sym_comparison,
    STATE(593), 3,
      sym_set_aggregate,
      sym_theory_atom,
      sym_body_aggregate,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [996] = 30,
    ACTIONS(191), 1,
      sym_identifier,
    ACTIONS(194), 1,
      anon_sym_DQUOTE,
    ACTIONS(197), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(200), 1,
      anon_sym_LBRACE,
    ACTIONS(206), 1,
      sym_anonymous,
    ACTIONS(209), 1,
      anon_sym_AMP,
    ACTIONS(212), 1,
      anon_sym_DASH,
    ACTIONS(215), 1,
      anon_sym_TILDE,
    ACTIONS(218), 1,
      anon_sym_PIPE,
    ACTIONS(221), 1,
      anon_sym_LPAREN,
    ACTIONS(224), 1,
      anon_sym_AT,
    ACTIONS(227), 1,
      sym_boolean_constant,
    ACTIONS(230), 1,
      sym_default_negation,
    ACTIONS(233), 1,
      sym_double_default_negation,
    ACTIONS(236), 1,
      sym_aggregate_function,
    STATE(13), 1,
      aux_sym_body_repeat1,
    STATE(88), 1,
      sym_literal_sign,
    STATE(355), 1,
      sym_term,
    STATE(490), 1,
      sym_simple_atom,
    STATE(696), 1,
      sym_body_literal,
    STATE(710), 1,
      sym__conditional_literal_0,
    STATE(711), 1,
      sym_lower,
    STATE(712), 1,
      sym__conditional_literal_n,
    STATE(810), 1,
      sym_conditional_literal,
    STATE(877), 1,
      sym_literal,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    STATE(426), 2,
      sym_symbolic_atom,
      sym_comparison,
    STATE(593), 3,
      sym_set_aggregate,
      sym_theory_atom,
      sym_body_aggregate,
    ACTIONS(203), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [1101] = 30,
    ACTIONS(11), 1,
      sym_identifier,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(17), 1,
      anon_sym_LBRACE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(23), 1,
      anon_sym_AMP,
    ACTIONS(25), 1,
      anon_sym_DASH,
    ACTIONS(27), 1,
      anon_sym_TILDE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(35), 1,
      sym_boolean_constant,
    ACTIONS(37), 1,
      sym_default_negation,
    ACTIONS(39), 1,
      sym_double_default_negation,
    ACTIONS(173), 1,
      sym_aggregate_function,
    STATE(13), 1,
      aux_sym_body_repeat1,
    STATE(77), 1,
      sym_literal_sign,
    STATE(355), 1,
      sym_term,
    STATE(490), 1,
      sym_simple_atom,
    STATE(588), 1,
      sym_body_literal,
    STATE(666), 1,
      sym_lower,
    STATE(674), 1,
      sym_conditional_literal,
    STATE(710), 1,
      sym__conditional_literal_0,
    STATE(712), 1,
      sym__conditional_literal_n,
    STATE(725), 1,
      sym_literal,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    STATE(426), 2,
      sym_symbolic_atom,
      sym_comparison,
    STATE(593), 3,
      sym_set_aggregate,
      sym_theory_atom,
      sym_body_aggregate,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [1206] = 5,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(243), 1,
      anon_sym_LBRACK,
    ACTIONS(241), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(239), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [1258] = 5,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(249), 1,
      anon_sym_LBRACK,
    ACTIONS(247), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(245), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [1310] = 5,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(255), 1,
      anon_sym_LBRACK,
    ACTIONS(253), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(251), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [1362] = 5,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(261), 1,
      anon_sym_LBRACK,
    ACTIONS(259), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(257), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [1414] = 5,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(267), 1,
      anon_sym_LBRACK,
    ACTIONS(265), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(263), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [1466] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(271), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(269), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [1515] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(275), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(273), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [1564] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(279), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(277), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [1613] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(283), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(281), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [1662] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(287), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(285), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [1711] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(291), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(289), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [1760] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(295), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(293), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [1809] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(299), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(297), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [1858] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(303), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(301), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [1907] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(307), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(305), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [1956] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(311), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(309), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [2005] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(315), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(313), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [2054] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(319), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(317), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [2103] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(323), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(321), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [2152] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(327), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(325), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [2201] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(331), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(329), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [2250] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(335), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(333), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [2299] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(339), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(337), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [2348] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(343), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(341), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [2397] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(347), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(345), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [2446] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(351), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(349), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [2495] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(355), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(353), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [2544] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(359), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(357), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [2593] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(363), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(361), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [2642] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(367), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(365), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [2691] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(371), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(369), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [2740] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(375), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(373), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [2789] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(379), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(377), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [2838] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(383), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(381), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [2887] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(387), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(385), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [2936] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(391), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(389), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [2985] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(395), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(393), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [3034] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(399), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(397), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [3083] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(403), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(401), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [3132] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(407), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(405), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [3181] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(411), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(409), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [3230] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(415), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(413), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [3279] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(419), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(417), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [3328] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(423), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(421), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [3377] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(427), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(425), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [3426] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(431), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(429), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [3475] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(435), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(433), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [3524] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(439), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(437), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [3573] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(443), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(441), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [3622] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(447), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(445), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [3671] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(451), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(449), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [3720] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(455), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(453), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [3769] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(459), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(457), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [3818] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(463), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(461), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [3867] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(467), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(465), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [3916] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(471), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(469), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [3965] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(475), 4,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
      sym_aggregate_function,
    ACTIONS(473), 34,
      ts_builtin_sym_end,
      anon_sym_PERCENT_STAR_BANG,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      anon_sym_COLON_DASH,
      anon_sym_COLON_TILDE,
      anon_sym_POUNDmaximize,
      anon_sym_POUNDmaximise,
      anon_sym_POUNDminimize,
      anon_sym_POUNDminimise,
      anon_sym_POUNDshow,
      anon_sym_POUNDdefined,
      anon_sym_POUNDproject,
      anon_sym_POUNDprogram,
      anon_sym_POUNDscript,
      anon_sym_POUNDconst,
      anon_sym_POUNDedge,
      anon_sym_POUNDheuristic,
      anon_sym_POUNDinclude,
      anon_sym_POUNDexternal,
      anon_sym_POUNDtheory,
  [4014] = 24,
    ACTIONS(11), 1,
      sym_identifier,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(25), 1,
      anon_sym_DASH,
    ACTIONS(27), 1,
      anon_sym_TILDE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(35), 1,
      sym_boolean_constant,
    ACTIONS(37), 1,
      sym_default_negation,
    ACTIONS(39), 1,
      sym_double_default_negation,
    STATE(83), 1,
      aux_sym_disjunction_repeat1,
    STATE(123), 1,
      sym_literal_sign,
    STATE(382), 1,
      sym_term,
    STATE(435), 1,
      sym_simple_atom,
    STATE(440), 1,
      sym_literal,
    STATE(520), 1,
      sym__conditional_literal_n,
    STATE(597), 1,
      sym__conditional_literal_0,
    STATE(702), 1,
      sym_conditional_literal,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    STATE(426), 2,
      sym_symbolic_atom,
      sym_comparison,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [4099] = 22,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(27), 1,
      anon_sym_TILDE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(37), 1,
      sym_default_negation,
    ACTIONS(39), 1,
      sym_double_default_negation,
    ACTIONS(477), 1,
      sym_identifier,
    ACTIONS(479), 1,
      anon_sym_DASH,
    ACTIONS(483), 1,
      sym_boolean_constant,
    STATE(132), 1,
      sym_literal_sign,
    STATE(388), 1,
      sym_term,
    STATE(437), 1,
      sym_literal,
    STATE(449), 1,
      sym_simple_atom,
    STATE(493), 1,
      sym_condition,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    STATE(447), 2,
      sym_symbolic_atom,
      sym_comparison,
    ACTIONS(481), 3,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [4180] = 22,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(27), 1,
      anon_sym_TILDE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(37), 1,
      sym_default_negation,
    ACTIONS(39), 1,
      sym_double_default_negation,
    ACTIONS(477), 1,
      sym_identifier,
    ACTIONS(479), 1,
      anon_sym_DASH,
    ACTIONS(483), 1,
      sym_boolean_constant,
    STATE(132), 1,
      sym_literal_sign,
    STATE(388), 1,
      sym_term,
    STATE(437), 1,
      sym_literal,
    STATE(449), 1,
      sym_simple_atom,
    STATE(665), 1,
      sym_condition,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(485), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
    STATE(447), 2,
      sym_symbolic_atom,
      sym_comparison,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [4260] = 23,
    ACTIONS(11), 1,
      sym_identifier,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(25), 1,
      anon_sym_DASH,
    ACTIONS(27), 1,
      anon_sym_TILDE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(35), 1,
      sym_boolean_constant,
    ACTIONS(37), 1,
      sym_default_negation,
    ACTIONS(39), 1,
      sym_double_default_negation,
    ACTIONS(487), 1,
      anon_sym_RBRACE2,
    STATE(123), 1,
      sym_literal_sign,
    STATE(382), 1,
      sym_term,
    STATE(435), 1,
      sym_simple_atom,
    STATE(594), 1,
      sym_literal,
    STATE(627), 1,
      sym_set_aggregate_element,
    STATE(836), 1,
      sym_set_aggregate_elements,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    STATE(426), 2,
      sym_symbolic_atom,
      sym_comparison,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [4342] = 22,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(27), 1,
      anon_sym_TILDE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(37), 1,
      sym_default_negation,
    ACTIONS(39), 1,
      sym_double_default_negation,
    ACTIONS(477), 1,
      sym_identifier,
    ACTIONS(479), 1,
      anon_sym_DASH,
    ACTIONS(483), 1,
      sym_boolean_constant,
    STATE(132), 1,
      sym_literal_sign,
    STATE(388), 1,
      sym_term,
    STATE(437), 1,
      sym_literal,
    STATE(449), 1,
      sym_simple_atom,
    STATE(667), 1,
      sym_condition,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(489), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
    STATE(447), 2,
      sym_symbolic_atom,
      sym_comparison,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [4422] = 21,
    ACTIONS(11), 1,
      sym_identifier,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(17), 1,
      anon_sym_LBRACE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(23), 1,
      anon_sym_AMP,
    ACTIONS(25), 1,
      anon_sym_DASH,
    ACTIONS(27), 1,
      anon_sym_TILDE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(35), 1,
      sym_boolean_constant,
    ACTIONS(173), 1,
      sym_aggregate_function,
    STATE(355), 1,
      sym_term,
    STATE(496), 1,
      sym_simple_atom,
    STATE(666), 1,
      sym_lower,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    STATE(426), 2,
      sym_symbolic_atom,
      sym_comparison,
    STATE(584), 3,
      sym_set_aggregate,
      sym_theory_atom,
      sym_body_aggregate,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [4500] = 23,
    ACTIONS(11), 1,
      sym_identifier,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(25), 1,
      anon_sym_DASH,
    ACTIONS(27), 1,
      anon_sym_TILDE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(35), 1,
      sym_boolean_constant,
    ACTIONS(37), 1,
      sym_default_negation,
    ACTIONS(39), 1,
      sym_double_default_negation,
    ACTIONS(491), 1,
      anon_sym_RBRACE2,
    STATE(123), 1,
      sym_literal_sign,
    STATE(382), 1,
      sym_term,
    STATE(435), 1,
      sym_simple_atom,
    STATE(594), 1,
      sym_literal,
    STATE(627), 1,
      sym_set_aggregate_element,
    STATE(783), 1,
      sym_set_aggregate_elements,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    STATE(426), 2,
      sym_symbolic_atom,
      sym_comparison,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [4582] = 22,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(27), 1,
      anon_sym_TILDE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(37), 1,
      sym_default_negation,
    ACTIONS(39), 1,
      sym_double_default_negation,
    ACTIONS(477), 1,
      sym_identifier,
    ACTIONS(479), 1,
      anon_sym_DASH,
    ACTIONS(483), 1,
      sym_boolean_constant,
    STATE(132), 1,
      sym_literal_sign,
    STATE(388), 1,
      sym_term,
    STATE(437), 1,
      sym_literal,
    STATE(449), 1,
      sym_simple_atom,
    STATE(662), 1,
      sym_condition,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(493), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
    STATE(447), 2,
      sym_symbolic_atom,
      sym_comparison,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [4662] = 22,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(27), 1,
      anon_sym_TILDE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(37), 1,
      sym_default_negation,
    ACTIONS(39), 1,
      sym_double_default_negation,
    ACTIONS(477), 1,
      sym_identifier,
    ACTIONS(479), 1,
      anon_sym_DASH,
    ACTIONS(483), 1,
      sym_boolean_constant,
    STATE(132), 1,
      sym_literal_sign,
    STATE(388), 1,
      sym_term,
    STATE(437), 1,
      sym_literal,
    STATE(449), 1,
      sym_simple_atom,
    STATE(637), 1,
      sym_condition,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(495), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
    STATE(447), 2,
      sym_symbolic_atom,
      sym_comparison,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [4742] = 22,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(27), 1,
      anon_sym_TILDE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(37), 1,
      sym_default_negation,
    ACTIONS(39), 1,
      sym_double_default_negation,
    ACTIONS(477), 1,
      sym_identifier,
    ACTIONS(479), 1,
      anon_sym_DASH,
    ACTIONS(483), 1,
      sym_boolean_constant,
    STATE(132), 1,
      sym_literal_sign,
    STATE(388), 1,
      sym_term,
    STATE(437), 1,
      sym_literal,
    STATE(449), 1,
      sym_simple_atom,
    STATE(659), 1,
      sym_condition,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(497), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
    STATE(447), 2,
      sym_symbolic_atom,
      sym_comparison,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [4822] = 22,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(27), 1,
      anon_sym_TILDE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(37), 1,
      sym_default_negation,
    ACTIONS(39), 1,
      sym_double_default_negation,
    ACTIONS(477), 1,
      sym_identifier,
    ACTIONS(479), 1,
      anon_sym_DASH,
    ACTIONS(483), 1,
      sym_boolean_constant,
    STATE(132), 1,
      sym_literal_sign,
    STATE(388), 1,
      sym_term,
    STATE(437), 1,
      sym_literal,
    STATE(449), 1,
      sym_simple_atom,
    STATE(708), 1,
      sym_condition,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(499), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
    STATE(447), 2,
      sym_symbolic_atom,
      sym_comparison,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [4902] = 23,
    ACTIONS(501), 1,
      sym_identifier,
    ACTIONS(504), 1,
      anon_sym_DQUOTE,
    ACTIONS(507), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(513), 1,
      sym_anonymous,
    ACTIONS(516), 1,
      anon_sym_DASH,
    ACTIONS(519), 1,
      anon_sym_TILDE,
    ACTIONS(522), 1,
      anon_sym_PIPE,
    ACTIONS(525), 1,
      anon_sym_LPAREN,
    ACTIONS(528), 1,
      anon_sym_AT,
    ACTIONS(531), 1,
      sym_boolean_constant,
    ACTIONS(534), 1,
      sym_default_negation,
    ACTIONS(537), 1,
      sym_double_default_negation,
    STATE(83), 1,
      aux_sym_disjunction_repeat1,
    STATE(123), 1,
      sym_literal_sign,
    STATE(382), 1,
      sym_term,
    STATE(435), 1,
      sym_simple_atom,
    STATE(494), 1,
      sym_literal,
    STATE(657), 1,
      sym__conditional_literal_n,
    STATE(795), 1,
      sym__conditional_literal_0,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    STATE(426), 2,
      sym_symbolic_atom,
      sym_comparison,
    ACTIONS(510), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [4984] = 22,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(27), 1,
      anon_sym_TILDE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(37), 1,
      sym_default_negation,
    ACTIONS(39), 1,
      sym_double_default_negation,
    ACTIONS(477), 1,
      sym_identifier,
    ACTIONS(479), 1,
      anon_sym_DASH,
    ACTIONS(483), 1,
      sym_boolean_constant,
    STATE(132), 1,
      sym_literal_sign,
    STATE(388), 1,
      sym_term,
    STATE(437), 1,
      sym_literal,
    STATE(449), 1,
      sym_simple_atom,
    STATE(681), 1,
      sym_condition,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(540), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
    STATE(447), 2,
      sym_symbolic_atom,
      sym_comparison,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [5064] = 22,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(27), 1,
      anon_sym_TILDE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(37), 1,
      sym_default_negation,
    ACTIONS(39), 1,
      sym_double_default_negation,
    ACTIONS(477), 1,
      sym_identifier,
    ACTIONS(479), 1,
      anon_sym_DASH,
    ACTIONS(483), 1,
      sym_boolean_constant,
    STATE(132), 1,
      sym_literal_sign,
    STATE(388), 1,
      sym_term,
    STATE(437), 1,
      sym_literal,
    STATE(449), 1,
      sym_simple_atom,
    STATE(690), 1,
      sym_condition,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(542), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
    STATE(447), 2,
      sym_symbolic_atom,
      sym_comparison,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [5144] = 22,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(27), 1,
      anon_sym_TILDE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(37), 1,
      sym_default_negation,
    ACTIONS(39), 1,
      sym_double_default_negation,
    ACTIONS(477), 1,
      sym_identifier,
    ACTIONS(479), 1,
      anon_sym_DASH,
    ACTIONS(483), 1,
      sym_boolean_constant,
    STATE(132), 1,
      sym_literal_sign,
    STATE(388), 1,
      sym_term,
    STATE(437), 1,
      sym_literal,
    STATE(449), 1,
      sym_simple_atom,
    STATE(691), 1,
      sym_condition,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(544), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
    STATE(447), 2,
      sym_symbolic_atom,
      sym_comparison,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [5224] = 23,
    ACTIONS(11), 1,
      sym_identifier,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(25), 1,
      anon_sym_DASH,
    ACTIONS(27), 1,
      anon_sym_TILDE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(35), 1,
      sym_boolean_constant,
    ACTIONS(37), 1,
      sym_default_negation,
    ACTIONS(39), 1,
      sym_double_default_negation,
    ACTIONS(546), 1,
      anon_sym_RBRACE2,
    STATE(123), 1,
      sym_literal_sign,
    STATE(382), 1,
      sym_term,
    STATE(435), 1,
      sym_simple_atom,
    STATE(594), 1,
      sym_literal,
    STATE(627), 1,
      sym_set_aggregate_element,
    STATE(814), 1,
      sym_set_aggregate_elements,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    STATE(426), 2,
      sym_symbolic_atom,
      sym_comparison,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [5306] = 21,
    ACTIONS(11), 1,
      sym_identifier,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(23), 1,
      anon_sym_AMP,
    ACTIONS(25), 1,
      anon_sym_DASH,
    ACTIONS(27), 1,
      anon_sym_TILDE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(35), 1,
      sym_boolean_constant,
    ACTIONS(548), 1,
      anon_sym_LBRACE,
    ACTIONS(550), 1,
      sym_aggregate_function,
    STATE(355), 1,
      sym_term,
    STATE(496), 1,
      sym_simple_atom,
    STATE(711), 1,
      sym_lower,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    STATE(426), 2,
      sym_symbolic_atom,
      sym_comparison,
    STATE(584), 3,
      sym_set_aggregate,
      sym_theory_atom,
      sym_body_aggregate,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [5384] = 23,
    ACTIONS(11), 1,
      sym_identifier,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(25), 1,
      anon_sym_DASH,
    ACTIONS(27), 1,
      anon_sym_TILDE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(35), 1,
      sym_boolean_constant,
    ACTIONS(37), 1,
      sym_default_negation,
    ACTIONS(39), 1,
      sym_double_default_negation,
    ACTIONS(552), 1,
      anon_sym_RBRACE2,
    STATE(123), 1,
      sym_literal_sign,
    STATE(382), 1,
      sym_term,
    STATE(435), 1,
      sym_simple_atom,
    STATE(594), 1,
      sym_literal,
    STATE(627), 1,
      sym_set_aggregate_element,
    STATE(855), 1,
      sym_set_aggregate_elements,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    STATE(426), 2,
      sym_symbolic_atom,
      sym_comparison,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [5466] = 20,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    ACTIONS(556), 1,
      anon_sym_STAR,
    ACTIONS(558), 1,
      anon_sym_COMMA2,
    ACTIONS(560), 1,
      anon_sym_RPAREN2,
    ACTIONS(562), 1,
      sym_empty_pool_item_first,
    STATE(344), 1,
      sym_term,
    STATE(551), 1,
      sym_tuple_pool_item,
    STATE(760), 1,
      sym__tuple_pool_n,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    STATE(650), 2,
      sym_projection,
      sym__term_or_projection,
    STATE(653), 2,
      sym__term_comma,
      sym__terms_trail,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [5541] = 20,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    ACTIONS(556), 1,
      anon_sym_STAR,
    ACTIONS(558), 1,
      anon_sym_COMMA2,
    ACTIONS(562), 1,
      sym_empty_pool_item_first,
    ACTIONS(564), 1,
      anon_sym_RPAREN2,
    STATE(344), 1,
      sym_term,
    STATE(617), 1,
      sym_tuple_pool_item,
    STATE(817), 1,
      sym__tuple_pool_n,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    STATE(650), 2,
      sym_projection,
      sym__term_or_projection,
    STATE(653), 2,
      sym__term_comma,
      sym__terms_trail,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [5616] = 20,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    ACTIONS(556), 1,
      anon_sym_STAR,
    ACTIONS(558), 1,
      anon_sym_COMMA2,
    ACTIONS(562), 1,
      sym_empty_pool_item_first,
    ACTIONS(566), 1,
      anon_sym_RPAREN2,
    STATE(344), 1,
      sym_term,
    STATE(630), 1,
      sym_tuple_pool_item,
    STATE(862), 1,
      sym__tuple_pool_n,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    STATE(650), 2,
      sym_projection,
      sym__term_or_projection,
    STATE(653), 2,
      sym__term_comma,
      sym__terms_trail,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [5691] = 22,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(27), 1,
      anon_sym_TILDE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(37), 1,
      sym_default_negation,
    ACTIONS(39), 1,
      sym_double_default_negation,
    ACTIONS(481), 1,
      anon_sym_SEMI,
    ACTIONS(483), 1,
      sym_boolean_constant,
    ACTIONS(568), 1,
      sym_identifier,
    ACTIONS(570), 1,
      anon_sym_DASH,
    STATE(124), 1,
      sym_literal_sign,
    STATE(384), 1,
      sym_term,
    STATE(449), 1,
      sym_simple_atom,
    STATE(493), 1,
      sym_condition,
    STATE(533), 1,
      sym_literal,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    STATE(447), 2,
      sym_symbolic_atom,
      sym_comparison,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [5770] = 21,
    ACTIONS(11), 1,
      sym_identifier,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(25), 1,
      anon_sym_DASH,
    ACTIONS(27), 1,
      anon_sym_TILDE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(35), 1,
      sym_boolean_constant,
    ACTIONS(37), 1,
      sym_default_negation,
    ACTIONS(39), 1,
      sym_double_default_negation,
    STATE(123), 1,
      sym_literal_sign,
    STATE(382), 1,
      sym_term,
    STATE(435), 1,
      sym_simple_atom,
    STATE(594), 1,
      sym_literal,
    STATE(668), 1,
      sym_set_aggregate_element,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    STATE(426), 2,
      sym_symbolic_atom,
      sym_comparison,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [5846] = 20,
    ACTIONS(11), 1,
      sym_identifier,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(25), 1,
      anon_sym_DASH,
    ACTIONS(27), 1,
      anon_sym_TILDE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(35), 1,
      sym_boolean_constant,
    ACTIONS(37), 1,
      sym_default_negation,
    ACTIONS(39), 1,
      sym_double_default_negation,
    STATE(123), 1,
      sym_literal_sign,
    STATE(382), 1,
      sym_term,
    STATE(435), 1,
      sym_simple_atom,
    STATE(625), 1,
      sym_literal,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    STATE(426), 2,
      sym_symbolic_atom,
      sym_comparison,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [5919] = 20,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(27), 1,
      anon_sym_TILDE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(37), 1,
      sym_default_negation,
    ACTIONS(39), 1,
      sym_double_default_negation,
    ACTIONS(477), 1,
      sym_identifier,
    ACTIONS(479), 1,
      anon_sym_DASH,
    ACTIONS(483), 1,
      sym_boolean_constant,
    STATE(132), 1,
      sym_literal_sign,
    STATE(388), 1,
      sym_term,
    STATE(443), 1,
      sym_literal,
    STATE(449), 1,
      sym_simple_atom,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    STATE(447), 2,
      sym_symbolic_atom,
      sym_comparison,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [5992] = 20,
    ACTIONS(11), 1,
      sym_identifier,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(25), 1,
      anon_sym_DASH,
    ACTIONS(27), 1,
      anon_sym_TILDE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(35), 1,
      sym_boolean_constant,
    ACTIONS(37), 1,
      sym_default_negation,
    ACTIONS(39), 1,
      sym_double_default_negation,
    STATE(123), 1,
      sym_literal_sign,
    STATE(382), 1,
      sym_term,
    STATE(435), 1,
      sym_simple_atom,
    STATE(609), 1,
      sym_literal,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    STATE(426), 2,
      sym_symbolic_atom,
      sym_comparison,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [6065] = 18,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    ACTIONS(556), 1,
      anon_sym_STAR,
    ACTIONS(558), 1,
      anon_sym_COMMA2,
    ACTIONS(572), 1,
      sym_empty_pool_item,
    STATE(344), 1,
      sym_term,
    STATE(687), 1,
      sym_tuple_pool_item,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    STATE(650), 2,
      sym_projection,
      sym__term_or_projection,
    STATE(653), 2,
      sym__term_comma,
      sym__terms_trail,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [6134] = 20,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(27), 1,
      anon_sym_TILDE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(37), 1,
      sym_default_negation,
    ACTIONS(39), 1,
      sym_double_default_negation,
    ACTIONS(483), 1,
      sym_boolean_constant,
    ACTIONS(568), 1,
      sym_identifier,
    ACTIONS(570), 1,
      anon_sym_DASH,
    STATE(124), 1,
      sym_literal_sign,
    STATE(384), 1,
      sym_term,
    STATE(443), 1,
      sym_literal,
    STATE(449), 1,
      sym_simple_atom,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    STATE(447), 2,
      sym_symbolic_atom,
      sym_comparison,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [6207] = 18,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    ACTIONS(556), 1,
      anon_sym_STAR,
    ACTIONS(574), 1,
      anon_sym_RPAREN2,
    ACTIONS(576), 1,
      sym_empty_pool_item_first,
    STATE(361), 1,
      sym_term,
    STATE(635), 1,
      sym__terms_or_projection,
    STATE(883), 1,
      sym__arg_pool_n,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    STATE(538), 2,
      sym_projection,
      sym__term_or_projection,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [6275] = 18,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    ACTIONS(556), 1,
      anon_sym_STAR,
    ACTIONS(576), 1,
      sym_empty_pool_item_first,
    ACTIONS(578), 1,
      anon_sym_RPAREN2,
    STATE(361), 1,
      sym_term,
    STATE(633), 1,
      sym__terms_or_projection,
    STATE(870), 1,
      sym__arg_pool_n,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    STATE(538), 2,
      sym_projection,
      sym__term_or_projection,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [6343] = 18,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    ACTIONS(556), 1,
      anon_sym_STAR,
    ACTIONS(576), 1,
      sym_empty_pool_item_first,
    ACTIONS(580), 1,
      anon_sym_RPAREN2,
    STATE(361), 1,
      sym_term,
    STATE(621), 1,
      sym__terms_or_projection,
    STATE(827), 1,
      sym__arg_pool_n,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    STATE(538), 2,
      sym_projection,
      sym__term_or_projection,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [6411] = 18,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    ACTIONS(556), 1,
      anon_sym_STAR,
    ACTIONS(576), 1,
      sym_empty_pool_item_first,
    ACTIONS(582), 1,
      anon_sym_RPAREN2,
    STATE(361), 1,
      sym_term,
    STATE(631), 1,
      sym__terms_or_projection,
    STATE(864), 1,
      sym__arg_pool_n,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    STATE(538), 2,
      sym_projection,
      sym__term_or_projection,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [6479] = 15,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(584), 1,
      sym_identifier,
    ACTIONS(586), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(590), 1,
      sym_anonymous,
    ACTIONS(594), 1,
      anon_sym_PIPE,
    ACTIONS(598), 1,
      anon_sym_LPAREN,
    ACTIONS(600), 1,
      anon_sym_AT,
    ACTIONS(602), 1,
      sym_relation,
    STATE(334), 1,
      sym_term,
    STATE(506), 1,
      sym_upper,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(592), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(588), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    ACTIONS(596), 4,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
    STATE(299), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [6540] = 15,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(584), 1,
      sym_identifier,
    ACTIONS(586), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(590), 1,
      sym_anonymous,
    ACTIONS(594), 1,
      anon_sym_PIPE,
    ACTIONS(598), 1,
      anon_sym_LPAREN,
    ACTIONS(600), 1,
      anon_sym_AT,
    ACTIONS(602), 1,
      sym_relation,
    STATE(334), 1,
      sym_term,
    STATE(530), 1,
      sym_upper,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(592), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(588), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    ACTIONS(604), 4,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
    STATE(299), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [6601] = 22,
    ACTIONS(606), 1,
      sym_identifier,
    ACTIONS(608), 1,
      anon_sym_DQUOTE,
    ACTIONS(610), 1,
      anon_sym_LBRACE,
    ACTIONS(614), 1,
      sym_anonymous,
    ACTIONS(616), 1,
      anon_sym_LPAREN,
    ACTIONS(618), 1,
      sym_theory_operator,
    ACTIONS(620), 1,
      anon_sym_LBRACK,
    ACTIONS(622), 1,
      anon_sym_RBRACE2,
    ACTIONS(624), 1,
      sym_default_negation,
    ACTIONS(626), 1,
      sym_colon,
    STATE(307), 1,
      sym_theory_operators,
    STATE(359), 1,
      aux_sym_theory_operators_repeat1,
    STATE(393), 1,
      sym_theory_root_term,
    STATE(408), 1,
      aux_sym_theory_unparsed_term_repeat1,
    STATE(478), 1,
      sym_theory_term,
    STATE(497), 1,
      sym_theory_unparsed_term,
    STATE(598), 1,
      sym_theory_terms,
    STATE(601), 1,
      sym_theory_element,
    STATE(772), 1,
      sym_theory_elements,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(612), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(467), 5,
      sym_string,
      sym_theory_function,
      sym_theory_tuple,
      sym_theory_list,
      sym_theory_set,
  [6676] = 15,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(584), 1,
      sym_identifier,
    ACTIONS(586), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(590), 1,
      sym_anonymous,
    ACTIONS(594), 1,
      anon_sym_PIPE,
    ACTIONS(598), 1,
      anon_sym_LPAREN,
    ACTIONS(600), 1,
      anon_sym_AT,
    ACTIONS(602), 1,
      sym_relation,
    STATE(334), 1,
      sym_term,
    STATE(513), 1,
      sym_upper,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(592), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(588), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    ACTIONS(628), 4,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
    STATE(299), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [6737] = 15,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(584), 1,
      sym_identifier,
    ACTIONS(586), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(590), 1,
      sym_anonymous,
    ACTIONS(594), 1,
      anon_sym_PIPE,
    ACTIONS(598), 1,
      anon_sym_LPAREN,
    ACTIONS(600), 1,
      anon_sym_AT,
    ACTIONS(602), 1,
      sym_relation,
    STATE(334), 1,
      sym_term,
    STATE(495), 1,
      sym_upper,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(592), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(588), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    ACTIONS(630), 4,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
    STATE(299), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [6798] = 22,
    ACTIONS(606), 1,
      sym_identifier,
    ACTIONS(608), 1,
      anon_sym_DQUOTE,
    ACTIONS(610), 1,
      anon_sym_LBRACE,
    ACTIONS(614), 1,
      sym_anonymous,
    ACTIONS(616), 1,
      anon_sym_LPAREN,
    ACTIONS(618), 1,
      sym_theory_operator,
    ACTIONS(620), 1,
      anon_sym_LBRACK,
    ACTIONS(624), 1,
      sym_default_negation,
    ACTIONS(626), 1,
      sym_colon,
    ACTIONS(632), 1,
      anon_sym_RBRACE2,
    STATE(307), 1,
      sym_theory_operators,
    STATE(359), 1,
      aux_sym_theory_operators_repeat1,
    STATE(393), 1,
      sym_theory_root_term,
    STATE(408), 1,
      aux_sym_theory_unparsed_term_repeat1,
    STATE(478), 1,
      sym_theory_term,
    STATE(497), 1,
      sym_theory_unparsed_term,
    STATE(598), 1,
      sym_theory_terms,
    STATE(601), 1,
      sym_theory_element,
    STATE(787), 1,
      sym_theory_elements,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(612), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(467), 5,
      sym_string,
      sym_theory_function,
      sym_theory_tuple,
      sym_theory_list,
      sym_theory_set,
  [6873] = 17,
    ACTIONS(634), 1,
      sym_identifier,
    ACTIONS(636), 1,
      anon_sym_DQUOTE,
    ACTIONS(638), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(642), 1,
      sym_anonymous,
    ACTIONS(646), 1,
      anon_sym_PIPE,
    ACTIONS(648), 1,
      anon_sym_LPAREN,
    ACTIONS(650), 1,
      anon_sym_AT,
    ACTIONS(652), 1,
      anon_sym_RBRACE2,
    ACTIONS(654), 1,
      sym_colon,
    STATE(325), 1,
      sym_term,
    STATE(614), 1,
      sym_terms,
    STATE(615), 1,
      sym_body_aggregate_element,
    STATE(840), 1,
      sym_body_aggregate_elements,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(644), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(640), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(252), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [6937] = 16,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    ACTIONS(556), 1,
      anon_sym_STAR,
    ACTIONS(656), 1,
      sym_empty_pool_item,
    STATE(361), 1,
      sym_term,
    STATE(678), 1,
      sym__terms_or_projection,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    STATE(538), 2,
      sym_projection,
      sym__term_or_projection,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [6999] = 15,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(584), 1,
      sym_identifier,
    ACTIONS(586), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(590), 1,
      sym_anonymous,
    ACTIONS(594), 1,
      anon_sym_PIPE,
    ACTIONS(598), 1,
      anon_sym_LPAREN,
    ACTIONS(600), 1,
      anon_sym_AT,
    ACTIONS(602), 1,
      sym_relation,
    STATE(334), 1,
      sym_term,
    STATE(634), 1,
      sym_upper,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(592), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(658), 3,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_DOT,
    ACTIONS(588), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(299), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [7059] = 17,
    ACTIONS(634), 1,
      sym_identifier,
    ACTIONS(636), 1,
      anon_sym_DQUOTE,
    ACTIONS(638), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(642), 1,
      sym_anonymous,
    ACTIONS(646), 1,
      anon_sym_PIPE,
    ACTIONS(648), 1,
      anon_sym_LPAREN,
    ACTIONS(650), 1,
      anon_sym_AT,
    ACTIONS(654), 1,
      sym_colon,
    ACTIONS(660), 1,
      anon_sym_RBRACE2,
    STATE(325), 1,
      sym_term,
    STATE(614), 1,
      sym_terms,
    STATE(615), 1,
      sym_body_aggregate_element,
    STATE(723), 1,
      sym_body_aggregate_elements,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(644), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(640), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(252), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [7123] = 17,
    ACTIONS(634), 1,
      sym_identifier,
    ACTIONS(636), 1,
      anon_sym_DQUOTE,
    ACTIONS(638), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(642), 1,
      sym_anonymous,
    ACTIONS(646), 1,
      anon_sym_PIPE,
    ACTIONS(648), 1,
      anon_sym_LPAREN,
    ACTIONS(650), 1,
      anon_sym_AT,
    ACTIONS(662), 1,
      anon_sym_RBRACE2,
    ACTIONS(664), 1,
      sym_colon,
    STATE(325), 1,
      sym_term,
    STATE(580), 1,
      sym_head_aggregate_element,
    STATE(727), 1,
      sym_head_aggregate_elements,
    STATE(830), 1,
      sym_terms,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(644), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(640), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(252), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [7187] = 15,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    ACTIONS(556), 1,
      anon_sym_STAR,
    STATE(361), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(666), 2,
      anon_sym_RPAREN2,
      anon_sym_SEMI,
    STATE(607), 2,
      sym_projection,
      sym__term_or_projection,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [7247] = 17,
    ACTIONS(634), 1,
      sym_identifier,
    ACTIONS(636), 1,
      anon_sym_DQUOTE,
    ACTIONS(638), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(642), 1,
      sym_anonymous,
    ACTIONS(646), 1,
      anon_sym_PIPE,
    ACTIONS(648), 1,
      anon_sym_LPAREN,
    ACTIONS(650), 1,
      anon_sym_AT,
    ACTIONS(664), 1,
      sym_colon,
    ACTIONS(668), 1,
      anon_sym_RBRACE2,
    STATE(325), 1,
      sym_term,
    STATE(580), 1,
      sym_head_aggregate_element,
    STATE(830), 1,
      sym_terms,
    STATE(878), 1,
      sym_head_aggregate_elements,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(644), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(640), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(252), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [7311] = 15,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    ACTIONS(556), 1,
      anon_sym_STAR,
    STATE(361), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(670), 2,
      anon_sym_RPAREN2,
      anon_sym_SEMI,
    STATE(607), 2,
      sym_projection,
      sym__term_or_projection,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [7371] = 17,
    ACTIONS(634), 1,
      sym_identifier,
    ACTIONS(636), 1,
      anon_sym_DQUOTE,
    ACTIONS(638), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(642), 1,
      sym_anonymous,
    ACTIONS(646), 1,
      anon_sym_PIPE,
    ACTIONS(648), 1,
      anon_sym_LPAREN,
    ACTIONS(650), 1,
      anon_sym_AT,
    ACTIONS(654), 1,
      sym_colon,
    ACTIONS(672), 1,
      anon_sym_RBRACE2,
    STATE(325), 1,
      sym_term,
    STATE(614), 1,
      sym_terms,
    STATE(615), 1,
      sym_body_aggregate_element,
    STATE(885), 1,
      sym_body_aggregate_elements,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(644), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(640), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(252), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [7435] = 17,
    ACTIONS(634), 1,
      sym_identifier,
    ACTIONS(636), 1,
      anon_sym_DQUOTE,
    ACTIONS(638), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(642), 1,
      sym_anonymous,
    ACTIONS(646), 1,
      anon_sym_PIPE,
    ACTIONS(648), 1,
      anon_sym_LPAREN,
    ACTIONS(650), 1,
      anon_sym_AT,
    ACTIONS(654), 1,
      sym_colon,
    ACTIONS(674), 1,
      anon_sym_RBRACE2,
    STATE(325), 1,
      sym_term,
    STATE(614), 1,
      sym_terms,
    STATE(615), 1,
      sym_body_aggregate_element,
    STATE(849), 1,
      sym_body_aggregate_elements,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(644), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(640), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(252), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [7499] = 15,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(584), 1,
      sym_identifier,
    ACTIONS(586), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(590), 1,
      sym_anonymous,
    ACTIONS(594), 1,
      anon_sym_PIPE,
    ACTIONS(598), 1,
      anon_sym_LPAREN,
    ACTIONS(600), 1,
      anon_sym_AT,
    ACTIONS(602), 1,
      sym_relation,
    STATE(334), 1,
      sym_term,
    STATE(550), 1,
      sym_upper,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(592), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(676), 3,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_DOT,
    ACTIONS(588), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(299), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [7559] = 15,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(584), 1,
      sym_identifier,
    ACTIONS(586), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(590), 1,
      sym_anonymous,
    ACTIONS(594), 1,
      anon_sym_PIPE,
    ACTIONS(598), 1,
      anon_sym_LPAREN,
    ACTIONS(600), 1,
      anon_sym_AT,
    ACTIONS(602), 1,
      sym_relation,
    STATE(334), 1,
      sym_term,
    STATE(612), 1,
      sym_upper,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(592), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(678), 3,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_DOT,
    ACTIONS(588), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(299), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [7619] = 15,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(584), 1,
      sym_identifier,
    ACTIONS(586), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(590), 1,
      sym_anonymous,
    ACTIONS(594), 1,
      anon_sym_PIPE,
    ACTIONS(598), 1,
      anon_sym_LPAREN,
    ACTIONS(600), 1,
      anon_sym_AT,
    ACTIONS(602), 1,
      sym_relation,
    STATE(334), 1,
      sym_term,
    STATE(613), 1,
      sym_upper,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(592), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(680), 3,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_DOT,
    ACTIONS(588), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(299), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [7679] = 16,
    ACTIONS(11), 1,
      sym_identifier,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(25), 1,
      anon_sym_DASH,
    ACTIONS(27), 1,
      anon_sym_TILDE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(35), 1,
      sym_boolean_constant,
    STATE(382), 1,
      sym_term,
    STATE(433), 1,
      sym_simple_atom,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    STATE(426), 2,
      sym_symbolic_atom,
      sym_comparison,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [7740] = 16,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(27), 1,
      anon_sym_TILDE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(483), 1,
      sym_boolean_constant,
    ACTIONS(568), 1,
      sym_identifier,
    ACTIONS(570), 1,
      anon_sym_DASH,
    STATE(384), 1,
      sym_term,
    STATE(453), 1,
      sym_simple_atom,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    STATE(447), 2,
      sym_symbolic_atom,
      sym_comparison,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [7801] = 5,
    ACTIONS(686), 1,
      anon_sym_LPAREN,
    STATE(161), 1,
      sym__arg_pool,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(684), 2,
      aux_sym_fstring_conversion_token1,
      anon_sym_STAR,
    ACTIONS(682), 22,
      aux_sym_fstring_accessor_token1,
      aux_sym_fstring_accessor_token2,
      aux_sym_fstring_spec_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACK,
      sym_relation,
      sym_aggregate_function,
  [7840] = 15,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(584), 1,
      sym_identifier,
    ACTIONS(586), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(590), 1,
      sym_anonymous,
    ACTIONS(594), 1,
      anon_sym_PIPE,
    ACTIONS(598), 1,
      anon_sym_LPAREN,
    ACTIONS(600), 1,
      anon_sym_AT,
    ACTIONS(602), 1,
      sym_relation,
    STATE(334), 1,
      sym_term,
    STATE(673), 1,
      sym_upper,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(592), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(688), 2,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
    ACTIONS(588), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(299), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [7899] = 15,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(584), 1,
      sym_identifier,
    ACTIONS(586), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(590), 1,
      sym_anonymous,
    ACTIONS(594), 1,
      anon_sym_PIPE,
    ACTIONS(598), 1,
      anon_sym_LPAREN,
    ACTIONS(600), 1,
      anon_sym_AT,
    ACTIONS(602), 1,
      sym_relation,
    STATE(334), 1,
      sym_term,
    STATE(689), 1,
      sym_upper,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(592), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(690), 2,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
    ACTIONS(588), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(299), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [7958] = 20,
    ACTIONS(606), 1,
      sym_identifier,
    ACTIONS(608), 1,
      anon_sym_DQUOTE,
    ACTIONS(610), 1,
      anon_sym_LBRACE,
    ACTIONS(614), 1,
      sym_anonymous,
    ACTIONS(616), 1,
      anon_sym_LPAREN,
    ACTIONS(618), 1,
      sym_theory_operator,
    ACTIONS(620), 1,
      anon_sym_LBRACK,
    ACTIONS(624), 1,
      sym_default_negation,
    ACTIONS(626), 1,
      sym_colon,
    STATE(307), 1,
      sym_theory_operators,
    STATE(359), 1,
      aux_sym_theory_operators_repeat1,
    STATE(393), 1,
      sym_theory_root_term,
    STATE(408), 1,
      aux_sym_theory_unparsed_term_repeat1,
    STATE(478), 1,
      sym_theory_term,
    STATE(497), 1,
      sym_theory_unparsed_term,
    STATE(598), 1,
      sym_theory_terms,
    STATE(683), 1,
      sym_theory_element,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(612), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(467), 5,
      sym_string,
      sym_theory_function,
      sym_theory_tuple,
      sym_theory_list,
      sym_theory_set,
  [8027] = 5,
    ACTIONS(686), 1,
      anon_sym_LPAREN,
    STATE(156), 1,
      sym__arg_pool,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(694), 2,
      aux_sym_fstring_conversion_token1,
      anon_sym_STAR,
    ACTIONS(692), 22,
      aux_sym_fstring_accessor_token1,
      aux_sym_fstring_accessor_token2,
      aux_sym_fstring_spec_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACK,
      sym_relation,
      sym_aggregate_function,
  [8066] = 20,
    ACTIONS(618), 1,
      sym_theory_operator,
    ACTIONS(624), 1,
      sym_default_negation,
    ACTIONS(696), 1,
      sym_identifier,
    ACTIONS(698), 1,
      anon_sym_DQUOTE,
    ACTIONS(700), 1,
      anon_sym_LBRACE,
    ACTIONS(704), 1,
      sym_anonymous,
    ACTIONS(706), 1,
      anon_sym_COMMA2,
    ACTIONS(708), 1,
      anon_sym_LPAREN,
    ACTIONS(710), 1,
      anon_sym_RPAREN2,
    ACTIONS(712), 1,
      anon_sym_LBRACK,
    STATE(317), 1,
      sym_theory_operators,
    STATE(359), 1,
      aux_sym_theory_operators_repeat1,
    STATE(366), 1,
      sym_theory_root_term,
    STATE(367), 1,
      aux_sym_theory_unparsed_term_repeat1,
    STATE(424), 1,
      sym_theory_unparsed_term,
    STATE(589), 1,
      sym_theory_term,
    STATE(743), 1,
      sym__theory_terms_trail,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(702), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(395), 5,
      sym_string,
      sym_theory_function,
      sym_theory_tuple,
      sym_theory_list,
      sym_theory_set,
  [8135] = 16,
    ACTIONS(634), 1,
      sym_identifier,
    ACTIONS(636), 1,
      anon_sym_DQUOTE,
    ACTIONS(638), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(642), 1,
      sym_anonymous,
    ACTIONS(646), 1,
      anon_sym_PIPE,
    ACTIONS(648), 1,
      anon_sym_LPAREN,
    ACTIONS(650), 1,
      anon_sym_AT,
    ACTIONS(714), 1,
      anon_sym_RBRACE2,
    STATE(328), 1,
      sym_term,
    STATE(539), 1,
      sym_weight,
    STATE(616), 1,
      sym_optimize_element,
    STATE(815), 1,
      sym_optimize_elements,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(644), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(640), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(252), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [8196] = 16,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(27), 1,
      anon_sym_TILDE,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(477), 1,
      sym_identifier,
    ACTIONS(479), 1,
      anon_sym_DASH,
    ACTIONS(483), 1,
      sym_boolean_constant,
    STATE(388), 1,
      sym_term,
    STATE(453), 1,
      sym_simple_atom,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    STATE(447), 2,
      sym_symbolic_atom,
      sym_comparison,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [8257] = 15,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    ACTIONS(716), 1,
      sym_relation,
    STATE(376), 1,
      sym_term,
    STATE(513), 1,
      sym_upper,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(628), 2,
      anon_sym_COMMA2,
      anon_sym_SEMI,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [8316] = 15,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    ACTIONS(716), 1,
      sym_relation,
    STATE(376), 1,
      sym_term,
    STATE(506), 1,
      sym_upper,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(596), 2,
      anon_sym_COMMA2,
      anon_sym_SEMI,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [8375] = 15,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    ACTIONS(716), 1,
      sym_relation,
    STATE(376), 1,
      sym_term,
    STATE(495), 1,
      sym_upper,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(630), 2,
      anon_sym_COMMA2,
      anon_sym_SEMI,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [8434] = 15,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    ACTIONS(716), 1,
      sym_relation,
    STATE(376), 1,
      sym_term,
    STATE(634), 1,
      sym_upper,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(658), 2,
      anon_sym_COMMA2,
      anon_sym_SEMI,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [8493] = 15,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    ACTIONS(716), 1,
      sym_relation,
    STATE(376), 1,
      sym_term,
    STATE(530), 1,
      sym_upper,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(604), 2,
      anon_sym_COMMA2,
      anon_sym_SEMI,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [8552] = 15,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    ACTIONS(716), 1,
      sym_relation,
    STATE(376), 1,
      sym_term,
    STATE(612), 1,
      sym_upper,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(678), 2,
      anon_sym_COMMA2,
      anon_sym_SEMI,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [8611] = 15,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    ACTIONS(716), 1,
      sym_relation,
    STATE(376), 1,
      sym_term,
    STATE(613), 1,
      sym_upper,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(680), 2,
      anon_sym_COMMA2,
      anon_sym_SEMI,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [8670] = 15,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    ACTIONS(716), 1,
      sym_relation,
    STATE(376), 1,
      sym_term,
    STATE(550), 1,
      sym_upper,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(676), 2,
      anon_sym_COMMA2,
      anon_sym_SEMI,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [8729] = 15,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(584), 1,
      sym_identifier,
    ACTIONS(586), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(590), 1,
      sym_anonymous,
    ACTIONS(594), 1,
      anon_sym_PIPE,
    ACTIONS(598), 1,
      anon_sym_LPAREN,
    ACTIONS(600), 1,
      anon_sym_AT,
    ACTIONS(602), 1,
      sym_relation,
    STATE(334), 1,
      sym_term,
    STATE(707), 1,
      sym_upper,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(592), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(718), 2,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
    ACTIONS(588), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(299), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [8788] = 16,
    ACTIONS(634), 1,
      sym_identifier,
    ACTIONS(636), 1,
      anon_sym_DQUOTE,
    ACTIONS(638), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(642), 1,
      sym_anonymous,
    ACTIONS(646), 1,
      anon_sym_PIPE,
    ACTIONS(648), 1,
      anon_sym_LPAREN,
    ACTIONS(650), 1,
      anon_sym_AT,
    ACTIONS(720), 1,
      anon_sym_RBRACE2,
    STATE(328), 1,
      sym_term,
    STATE(539), 1,
      sym_weight,
    STATE(616), 1,
      sym_optimize_element,
    STATE(858), 1,
      sym_optimize_elements,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(644), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(640), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(252), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [8849] = 20,
    ACTIONS(618), 1,
      sym_theory_operator,
    ACTIONS(624), 1,
      sym_default_negation,
    ACTIONS(696), 1,
      sym_identifier,
    ACTIONS(698), 1,
      anon_sym_DQUOTE,
    ACTIONS(700), 1,
      anon_sym_LBRACE,
    ACTIONS(704), 1,
      sym_anonymous,
    ACTIONS(706), 1,
      anon_sym_COMMA2,
    ACTIONS(708), 1,
      anon_sym_LPAREN,
    ACTIONS(712), 1,
      anon_sym_LBRACK,
    ACTIONS(722), 1,
      anon_sym_RPAREN2,
    STATE(317), 1,
      sym_theory_operators,
    STATE(359), 1,
      aux_sym_theory_operators_repeat1,
    STATE(366), 1,
      sym_theory_root_term,
    STATE(367), 1,
      aux_sym_theory_unparsed_term_repeat1,
    STATE(424), 1,
      sym_theory_unparsed_term,
    STATE(589), 1,
      sym_theory_term,
    STATE(847), 1,
      sym__theory_terms_trail,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(702), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(395), 5,
      sym_string,
      sym_theory_function,
      sym_theory_tuple,
      sym_theory_list,
      sym_theory_set,
  [8918] = 15,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(584), 1,
      sym_identifier,
    ACTIONS(586), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(590), 1,
      sym_anonymous,
    ACTIONS(594), 1,
      anon_sym_PIPE,
    ACTIONS(598), 1,
      anon_sym_LPAREN,
    ACTIONS(600), 1,
      anon_sym_AT,
    ACTIONS(602), 1,
      sym_relation,
    STATE(334), 1,
      sym_term,
    STATE(688), 1,
      sym_upper,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(592), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(724), 2,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
    ACTIONS(588), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(299), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [8977] = 19,
    ACTIONS(618), 1,
      sym_theory_operator,
    ACTIONS(624), 1,
      sym_default_negation,
    ACTIONS(696), 1,
      sym_identifier,
    ACTIONS(698), 1,
      anon_sym_DQUOTE,
    ACTIONS(700), 1,
      anon_sym_LBRACE,
    ACTIONS(704), 1,
      sym_anonymous,
    ACTIONS(708), 1,
      anon_sym_LPAREN,
    ACTIONS(712), 1,
      anon_sym_LBRACK,
    ACTIONS(726), 1,
      anon_sym_RBRACE2,
    STATE(317), 1,
      sym_theory_operators,
    STATE(359), 1,
      aux_sym_theory_operators_repeat1,
    STATE(366), 1,
      sym_theory_root_term,
    STATE(367), 1,
      aux_sym_theory_unparsed_term_repeat1,
    STATE(424), 1,
      sym_theory_unparsed_term,
    STATE(487), 1,
      sym_theory_term,
    STATE(735), 1,
      sym_theory_terms,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(702), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(395), 5,
      sym_string,
      sym_theory_function,
      sym_theory_tuple,
      sym_theory_list,
      sym_theory_set,
  [9043] = 14,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    ACTIONS(556), 1,
      anon_sym_STAR,
    STATE(361), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    STATE(607), 2,
      sym_projection,
      sym__term_or_projection,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [9099] = 15,
    ACTIONS(634), 1,
      sym_identifier,
    ACTIONS(636), 1,
      anon_sym_DQUOTE,
    ACTIONS(638), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(642), 1,
      sym_anonymous,
    ACTIONS(646), 1,
      anon_sym_PIPE,
    ACTIONS(648), 1,
      anon_sym_LPAREN,
    ACTIONS(650), 1,
      anon_sym_AT,
    ACTIONS(654), 1,
      sym_colon,
    STATE(325), 1,
      sym_term,
    STATE(614), 1,
      sym_terms,
    STATE(692), 1,
      sym_body_aggregate_element,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(644), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(640), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(252), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [9157] = 19,
    ACTIONS(618), 1,
      sym_theory_operator,
    ACTIONS(624), 1,
      sym_default_negation,
    ACTIONS(696), 1,
      sym_identifier,
    ACTIONS(698), 1,
      anon_sym_DQUOTE,
    ACTIONS(700), 1,
      anon_sym_LBRACE,
    ACTIONS(704), 1,
      sym_anonymous,
    ACTIONS(708), 1,
      anon_sym_LPAREN,
    ACTIONS(712), 1,
      anon_sym_LBRACK,
    ACTIONS(728), 1,
      anon_sym_RBRACK,
    STATE(317), 1,
      sym_theory_operators,
    STATE(359), 1,
      aux_sym_theory_operators_repeat1,
    STATE(366), 1,
      sym_theory_root_term,
    STATE(367), 1,
      aux_sym_theory_unparsed_term_repeat1,
    STATE(424), 1,
      sym_theory_unparsed_term,
    STATE(487), 1,
      sym_theory_term,
    STATE(753), 1,
      sym_theory_terms,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(702), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(395), 5,
      sym_string,
      sym_theory_function,
      sym_theory_tuple,
      sym_theory_list,
      sym_theory_set,
  [9223] = 19,
    ACTIONS(618), 1,
      sym_theory_operator,
    ACTIONS(624), 1,
      sym_default_negation,
    ACTIONS(696), 1,
      sym_identifier,
    ACTIONS(698), 1,
      anon_sym_DQUOTE,
    ACTIONS(700), 1,
      anon_sym_LBRACE,
    ACTIONS(704), 1,
      sym_anonymous,
    ACTIONS(708), 1,
      anon_sym_LPAREN,
    ACTIONS(712), 1,
      anon_sym_LBRACK,
    ACTIONS(730), 1,
      anon_sym_RPAREN2,
    STATE(317), 1,
      sym_theory_operators,
    STATE(359), 1,
      aux_sym_theory_operators_repeat1,
    STATE(366), 1,
      sym_theory_root_term,
    STATE(367), 1,
      aux_sym_theory_unparsed_term_repeat1,
    STATE(424), 1,
      sym_theory_unparsed_term,
    STATE(487), 1,
      sym_theory_term,
    STATE(851), 1,
      sym_theory_terms,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(702), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(395), 5,
      sym_string,
      sym_theory_function,
      sym_theory_tuple,
      sym_theory_list,
      sym_theory_set,
  [9289] = 15,
    ACTIONS(634), 1,
      sym_identifier,
    ACTIONS(636), 1,
      anon_sym_DQUOTE,
    ACTIONS(638), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(642), 1,
      sym_anonymous,
    ACTIONS(646), 1,
      anon_sym_PIPE,
    ACTIONS(648), 1,
      anon_sym_LPAREN,
    ACTIONS(650), 1,
      anon_sym_AT,
    ACTIONS(664), 1,
      sym_colon,
    STATE(325), 1,
      sym_term,
    STATE(704), 1,
      sym_head_aggregate_element,
    STATE(830), 1,
      sym_terms,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(644), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(640), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(252), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [9347] = 19,
    ACTIONS(618), 1,
      sym_theory_operator,
    ACTIONS(624), 1,
      sym_default_negation,
    ACTIONS(696), 1,
      sym_identifier,
    ACTIONS(698), 1,
      anon_sym_DQUOTE,
    ACTIONS(700), 1,
      anon_sym_LBRACE,
    ACTIONS(704), 1,
      sym_anonymous,
    ACTIONS(708), 1,
      anon_sym_LPAREN,
    ACTIONS(712), 1,
      anon_sym_LBRACK,
    ACTIONS(732), 1,
      anon_sym_RBRACE2,
    STATE(317), 1,
      sym_theory_operators,
    STATE(359), 1,
      aux_sym_theory_operators_repeat1,
    STATE(366), 1,
      sym_theory_root_term,
    STATE(367), 1,
      aux_sym_theory_unparsed_term_repeat1,
    STATE(424), 1,
      sym_theory_unparsed_term,
    STATE(487), 1,
      sym_theory_term,
    STATE(846), 1,
      sym_theory_terms,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(702), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(395), 5,
      sym_string,
      sym_theory_function,
      sym_theory_tuple,
      sym_theory_list,
      sym_theory_set,
  [9413] = 19,
    ACTIONS(618), 1,
      sym_theory_operator,
    ACTIONS(624), 1,
      sym_default_negation,
    ACTIONS(696), 1,
      sym_identifier,
    ACTIONS(698), 1,
      anon_sym_DQUOTE,
    ACTIONS(700), 1,
      anon_sym_LBRACE,
    ACTIONS(704), 1,
      sym_anonymous,
    ACTIONS(708), 1,
      anon_sym_LPAREN,
    ACTIONS(712), 1,
      anon_sym_LBRACK,
    ACTIONS(734), 1,
      anon_sym_RPAREN2,
    STATE(317), 1,
      sym_theory_operators,
    STATE(359), 1,
      aux_sym_theory_operators_repeat1,
    STATE(366), 1,
      sym_theory_root_term,
    STATE(367), 1,
      aux_sym_theory_unparsed_term_repeat1,
    STATE(424), 1,
      sym_theory_unparsed_term,
    STATE(487), 1,
      sym_theory_term,
    STATE(764), 1,
      sym_theory_terms,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(702), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(395), 5,
      sym_string,
      sym_theory_function,
      sym_theory_tuple,
      sym_theory_list,
      sym_theory_set,
  [9479] = 19,
    ACTIONS(618), 1,
      sym_theory_operator,
    ACTIONS(624), 1,
      sym_default_negation,
    ACTIONS(696), 1,
      sym_identifier,
    ACTIONS(698), 1,
      anon_sym_DQUOTE,
    ACTIONS(700), 1,
      anon_sym_LBRACE,
    ACTIONS(704), 1,
      sym_anonymous,
    ACTIONS(708), 1,
      anon_sym_LPAREN,
    ACTIONS(712), 1,
      anon_sym_LBRACK,
    ACTIONS(736), 1,
      anon_sym_RBRACK,
    STATE(317), 1,
      sym_theory_operators,
    STATE(359), 1,
      aux_sym_theory_operators_repeat1,
    STATE(366), 1,
      sym_theory_root_term,
    STATE(367), 1,
      aux_sym_theory_unparsed_term_repeat1,
    STATE(424), 1,
      sym_theory_unparsed_term,
    STATE(487), 1,
      sym_theory_term,
    STATE(848), 1,
      sym_theory_terms,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(702), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(395), 5,
      sym_string,
      sym_theory_function,
      sym_theory_tuple,
      sym_theory_list,
      sym_theory_set,
  [9545] = 13,
    ACTIONS(634), 1,
      sym_identifier,
    ACTIONS(636), 1,
      anon_sym_DQUOTE,
    ACTIONS(638), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(642), 1,
      sym_anonymous,
    ACTIONS(646), 1,
      anon_sym_PIPE,
    ACTIONS(648), 1,
      anon_sym_LPAREN,
    ACTIONS(650), 1,
      anon_sym_AT,
    STATE(260), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(644), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(738), 2,
      anon_sym_LBRACE,
      sym_aggregate_function,
    ACTIONS(640), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(252), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [9598] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(742), 2,
      aux_sym_fstring_conversion_token1,
      anon_sym_STAR,
    ACTIONS(740), 22,
      aux_sym_fstring_accessor_token1,
      aux_sym_fstring_accessor_token2,
      aux_sym_fstring_spec_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACK,
      sym_relation,
      sym_aggregate_function,
  [9631] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(746), 2,
      aux_sym_fstring_conversion_token1,
      anon_sym_STAR,
    ACTIONS(744), 22,
      aux_sym_fstring_accessor_token1,
      aux_sym_fstring_accessor_token2,
      aux_sym_fstring_spec_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACK,
      sym_relation,
      sym_aggregate_function,
  [9664] = 18,
    ACTIONS(618), 1,
      sym_theory_operator,
    ACTIONS(624), 1,
      sym_default_negation,
    ACTIONS(696), 1,
      sym_identifier,
    ACTIONS(698), 1,
      anon_sym_DQUOTE,
    ACTIONS(700), 1,
      anon_sym_LBRACE,
    ACTIONS(704), 1,
      sym_anonymous,
    ACTIONS(708), 1,
      anon_sym_LPAREN,
    ACTIONS(712), 1,
      anon_sym_LBRACK,
    ACTIONS(748), 1,
      anon_sym_RPAREN2,
    STATE(317), 1,
      sym_theory_operators,
    STATE(359), 1,
      aux_sym_theory_operators_repeat1,
    STATE(366), 1,
      sym_theory_root_term,
    STATE(367), 1,
      aux_sym_theory_unparsed_term_repeat1,
    STATE(424), 1,
      sym_theory_unparsed_term,
    STATE(491), 1,
      sym_theory_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(702), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(395), 5,
      sym_string,
      sym_theory_function,
      sym_theory_tuple,
      sym_theory_list,
      sym_theory_set,
  [9727] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(752), 2,
      aux_sym_fstring_conversion_token1,
      anon_sym_STAR,
    ACTIONS(750), 22,
      aux_sym_fstring_accessor_token1,
      aux_sym_fstring_accessor_token2,
      aux_sym_fstring_spec_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACK,
      sym_relation,
      sym_aggregate_function,
  [9760] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(756), 2,
      aux_sym_fstring_conversion_token1,
      anon_sym_STAR,
    ACTIONS(754), 22,
      aux_sym_fstring_accessor_token1,
      aux_sym_fstring_accessor_token2,
      aux_sym_fstring_spec_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACK,
      sym_relation,
      sym_aggregate_function,
  [9793] = 14,
    ACTIONS(634), 1,
      sym_identifier,
    ACTIONS(636), 1,
      anon_sym_DQUOTE,
    ACTIONS(638), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(642), 1,
      sym_anonymous,
    ACTIONS(646), 1,
      anon_sym_PIPE,
    ACTIONS(648), 1,
      anon_sym_LPAREN,
    ACTIONS(650), 1,
      anon_sym_AT,
    STATE(328), 1,
      sym_term,
    STATE(539), 1,
      sym_weight,
    STATE(671), 1,
      sym_optimize_element,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(644), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(640), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(252), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [9848] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(760), 2,
      aux_sym_fstring_conversion_token1,
      anon_sym_STAR,
    ACTIONS(758), 22,
      aux_sym_fstring_accessor_token1,
      aux_sym_fstring_accessor_token2,
      aux_sym_fstring_spec_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACK,
      sym_relation,
      sym_aggregate_function,
  [9881] = 18,
    ACTIONS(618), 1,
      sym_theory_operator,
    ACTIONS(624), 1,
      sym_default_negation,
    ACTIONS(696), 1,
      sym_identifier,
    ACTIONS(698), 1,
      anon_sym_DQUOTE,
    ACTIONS(700), 1,
      anon_sym_LBRACE,
    ACTIONS(704), 1,
      sym_anonymous,
    ACTIONS(708), 1,
      anon_sym_LPAREN,
    ACTIONS(712), 1,
      anon_sym_LBRACK,
    ACTIONS(762), 1,
      anon_sym_RPAREN2,
    STATE(317), 1,
      sym_theory_operators,
    STATE(359), 1,
      aux_sym_theory_operators_repeat1,
    STATE(366), 1,
      sym_theory_root_term,
    STATE(367), 1,
      aux_sym_theory_unparsed_term_repeat1,
    STATE(424), 1,
      sym_theory_unparsed_term,
    STATE(491), 1,
      sym_theory_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(702), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(395), 5,
      sym_string,
      sym_theory_function,
      sym_theory_tuple,
      sym_theory_list,
      sym_theory_set,
  [9944] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(766), 2,
      aux_sym_fstring_conversion_token1,
      anon_sym_STAR,
    ACTIONS(764), 22,
      aux_sym_fstring_accessor_token1,
      aux_sym_fstring_accessor_token2,
      aux_sym_fstring_spec_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACK,
      sym_relation,
      sym_aggregate_function,
  [9977] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(770), 2,
      aux_sym_fstring_conversion_token1,
      anon_sym_STAR,
    ACTIONS(768), 22,
      aux_sym_fstring_accessor_token1,
      aux_sym_fstring_accessor_token2,
      aux_sym_fstring_spec_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACK,
      sym_relation,
      sym_aggregate_function,
  [10010] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(774), 2,
      aux_sym_fstring_conversion_token1,
      anon_sym_STAR,
    ACTIONS(772), 22,
      aux_sym_fstring_accessor_token1,
      aux_sym_fstring_accessor_token2,
      aux_sym_fstring_spec_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACK,
      sym_relation,
      sym_aggregate_function,
  [10043] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(778), 2,
      aux_sym_fstring_conversion_token1,
      anon_sym_STAR,
    ACTIONS(776), 22,
      aux_sym_fstring_accessor_token1,
      aux_sym_fstring_accessor_token2,
      aux_sym_fstring_spec_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACK,
      sym_relation,
      sym_aggregate_function,
  [10076] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(782), 2,
      aux_sym_fstring_conversion_token1,
      anon_sym_STAR,
    ACTIONS(780), 22,
      aux_sym_fstring_accessor_token1,
      aux_sym_fstring_accessor_token2,
      aux_sym_fstring_spec_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACK,
      sym_relation,
      sym_aggregate_function,
  [10109] = 15,
    ACTIONS(636), 1,
      anon_sym_DQUOTE,
    ACTIONS(638), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(642), 1,
      sym_anonymous,
    ACTIONS(644), 1,
      anon_sym_TILDE,
    ACTIONS(646), 1,
      anon_sym_PIPE,
    ACTIONS(648), 1,
      anon_sym_LPAREN,
    ACTIONS(650), 1,
      anon_sym_AT,
    ACTIONS(784), 1,
      sym_identifier,
    ACTIONS(786), 1,
      anon_sym_DASH,
    ACTIONS(788), 1,
      anon_sym_DOT,
    STATE(375), 1,
      sym_term,
    STATE(823), 1,
      sym_signature,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(640), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(252), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [10166] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(792), 2,
      aux_sym_fstring_conversion_token1,
      anon_sym_STAR,
    ACTIONS(790), 22,
      aux_sym_fstring_accessor_token1,
      aux_sym_fstring_accessor_token2,
      aux_sym_fstring_spec_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACK,
      sym_relation,
      sym_aggregate_function,
  [10199] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(796), 2,
      aux_sym_fstring_conversion_token1,
      anon_sym_STAR,
    ACTIONS(794), 22,
      aux_sym_fstring_accessor_token1,
      aux_sym_fstring_accessor_token2,
      aux_sym_fstring_spec_token1,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACK,
      sym_relation,
      sym_aggregate_function,
  [10232] = 13,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    STATE(358), 1,
      sym_term,
    STATE(660), 1,
      sym_weight,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [10284] = 13,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    STATE(357), 1,
      sym_term,
    STATE(838), 1,
      sym_terms,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [10336] = 17,
    ACTIONS(618), 1,
      sym_theory_operator,
    ACTIONS(624), 1,
      sym_default_negation,
    ACTIONS(696), 1,
      sym_identifier,
    ACTIONS(698), 1,
      anon_sym_DQUOTE,
    ACTIONS(700), 1,
      anon_sym_LBRACE,
    ACTIONS(704), 1,
      sym_anonymous,
    ACTIONS(708), 1,
      anon_sym_LPAREN,
    ACTIONS(712), 1,
      anon_sym_LBRACK,
    STATE(317), 1,
      sym_theory_operators,
    STATE(359), 1,
      aux_sym_theory_operators_repeat1,
    STATE(366), 1,
      sym_theory_root_term,
    STATE(367), 1,
      aux_sym_theory_unparsed_term_repeat1,
    STATE(424), 1,
      sym_theory_unparsed_term,
    STATE(491), 1,
      sym_theory_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(702), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(395), 5,
      sym_string,
      sym_theory_function,
      sym_theory_tuple,
      sym_theory_list,
      sym_theory_set,
  [10396] = 13,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    STATE(383), 1,
      sym_term,
    STATE(663), 1,
      sym_edge_pair,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [10448] = 13,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    STATE(383), 1,
      sym_term,
    STATE(560), 1,
      sym_edge_pair,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [10500] = 13,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    STATE(358), 1,
      sym_term,
    STATE(722), 1,
      sym_weight,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [10552] = 13,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    STATE(358), 1,
      sym_term,
    STATE(698), 1,
      sym_weight,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [10604] = 13,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    STATE(358), 1,
      sym_term,
    STATE(754), 1,
      sym_weight,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [10656] = 13,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    STATE(358), 1,
      sym_term,
    STATE(719), 1,
      sym_weight,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [10708] = 13,
    ACTIONS(634), 1,
      sym_identifier,
    ACTIONS(636), 1,
      anon_sym_DQUOTE,
    ACTIONS(638), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(642), 1,
      sym_anonymous,
    ACTIONS(646), 1,
      anon_sym_PIPE,
    ACTIONS(648), 1,
      anon_sym_LPAREN,
    ACTIONS(650), 1,
      anon_sym_AT,
    STATE(325), 1,
      sym_term,
    STATE(553), 1,
      sym_terms,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(644), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(640), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(252), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [10760] = 13,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    STATE(357), 1,
      sym_term,
    STATE(804), 1,
      sym_terms,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [10812] = 17,
    ACTIONS(606), 1,
      sym_identifier,
    ACTIONS(608), 1,
      anon_sym_DQUOTE,
    ACTIONS(610), 1,
      anon_sym_LBRACE,
    ACTIONS(614), 1,
      sym_anonymous,
    ACTIONS(616), 1,
      anon_sym_LPAREN,
    ACTIONS(618), 1,
      sym_theory_operator,
    ACTIONS(620), 1,
      anon_sym_LBRACK,
    ACTIONS(624), 1,
      sym_default_negation,
    STATE(307), 1,
      sym_theory_operators,
    STATE(359), 1,
      aux_sym_theory_operators_repeat1,
    STATE(393), 1,
      sym_theory_root_term,
    STATE(408), 1,
      aux_sym_theory_unparsed_term_repeat1,
    STATE(497), 1,
      sym_theory_unparsed_term,
    STATE(498), 1,
      sym_theory_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(612), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(467), 5,
      sym_string,
      sym_theory_function,
      sym_theory_tuple,
      sym_theory_list,
      sym_theory_set,
  [10872] = 17,
    ACTIONS(618), 1,
      sym_theory_operator,
    ACTIONS(624), 1,
      sym_default_negation,
    ACTIONS(696), 1,
      sym_identifier,
    ACTIONS(698), 1,
      anon_sym_DQUOTE,
    ACTIONS(700), 1,
      anon_sym_LBRACE,
    ACTIONS(704), 1,
      sym_anonymous,
    ACTIONS(708), 1,
      anon_sym_LPAREN,
    ACTIONS(712), 1,
      anon_sym_LBRACK,
    STATE(317), 1,
      sym_theory_operators,
    STATE(359), 1,
      aux_sym_theory_operators_repeat1,
    STATE(366), 1,
      sym_theory_root_term,
    STATE(367), 1,
      aux_sym_theory_unparsed_term_repeat1,
    STATE(424), 1,
      sym_theory_unparsed_term,
    STATE(503), 1,
      sym_theory_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(702), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(395), 5,
      sym_string,
      sym_theory_function,
      sym_theory_tuple,
      sym_theory_list,
      sym_theory_set,
  [10932] = 17,
    ACTIONS(618), 1,
      sym_theory_operator,
    ACTIONS(624), 1,
      sym_default_negation,
    ACTIONS(696), 1,
      sym_identifier,
    ACTIONS(698), 1,
      anon_sym_DQUOTE,
    ACTIONS(700), 1,
      anon_sym_LBRACE,
    ACTIONS(704), 1,
      sym_anonymous,
    ACTIONS(708), 1,
      anon_sym_LPAREN,
    ACTIONS(712), 1,
      anon_sym_LBRACK,
    STATE(317), 1,
      sym_theory_operators,
    STATE(359), 1,
      aux_sym_theory_operators_repeat1,
    STATE(366), 1,
      sym_theory_root_term,
    STATE(367), 1,
      aux_sym_theory_unparsed_term_repeat1,
    STATE(424), 1,
      sym_theory_unparsed_term,
    STATE(502), 1,
      sym_theory_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(702), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(395), 5,
      sym_string,
      sym_theory_function,
      sym_theory_tuple,
      sym_theory_list,
      sym_theory_set,
  [10992] = 12,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    ACTIONS(798), 1,
      anon_sym_DQUOTE,
    STATE(323), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(800), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [11041] = 7,
    ACTIONS(694), 1,
      anon_sym_STAR,
    ACTIONS(804), 1,
      anon_sym_LPAREN,
    ACTIONS(806), 1,
      anon_sym_DOT,
    STATE(247), 1,
      sym__arg_pool,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(802), 6,
      sym_colon,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      anon_sym_COLON_DASH,
    ACTIONS(692), 12,
      anon_sym_LBRACE,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_DOT_DOT,
      sym_relation,
      sym_aggregate_function,
  [11080] = 12,
    ACTIONS(634), 1,
      sym_identifier,
    ACTIONS(636), 1,
      anon_sym_DQUOTE,
    ACTIONS(638), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(642), 1,
      sym_anonymous,
    ACTIONS(646), 1,
      anon_sym_PIPE,
    ACTIONS(648), 1,
      anon_sym_LPAREN,
    ACTIONS(650), 1,
      anon_sym_AT,
    STATE(336), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(644), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(640), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(252), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [11129] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(810), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(808), 20,
      anon_sym_LBRACE,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACK,
      anon_sym_RBRACE2,
      sym_relation,
      sym_aggregate_function,
      anon_sym_COLON_DASH,
  [11160] = 12,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(584), 1,
      sym_identifier,
    ACTIONS(586), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(590), 1,
      sym_anonymous,
    ACTIONS(594), 1,
      anon_sym_PIPE,
    ACTIONS(598), 1,
      anon_sym_LPAREN,
    ACTIONS(600), 1,
      anon_sym_AT,
    STATE(340), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(592), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(588), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(299), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [11209] = 12,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    STATE(377), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [11258] = 12,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    STATE(387), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [11307] = 12,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    STATE(363), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [11356] = 12,
    ACTIONS(634), 1,
      sym_identifier,
    ACTIONS(636), 1,
      anon_sym_DQUOTE,
    ACTIONS(638), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(642), 1,
      sym_anonymous,
    ACTIONS(646), 1,
      anon_sym_PIPE,
    ACTIONS(648), 1,
      anon_sym_LPAREN,
    ACTIONS(650), 1,
      anon_sym_AT,
    STATE(260), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(644), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(640), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(252), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [11405] = 12,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    STATE(378), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [11454] = 12,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    STATE(379), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [11503] = 12,
    ACTIONS(634), 1,
      sym_identifier,
    ACTIONS(636), 1,
      anon_sym_DQUOTE,
    ACTIONS(638), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(642), 1,
      sym_anonymous,
    ACTIONS(646), 1,
      anon_sym_PIPE,
    ACTIONS(648), 1,
      anon_sym_LPAREN,
    ACTIONS(650), 1,
      anon_sym_AT,
    STATE(281), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(644), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(640), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(252), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [11552] = 12,
    ACTIONS(636), 1,
      anon_sym_DQUOTE,
    ACTIONS(638), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(642), 1,
      sym_anonymous,
    ACTIONS(646), 1,
      anon_sym_PIPE,
    ACTIONS(648), 1,
      anon_sym_LPAREN,
    ACTIONS(650), 1,
      anon_sym_AT,
    ACTIONS(812), 1,
      sym_identifier,
    STATE(257), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(644), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(640), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(252), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [11601] = 12,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    STATE(250), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [11650] = 12,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    STATE(342), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [11699] = 12,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    STATE(263), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [11748] = 12,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    STATE(381), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [11797] = 12,
    ACTIONS(634), 1,
      sym_identifier,
    ACTIONS(636), 1,
      anon_sym_DQUOTE,
    ACTIONS(638), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(642), 1,
      sym_anonymous,
    ACTIONS(646), 1,
      anon_sym_PIPE,
    ACTIONS(648), 1,
      anon_sym_LPAREN,
    ACTIONS(650), 1,
      anon_sym_AT,
    STATE(257), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(644), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(640), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(252), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [11846] = 12,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    STATE(264), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [11895] = 12,
    ACTIONS(634), 1,
      sym_identifier,
    ACTIONS(636), 1,
      anon_sym_DQUOTE,
    ACTIONS(638), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(642), 1,
      sym_anonymous,
    ACTIONS(646), 1,
      anon_sym_PIPE,
    ACTIONS(648), 1,
      anon_sym_LPAREN,
    ACTIONS(650), 1,
      anon_sym_AT,
    STATE(249), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(644), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(640), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(252), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [11944] = 12,
    ACTIONS(634), 1,
      sym_identifier,
    ACTIONS(636), 1,
      anon_sym_DQUOTE,
    ACTIONS(638), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(642), 1,
      sym_anonymous,
    ACTIONS(646), 1,
      anon_sym_PIPE,
    ACTIONS(648), 1,
      anon_sym_LPAREN,
    ACTIONS(650), 1,
      anon_sym_AT,
    STATE(276), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(644), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(640), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(252), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [11993] = 12,
    ACTIONS(634), 1,
      sym_identifier,
    ACTIONS(636), 1,
      anon_sym_DQUOTE,
    ACTIONS(638), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(642), 1,
      sym_anonymous,
    ACTIONS(646), 1,
      anon_sym_PIPE,
    ACTIONS(648), 1,
      anon_sym_LPAREN,
    ACTIONS(650), 1,
      anon_sym_AT,
    STATE(277), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(644), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(640), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(252), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [12042] = 12,
    ACTIONS(634), 1,
      sym_identifier,
    ACTIONS(636), 1,
      anon_sym_DQUOTE,
    ACTIONS(638), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(642), 1,
      sym_anonymous,
    ACTIONS(646), 1,
      anon_sym_PIPE,
    ACTIONS(648), 1,
      anon_sym_LPAREN,
    ACTIONS(650), 1,
      anon_sym_AT,
    STATE(278), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(644), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(640), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(252), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [12091] = 12,
    ACTIONS(634), 1,
      sym_identifier,
    ACTIONS(636), 1,
      anon_sym_DQUOTE,
    ACTIONS(638), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(642), 1,
      sym_anonymous,
    ACTIONS(646), 1,
      anon_sym_PIPE,
    ACTIONS(648), 1,
      anon_sym_LPAREN,
    ACTIONS(650), 1,
      anon_sym_AT,
    STATE(266), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(644), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(640), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(252), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [12140] = 12,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    STATE(365), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [12189] = 12,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    STATE(330), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [12238] = 12,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    STATE(265), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [12287] = 12,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    STATE(370), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [12336] = 12,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    STATE(371), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [12385] = 12,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    STATE(337), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [12434] = 12,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    STATE(269), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [12483] = 12,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    STATE(270), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [12532] = 12,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(584), 1,
      sym_identifier,
    ACTIONS(586), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(590), 1,
      sym_anonymous,
    ACTIONS(594), 1,
      anon_sym_PIPE,
    ACTIONS(598), 1,
      anon_sym_LPAREN,
    ACTIONS(600), 1,
      anon_sym_AT,
    STATE(302), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(592), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(588), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(299), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [12581] = 7,
    ACTIONS(694), 1,
      anon_sym_STAR,
    ACTIONS(804), 1,
      anon_sym_LPAREN,
    ACTIONS(816), 1,
      anon_sym_DOT,
    STATE(246), 1,
      sym__arg_pool,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(814), 6,
      sym_colon,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      anon_sym_COLON_DASH,
    ACTIONS(692), 12,
      anon_sym_LBRACE,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_DOT_DOT,
      sym_relation,
      sym_aggregate_function,
  [12620] = 12,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(584), 1,
      sym_identifier,
    ACTIONS(586), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(590), 1,
      sym_anonymous,
    ACTIONS(594), 1,
      anon_sym_PIPE,
    ACTIONS(598), 1,
      anon_sym_LPAREN,
    ACTIONS(600), 1,
      anon_sym_AT,
    STATE(288), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(592), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(588), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(299), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [12669] = 12,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(584), 1,
      sym_identifier,
    ACTIONS(586), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(590), 1,
      sym_anonymous,
    ACTIONS(594), 1,
      anon_sym_PIPE,
    ACTIONS(598), 1,
      anon_sym_LPAREN,
    ACTIONS(600), 1,
      anon_sym_AT,
    STATE(289), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(592), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(588), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(299), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [12718] = 12,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(584), 1,
      sym_identifier,
    ACTIONS(586), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(590), 1,
      sym_anonymous,
    ACTIONS(594), 1,
      anon_sym_PIPE,
    ACTIONS(598), 1,
      anon_sym_LPAREN,
    ACTIONS(600), 1,
      anon_sym_AT,
    STATE(290), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(592), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(588), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(299), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [12767] = 12,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(584), 1,
      sym_identifier,
    ACTIONS(586), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(590), 1,
      sym_anonymous,
    ACTIONS(594), 1,
      anon_sym_PIPE,
    ACTIONS(598), 1,
      anon_sym_LPAREN,
    ACTIONS(600), 1,
      anon_sym_AT,
    STATE(291), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(592), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(588), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(299), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [12816] = 12,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(584), 1,
      sym_identifier,
    ACTIONS(586), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(590), 1,
      sym_anonymous,
    ACTIONS(594), 1,
      anon_sym_PIPE,
    ACTIONS(598), 1,
      anon_sym_LPAREN,
    ACTIONS(600), 1,
      anon_sym_AT,
    STATE(292), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(592), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(588), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(299), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [12865] = 12,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    STATE(374), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [12914] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(820), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(818), 20,
      anon_sym_LBRACE,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACK,
      anon_sym_RBRACE2,
      sym_relation,
      sym_aggregate_function,
      anon_sym_COLON_DASH,
  [12945] = 12,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(822), 1,
      sym_identifier,
    STATE(250), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [12994] = 12,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    ACTIONS(798), 1,
      anon_sym_DQUOTE,
    STATE(331), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(800), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [13043] = 12,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    ACTIONS(798), 1,
      anon_sym_DQUOTE,
    STATE(329), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(800), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [13092] = 12,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    ACTIONS(798), 1,
      anon_sym_DQUOTE,
    STATE(333), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(800), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [13141] = 12,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    ACTIONS(798), 1,
      anon_sym_DQUOTE,
    STATE(332), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(800), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [13190] = 12,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    ACTIONS(798), 1,
      anon_sym_DQUOTE,
    STATE(322), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(800), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [13239] = 12,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(584), 1,
      sym_identifier,
    ACTIONS(586), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(590), 1,
      sym_anonymous,
    ACTIONS(594), 1,
      anon_sym_PIPE,
    ACTIONS(598), 1,
      anon_sym_LPAREN,
    ACTIONS(600), 1,
      anon_sym_AT,
    STATE(279), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(592), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(588), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(299), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [13288] = 12,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(824), 1,
      sym_identifier,
    STATE(250), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [13337] = 12,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(584), 1,
      sym_identifier,
    ACTIONS(586), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(590), 1,
      sym_anonymous,
    ACTIONS(594), 1,
      anon_sym_PIPE,
    ACTIONS(598), 1,
      anon_sym_LPAREN,
    ACTIONS(600), 1,
      anon_sym_AT,
    STATE(298), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(592), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(588), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(299), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [13386] = 12,
    ACTIONS(634), 1,
      sym_identifier,
    ACTIONS(636), 1,
      anon_sym_DQUOTE,
    ACTIONS(638), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(642), 1,
      sym_anonymous,
    ACTIONS(646), 1,
      anon_sym_PIPE,
    ACTIONS(648), 1,
      anon_sym_LPAREN,
    ACTIONS(650), 1,
      anon_sym_AT,
    STATE(338), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(644), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(640), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(252), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [13435] = 12,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    STATE(380), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [13484] = 12,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    ACTIONS(798), 1,
      anon_sym_DQUOTE,
    STATE(258), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(800), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [13533] = 12,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    STATE(360), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [13582] = 12,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(826), 1,
      sym_identifier,
    STATE(250), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [13631] = 12,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(15), 1,
      anon_sym_f_DQUOTE,
    ACTIONS(21), 1,
      sym_anonymous,
    ACTIONS(29), 1,
      anon_sym_PIPE,
    ACTIONS(31), 1,
      anon_sym_LPAREN,
    ACTIONS(33), 1,
      anon_sym_AT,
    ACTIONS(554), 1,
      sym_identifier,
    STATE(362), 1,
      sym_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(27), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(19), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(158), 8,
      sym_string,
      sym_fstring,
      sym_binary_operation,
      sym_unary_operation,
      sym_abs,
      sym_function,
      sym_external_function,
      sym_tuple,
  [13680] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(782), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(780), 19,
      sym_colon,
      anon_sym_LBRACE,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACE2,
      sym_relation,
      sym_aggregate_function,
      anon_sym_COLON_DASH,
  [13710] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(774), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(772), 19,
      sym_colon,
      anon_sym_LBRACE,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACE2,
      sym_relation,
      sym_aggregate_function,
      anon_sym_COLON_DASH,
  [13740] = 5,
    ACTIONS(804), 1,
      anon_sym_LPAREN,
    STATE(271), 1,
      sym__arg_pool,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(694), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(692), 17,
      sym_colon,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [13774] = 5,
    ACTIONS(804), 1,
      anon_sym_LPAREN,
    STATE(274), 1,
      sym__arg_pool,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(684), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(682), 17,
      sym_colon,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [13808] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(770), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(768), 19,
      sym_colon,
      anon_sym_LBRACE,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACE2,
      sym_relation,
      sym_aggregate_function,
      anon_sym_COLON_DASH,
  [13838] = 5,
    ACTIONS(746), 1,
      anon_sym_STAR,
    ACTIONS(830), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(828), 6,
      sym_colon,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      anon_sym_COLON_DASH,
    ACTIONS(744), 12,
      anon_sym_LBRACE,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_DOT_DOT,
      sym_relation,
      sym_aggregate_function,
  [13871] = 5,
    ACTIONS(746), 1,
      anon_sym_STAR,
    ACTIONS(834), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(832), 6,
      sym_colon,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      anon_sym_COLON_DASH,
    ACTIONS(744), 12,
      anon_sym_LBRACE,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_DOT_DOT,
      sym_relation,
      sym_aggregate_function,
  [13904] = 11,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(836), 1,
      sym_identifier,
    ACTIONS(842), 1,
      anon_sym_PIPE,
    ACTIONS(844), 1,
      anon_sym_COMMA2,
    ACTIONS(846), 1,
      anon_sym_LPAREN,
    ACTIONS(848), 1,
      anon_sym_RPAREN2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(840), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    STATE(812), 2,
      sym__const_term_comma,
      sym__const_terms_trail,
    ACTIONS(838), 3,
      sym_supremum,
      sym_infimum,
      sym_number,
    STATE(364), 7,
      sym_string,
      sym__const_term,
      sym_const_binary_operation,
      sym_const_unary_operation,
      sym_const_abs,
      sym_const_function,
      sym_const_tuple,
  [13949] = 7,
    ACTIONS(854), 1,
      anon_sym_STAR,
    ACTIONS(858), 1,
      anon_sym_STAR_STAR,
    ACTIONS(860), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(852), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(856), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(850), 12,
      sym_colon,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [13985] = 4,
    ACTIONS(864), 1,
      anon_sym_STAR,
    ACTIONS(866), 1,
      anon_sym_STAR_STAR,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(862), 17,
      anon_sym_LBRACE,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACK,
      sym_relation,
      sym_aggregate_function,
  [14015] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(766), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(764), 17,
      sym_colon,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [14043] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(752), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(750), 17,
      sym_colon,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [14071] = 7,
    ACTIONS(694), 1,
      anon_sym_STAR,
    ACTIONS(816), 1,
      anon_sym_DOT,
    ACTIONS(868), 1,
      anon_sym_LPAREN,
    STATE(300), 1,
      sym__arg_pool,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(814), 5,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      anon_sym_COLON_DASH,
    ACTIONS(692), 10,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_DOT_DOT,
      sym_relation,
  [14107] = 5,
    ACTIONS(868), 1,
      anon_sym_LPAREN,
    STATE(284), 1,
      sym__arg_pool,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(694), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(692), 15,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [14139] = 7,
    ACTIONS(694), 1,
      anon_sym_STAR,
    ACTIONS(806), 1,
      anon_sym_DOT,
    ACTIONS(868), 1,
      anon_sym_LPAREN,
    STATE(304), 1,
      sym__arg_pool,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(802), 5,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      anon_sym_COLON_DASH,
    ACTIONS(692), 10,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_DOT_DOT,
      sym_relation,
  [14175] = 5,
    ACTIONS(868), 1,
      anon_sym_LPAREN,
    STATE(287), 1,
      sym__arg_pool,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(684), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(682), 15,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [14207] = 4,
    ACTIONS(858), 1,
      anon_sym_STAR_STAR,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(864), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(862), 16,
      sym_colon,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [14237] = 16,
    ACTIONS(870), 1,
      aux_sym_fstring_accessor_token1,
    ACTIONS(872), 1,
      aux_sym_fstring_accessor_token2,
    ACTIONS(874), 1,
      aux_sym_fstring_conversion_token1,
    ACTIONS(876), 1,
      aux_sym_fstring_spec_token1,
    ACTIONS(878), 1,
      anon_sym_RBRACE,
    ACTIONS(884), 1,
      anon_sym_STAR,
    ACTIONS(888), 1,
      anon_sym_STAR_STAR,
    ACTIONS(890), 1,
      anon_sym_DOT_DOT,
    STATE(465), 1,
      aux_sym_fstring_accessor_repeat1,
    STATE(484), 1,
      sym_fstring_accessor,
    STATE(582), 1,
      sym_fstring_conversion,
    STATE(718), 1,
      sym_fstring_spec,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(882), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(886), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(880), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
  [14291] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(778), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(776), 17,
      sym_colon,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [14319] = 11,
    ACTIONS(854), 1,
      anon_sym_STAR,
    ACTIONS(858), 1,
      anon_sym_STAR_STAR,
    ACTIONS(896), 1,
      anon_sym_DOT_DOT,
    ACTIONS(898), 1,
      sym_relation,
    ACTIONS(900), 1,
      anon_sym_DOT,
    STATE(412), 1,
      aux_sym_comparison_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(852), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(856), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(892), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
    ACTIONS(894), 6,
      sym_colon,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      anon_sym_COLON_DASH,
  [14363] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(820), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(818), 17,
      sym_colon,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [14391] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(796), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(794), 17,
      sym_colon,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [14419] = 6,
    ACTIONS(866), 1,
      anon_sym_STAR_STAR,
    ACTIONS(904), 1,
      anon_sym_STAR,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(902), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(906), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(850), 13,
      anon_sym_LBRACE,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACK,
      sym_relation,
      sym_aggregate_function,
  [14453] = 5,
    ACTIONS(866), 1,
      anon_sym_STAR_STAR,
    ACTIONS(904), 1,
      anon_sym_STAR,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(906), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(850), 15,
      anon_sym_LBRACE,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACK,
      sym_relation,
      sym_aggregate_function,
  [14485] = 4,
    ACTIONS(860), 1,
      anon_sym_STAR,
    ACTIONS(866), 1,
      anon_sym_STAR_STAR,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(850), 17,
      anon_sym_LBRACE,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACK,
      sym_relation,
      sym_aggregate_function,
  [14515] = 8,
    ACTIONS(854), 1,
      anon_sym_STAR,
    ACTIONS(858), 1,
      anon_sym_STAR_STAR,
    ACTIONS(860), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(852), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(856), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(892), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
    ACTIONS(850), 9,
      sym_colon,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [14553] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(810), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(808), 17,
      sym_colon,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [14581] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(792), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(790), 17,
      sym_colon,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [14609] = 4,
    ACTIONS(860), 1,
      anon_sym_STAR,
    ACTIONS(866), 1,
      anon_sym_STAR_STAR,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(850), 17,
      anon_sym_LBRACE,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACK,
      sym_relation,
      sym_aggregate_function,
  [14639] = 7,
    ACTIONS(866), 1,
      anon_sym_STAR_STAR,
    ACTIONS(904), 1,
      anon_sym_STAR,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(902), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(906), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(908), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
    ACTIONS(850), 10,
      anon_sym_LBRACE,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACK,
      sym_relation,
      sym_aggregate_function,
  [14675] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(746), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(744), 17,
      sym_colon,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [14703] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(742), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(740), 17,
      sym_colon,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [14731] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(756), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(754), 17,
      sym_colon,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [14759] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(760), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(758), 17,
      sym_colon,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [14787] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(910), 3,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
    ACTIONS(912), 16,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
      sym_aggregate_function,
  [14815] = 6,
    ACTIONS(854), 1,
      anon_sym_STAR,
    ACTIONS(858), 1,
      anon_sym_STAR_STAR,
    ACTIONS(860), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(856), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(850), 14,
      sym_colon,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [14849] = 4,
    ACTIONS(858), 1,
      anon_sym_STAR_STAR,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(860), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(850), 16,
      sym_colon,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [14879] = 4,
    ACTIONS(858), 1,
      anon_sym_STAR_STAR,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(860), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(850), 16,
      sym_colon,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_AT,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [14909] = 11,
    ACTIONS(900), 1,
      anon_sym_DOT,
    ACTIONS(918), 1,
      anon_sym_STAR,
    ACTIONS(922), 1,
      anon_sym_STAR_STAR,
    ACTIONS(924), 1,
      anon_sym_DOT_DOT,
    ACTIONS(926), 1,
      sym_relation,
    STATE(415), 1,
      aux_sym_comparison_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(916), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(920), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(914), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
    ACTIONS(894), 5,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      anon_sym_COLON_DASH,
  [14952] = 10,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(836), 1,
      sym_identifier,
    ACTIONS(842), 1,
      anon_sym_PIPE,
    ACTIONS(846), 1,
      anon_sym_LPAREN,
    ACTIONS(930), 1,
      anon_sym_RPAREN2,
    STATE(853), 1,
      sym_const_terms,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(840), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(928), 3,
      sym_supremum,
      sym_infimum,
      sym_number,
    STATE(373), 7,
      sym_string,
      sym__const_term,
      sym_const_binary_operation,
      sym_const_unary_operation,
      sym_const_abs,
      sym_const_function,
      sym_const_tuple,
  [14993] = 9,
    ACTIONS(854), 1,
      anon_sym_STAR,
    ACTIONS(858), 1,
      anon_sym_STAR_STAR,
    ACTIONS(896), 1,
      anon_sym_DOT_DOT,
    ACTIONS(934), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(852), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(856), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(892), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
    ACTIONS(932), 7,
      sym_colon,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [15032] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(778), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(776), 15,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [15058] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(792), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(790), 15,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [15084] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(746), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(744), 15,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [15110] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(742), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(740), 15,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [15136] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(756), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(754), 15,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [15162] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(760), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(758), 15,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [15188] = 7,
    ACTIONS(860), 1,
      anon_sym_DOT,
    ACTIONS(918), 1,
      anon_sym_STAR,
    ACTIONS(922), 1,
      anon_sym_STAR_STAR,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(916), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(920), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(850), 10,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [15222] = 6,
    ACTIONS(860), 1,
      anon_sym_DOT,
    ACTIONS(918), 1,
      anon_sym_STAR,
    ACTIONS(922), 1,
      anon_sym_STAR_STAR,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(920), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(850), 12,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [15254] = 4,
    ACTIONS(922), 1,
      anon_sym_STAR_STAR,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(860), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(850), 14,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [15282] = 4,
    ACTIONS(922), 1,
      anon_sym_STAR_STAR,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(860), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(850), 14,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [15310] = 8,
    ACTIONS(860), 1,
      anon_sym_DOT,
    ACTIONS(918), 1,
      anon_sym_STAR,
    ACTIONS(922), 1,
      anon_sym_STAR_STAR,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(916), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(920), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(914), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
    ACTIONS(850), 7,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [15346] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(766), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(764), 15,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [15372] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(770), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(768), 15,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [15398] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(782), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(780), 15,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [15424] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(774), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(772), 15,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [15450] = 9,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(836), 1,
      sym_identifier,
    ACTIONS(842), 1,
      anon_sym_PIPE,
    ACTIONS(846), 1,
      anon_sym_LPAREN,
    ACTIONS(938), 1,
      anon_sym_RPAREN2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(840), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(936), 3,
      sym_supremum,
      sym_infimum,
      sym_number,
    STATE(385), 7,
      sym_string,
      sym__const_term,
      sym_const_binary_operation,
      sym_const_unary_operation,
      sym_const_abs,
      sym_const_function,
      sym_const_tuple,
  [15488] = 9,
    ACTIONS(918), 1,
      anon_sym_STAR,
    ACTIONS(922), 1,
      anon_sym_STAR_STAR,
    ACTIONS(924), 1,
      anon_sym_DOT_DOT,
    ACTIONS(934), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(916), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(920), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(914), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
    ACTIONS(932), 6,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [15526] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(752), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(750), 15,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [15552] = 5,
    ACTIONS(746), 1,
      anon_sym_STAR,
    ACTIONS(830), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(828), 5,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      anon_sym_COLON_DASH,
    ACTIONS(744), 10,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_DOT_DOT,
      sym_relation,
  [15582] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(796), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(794), 15,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [15608] = 4,
    ACTIONS(922), 1,
      anon_sym_STAR_STAR,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(864), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(862), 14,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_DOT_DOT,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      sym_relation,
      anon_sym_COLON_DASH,
  [15636] = 9,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(836), 1,
      sym_identifier,
    ACTIONS(842), 1,
      anon_sym_PIPE,
    ACTIONS(846), 1,
      anon_sym_LPAREN,
    ACTIONS(940), 1,
      anon_sym_RPAREN2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(840), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(936), 3,
      sym_supremum,
      sym_infimum,
      sym_number,
    STATE(385), 7,
      sym_string,
      sym__const_term,
      sym_const_binary_operation,
      sym_const_unary_operation,
      sym_const_abs,
      sym_const_function,
      sym_const_tuple,
  [15674] = 5,
    ACTIONS(746), 1,
      anon_sym_STAR,
    ACTIONS(834), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(832), 5,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      anon_sym_COLON_DASH,
    ACTIONS(744), 10,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_DOT_DOT,
      sym_relation,
  [15704] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(942), 2,
      sym_identifier,
      sym_anonymous,
    ACTIONS(944), 15,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_AMP,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_aggregate_function,
  [15730] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(946), 3,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
    ACTIONS(948), 13,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
  [15755] = 10,
    ACTIONS(608), 1,
      anon_sym_DQUOTE,
    ACTIONS(610), 1,
      anon_sym_LBRACE,
    ACTIONS(614), 1,
      sym_anonymous,
    ACTIONS(616), 1,
      anon_sym_LPAREN,
    ACTIONS(620), 1,
      anon_sym_LBRACK,
    ACTIONS(950), 1,
      sym_identifier,
    STATE(444), 1,
      sym_theory_root_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(612), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(467), 5,
      sym_string,
      sym_theory_function,
      sym_theory_tuple,
      sym_theory_list,
      sym_theory_set,
  [15794] = 8,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(836), 1,
      sym_identifier,
    ACTIONS(842), 1,
      anon_sym_PIPE,
    ACTIONS(846), 1,
      anon_sym_LPAREN,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(840), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(952), 3,
      sym_supremum,
      sym_infimum,
      sym_number,
    STATE(345), 7,
      sym_string,
      sym__const_term,
      sym_const_binary_operation,
      sym_const_unary_operation,
      sym_const_abs,
      sym_const_function,
      sym_const_tuple,
  [15829] = 8,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(836), 1,
      sym_identifier,
    ACTIONS(842), 1,
      anon_sym_PIPE,
    ACTIONS(846), 1,
      anon_sym_LPAREN,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(840), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(954), 3,
      sym_supremum,
      sym_infimum,
      sym_number,
    STATE(351), 7,
      sym_string,
      sym__const_term,
      sym_const_binary_operation,
      sym_const_unary_operation,
      sym_const_abs,
      sym_const_function,
      sym_const_tuple,
  [15864] = 8,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(836), 1,
      sym_identifier,
    ACTIONS(842), 1,
      anon_sym_PIPE,
    ACTIONS(846), 1,
      anon_sym_LPAREN,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(840), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(956), 3,
      sym_supremum,
      sym_infimum,
      sym_number,
    STATE(352), 7,
      sym_string,
      sym__const_term,
      sym_const_binary_operation,
      sym_const_unary_operation,
      sym_const_abs,
      sym_const_function,
      sym_const_tuple,
  [15899] = 8,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(836), 1,
      sym_identifier,
    ACTIONS(842), 1,
      anon_sym_PIPE,
    ACTIONS(846), 1,
      anon_sym_LPAREN,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(840), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(958), 3,
      sym_supremum,
      sym_infimum,
      sym_number,
    STATE(353), 7,
      sym_string,
      sym__const_term,
      sym_const_binary_operation,
      sym_const_unary_operation,
      sym_const_abs,
      sym_const_function,
      sym_const_tuple,
  [15934] = 8,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(836), 1,
      sym_identifier,
    ACTIONS(842), 1,
      anon_sym_PIPE,
    ACTIONS(846), 1,
      anon_sym_LPAREN,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(840), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(960), 3,
      sym_supremum,
      sym_infimum,
      sym_number,
    STATE(354), 7,
      sym_string,
      sym__const_term,
      sym_const_binary_operation,
      sym_const_unary_operation,
      sym_const_abs,
      sym_const_function,
      sym_const_tuple,
  [15969] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(962), 3,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
    ACTIONS(964), 13,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
  [15994] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(962), 3,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
    ACTIONS(964), 13,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
  [16019] = 8,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(836), 1,
      sym_identifier,
    ACTIONS(842), 1,
      anon_sym_PIPE,
    ACTIONS(846), 1,
      anon_sym_LPAREN,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(840), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(936), 3,
      sym_supremum,
      sym_infimum,
      sym_number,
    STATE(385), 7,
      sym_string,
      sym__const_term,
      sym_const_binary_operation,
      sym_const_unary_operation,
      sym_const_abs,
      sym_const_function,
      sym_const_tuple,
  [16054] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(966), 3,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
    ACTIONS(968), 13,
      anon_sym_DQUOTE,
      anon_sym_f_DQUOTE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_DASH,
      anon_sym_TILDE,
      anon_sym_PIPE,
      anon_sym_LPAREN,
      anon_sym_AT,
      sym_boolean_constant,
      sym_double_default_negation,
  [16079] = 10,
    ACTIONS(698), 1,
      anon_sym_DQUOTE,
    ACTIONS(700), 1,
      anon_sym_LBRACE,
    ACTIONS(704), 1,
      sym_anonymous,
    ACTIONS(708), 1,
      anon_sym_LPAREN,
    ACTIONS(712), 1,
      anon_sym_LBRACK,
    ACTIONS(970), 1,
      sym_identifier,
    STATE(400), 1,
      sym_theory_root_term,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(702), 4,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
    STATE(395), 5,
      sym_string,
      sym_theory_function,
      sym_theory_tuple,
      sym_theory_list,
      sym_theory_set,
  [16118] = 8,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(836), 1,
      sym_identifier,
    ACTIONS(842), 1,
      anon_sym_PIPE,
    ACTIONS(846), 1,
      anon_sym_LPAREN,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(840), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(972), 3,
      sym_supremum,
      sym_infimum,
      sym_number,
    STATE(390), 7,
      sym_string,
      sym__const_term,
      sym_const_binary_operation,
      sym_const_unary_operation,
      sym_const_abs,
      sym_const_function,
      sym_const_tuple,
  [16153] = 6,
    ACTIONS(686), 1,
      anon_sym_LPAREN,
    ACTIONS(694), 1,
      anon_sym_STAR,
    STATE(300), 1,
      sym__arg_pool,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(814), 3,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_SEMI,
    ACTIONS(692), 10,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_DOT_DOT,
      sym_relation,
  [16184] = 6,
    ACTIONS(686), 1,
      anon_sym_LPAREN,
    ACTIONS(694), 1,
      anon_sym_STAR,
    STATE(304), 1,
      sym__arg_pool,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(802), 3,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_SEMI,
    ACTIONS(692), 10,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_DOT_DOT,
      sym_relation,
  [16215] = 8,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(836), 1,
      sym_identifier,
    ACTIONS(842), 1,
      anon_sym_PIPE,
    ACTIONS(846), 1,
      anon_sym_LPAREN,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(840), 2,
      anon_sym_DASH,
      anon_sym_TILDE,
    ACTIONS(974), 3,
      sym_supremum,
      sym_infimum,
      sym_number,
    STATE(391), 7,
      sym_string,
      sym__const_term,
      sym_const_binary_operation,
      sym_const_unary_operation,
      sym_const_abs,
      sym_const_function,
      sym_const_tuple,
  [16250] = 4,
    ACTIONS(860), 1,
      anon_sym_STAR,
    ACTIONS(888), 1,
      anon_sym_STAR_STAR,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(850), 13,
      aux_sym_fstring_accessor_token1,
      aux_sym_fstring_accessor_token2,
      aux_sym_fstring_conversion_token1,
      aux_sym_fstring_spec_token1,
      anon_sym_RBRACE,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_DOT_DOT,
  [16276] = 7,
    ACTIONS(884), 1,
      anon_sym_STAR,
    ACTIONS(888), 1,
      anon_sym_STAR_STAR,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(882), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(886), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(880), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
    ACTIONS(850), 6,
      aux_sym_fstring_accessor_token1,
      aux_sym_fstring_accessor_token2,
      aux_sym_fstring_conversion_token1,
      aux_sym_fstring_spec_token1,
      anon_sym_RBRACE,
      anon_sym_DOT_DOT,
  [16308] = 3,
    ACTIONS(810), 1,
      anon_sym_STAR,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(808), 14,
      aux_sym_fstring_accessor_token1,
      aux_sym_fstring_accessor_token2,
      aux_sym_fstring_conversion_token1,
      aux_sym_fstring_spec_token1,
      anon_sym_RBRACE,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_DOT_DOT,
  [16332] = 10,
    ACTIONS(854), 1,
      anon_sym_STAR,
    ACTIONS(858), 1,
      anon_sym_STAR_STAR,
    ACTIONS(896), 1,
      anon_sym_DOT_DOT,
    ACTIONS(976), 1,
      anon_sym_COMMA2,
    STATE(480), 1,
      aux_sym_terms_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(852), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(856), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(892), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
    ACTIONS(978), 3,
      sym_colon,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [16370] = 3,
    ACTIONS(820), 1,
      anon_sym_STAR,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(818), 14,
      aux_sym_fstring_accessor_token1,
      aux_sym_fstring_accessor_token2,
      aux_sym_fstring_conversion_token1,
      aux_sym_fstring_spec_token1,
      anon_sym_RBRACE,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_DOT_DOT,
  [16394] = 5,
    ACTIONS(982), 1,
      anon_sym_STAR,
    ACTIONS(984), 1,
      anon_sym_LPAREN,
    STATE(341), 1,
      sym__const_arg_pool,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(980), 12,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_DOT,
  [16422] = 9,
    ACTIONS(854), 1,
      anon_sym_STAR,
    ACTIONS(858), 1,
      anon_sym_STAR_STAR,
    ACTIONS(896), 1,
      anon_sym_DOT_DOT,
    ACTIONS(988), 1,
      anon_sym_AT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(852), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(856), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(892), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
    ACTIONS(986), 4,
      sym_colon,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [16458] = 6,
    ACTIONS(884), 1,
      anon_sym_STAR,
    ACTIONS(888), 1,
      anon_sym_STAR_STAR,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(882), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(886), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(850), 9,
      aux_sym_fstring_accessor_token1,
      aux_sym_fstring_accessor_token2,
      aux_sym_fstring_conversion_token1,
      aux_sym_fstring_spec_token1,
      anon_sym_RBRACE,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_DOT_DOT,
  [16488] = 10,
    ACTIONS(866), 1,
      anon_sym_STAR_STAR,
    ACTIONS(904), 1,
      anon_sym_STAR,
    ACTIONS(990), 1,
      anon_sym_DOT_DOT,
    ACTIONS(992), 1,
      sym_relation,
    STATE(481), 1,
      aux_sym_comparison_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(902), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(906), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(894), 3,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_SEMI,
    ACTIONS(908), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
  [16526] = 4,
    ACTIONS(864), 1,
      anon_sym_STAR,
    ACTIONS(888), 1,
      anon_sym_STAR_STAR,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(862), 13,
      aux_sym_fstring_accessor_token1,
      aux_sym_fstring_accessor_token2,
      aux_sym_fstring_conversion_token1,
      aux_sym_fstring_spec_token1,
      anon_sym_RBRACE,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_DOT_DOT,
  [16552] = 4,
    ACTIONS(860), 1,
      anon_sym_STAR,
    ACTIONS(888), 1,
      anon_sym_STAR_STAR,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(850), 13,
      aux_sym_fstring_accessor_token1,
      aux_sym_fstring_accessor_token2,
      aux_sym_fstring_conversion_token1,
      aux_sym_fstring_spec_token1,
      anon_sym_RBRACE,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_DOT_DOT,
  [16578] = 5,
    ACTIONS(884), 1,
      anon_sym_STAR,
    ACTIONS(888), 1,
      anon_sym_STAR_STAR,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(886), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(850), 11,
      aux_sym_fstring_accessor_token1,
      aux_sym_fstring_accessor_token2,
      aux_sym_fstring_conversion_token1,
      aux_sym_fstring_spec_token1,
      anon_sym_RBRACE,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_DOT_DOT,
  [16606] = 9,
    ACTIONS(918), 1,
      anon_sym_STAR,
    ACTIONS(922), 1,
      anon_sym_STAR_STAR,
    ACTIONS(924), 1,
      anon_sym_DOT_DOT,
    ACTIONS(996), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(916), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(920), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(914), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
    ACTIONS(994), 3,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_COLON_DASH,
  [16641] = 6,
    ACTIONS(804), 1,
      anon_sym_LPAREN,
    ACTIONS(998), 1,
      anon_sym_SLASH,
    STATE(271), 1,
      sym__arg_pool,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(694), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(692), 9,
      sym_colon,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_DOT_DOT,
  [16670] = 8,
    ACTIONS(854), 1,
      anon_sym_STAR,
    ACTIONS(858), 1,
      anon_sym_STAR_STAR,
    ACTIONS(896), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(852), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(856), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(892), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
    ACTIONS(1001), 4,
      sym_colon,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [16703] = 8,
    ACTIONS(866), 1,
      anon_sym_STAR_STAR,
    ACTIONS(904), 1,
      anon_sym_STAR,
    ACTIONS(990), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(902), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(906), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(908), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
    ACTIONS(932), 4,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      sym_relation,
  [16736] = 8,
    ACTIONS(854), 1,
      anon_sym_STAR,
    ACTIONS(858), 1,
      anon_sym_STAR_STAR,
    ACTIONS(896), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(852), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(856), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(892), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
    ACTIONS(1003), 4,
      sym_colon,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [16769] = 6,
    ACTIONS(804), 1,
      anon_sym_LPAREN,
    ACTIONS(1005), 1,
      anon_sym_SLASH,
    STATE(271), 1,
      sym__arg_pool,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(694), 2,
      anon_sym_STAR,
      anon_sym_DOT,
    ACTIONS(692), 9,
      sym_colon,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_DOT_DOT,
  [16798] = 9,
    ACTIONS(918), 1,
      anon_sym_STAR,
    ACTIONS(922), 1,
      anon_sym_STAR_STAR,
    ACTIONS(924), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1010), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(916), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(920), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(914), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
    ACTIONS(1008), 3,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_COLON_DASH,
  [16833] = 3,
    ACTIONS(1014), 1,
      anon_sym_STAR,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1012), 12,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_DOT,
  [16855] = 10,
    ACTIONS(866), 1,
      anon_sym_STAR_STAR,
    ACTIONS(904), 1,
      anon_sym_STAR,
    ACTIONS(990), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1016), 1,
      anon_sym_PIPE,
    ACTIONS(1018), 1,
      anon_sym_SEMI,
    STATE(557), 1,
      aux_sym_abs_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(902), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(906), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(908), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
  [16891] = 6,
    ACTIONS(1024), 1,
      sym_theory_operator,
    ACTIONS(1027), 1,
      sym_default_negation,
    STATE(343), 1,
      aux_sym_theory_operators_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1020), 2,
      sym_identifier,
      sym_anonymous,
    ACTIONS(1022), 8,
      anon_sym_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
  [16919] = 9,
    ACTIONS(866), 1,
      anon_sym_STAR_STAR,
    ACTIONS(904), 1,
      anon_sym_STAR,
    ACTIONS(990), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1030), 1,
      anon_sym_COMMA2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(902), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(906), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(1032), 2,
      anon_sym_RPAREN2,
      anon_sym_SEMI,
    ACTIONS(908), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
  [16953] = 4,
    ACTIONS(1036), 1,
      anon_sym_STAR,
    ACTIONS(1038), 1,
      anon_sym_STAR_STAR,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1034), 11,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_DOT,
  [16977] = 3,
    ACTIONS(1042), 1,
      anon_sym_STAR,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1040), 12,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_DOT,
  [16999] = 3,
    ACTIONS(1046), 1,
      anon_sym_STAR,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1044), 12,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_DOT,
  [17021] = 3,
    ACTIONS(1050), 1,
      anon_sym_STAR,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1048), 12,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_DOT,
  [17043] = 3,
    ACTIONS(1054), 1,
      anon_sym_STAR,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1052), 12,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_DOT,
  [17065] = 3,
    ACTIONS(1058), 1,
      anon_sym_STAR,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1056), 12,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_DOT,
  [17087] = 6,
    ACTIONS(1038), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1064), 1,
      anon_sym_STAR,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1062), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(1066), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(1060), 7,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_DOT,
  [17115] = 5,
    ACTIONS(1038), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1064), 1,
      anon_sym_STAR,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1066), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(1060), 9,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_DOT,
  [17141] = 4,
    ACTIONS(1038), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1068), 1,
      anon_sym_STAR,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1060), 11,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_DOT,
  [17165] = 4,
    ACTIONS(1038), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1068), 1,
      anon_sym_STAR,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1060), 11,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_DOT,
  [17189] = 9,
    ACTIONS(866), 1,
      anon_sym_STAR_STAR,
    ACTIONS(904), 1,
      anon_sym_STAR,
    ACTIONS(990), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1072), 1,
      sym_relation,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(902), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(906), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(1070), 2,
      anon_sym_LBRACE,
      sym_aggregate_function,
    ACTIONS(908), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
  [17223] = 3,
    ACTIONS(1076), 1,
      anon_sym_STAR,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1074), 12,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
      anon_sym_PLUS,
      anon_sym_DASH,
      anon_sym_SLASH,
      anon_sym_BSLASH,
      anon_sym_STAR_STAR,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_DOT,
  [17245] = 10,
    ACTIONS(866), 1,
      anon_sym_STAR_STAR,
    ACTIONS(904), 1,
      anon_sym_STAR,
    ACTIONS(978), 1,
      anon_sym_RBRACK,
    ACTIONS(990), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1078), 1,
      anon_sym_COMMA2,
    STATE(585), 1,
      aux_sym_terms_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(902), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(906), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(908), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
  [17281] = 9,
    ACTIONS(866), 1,
      anon_sym_STAR_STAR,
    ACTIONS(904), 1,
      anon_sym_STAR,
    ACTIONS(990), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1080), 1,
      anon_sym_AT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(902), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(906), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(986), 2,
      anon_sym_COMMA2,
      anon_sym_RBRACK,
    ACTIONS(908), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
  [17315] = 6,
    ACTIONS(624), 1,
      sym_default_negation,
    ACTIONS(1086), 1,
      sym_theory_operator,
    STATE(343), 1,
      aux_sym_theory_operators_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1082), 2,
      sym_identifier,
      sym_anonymous,
    ACTIONS(1084), 8,
      anon_sym_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_LPAREN,
      anon_sym_LBRACK,
  [17343] = 10,
    ACTIONS(866), 1,
      anon_sym_STAR_STAR,
    ACTIONS(904), 1,
      anon_sym_STAR,
    ACTIONS(990), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1018), 1,
      anon_sym_SEMI,
    ACTIONS(1088), 1,
      anon_sym_PIPE,
    STATE(622), 1,
      aux_sym_abs_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(902), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(906), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(908), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
  [17379] = 8,
    ACTIONS(866), 1,
      anon_sym_STAR_STAR,
    ACTIONS(904), 1,
      anon_sym_STAR,
    ACTIONS(990), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(902), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(906), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(908), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
    ACTIONS(1030), 3,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_SEMI,
  [17411] = 10,
    ACTIONS(866), 1,
      anon_sym_STAR_STAR,
    ACTIONS(904), 1,
      anon_sym_STAR,
    ACTIONS(990), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1018), 1,
      anon_sym_SEMI,
    ACTIONS(1090), 1,
      anon_sym_PIPE,
    STATE(632), 1,
      aux_sym_abs_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(902), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(906), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(908), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
  [17447] = 8,
    ACTIONS(866), 1,
      anon_sym_STAR_STAR,
    ACTIONS(904), 1,
      anon_sym_STAR,
    ACTIONS(990), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(902), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(906), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(1092), 2,
      anon_sym_RPAREN2,
      anon_sym_SEMI,
    ACTIONS(908), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
  [17478] = 9,
    ACTIONS(1038), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1064), 1,
      anon_sym_STAR,
    ACTIONS(1096), 1,
      anon_sym_COMMA2,
    ACTIONS(1098), 1,
      anon_sym_RPAREN2,
    STATE(629), 1,
      aux_sym_const_terms_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1062), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(1066), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(1094), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
  [17511] = 8,
    ACTIONS(866), 1,
      anon_sym_STAR_STAR,
    ACTIONS(904), 1,
      anon_sym_STAR,
    ACTIONS(990), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(902), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(906), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(1008), 2,
      anon_sym_COMMA2,
      anon_sym_SEMI,
    ACTIONS(908), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
  [17542] = 8,
    ACTIONS(618), 1,
      sym_theory_operator,
    ACTIONS(1104), 1,
      sym_default_negation,
    STATE(317), 1,
      sym_theory_operators,
    STATE(359), 1,
      aux_sym_theory_operators_repeat1,
    STATE(368), 1,
      aux_sym_theory_unparsed_term_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1102), 3,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
    ACTIONS(1100), 4,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_RBRACK,
      anon_sym_RBRACE2,
  [17573] = 8,
    ACTIONS(618), 1,
      sym_theory_operator,
    ACTIONS(1104), 1,
      sym_default_negation,
    STATE(317), 1,
      sym_theory_operators,
    STATE(359), 1,
      aux_sym_theory_operators_repeat1,
    STATE(369), 1,
      aux_sym_theory_unparsed_term_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1108), 3,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
    ACTIONS(1106), 4,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_RBRACK,
      anon_sym_RBRACE2,
  [17604] = 8,
    ACTIONS(618), 1,
      sym_theory_operator,
    ACTIONS(1104), 1,
      sym_default_negation,
    STATE(317), 1,
      sym_theory_operators,
    STATE(359), 1,
      aux_sym_theory_operators_repeat1,
    STATE(369), 1,
      aux_sym_theory_unparsed_term_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1112), 3,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
    ACTIONS(1110), 4,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_RBRACK,
      anon_sym_RBRACE2,
  [17635] = 8,
    ACTIONS(1118), 1,
      sym_theory_operator,
    ACTIONS(1121), 1,
      sym_default_negation,
    STATE(317), 1,
      sym_theory_operators,
    STATE(359), 1,
      aux_sym_theory_operators_repeat1,
    STATE(369), 1,
      aux_sym_theory_unparsed_term_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1116), 3,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
    ACTIONS(1114), 4,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_RBRACK,
      anon_sym_RBRACE2,
  [17666] = 8,
    ACTIONS(866), 1,
      anon_sym_STAR_STAR,
    ACTIONS(904), 1,
      anon_sym_STAR,
    ACTIONS(990), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(902), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(906), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(1003), 2,
      anon_sym_COMMA2,
      anon_sym_RBRACK,
    ACTIONS(908), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
  [17697] = 8,
    ACTIONS(866), 1,
      anon_sym_STAR_STAR,
    ACTIONS(904), 1,
      anon_sym_STAR,
    ACTIONS(990), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(902), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(906), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(1001), 2,
      anon_sym_COMMA2,
      anon_sym_RBRACK,
    ACTIONS(908), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
  [17728] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1124), 3,
      sym_identifier,
      sym_anonymous,
      sym_default_negation,
    ACTIONS(1126), 9,
      anon_sym_DQUOTE,
      anon_sym_LBRACE,
      sym_supremum,
      sym_infimum,
      sym_number,
      sym_variable,
      anon_sym_LPAREN,
      sym_theory_operator,
      anon_sym_LBRACK,
  [17749] = 9,
    ACTIONS(1038), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1064), 1,
      anon_sym_STAR,
    ACTIONS(1128), 1,
      anon_sym_COMMA2,
    ACTIONS(1130), 1,
      anon_sym_RPAREN2,
    STATE(552), 1,
      aux_sym_const_terms_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1062), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(1066), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(1094), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
  [17782] = 8,
    ACTIONS(866), 1,
      anon_sym_STAR_STAR,
    ACTIONS(904), 1,
      anon_sym_STAR,
    ACTIONS(990), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(902), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(906), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(1132), 2,
      anon_sym_PIPE,
      anon_sym_SEMI,
    ACTIONS(908), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
  [17813] = 9,
    ACTIONS(854), 1,
      anon_sym_STAR,
    ACTIONS(858), 1,
      anon_sym_STAR_STAR,
    ACTIONS(896), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1134), 1,
      anon_sym_DOT,
    ACTIONS(1136), 1,
      sym_colon,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(852), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(856), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(892), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
  [17846] = 8,
    ACTIONS(866), 1,
      anon_sym_STAR_STAR,
    ACTIONS(904), 1,
      anon_sym_STAR,
    ACTIONS(990), 1,
      anon_sym_DOT_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(902), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(906), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(994), 2,
      anon_sym_COMMA2,
      anon_sym_SEMI,
    ACTIONS(908), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
  [17877] = 8,
    ACTIONS(866), 1,
      anon_sym_STAR_STAR,
    ACTIONS(904), 1,
      anon_sym_STAR,
    ACTIONS(990), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1138), 1,
      anon_sym_RBRACK,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(902), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(906), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(908), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
  [17907] = 8,
    ACTIONS(866), 1,
      anon_sym_STAR_STAR,
    ACTIONS(904), 1,
      anon_sym_STAR,
    ACTIONS(990), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1140), 1,
      anon_sym_RBRACK,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(902), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(906), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(908), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
  [17937] = 8,
    ACTIONS(866), 1,
      anon_sym_STAR_STAR,
    ACTIONS(904), 1,
      anon_sym_STAR,
    ACTIONS(990), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1142), 1,
      anon_sym_RBRACK,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(902), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(906), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(908), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
  [17967] = 8,
    ACTIONS(866), 1,
      anon_sym_STAR_STAR,
    ACTIONS(904), 1,
      anon_sym_STAR,
    ACTIONS(990), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1144), 1,
      anon_sym_RBRACK,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(902), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(906), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(908), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
  [17997] = 8,
    ACTIONS(866), 1,
      anon_sym_STAR_STAR,
    ACTIONS(904), 1,
      anon_sym_STAR,
    ACTIONS(990), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1146), 1,
      anon_sym_RBRACK,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(902), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(906), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(908), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
  [18027] = 8,
    ACTIONS(866), 1,
      anon_sym_STAR_STAR,
    ACTIONS(904), 1,
      anon_sym_STAR,
    ACTIONS(990), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1148), 1,
      sym_relation,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(902), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(906), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(908), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
  [18057] = 8,
    ACTIONS(866), 1,
      anon_sym_STAR_STAR,
    ACTIONS(904), 1,
      anon_sym_STAR,
    ACTIONS(990), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1150), 1,
      anon_sym_COMMA2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(902), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(906), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(908), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
  [18087] = 8,
    ACTIONS(866), 1,
      anon_sym_STAR_STAR,
    ACTIONS(904), 1,
      anon_sym_STAR,
    ACTIONS(990), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1152), 1,
      sym_relation,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(902), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(906), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(908), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
  [18117] = 7,
    ACTIONS(1038), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1064), 1,
      anon_sym_STAR,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1062), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(1066), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(1154), 2,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
    ACTIONS(1094), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
  [18145] = 5,
    ACTIONS(1158), 1,
      anon_sym_LPAREN,
    STATE(396), 1,
      sym__theory_arguments,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1160), 3,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
    ACTIONS(1156), 6,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      sym_theory_operator,
      anon_sym_RBRACK,
      anon_sym_RBRACE2,
      sym_default_negation,
  [18169] = 8,
    ACTIONS(866), 1,
      anon_sym_STAR_STAR,
    ACTIONS(904), 1,
      anon_sym_STAR,
    ACTIONS(990), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1162), 1,
      anon_sym_RBRACK,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(902), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(906), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(908), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
  [18199] = 8,
    ACTIONS(866), 1,
      anon_sym_STAR_STAR,
    ACTIONS(904), 1,
      anon_sym_STAR,
    ACTIONS(990), 1,
      anon_sym_DOT_DOT,
    ACTIONS(1164), 1,
      sym_relation,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(902), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(906), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(908), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
  [18229] = 9,
    ACTIONS(1166), 1,
      anon_sym_LBRACE,
    ACTIONS(1168), 1,
      anon_sym_COMMA2,
    ACTIONS(1170), 1,
      anon_sym_LPAREN,
    ACTIONS(1174), 1,
      sym_theory_operator,
    ACTIONS(1176), 1,
      sym_default_negation,
    STATE(421), 1,
      sym__arg_pool,
    STATE(540), 1,
      sym_theory_atom_upper,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1172), 3,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [18260] = 7,
    ACTIONS(1038), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1064), 1,
      anon_sym_STAR,
    ACTIONS(1178), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1062), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(1066), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(1094), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
  [18287] = 7,
    ACTIONS(1038), 1,
      anon_sym_STAR_STAR,
    ACTIONS(1064), 1,
      anon_sym_STAR,
    ACTIONS(1180), 1,
      anon_sym_PIPE,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1062), 2,
      anon_sym_PLUS,
      anon_sym_DASH,
    ACTIONS(1066), 2,
      anon_sym_SLASH,
      anon_sym_BSLASH,
    ACTIONS(1094), 3,
      anon_sym_CARET,
      anon_sym_QMARK,
      anon_sym_AMP,
  [18314] = 7,
    ACTIONS(1182), 1,
      anon_sym_STAR_PERCENT,
    ACTIONS(1186), 1,
      sym__doc_token_args,
    STATE(420), 1,
      aux_sym_doc_desc_repeat1,
    STATE(578), 1,
      sym_doc_desc,
    STATE(821), 1,
      sym_doc_args,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1184), 5,
      sym_doc_fragment_string,
      sym_doc_fragment_emph,
      sym_doc_fragment_bold,
      sym_doc_fragment_italic,
      sym_doc_fragment_code,
  [18341] = 8,
    ACTIONS(618), 1,
      sym_theory_operator,
    ACTIONS(1102), 1,
      anon_sym_SEMI,
    ACTIONS(1104), 1,
      sym_default_negation,
    STATE(307), 1,
      sym_theory_operators,
    STATE(359), 1,
      aux_sym_theory_operators_repeat1,
    STATE(401), 1,
      aux_sym_theory_unparsed_term_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1100), 3,
      sym_colon,
      anon_sym_COMMA2,
      anon_sym_RBRACE2,
  [18369] = 4,
    ACTIONS(1188), 1,
      sym_relation,
    STATE(394), 1,
      aux_sym_comparison_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(932), 7,
      sym_colon,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [18389] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1193), 3,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
    ACTIONS(1191), 6,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      sym_theory_operator,
      anon_sym_RBRACK,
      anon_sym_RBRACE2,
      sym_default_negation,
  [18407] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1197), 3,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
    ACTIONS(1195), 6,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      sym_theory_operator,
      anon_sym_RBRACK,
      anon_sym_RBRACE2,
      sym_default_negation,
  [18425] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1201), 3,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
    ACTIONS(1199), 6,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      sym_theory_operator,
      anon_sym_RBRACK,
      anon_sym_RBRACE2,
      sym_default_negation,
  [18443] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1205), 3,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
    ACTIONS(1203), 6,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      sym_theory_operator,
      anon_sym_RBRACK,
      anon_sym_RBRACE2,
      sym_default_negation,
  [18461] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1209), 3,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
    ACTIONS(1207), 6,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      sym_theory_operator,
      anon_sym_RBRACK,
      anon_sym_RBRACE2,
      sym_default_negation,
  [18479] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1116), 3,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
    ACTIONS(1114), 6,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      sym_theory_operator,
      anon_sym_RBRACK,
      anon_sym_RBRACE2,
      sym_default_negation,
  [18497] = 8,
    ACTIONS(618), 1,
      sym_theory_operator,
    ACTIONS(1104), 1,
      sym_default_negation,
    ACTIONS(1112), 1,
      anon_sym_SEMI,
    STATE(307), 1,
      sym_theory_operators,
    STATE(359), 1,
      aux_sym_theory_operators_repeat1,
    STATE(407), 1,
      aux_sym_theory_unparsed_term_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1110), 3,
      sym_colon,
      anon_sym_COMMA2,
      anon_sym_RBRACE2,
  [18525] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1213), 3,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
    ACTIONS(1211), 6,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      sym_theory_operator,
      anon_sym_RBRACK,
      anon_sym_RBRACE2,
      sym_default_negation,
  [18543] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1217), 3,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
    ACTIONS(1215), 6,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      sym_theory_operator,
      anon_sym_RBRACK,
      anon_sym_RBRACE2,
      sym_default_negation,
  [18561] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1221), 3,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
    ACTIONS(1219), 6,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      sym_theory_operator,
      anon_sym_RBRACK,
      anon_sym_RBRACE2,
      sym_default_negation,
  [18579] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1225), 3,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
    ACTIONS(1223), 6,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      sym_theory_operator,
      anon_sym_RBRACK,
      anon_sym_RBRACE2,
      sym_default_negation,
  [18597] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1229), 3,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
    ACTIONS(1227), 6,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      sym_theory_operator,
      anon_sym_RBRACK,
      anon_sym_RBRACE2,
      sym_default_negation,
  [18615] = 8,
    ACTIONS(1116), 1,
      anon_sym_SEMI,
    ACTIONS(1118), 1,
      sym_theory_operator,
    ACTIONS(1121), 1,
      sym_default_negation,
    STATE(307), 1,
      sym_theory_operators,
    STATE(359), 1,
      aux_sym_theory_operators_repeat1,
    STATE(407), 1,
      aux_sym_theory_unparsed_term_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1114), 3,
      sym_colon,
      anon_sym_COMMA2,
      anon_sym_RBRACE2,
  [18643] = 8,
    ACTIONS(618), 1,
      sym_theory_operator,
    ACTIONS(1104), 1,
      sym_default_negation,
    ACTIONS(1108), 1,
      anon_sym_SEMI,
    STATE(307), 1,
      sym_theory_operators,
    STATE(359), 1,
      aux_sym_theory_operators_repeat1,
    STATE(407), 1,
      aux_sym_theory_unparsed_term_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1106), 3,
      sym_colon,
      anon_sym_COMMA2,
      anon_sym_RBRACE2,
  [18671] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(820), 3,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
    ACTIONS(818), 6,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      sym_theory_operator,
      anon_sym_RBRACK,
      anon_sym_RBRACE2,
      sym_default_negation,
  [18689] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(810), 3,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
    ACTIONS(808), 6,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      sym_theory_operator,
      anon_sym_RBRACK,
      anon_sym_RBRACE2,
      sym_default_negation,
  [18707] = 5,
    STATE(419), 1,
      aux_sym_doc_desc_repeat1,
    STATE(644), 1,
      sym_doc_desc,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1231), 2,
      sym__doc_token_minus,
      anon_sym_STAR_PERCENT,
    ACTIONS(1233), 5,
      sym_doc_fragment_string,
      sym_doc_fragment_emph,
      sym_doc_fragment_bold,
      sym_doc_fragment_italic,
      sym_doc_fragment_code,
  [18729] = 4,
    ACTIONS(898), 1,
      sym_relation,
    STATE(394), 1,
      aux_sym_comparison_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1235), 7,
      sym_colon,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [18749] = 4,
    STATE(413), 1,
      aux_sym_doc_desc_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1237), 2,
      sym__doc_token_args,
      anon_sym_STAR_PERCENT,
    ACTIONS(1239), 5,
      sym_doc_fragment_string,
      sym_doc_fragment_emph,
      sym_doc_fragment_bold,
      sym_doc_fragment_italic,
      sym_doc_fragment_code,
  [18768] = 10,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(1242), 1,
      aux_sym_fstring_accessor_token3,
    ACTIONS(1244), 1,
      sym_fstring_align,
    ACTIONS(1246), 1,
      sym_fstring_sign,
    ACTIONS(1248), 1,
      sym_fstring_alternate,
    ACTIONS(1250), 1,
      sym_fstring_grouping,
    ACTIONS(1252), 1,
      sym_fstring_type,
    ACTIONS(1254), 1,
      anon_sym_RBRACE,
    STATE(577), 1,
      sym_fstring_width,
  [18799] = 4,
    ACTIONS(926), 1,
      sym_relation,
    STATE(417), 1,
      aux_sym_comparison_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1235), 6,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [18818] = 5,
    ACTIONS(1160), 1,
      anon_sym_SEMI,
    ACTIONS(1256), 1,
      anon_sym_LPAREN,
    STATE(468), 1,
      sym__theory_arguments,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1156), 5,
      sym_colon,
      anon_sym_COMMA2,
      sym_theory_operator,
      anon_sym_RBRACE2,
      sym_default_negation,
  [18839] = 4,
    ACTIONS(1258), 1,
      sym_relation,
    STATE(417), 1,
      aux_sym_comparison_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(932), 6,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [18858] = 4,
    STATE(418), 1,
      aux_sym_doc_desc_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1237), 2,
      sym__doc_token_minus,
      anon_sym_STAR_PERCENT,
    ACTIONS(1261), 5,
      sym_doc_fragment_string,
      sym_doc_fragment_emph,
      sym_doc_fragment_bold,
      sym_doc_fragment_italic,
      sym_doc_fragment_code,
  [18877] = 4,
    STATE(418), 1,
      aux_sym_doc_desc_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1264), 2,
      sym__doc_token_minus,
      anon_sym_STAR_PERCENT,
    ACTIONS(1266), 5,
      sym_doc_fragment_string,
      sym_doc_fragment_emph,
      sym_doc_fragment_bold,
      sym_doc_fragment_italic,
      sym_doc_fragment_code,
  [18896] = 4,
    STATE(413), 1,
      aux_sym_doc_desc_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1264), 2,
      sym__doc_token_args,
      anon_sym_STAR_PERCENT,
    ACTIONS(1268), 5,
      sym_doc_fragment_string,
      sym_doc_fragment_emph,
      sym_doc_fragment_bold,
      sym_doc_fragment_italic,
      sym_doc_fragment_code,
  [18915] = 7,
    ACTIONS(1174), 1,
      sym_theory_operator,
    ACTIONS(1176), 1,
      sym_default_negation,
    ACTIONS(1270), 1,
      anon_sym_LBRACE,
    ACTIONS(1272), 1,
      anon_sym_COMMA2,
    STATE(505), 1,
      sym_theory_atom_upper,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1274), 3,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [18940] = 3,
    ACTIONS(1278), 1,
      sym__doc_token_paren,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1276), 7,
      sym_doc_fragment_string,
      sym__doc_token_args,
      anon_sym_STAR_PERCENT,
      sym_doc_fragment_emph,
      sym_doc_fragment_bold,
      sym_doc_fragment_italic,
      sym_doc_fragment_code,
  [18957] = 6,
    ACTIONS(1174), 1,
      sym_theory_operator,
    ACTIONS(1176), 1,
      sym_default_negation,
    ACTIONS(1280), 1,
      anon_sym_COMMA2,
    STATE(511), 1,
      sym_theory_atom_upper,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1282), 3,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [18979] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1100), 7,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_SEMI,
      anon_sym_RBRACK,
      anon_sym_RBRACE2,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [18993] = 9,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(1284), 1,
      aux_sym_fstring_accessor_token3,
    ACTIONS(1286), 1,
      sym_fstring_sign,
    ACTIONS(1288), 1,
      sym_fstring_alternate,
    ACTIONS(1290), 1,
      sym_fstring_grouping,
    ACTIONS(1292), 1,
      sym_fstring_type,
    ACTIONS(1294), 1,
      anon_sym_RBRACE,
    STATE(595), 1,
      sym_fstring_width,
  [19021] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1296), 7,
      sym_colon,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [19035] = 4,
    ACTIONS(1300), 1,
      anon_sym_COMMA2,
    STATE(439), 1,
      aux_sym_condition_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1298), 5,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [19053] = 6,
    ACTIONS(1174), 1,
      sym_theory_operator,
    ACTIONS(1176), 1,
      sym_default_negation,
    ACTIONS(1302), 1,
      anon_sym_COMMA2,
    STATE(524), 1,
      sym_theory_atom_upper,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1304), 3,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [19075] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1306), 7,
      sym_doc_fragment_string,
      sym__doc_token_args,
      anon_sym_STAR_PERCENT,
      sym_doc_fragment_emph,
      sym_doc_fragment_bold,
      sym_doc_fragment_italic,
      sym_doc_fragment_code,
  [19089] = 6,
    ACTIONS(1174), 1,
      sym_theory_operator,
    ACTIONS(1176), 1,
      sym_default_negation,
    ACTIONS(1308), 1,
      anon_sym_COMMA2,
    STATE(529), 1,
      sym_theory_atom_upper,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1310), 3,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [19111] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1312), 7,
      sym_doc_fragment_string,
      sym__doc_token_args,
      anon_sym_STAR_PERCENT,
      sym_doc_fragment_emph,
      sym_doc_fragment_bold,
      sym_doc_fragment_italic,
      sym_doc_fragment_code,
  [19125] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(774), 3,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
    ACTIONS(772), 4,
      anon_sym_LBRACE,
      anon_sym_COMMA2,
      sym_theory_operator,
      sym_default_negation,
  [19141] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1314), 7,
      sym_colon,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [19155] = 6,
    ACTIONS(1174), 1,
      sym_theory_operator,
    ACTIONS(1176), 1,
      sym_default_negation,
    ACTIONS(1316), 1,
      anon_sym_COMMA2,
    STATE(541), 1,
      sym_theory_atom_upper,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1318), 3,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [19177] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1320), 7,
      sym_colon,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [19191] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(770), 3,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
    ACTIONS(768), 4,
      anon_sym_LBRACE,
      anon_sym_COMMA2,
      sym_theory_operator,
      sym_default_negation,
  [19207] = 4,
    ACTIONS(1300), 1,
      anon_sym_COMMA2,
    STATE(427), 1,
      aux_sym_condition_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1322), 5,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [19225] = 3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(782), 3,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
    ACTIONS(780), 4,
      anon_sym_LBRACE,
      anon_sym_COMMA2,
      sym_theory_operator,
      sym_default_negation,
  [19241] = 4,
    ACTIONS(1326), 1,
      anon_sym_COMMA2,
    STATE(439), 1,
      aux_sym_condition_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1324), 5,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [19259] = 4,
    ACTIONS(1333), 1,
      sym_colon,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1331), 2,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
    ACTIONS(1329), 3,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_SEMI,
  [19276] = 3,
    ACTIONS(1205), 1,
      anon_sym_SEMI,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1203), 5,
      sym_colon,
      anon_sym_COMMA2,
      sym_theory_operator,
      anon_sym_RBRACE2,
      sym_default_negation,
  [19291] = 3,
    ACTIONS(1209), 1,
      anon_sym_SEMI,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1207), 5,
      sym_colon,
      anon_sym_COMMA2,
      sym_theory_operator,
      anon_sym_RBRACE2,
      sym_default_negation,
  [19306] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1324), 6,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [19319] = 3,
    ACTIONS(1116), 1,
      anon_sym_SEMI,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1114), 5,
      sym_colon,
      anon_sym_COMMA2,
      sym_theory_operator,
      anon_sym_RBRACE2,
      sym_default_negation,
  [19334] = 7,
    ACTIONS(1284), 1,
      aux_sym_fstring_accessor_token3,
    ACTIONS(1335), 1,
      sym_fstring_alternate,
    ACTIONS(1337), 1,
      sym_fstring_grouping,
    ACTIONS(1339), 1,
      sym_fstring_type,
    ACTIONS(1341), 1,
      anon_sym_RBRACE,
    STATE(548), 1,
      sym_fstring_width,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [19357] = 3,
    ACTIONS(1213), 1,
      anon_sym_SEMI,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1211), 5,
      sym_colon,
      anon_sym_COMMA2,
      sym_theory_operator,
      anon_sym_RBRACE2,
      sym_default_negation,
  [19372] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1296), 6,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [19385] = 3,
    ACTIONS(820), 1,
      anon_sym_SEMI,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(818), 5,
      sym_colon,
      anon_sym_COMMA2,
      sym_theory_operator,
      anon_sym_RBRACE2,
      sym_default_negation,
  [19400] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1320), 6,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [19413] = 3,
    ACTIONS(1229), 1,
      anon_sym_SEMI,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1227), 5,
      sym_colon,
      anon_sym_COMMA2,
      sym_theory_operator,
      anon_sym_RBRACE2,
      sym_default_negation,
  [19428] = 3,
    ACTIONS(1217), 1,
      anon_sym_SEMI,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1215), 5,
      sym_colon,
      anon_sym_COMMA2,
      sym_theory_operator,
      anon_sym_RBRACE2,
      sym_default_negation,
  [19443] = 3,
    ACTIONS(810), 1,
      anon_sym_SEMI,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(808), 5,
      sym_colon,
      anon_sym_COMMA2,
      sym_theory_operator,
      anon_sym_RBRACE2,
      sym_default_negation,
  [19458] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1314), 6,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [19471] = 6,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(1343), 1,
      anon_sym_DQUOTE,
    ACTIONS(1347), 1,
      anon_sym_LBRACE,
    ACTIONS(1345), 2,
      sym__fstring_fragment,
      sym__fstring_escape,
    STATE(463), 2,
      sym_fstring_field,
      aux_sym_fstring_repeat1,
  [19492] = 3,
    ACTIONS(1221), 1,
      anon_sym_SEMI,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1219), 5,
      sym_colon,
      anon_sym_COMMA2,
      sym_theory_operator,
      anon_sym_RBRACE2,
      sym_default_negation,
  [19507] = 3,
    ACTIONS(1225), 1,
      anon_sym_SEMI,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1223), 5,
      sym_colon,
      anon_sym_COMMA2,
      sym_theory_operator,
      anon_sym_RBRACE2,
      sym_default_negation,
  [19522] = 6,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(1349), 1,
      anon_sym_DQUOTE,
    ACTIONS(1354), 1,
      anon_sym_LBRACE,
    ACTIONS(1351), 2,
      sym__fstring_fragment,
      sym__fstring_escape,
    STATE(457), 2,
      sym_fstring_field,
      aux_sym_fstring_repeat1,
  [19543] = 6,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(1347), 1,
      anon_sym_LBRACE,
    ACTIONS(1357), 1,
      anon_sym_DQUOTE,
    ACTIONS(1359), 2,
      sym__fstring_fragment,
      sym__fstring_escape,
    STATE(459), 2,
      sym_fstring_field,
      aux_sym_fstring_repeat1,
  [19564] = 6,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(1347), 1,
      anon_sym_LBRACE,
    ACTIONS(1361), 1,
      anon_sym_DQUOTE,
    ACTIONS(1363), 2,
      sym__fstring_fragment,
      sym__fstring_escape,
    STATE(457), 2,
      sym_fstring_field,
      aux_sym_fstring_repeat1,
  [19585] = 7,
    ACTIONS(1284), 1,
      aux_sym_fstring_accessor_token3,
    ACTIONS(1288), 1,
      sym_fstring_alternate,
    ACTIONS(1290), 1,
      sym_fstring_grouping,
    ACTIONS(1292), 1,
      sym_fstring_type,
    ACTIONS(1294), 1,
      anon_sym_RBRACE,
    STATE(595), 1,
      sym_fstring_width,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [19608] = 4,
    ACTIONS(1333), 1,
      sym_colon,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1365), 2,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
    ACTIONS(1329), 3,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_SEMI,
  [19625] = 5,
    ACTIONS(1367), 1,
      aux_sym_fstring_accessor_token1,
    ACTIONS(1370), 1,
      aux_sym_fstring_accessor_token2,
    STATE(462), 1,
      aux_sym_fstring_accessor_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1373), 3,
      aux_sym_fstring_conversion_token1,
      aux_sym_fstring_spec_token1,
      anon_sym_RBRACE,
  [19644] = 6,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(1347), 1,
      anon_sym_LBRACE,
    ACTIONS(1375), 1,
      anon_sym_DQUOTE,
    ACTIONS(1363), 2,
      sym__fstring_fragment,
      sym__fstring_escape,
    STATE(457), 2,
      sym_fstring_field,
      aux_sym_fstring_repeat1,
  [19665] = 6,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(1347), 1,
      anon_sym_LBRACE,
    ACTIONS(1377), 1,
      anon_sym_DQUOTE,
    ACTIONS(1379), 2,
      sym__fstring_fragment,
      sym__fstring_escape,
    STATE(466), 2,
      sym_fstring_field,
      aux_sym_fstring_repeat1,
  [19686] = 5,
    ACTIONS(872), 1,
      aux_sym_fstring_accessor_token2,
    ACTIONS(1381), 1,
      aux_sym_fstring_accessor_token1,
    STATE(462), 1,
      aux_sym_fstring_accessor_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1383), 3,
      aux_sym_fstring_conversion_token1,
      aux_sym_fstring_spec_token1,
      anon_sym_RBRACE,
  [19705] = 6,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(1347), 1,
      anon_sym_LBRACE,
    ACTIONS(1385), 1,
      anon_sym_DQUOTE,
    ACTIONS(1363), 2,
      sym__fstring_fragment,
      sym__fstring_escape,
    STATE(457), 2,
      sym_fstring_field,
      aux_sym_fstring_repeat1,
  [19726] = 3,
    ACTIONS(1193), 1,
      anon_sym_SEMI,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1191), 5,
      sym_colon,
      anon_sym_COMMA2,
      sym_theory_operator,
      anon_sym_RBRACE2,
      sym_default_negation,
  [19741] = 3,
    ACTIONS(1197), 1,
      anon_sym_SEMI,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1195), 5,
      sym_colon,
      anon_sym_COMMA2,
      sym_theory_operator,
      anon_sym_RBRACE2,
      sym_default_negation,
  [19756] = 3,
    ACTIONS(1201), 1,
      anon_sym_SEMI,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1199), 5,
      sym_colon,
      anon_sym_COMMA2,
      sym_theory_operator,
      anon_sym_RBRACE2,
      sym_default_negation,
  [19771] = 5,
    ACTIONS(804), 1,
      anon_sym_LPAREN,
    ACTIONS(1387), 1,
      anon_sym_SLASH,
    STATE(676), 1,
      sym__arg_pool,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(814), 2,
      sym_colon,
      anon_sym_DOT,
  [19789] = 6,
    ACTIONS(1284), 1,
      aux_sym_fstring_accessor_token3,
    ACTIONS(1337), 1,
      sym_fstring_grouping,
    ACTIONS(1339), 1,
      sym_fstring_type,
    ACTIONS(1341), 1,
      anon_sym_RBRACE,
    STATE(548), 1,
      sym_fstring_width,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [19809] = 6,
    ACTIONS(1284), 1,
      aux_sym_fstring_accessor_token3,
    ACTIONS(1290), 1,
      sym_fstring_grouping,
    ACTIONS(1292), 1,
      sym_fstring_type,
    ACTIONS(1294), 1,
      anon_sym_RBRACE,
    STATE(595), 1,
      sym_fstring_width,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [19829] = 4,
    ACTIONS(1389), 1,
      anon_sym_COMMA2,
    STATE(475), 1,
      aux_sym_theory_terms_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1391), 3,
      sym_colon,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [19845] = 4,
    ACTIONS(1393), 1,
      anon_sym_COMMA2,
    STATE(477), 1,
      aux_sym_theory_terms_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1391), 3,
      anon_sym_RPAREN2,
      anon_sym_RBRACK,
      anon_sym_RBRACE2,
  [19861] = 4,
    ACTIONS(1395), 1,
      anon_sym_COMMA2,
    STATE(475), 1,
      aux_sym_theory_terms_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1398), 3,
      sym_colon,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [19877] = 6,
    ACTIONS(1400), 1,
      sym_identifier,
    ACTIONS(1402), 1,
      anon_sym_AMP,
    ACTIONS(1404), 1,
      anon_sym_RBRACE2,
    STATE(509), 1,
      sym_theory_term_definition,
    STATE(636), 1,
      sym_theory_atom_definition,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [19897] = 4,
    ACTIONS(1406), 1,
      anon_sym_COMMA2,
    STATE(477), 1,
      aux_sym_theory_terms_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1398), 3,
      anon_sym_RPAREN2,
      anon_sym_RBRACK,
      anon_sym_RBRACE2,
  [19913] = 4,
    ACTIONS(1389), 1,
      anon_sym_COMMA2,
    STATE(473), 1,
      aux_sym_theory_terms_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1409), 3,
      sym_colon,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [19929] = 6,
    ACTIONS(1411), 1,
      sym_theory_operator,
    ACTIONS(1413), 1,
      anon_sym_RBRACE2,
    ACTIONS(1415), 1,
      sym_default_negation,
    STATE(543), 1,
      sym_theory_operator_definition,
    STATE(733), 1,
      sym_theory_operator_definitions,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [19949] = 4,
    ACTIONS(976), 1,
      anon_sym_COMMA2,
    STATE(483), 1,
      aux_sym_terms_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1417), 3,
      sym_colon,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [19965] = 4,
    ACTIONS(992), 1,
      sym_relation,
    STATE(482), 1,
      aux_sym_comparison_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1235), 3,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_SEMI,
  [19981] = 4,
    ACTIONS(1419), 1,
      sym_relation,
    STATE(482), 1,
      aux_sym_comparison_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(932), 3,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_SEMI,
  [19997] = 4,
    ACTIONS(1422), 1,
      anon_sym_COMMA2,
    STATE(483), 1,
      aux_sym_terms_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1003), 3,
      sym_colon,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [20013] = 6,
    ACTIONS(874), 1,
      aux_sym_fstring_conversion_token1,
    ACTIONS(876), 1,
      aux_sym_fstring_spec_token1,
    ACTIONS(1425), 1,
      anon_sym_RBRACE,
    STATE(542), 1,
      sym_fstring_conversion,
    STATE(796), 1,
      sym_fstring_spec,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [20033] = 6,
    ACTIONS(1284), 1,
      aux_sym_fstring_accessor_token3,
    ACTIONS(1427), 1,
      sym_fstring_grouping,
    ACTIONS(1429), 1,
      sym_fstring_type,
    ACTIONS(1431), 1,
      anon_sym_RBRACE,
    STATE(563), 1,
      sym_fstring_width,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [20053] = 5,
    ACTIONS(804), 1,
      anon_sym_LPAREN,
    ACTIONS(1433), 1,
      anon_sym_SLASH,
    STATE(703), 1,
      sym__arg_pool,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(802), 2,
      sym_colon,
      anon_sym_DOT,
  [20071] = 4,
    ACTIONS(1393), 1,
      anon_sym_COMMA2,
    STATE(474), 1,
      aux_sym_theory_terms_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1409), 3,
      anon_sym_RPAREN2,
      anon_sym_RBRACK,
      anon_sym_RBRACE2,
  [20087] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1435), 5,
      aux_sym_fstring_accessor_token1,
      aux_sym_fstring_accessor_token2,
      aux_sym_fstring_conversion_token1,
      aux_sym_fstring_spec_token1,
      anon_sym_RBRACE,
  [20099] = 4,
    ACTIONS(1437), 1,
      anon_sym_COMMA2,
    STATE(508), 1,
      aux_sym__terms_or_projection_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1439), 2,
      anon_sym_RPAREN2,
      anon_sym_SEMI,
  [20114] = 3,
    ACTIONS(1320), 1,
      sym_colon,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1441), 3,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_DOT,
  [20127] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1398), 4,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_RBRACK,
      anon_sym_RBRACE2,
  [20138] = 4,
    ACTIONS(804), 1,
      anon_sym_LPAREN,
    STATE(703), 1,
      sym__arg_pool,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(802), 2,
      sym_colon,
      anon_sym_DOT,
  [20153] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1443), 4,
      anon_sym_PIPE,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [20164] = 3,
    ACTIONS(1445), 1,
      sym_colon,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1329), 3,
      anon_sym_PIPE,
      anon_sym_COMMA2,
      anon_sym_SEMI,
  [20177] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1447), 4,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [20188] = 3,
    ACTIONS(1314), 1,
      sym_colon,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1449), 3,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_DOT,
  [20201] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1100), 4,
      sym_colon,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [20212] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1398), 4,
      sym_colon,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [20223] = 5,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(1451), 1,
      anon_sym_DQUOTE,
    STATE(499), 1,
      aux_sym_string_repeat1,
    ACTIONS(1453), 2,
      sym__string_fragment,
      sym__string_escape,
  [20240] = 4,
    ACTIONS(1456), 1,
      anon_sym_COMMA2,
    STATE(508), 1,
      aux_sym__terms_or_projection_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1458), 2,
      anon_sym_RPAREN2,
      anon_sym_SEMI,
  [20255] = 5,
    ACTIONS(1400), 1,
      sym_identifier,
    ACTIONS(1402), 1,
      anon_sym_AMP,
    STATE(669), 1,
      sym_theory_term_definition,
    STATE(675), 1,
      sym_theory_atom_definition,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [20272] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1460), 4,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [20283] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1462), 4,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [20294] = 4,
    ACTIONS(804), 1,
      anon_sym_LPAREN,
    STATE(676), 1,
      sym__arg_pool,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(814), 2,
      sym_colon,
      anon_sym_DOT,
  [20309] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1464), 4,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [20320] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1466), 4,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [20331] = 5,
    ACTIONS(1468), 1,
      anon_sym_SEMI,
    ACTIONS(1470), 1,
      anon_sym_RBRACE2,
    STATE(545), 1,
      aux_sym__theory_definitions_repeat1,
    STATE(546), 1,
      aux_sym__theory_definitions_repeat2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [20348] = 4,
    ACTIONS(1472), 1,
      anon_sym_COMMA2,
    STATE(508), 1,
      aux_sym__terms_or_projection_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1475), 2,
      anon_sym_RPAREN2,
      anon_sym_SEMI,
  [20363] = 5,
    ACTIONS(1468), 1,
      anon_sym_SEMI,
    ACTIONS(1477), 1,
      anon_sym_RBRACE2,
    STATE(507), 1,
      aux_sym__theory_definitions_repeat2,
    STATE(587), 1,
      aux_sym__theory_definitions_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [20380] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(1479), 2,
      anon_sym_DQUOTE,
      anon_sym_LBRACE,
    ACTIONS(1481), 2,
      sym__fstring_fragment,
      sym__fstring_escape,
  [20395] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1483), 4,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [20406] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(1485), 2,
      anon_sym_DQUOTE,
      anon_sym_LBRACE,
    ACTIONS(1487), 2,
      sym__fstring_fragment,
      sym__fstring_escape,
  [20421] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1489), 4,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [20432] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(1491), 2,
      anon_sym_DQUOTE,
      anon_sym_LBRACE,
    ACTIONS(1493), 2,
      sym__fstring_fragment,
      sym__fstring_escape,
  [20447] = 5,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(1495), 1,
      anon_sym_DQUOTE,
    STATE(522), 1,
      aux_sym_string_repeat1,
    ACTIONS(1497), 2,
      sym__string_fragment,
      sym__string_escape,
  [20464] = 5,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(1499), 1,
      anon_sym_DQUOTE,
    STATE(518), 1,
      aux_sym_string_repeat1,
    ACTIONS(1501), 2,
      sym__string_fragment,
      sym__string_escape,
  [20481] = 4,
    ACTIONS(1503), 1,
      anon_sym_STAR_PERCENT,
    ACTIONS(1505), 1,
      sym__doc_token_minus,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    STATE(532), 2,
      sym_doc_arg,
      aux_sym_doc_args_repeat1,
  [20496] = 5,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(1507), 1,
      anon_sym_DQUOTE,
    STATE(499), 1,
      aux_sym_string_repeat1,
    ACTIONS(1509), 2,
      sym__string_fragment,
      sym__string_escape,
  [20513] = 4,
    ACTIONS(1511), 1,
      anon_sym_STAR_PERCENT,
    ACTIONS(1513), 1,
      sym__doc_token_minus,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    STATE(519), 2,
      sym_doc_arg,
      aux_sym_doc_args_repeat1,
  [20528] = 4,
    ACTIONS(1516), 1,
      anon_sym_PIPE,
    ACTIONS(1518), 1,
      anon_sym_SEMI,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1520), 2,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [20543] = 4,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(1522), 2,
      anon_sym_DQUOTE,
      anon_sym_LBRACE,
    ACTIONS(1524), 2,
      sym__fstring_fragment,
      sym__fstring_escape,
  [20558] = 5,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(1526), 1,
      anon_sym_DQUOTE,
    STATE(499), 1,
      aux_sym_string_repeat1,
    ACTIONS(1509), 2,
      sym__string_fragment,
      sym__string_escape,
  [20575] = 5,
    ACTIONS(1528), 1,
      sym_identifier,
    ACTIONS(1530), 1,
      anon_sym_DASH,
    STATE(655), 1,
      sym_symbolic_atom,
    STATE(778), 1,
      sym_signature,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [20592] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1532), 4,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [20603] = 5,
    ACTIONS(1534), 1,
      sym_theory_operator,
    ACTIONS(1536), 1,
      anon_sym_RBRACE2,
    ACTIONS(1538), 1,
      sym_default_negation,
    STATE(868), 1,
      sym__theory_operators_sep,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [20620] = 4,
    ACTIONS(1540), 1,
      anon_sym_COMMA2,
    STATE(527), 1,
      aux_sym_condition_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1298), 2,
      anon_sym_PIPE,
      anon_sym_SEMI,
  [20635] = 4,
    ACTIONS(1542), 1,
      anon_sym_COMMA2,
    STATE(527), 1,
      aux_sym_condition_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1324), 2,
      anon_sym_PIPE,
      anon_sym_SEMI,
  [20650] = 5,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(1545), 1,
      anon_sym_DQUOTE,
    STATE(531), 1,
      aux_sym_string_repeat1,
    ACTIONS(1547), 2,
      sym__string_fragment,
      sym__string_escape,
  [20667] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1549), 4,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [20678] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1551), 4,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [20689] = 5,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(1553), 1,
      anon_sym_DQUOTE,
    STATE(499), 1,
      aux_sym_string_repeat1,
    ACTIONS(1509), 2,
      sym__string_fragment,
      sym__string_escape,
  [20706] = 4,
    ACTIONS(1505), 1,
      sym__doc_token_minus,
    ACTIONS(1555), 1,
      anon_sym_STAR_PERCENT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    STATE(519), 2,
      sym_doc_arg,
      aux_sym_doc_args_repeat1,
  [20721] = 4,
    ACTIONS(1540), 1,
      anon_sym_COMMA2,
    STATE(526), 1,
      aux_sym_condition_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1322), 2,
      anon_sym_PIPE,
      anon_sym_SEMI,
  [20736] = 5,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(1557), 1,
      anon_sym_DQUOTE,
    STATE(535), 1,
      aux_sym_string_repeat1,
    ACTIONS(1559), 2,
      sym__string_fragment,
      sym__string_escape,
  [20753] = 5,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(1561), 1,
      anon_sym_DQUOTE,
    STATE(499), 1,
      aux_sym_string_repeat1,
    ACTIONS(1509), 2,
      sym__string_fragment,
      sym__string_escape,
  [20770] = 5,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(1563), 1,
      anon_sym_DQUOTE,
    STATE(537), 1,
      aux_sym_string_repeat1,
    ACTIONS(1565), 2,
      sym__string_fragment,
      sym__string_escape,
  [20787] = 5,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(1567), 1,
      anon_sym_DQUOTE,
    STATE(499), 1,
      aux_sym_string_repeat1,
    ACTIONS(1509), 2,
      sym__string_fragment,
      sym__string_escape,
  [20804] = 4,
    ACTIONS(1437), 1,
      anon_sym_COMMA2,
    STATE(489), 1,
      aux_sym__terms_or_projection_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1569), 2,
      anon_sym_RPAREN2,
      anon_sym_SEMI,
  [20819] = 4,
    ACTIONS(1571), 1,
      anon_sym_COMMA2,
    ACTIONS(1575), 1,
      sym_colon,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1573), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [20834] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1577), 4,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [20845] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1579), 4,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [20856] = 4,
    ACTIONS(876), 1,
      aux_sym_fstring_spec_token1,
    ACTIONS(1581), 1,
      anon_sym_RBRACE,
    STATE(762), 1,
      sym_fstring_spec,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [20870] = 4,
    ACTIONS(1583), 1,
      anon_sym_SEMI,
    ACTIONS(1585), 1,
      anon_sym_RBRACE2,
    STATE(559), 1,
      aux_sym_theory_operator_definitions_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [20884] = 4,
    ACTIONS(1587), 1,
      anon_sym_SEMI,
    ACTIONS(1590), 1,
      anon_sym_RBRACE2,
    STATE(544), 1,
      aux_sym__theory_definitions_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [20898] = 4,
    ACTIONS(1592), 1,
      anon_sym_SEMI,
    ACTIONS(1594), 1,
      anon_sym_RBRACE2,
    STATE(544), 1,
      aux_sym__theory_definitions_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [20912] = 4,
    ACTIONS(1596), 1,
      anon_sym_SEMI,
    ACTIONS(1599), 1,
      anon_sym_RBRACE2,
    STATE(546), 1,
      aux_sym__theory_definitions_repeat2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [20926] = 4,
    ACTIONS(1601), 1,
      anon_sym_SEMI,
    ACTIONS(1603), 1,
      anon_sym_RBRACE2,
    STATE(610), 1,
      aux_sym_body_aggregate_elements_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [20940] = 4,
    ACTIONS(1427), 1,
      sym_fstring_grouping,
    ACTIONS(1429), 1,
      sym_fstring_type,
    ACTIONS(1431), 1,
      anon_sym_RBRACE,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [20954] = 4,
    ACTIONS(1605), 1,
      sym_identifier,
    ACTIONS(1607), 1,
      anon_sym_DASH,
    STATE(661), 1,
      sym_symbolic_atom,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [20968] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1609), 3,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_DOT,
  [20978] = 4,
    ACTIONS(1611), 1,
      anon_sym_RPAREN2,
    ACTIONS(1613), 1,
      anon_sym_SEMI,
    STATE(561), 1,
      aux_sym__tuple_pool_n_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [20992] = 4,
    ACTIONS(1128), 1,
      anon_sym_COMMA2,
    ACTIONS(1615), 1,
      anon_sym_RPAREN2,
    STATE(555), 1,
      aux_sym_const_terms_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21006] = 3,
    ACTIONS(1619), 1,
      sym_colon,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1617), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [21018] = 4,
    ACTIONS(1621), 1,
      sym_identifier,
    ACTIONS(1623), 1,
      anon_sym_RPAREN2,
    STATE(845), 1,
      sym_parameters,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21032] = 4,
    ACTIONS(1154), 1,
      anon_sym_RPAREN2,
    ACTIONS(1625), 1,
      anon_sym_COMMA2,
    STATE(555), 1,
      aux_sym_const_terms_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21046] = 4,
    ACTIONS(1628), 1,
      anon_sym_SEMI,
    ACTIONS(1631), 1,
      anon_sym_RBRACE2,
    STATE(556), 1,
      aux_sym_optimize_elements_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21060] = 4,
    ACTIONS(1018), 1,
      anon_sym_SEMI,
    ACTIONS(1633), 1,
      anon_sym_PIPE,
    STATE(605), 1,
      aux_sym_abs_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21074] = 4,
    ACTIONS(1411), 1,
      sym_theory_operator,
    ACTIONS(1415), 1,
      sym_default_negation,
    STATE(701), 1,
      sym_theory_operator_definition,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21088] = 4,
    ACTIONS(1583), 1,
      anon_sym_SEMI,
    ACTIONS(1635), 1,
      anon_sym_RBRACE2,
    STATE(564), 1,
      aux_sym_theory_operator_definitions_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21102] = 4,
    ACTIONS(1637), 1,
      anon_sym_RPAREN2,
    ACTIONS(1639), 1,
      anon_sym_SEMI,
    STATE(590), 1,
      aux_sym_edge_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21116] = 4,
    ACTIONS(1613), 1,
      anon_sym_SEMI,
    ACTIONS(1641), 1,
      anon_sym_RPAREN2,
    STATE(606), 1,
      aux_sym__tuple_pool_n_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21130] = 4,
    ACTIONS(1643), 1,
      anon_sym_COMMA,
    ACTIONS(1645), 1,
      anon_sym_RPAREN,
    STATE(565), 1,
      aux_sym_variables_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21144] = 4,
    ACTIONS(1647), 1,
      sym_fstring_grouping,
    ACTIONS(1649), 1,
      sym_fstring_type,
    ACTIONS(1651), 1,
      anon_sym_RBRACE,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21158] = 4,
    ACTIONS(1653), 1,
      anon_sym_SEMI,
    ACTIONS(1656), 1,
      anon_sym_RBRACE2,
    STATE(564), 1,
      aux_sym_theory_operator_definitions_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21172] = 4,
    ACTIONS(1643), 1,
      anon_sym_COMMA,
    ACTIONS(1658), 1,
      anon_sym_RPAREN,
    STATE(569), 1,
      aux_sym_variables_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21186] = 4,
    ACTIONS(1660), 1,
      anon_sym_COMMA2,
    ACTIONS(1662), 1,
      anon_sym_RPAREN2,
    STATE(619), 1,
      aux_sym_parameters_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21200] = 4,
    ACTIONS(1664), 1,
      sym_theory_operator_arity,
    ACTIONS(1666), 1,
      anon_sym_binary,
    STATE(777), 1,
      sym__theory_operator_arity_binary,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21214] = 4,
    ACTIONS(1666), 1,
      anon_sym_binary,
    ACTIONS(1668), 1,
      sym_theory_operator_arity,
    STATE(793), 1,
      sym__theory_operator_arity_binary,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21228] = 4,
    ACTIONS(1670), 1,
      anon_sym_COMMA,
    ACTIONS(1673), 1,
      anon_sym_RPAREN,
    STATE(569), 1,
      aux_sym_variables_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21242] = 4,
    ACTIONS(1675), 1,
      anon_sym_COMMA2,
    ACTIONS(1677), 1,
      anon_sym_RBRACE2,
    STATE(572), 1,
      aux_sym__theory_operators_sep_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21256] = 4,
    ACTIONS(1675), 1,
      anon_sym_COMMA2,
    ACTIONS(1679), 1,
      anon_sym_RBRACE2,
    STATE(573), 1,
      aux_sym__theory_operators_sep_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21270] = 4,
    ACTIONS(1675), 1,
      anon_sym_COMMA2,
    ACTIONS(1681), 1,
      anon_sym_RBRACE2,
    STATE(574), 1,
      aux_sym__theory_operators_sep_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21284] = 4,
    ACTIONS(1675), 1,
      anon_sym_COMMA2,
    ACTIONS(1683), 1,
      anon_sym_RBRACE2,
    STATE(574), 1,
      aux_sym__theory_operators_sep_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21298] = 4,
    ACTIONS(1685), 1,
      anon_sym_COMMA2,
    ACTIONS(1688), 1,
      anon_sym_RBRACE2,
    STATE(574), 1,
      aux_sym__theory_operators_sep_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21312] = 4,
    ACTIONS(13), 1,
      anon_sym_DQUOTE,
    ACTIONS(1690), 1,
      anon_sym_LT,
    STATE(736), 1,
      sym_string,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21326] = 4,
    ACTIONS(1605), 1,
      sym_identifier,
    ACTIONS(1607), 1,
      anon_sym_DASH,
    STATE(672), 1,
      sym_symbolic_atom,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21340] = 4,
    ACTIONS(1290), 1,
      sym_fstring_grouping,
    ACTIONS(1292), 1,
      sym_fstring_type,
    ACTIONS(1294), 1,
      anon_sym_RBRACE,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21354] = 4,
    ACTIONS(1186), 1,
      sym__doc_token_args,
    ACTIONS(1692), 1,
      anon_sym_STAR_PERCENT,
    STATE(756), 1,
      sym_doc_args,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21368] = 4,
    ACTIONS(1694), 1,
      anon_sym_RPAREN2,
    ACTIONS(1696), 1,
      anon_sym_SEMI,
    STATE(603), 1,
      aux_sym__arg_pool_n_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21382] = 4,
    ACTIONS(1698), 1,
      anon_sym_SEMI,
    ACTIONS(1700), 1,
      anon_sym_RBRACE2,
    STATE(611), 1,
      aux_sym_head_aggregate_elements_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21396] = 4,
    ACTIONS(1694), 1,
      anon_sym_RPAREN2,
    ACTIONS(1696), 1,
      anon_sym_SEMI,
    STATE(603), 1,
      aux_sym__arg_pool_n_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21410] = 4,
    ACTIONS(876), 1,
      aux_sym_fstring_spec_token1,
    ACTIONS(1425), 1,
      anon_sym_RBRACE,
    STATE(796), 1,
      sym_fstring_spec,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21424] = 4,
    ACTIONS(1702), 1,
      anon_sym_RPAREN2,
    ACTIONS(1704), 1,
      anon_sym_SEMI,
    STATE(583), 1,
      aux_sym_edge_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21438] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1449), 3,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_DOT,
  [21448] = 4,
    ACTIONS(1078), 1,
      anon_sym_COMMA2,
    ACTIONS(1417), 1,
      anon_sym_RBRACK,
    STATE(592), 1,
      aux_sym_terms_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21462] = 4,
    ACTIONS(1707), 1,
      anon_sym_SEMI,
    ACTIONS(1710), 1,
      anon_sym_RBRACE2,
    STATE(586), 1,
      aux_sym_set_aggregate_elements_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21476] = 4,
    ACTIONS(1470), 1,
      anon_sym_RBRACE2,
    ACTIONS(1592), 1,
      anon_sym_SEMI,
    STATE(544), 1,
      aux_sym__theory_definitions_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21490] = 3,
    ACTIONS(1714), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1712), 2,
      anon_sym_COMMA2,
      anon_sym_SEMI,
  [21502] = 4,
    ACTIONS(1716), 1,
      anon_sym_COMMA2,
    ACTIONS(1718), 1,
      anon_sym_RPAREN2,
    STATE(600), 1,
      aux_sym_theory_terms_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21516] = 4,
    ACTIONS(1639), 1,
      anon_sym_SEMI,
    ACTIONS(1720), 1,
      anon_sym_RPAREN2,
    STATE(583), 1,
      aux_sym_edge_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21530] = 4,
    ACTIONS(1722), 1,
      anon_sym_RPAREN,
    ACTIONS(1724), 1,
      sym_doc_var,
    STATE(854), 1,
      sym_variables,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21544] = 4,
    ACTIONS(1003), 1,
      anon_sym_RBRACK,
    ACTIONS(1726), 1,
      anon_sym_COMMA2,
    STATE(592), 1,
      aux_sym_terms_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21558] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1441), 3,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_DOT,
  [21568] = 3,
    ACTIONS(1731), 1,
      sym_colon,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1729), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [21580] = 4,
    ACTIONS(1337), 1,
      sym_fstring_grouping,
    ACTIONS(1339), 1,
      sym_fstring_type,
    ACTIONS(1341), 1,
      anon_sym_RBRACE,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21594] = 4,
    ACTIONS(1733), 1,
      anon_sym_SEMI,
    ACTIONS(1735), 1,
      anon_sym_RBRACE2,
    STATE(586), 1,
      aux_sym_set_aggregate_elements_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21608] = 3,
    ACTIONS(1737), 1,
      anon_sym_SEMI,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1739), 2,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [21620] = 3,
    ACTIONS(1743), 1,
      sym_colon,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1741), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [21632] = 3,
    ACTIONS(1745), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1712), 2,
      anon_sym_COMMA2,
      anon_sym_SEMI,
  [21644] = 4,
    ACTIONS(748), 1,
      anon_sym_RPAREN2,
    ACTIONS(1747), 1,
      anon_sym_COMMA2,
    STATE(477), 1,
      aux_sym_theory_terms_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21658] = 4,
    ACTIONS(1749), 1,
      anon_sym_SEMI,
    ACTIONS(1751), 1,
      anon_sym_RBRACE2,
    STATE(602), 1,
      aux_sym_theory_elements_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21672] = 4,
    ACTIONS(1749), 1,
      anon_sym_SEMI,
    ACTIONS(1753), 1,
      anon_sym_RBRACE2,
    STATE(604), 1,
      aux_sym_theory_elements_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21686] = 4,
    ACTIONS(1755), 1,
      anon_sym_RPAREN2,
    ACTIONS(1757), 1,
      anon_sym_SEMI,
    STATE(603), 1,
      aux_sym__arg_pool_n_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21700] = 4,
    ACTIONS(1760), 1,
      anon_sym_SEMI,
    ACTIONS(1763), 1,
      anon_sym_RBRACE2,
    STATE(604), 1,
      aux_sym_theory_elements_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21714] = 4,
    ACTIONS(1132), 1,
      anon_sym_PIPE,
    ACTIONS(1765), 1,
      anon_sym_SEMI,
    STATE(605), 1,
      aux_sym_abs_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21728] = 4,
    ACTIONS(1768), 1,
      anon_sym_RPAREN2,
    ACTIONS(1770), 1,
      anon_sym_SEMI,
    STATE(606), 1,
      aux_sym__tuple_pool_n_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21742] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1475), 3,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_SEMI,
  [21752] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1773), 3,
      sym_fstring_grouping,
      sym_fstring_type,
      anon_sym_RBRACE,
  [21762] = 3,
    ACTIONS(1777), 1,
      sym_colon,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1775), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [21774] = 4,
    ACTIONS(1779), 1,
      anon_sym_SEMI,
    ACTIONS(1782), 1,
      anon_sym_RBRACE2,
    STATE(610), 1,
      aux_sym_body_aggregate_elements_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21788] = 4,
    ACTIONS(1698), 1,
      anon_sym_SEMI,
    ACTIONS(1784), 1,
      anon_sym_RBRACE2,
    STATE(628), 1,
      aux_sym_head_aggregate_elements_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21802] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1786), 3,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_DOT,
  [21812] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1788), 3,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_DOT,
  [21822] = 3,
    ACTIONS(1792), 1,
      sym_colon,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1790), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [21834] = 4,
    ACTIONS(1601), 1,
      anon_sym_SEMI,
    ACTIONS(1794), 1,
      anon_sym_RBRACE2,
    STATE(547), 1,
      aux_sym_body_aggregate_elements_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21848] = 4,
    ACTIONS(1796), 1,
      anon_sym_SEMI,
    ACTIONS(1798), 1,
      anon_sym_RBRACE2,
    STATE(620), 1,
      aux_sym_optimize_elements_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21862] = 4,
    ACTIONS(1613), 1,
      anon_sym_SEMI,
    ACTIONS(1800), 1,
      anon_sym_RPAREN2,
    STATE(561), 1,
      aux_sym__tuple_pool_n_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21876] = 4,
    ACTIONS(1802), 1,
      sym_identifier,
    ACTIONS(1804), 1,
      anon_sym_DASH,
    STATE(758), 1,
      sym_signature,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21890] = 4,
    ACTIONS(1806), 1,
      anon_sym_COMMA2,
    ACTIONS(1809), 1,
      anon_sym_RPAREN2,
    STATE(619), 1,
      aux_sym_parameters_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21904] = 4,
    ACTIONS(1796), 1,
      anon_sym_SEMI,
    ACTIONS(1811), 1,
      anon_sym_RBRACE2,
    STATE(556), 1,
      aux_sym_optimize_elements_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21918] = 4,
    ACTIONS(1696), 1,
      anon_sym_SEMI,
    ACTIONS(1813), 1,
      anon_sym_RPAREN2,
    STATE(581), 1,
      aux_sym__arg_pool_n_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21932] = 4,
    ACTIONS(1018), 1,
      anon_sym_SEMI,
    ACTIONS(1815), 1,
      anon_sym_PIPE,
    STATE(605), 1,
      aux_sym_abs_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21946] = 3,
    STATE(813), 1,
      sym_boolean,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1817), 2,
      anon_sym_true,
      anon_sym_false,
  [21958] = 4,
    ACTIONS(1660), 1,
      anon_sym_COMMA2,
    ACTIONS(1819), 1,
      anon_sym_RPAREN2,
    STATE(566), 1,
      aux_sym_parameters_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [21972] = 3,
    ACTIONS(1823), 1,
      sym_colon,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1821), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [21984] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1825), 3,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
      anon_sym_SEMI,
  [21994] = 4,
    ACTIONS(1733), 1,
      anon_sym_SEMI,
    ACTIONS(1827), 1,
      anon_sym_RBRACE2,
    STATE(596), 1,
      aux_sym_set_aggregate_elements_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22008] = 4,
    ACTIONS(1829), 1,
      anon_sym_SEMI,
    ACTIONS(1832), 1,
      anon_sym_RBRACE2,
    STATE(628), 1,
      aux_sym_head_aggregate_elements_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22022] = 4,
    ACTIONS(1834), 1,
      anon_sym_COMMA2,
    ACTIONS(1836), 1,
      anon_sym_RPAREN2,
    STATE(555), 1,
      aux_sym_const_terms_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22036] = 4,
    ACTIONS(1613), 1,
      anon_sym_SEMI,
    ACTIONS(1838), 1,
      anon_sym_RPAREN2,
    STATE(561), 1,
      aux_sym__tuple_pool_n_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22050] = 4,
    ACTIONS(1696), 1,
      anon_sym_SEMI,
    ACTIONS(1840), 1,
      anon_sym_RPAREN2,
    STATE(581), 1,
      aux_sym__arg_pool_n_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22064] = 4,
    ACTIONS(1018), 1,
      anon_sym_SEMI,
    ACTIONS(1842), 1,
      anon_sym_PIPE,
    STATE(605), 1,
      aux_sym_abs_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22078] = 4,
    ACTIONS(1696), 1,
      anon_sym_SEMI,
    ACTIONS(1844), 1,
      anon_sym_RPAREN2,
    STATE(581), 1,
      aux_sym__arg_pool_n_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22092] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1846), 3,
      anon_sym_COMMA2,
      anon_sym_SEMI,
      anon_sym_DOT,
  [22102] = 4,
    ACTIONS(1696), 1,
      anon_sym_SEMI,
    ACTIONS(1848), 1,
      anon_sym_RPAREN2,
    STATE(581), 1,
      aux_sym__arg_pool_n_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22116] = 4,
    ACTIONS(1477), 1,
      anon_sym_RBRACE2,
    ACTIONS(1592), 1,
      anon_sym_SEMI,
    STATE(587), 1,
      aux_sym__theory_definitions_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22130] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1850), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [22139] = 3,
    ACTIONS(1429), 1,
      sym_fstring_type,
    ACTIONS(1431), 1,
      anon_sym_RBRACE,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22150] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1852), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [22159] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1854), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [22168] = 3,
    ACTIONS(1856), 1,
      anon_sym_LBRACE,
    ACTIONS(1858), 1,
      sym_theory_atom_type,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22179] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1860), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [22188] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1862), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [22197] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1864), 2,
      sym__doc_token_minus,
      anon_sym_STAR_PERCENT,
  [22206] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1866), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [22215] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1868), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [22224] = 3,
    ACTIONS(1870), 1,
      sym_theory_operator,
    ACTIONS(1872), 1,
      sym_default_negation,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22235] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1688), 2,
      anon_sym_COMMA2,
      anon_sym_RBRACE2,
  [22244] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1874), 2,
      anon_sym_COMMA2,
      anon_sym_RBRACE2,
  [22253] = 3,
    ACTIONS(1876), 1,
      anon_sym_COMMA2,
    STATE(500), 1,
      aux_sym__terms_or_projection_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22264] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1878), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [22273] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1880), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [22282] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1032), 2,
      anon_sym_RPAREN2,
      anon_sym_SEMI,
  [22291] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1882), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [22300] = 3,
    ACTIONS(1884), 1,
      anon_sym_DOT,
    ACTIONS(1886), 1,
      sym_colon,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22311] = 3,
    ACTIONS(1888), 1,
      anon_sym_LPAREN,
    ACTIONS(1890), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22322] = 3,
    ACTIONS(1516), 1,
      anon_sym_PIPE,
    ACTIONS(1518), 1,
      anon_sym_SEMI,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22333] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1365), 2,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [22342] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1892), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [22351] = 3,
    ACTIONS(1894), 1,
      anon_sym_COMMA2,
    ACTIONS(1896), 1,
      anon_sym_RBRACK,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22362] = 3,
    ACTIONS(1898), 1,
      anon_sym_DOT,
    ACTIONS(1900), 1,
      sym_colon,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22373] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1902), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [22382] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1904), 2,
      anon_sym_RPAREN2,
      anon_sym_SEMI,
  [22391] = 3,
    ACTIONS(1906), 1,
      anon_sym_DOT,
    ACTIONS(1908), 1,
      sym_colon,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22402] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1910), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [22411] = 3,
    ACTIONS(1912), 1,
      anon_sym_LBRACE,
    ACTIONS(1914), 1,
      sym_aggregate_function,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22422] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1916), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [22431] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1710), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [22440] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1599), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [22449] = 3,
    ACTIONS(1402), 1,
      anon_sym_AMP,
    STATE(675), 1,
      sym_theory_atom_definition,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22460] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1631), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [22469] = 3,
    ACTIONS(1918), 1,
      anon_sym_DOT,
    ACTIONS(1920), 1,
      sym_colon,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22480] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1922), 2,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [22489] = 3,
    ACTIONS(1712), 1,
      anon_sym_SEMI,
    ACTIONS(1714), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22500] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1590), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [22509] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(828), 2,
      sym_colon,
      anon_sym_DOT,
  [22518] = 3,
    ACTIONS(1924), 1,
      anon_sym_DOT,
    ACTIONS(1926), 1,
      sym_colon,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22529] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1928), 2,
      anon_sym_RPAREN2,
      anon_sym_SEMI,
  [22538] = 3,
    ACTIONS(1712), 1,
      anon_sym_SEMI,
    ACTIONS(1745), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22549] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1930), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [22558] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1932), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [22567] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1934), 2,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [22576] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1763), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [22585] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1936), 2,
      aux_sym_fstring_spec_token1,
      anon_sym_RBRACE,
  [22594] = 3,
    ACTIONS(1938), 1,
      anon_sym_DOT,
    ACTIONS(1940), 1,
      anon_sym_COLON_DASH,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22605] = 3,
    ACTIONS(1400), 1,
      sym_identifier,
    STATE(669), 1,
      sym_theory_term_definition,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22616] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1768), 2,
      anon_sym_RPAREN2,
      anon_sym_SEMI,
  [22625] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1942), 2,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [22634] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1944), 2,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [22643] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1946), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [22652] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1948), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [22661] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1782), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [22670] = 3,
    ACTIONS(1649), 1,
      sym_fstring_type,
    ACTIONS(1651), 1,
      anon_sym_RBRACE,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22681] = 3,
    ACTIONS(1950), 1,
      aux_sym_doc_predicate_token1,
    STATE(392), 1,
      sym_doc_predicate,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22692] = 3,
    ACTIONS(1912), 1,
      anon_sym_LBRACE,
    ACTIONS(1952), 1,
      sym_aggregate_function,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22703] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1712), 2,
      anon_sym_COMMA2,
      anon_sym_SEMI,
  [22712] = 3,
    ACTIONS(1613), 1,
      anon_sym_SEMI,
    STATE(561), 1,
      aux_sym__tuple_pool_n_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22723] = 3,
    ACTIONS(1954), 1,
      anon_sym_COMMA2,
    ACTIONS(1956), 1,
      anon_sym_RBRACK,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22734] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1809), 2,
      anon_sym_COMMA2,
      anon_sym_RPAREN2,
  [22743] = 3,
    ACTIONS(1292), 1,
      sym_fstring_type,
    ACTIONS(1294), 1,
      anon_sym_RBRACE,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22754] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1656), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [22763] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1331), 2,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [22772] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(832), 2,
      sym_colon,
      anon_sym_DOT,
  [22781] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1832), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [22790] = 3,
    ACTIONS(1696), 1,
      anon_sym_SEMI,
    STATE(579), 1,
      aux_sym__arg_pool_n_repeat1,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22801] = 3,
    ACTIONS(1958), 1,
      sym_fstring_type,
    ACTIONS(1960), 1,
      anon_sym_RBRACE,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22812] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1962), 2,
      anon_sym_DOT,
      anon_sym_COLON_DASH,
  [22821] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1964), 2,
      anon_sym_SEMI,
      anon_sym_RBRACE2,
  [22830] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1966), 2,
      anon_sym_RPAREN2,
      anon_sym_SEMI,
  [22839] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1739), 2,
      anon_sym_SEMI,
      anon_sym_DOT,
  [22848] = 3,
    ACTIONS(1968), 1,
      anon_sym_LBRACE,
    ACTIONS(1970), 1,
      sym_aggregate_function,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22859] = 2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
    ACTIONS(1520), 2,
      anon_sym_SEMI,
      anon_sym_DOT,
  [22868] = 3,
    ACTIONS(1339), 1,
      sym_fstring_type,
    ACTIONS(1341), 1,
      anon_sym_RBRACE,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22879] = 2,
    ACTIONS(1972), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22887] = 2,
    ACTIONS(1974), 1,
      sym_colon,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22895] = 2,
    ACTIONS(1976), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22903] = 2,
    ACTIONS(1978), 1,
      sym_colon,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22911] = 2,
    ACTIONS(1425), 1,
      anon_sym_RBRACE,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22919] = 2,
    ACTIONS(1980), 1,
      anon_sym_COMMA2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22927] = 2,
    ACTIONS(1982), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22935] = 2,
    ACTIONS(1984), 1,
      anon_sym_LBRACK,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22943] = 2,
    ACTIONS(1986), 1,
      anon_sym_COMMA2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22951] = 2,
    ACTIONS(1988), 1,
      anon_sym_RBRACE2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22959] = 2,
    ACTIONS(1990), 1,
      anon_sym_LBRACE,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22967] = 2,
    ACTIONS(1333), 1,
      sym_colon,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22975] = 2,
    ACTIONS(1992), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22983] = 2,
    ACTIONS(1994), 1,
      anon_sym_RBRACE2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22991] = 2,
    ACTIONS(1996), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [22999] = 2,
    ACTIONS(1998), 1,
      anon_sym_SLASH,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23007] = 2,
    ACTIONS(2000), 1,
      anon_sym_COMMA2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23015] = 3,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(2002), 1,
      aux_sym_doc_comment_token1,
  [23025] = 2,
    ACTIONS(2004), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23033] = 2,
    ACTIONS(2006), 1,
      anon_sym_RBRACE2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23041] = 2,
    ACTIONS(2008), 1,
      sym_number,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23049] = 2,
    ACTIONS(2010), 1,
      anon_sym_RBRACE2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23057] = 2,
    ACTIONS(2012), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23065] = 2,
    ACTIONS(2014), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23073] = 2,
    ACTIONS(2016), 1,
      sym_theory_operator_associativity,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23081] = 2,
    ACTIONS(2018), 1,
      anon_sym_RPAREN2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23089] = 2,
    ACTIONS(2020), 1,
      anon_sym_COMMA2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23097] = 2,
    ACTIONS(2022), 1,
      anon_sym_RBRACK,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23105] = 3,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(2024), 1,
      aux_sym_doc_comment_token1,
  [23115] = 2,
    ACTIONS(2026), 1,
      anon_sym_RPAREN2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23123] = 2,
    ACTIONS(2028), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23131] = 2,
    ACTIONS(2030), 1,
      sym_theory_atom_type,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23139] = 2,
    ACTIONS(2032), 1,
      anon_sym_RPAREN2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23147] = 2,
    ACTIONS(2034), 1,
      anon_sym_COMMA2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23155] = 2,
    ACTIONS(2036), 1,
      sym_theory_operator_associativity,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23163] = 2,
    ACTIONS(2038), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23171] = 2,
    ACTIONS(2040), 1,
      sym_theory_atom_type,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23179] = 3,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(2042), 1,
      aux_sym_doc_comment_token1,
  [23189] = 2,
    ACTIONS(2044), 1,
      anon_sym_LBRACE,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23197] = 2,
    ACTIONS(2046), 1,
      anon_sym_RBRACK,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23205] = 2,
    ACTIONS(2048), 1,
      anon_sym_COMMA2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23213] = 2,
    ACTIONS(2050), 1,
      anon_sym_POUNDend,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23221] = 2,
    ACTIONS(2052), 1,
      anon_sym_STAR_PERCENT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23229] = 2,
    ACTIONS(2054), 1,
      anon_sym_LPAREN,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23237] = 2,
    ACTIONS(2056), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23245] = 2,
    ACTIONS(1341), 1,
      anon_sym_RBRACE,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23253] = 2,
    ACTIONS(1611), 1,
      anon_sym_RPAREN2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23261] = 2,
    ACTIONS(2058), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23269] = 2,
    ACTIONS(2060), 1,
      anon_sym_RBRACE,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23277] = 2,
    ACTIONS(2062), 1,
      anon_sym_LPAREN,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23285] = 2,
    ACTIONS(2064), 1,
      anon_sym_RPAREN2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23293] = 2,
    ACTIONS(2066), 1,
      sym_number,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23301] = 2,
    ACTIONS(2068), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23309] = 2,
    ACTIONS(2070), 1,
      sym_number,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23317] = 3,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(2072), 1,
      aux_sym_doc_comment_token1,
  [23327] = 2,
    ACTIONS(2074), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23335] = 2,
    ACTIONS(2076), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23343] = 2,
    ACTIONS(2078), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23351] = 2,
    ACTIONS(2080), 1,
      anon_sym_RBRACE2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23359] = 2,
    ACTIONS(2082), 1,
      sym_colon,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23367] = 2,
    ACTIONS(2084), 1,
      anon_sym_COMMA2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23375] = 2,
    ACTIONS(2086), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23383] = 2,
    ACTIONS(2088), 1,
      aux_sym_fstring_accessor_token4,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23391] = 2,
    ACTIONS(2090), 1,
      anon_sym_COMMA2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23399] = 2,
    ACTIONS(2092), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23407] = 2,
    ACTIONS(2094), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23415] = 2,
    ACTIONS(2096), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23423] = 2,
    ACTIONS(2098), 1,
      anon_sym_RPAREN2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23431] = 2,
    ACTIONS(2100), 1,
      anon_sym_RBRACE,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23439] = 2,
    ACTIONS(2102), 1,
      anon_sym_RBRACE2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23447] = 2,
    ACTIONS(2104), 1,
      anon_sym_LBRACE,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23455] = 2,
    ACTIONS(2106), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23463] = 3,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(2108), 1,
      aux_sym_doc_comment_token1,
  [23473] = 2,
    ACTIONS(2110), 1,
      anon_sym_RBRACE2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23481] = 3,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(2112), 1,
      aux_sym_doc_comment_token1,
  [23491] = 2,
    ACTIONS(2114), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23499] = 2,
    ACTIONS(2116), 1,
      anon_sym_LBRACE,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23507] = 2,
    ACTIONS(2118), 1,
      anon_sym_LBRACK,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23515] = 2,
    ACTIONS(2120), 1,
      anon_sym_LBRACK,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23523] = 2,
    ACTIONS(2122), 1,
      anon_sym_COMMA2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23531] = 3,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(2124), 1,
      aux_sym_doc_comment_token1,
  [23541] = 2,
    ACTIONS(1737), 1,
      anon_sym_SEMI,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23549] = 2,
    ACTIONS(1581), 1,
      anon_sym_RBRACE,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23557] = 2,
    ACTIONS(1651), 1,
      anon_sym_RBRACE,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23565] = 3,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(2126), 1,
      aux_sym_doc_comment_token1,
  [23575] = 2,
    ACTIONS(2128), 1,
      anon_sym_RBRACK,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23583] = 3,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(2130), 1,
      aux_sym_doc_comment_token1,
  [23593] = 2,
    ACTIONS(2132), 1,
      anon_sym_LBRACE,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23601] = 2,
    ACTIONS(2134), 1,
      anon_sym_EQ,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23609] = 2,
    ACTIONS(2136), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23617] = 2,
    ACTIONS(2138), 1,
      anon_sym_RBRACK,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23625] = 2,
    ACTIONS(2140), 1,
      anon_sym_LBRACK,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23633] = 2,
    ACTIONS(2142), 1,
      sym_doc_var,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23641] = 2,
    ACTIONS(2144), 1,
      anon_sym_LBRACE,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23649] = 2,
    ACTIONS(2146), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23657] = 2,
    ACTIONS(2148), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23665] = 2,
    ACTIONS(1712), 1,
      anon_sym_SEMI,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23673] = 2,
    ACTIONS(2150), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23681] = 2,
    ACTIONS(1098), 1,
      anon_sym_RPAREN2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23689] = 2,
    ACTIONS(2152), 1,
      anon_sym_RBRACK,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23697] = 2,
    ACTIONS(2154), 1,
      anon_sym_RBRACE2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23705] = 2,
    ACTIONS(2156), 1,
      anon_sym_RBRACE2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23713] = 2,
    ACTIONS(2158), 1,
      anon_sym_GT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23721] = 2,
    ACTIONS(1800), 1,
      anon_sym_RPAREN2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23729] = 2,
    ACTIONS(2160), 1,
      anon_sym_LBRACE,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23737] = 2,
    ACTIONS(2162), 1,
      anon_sym_LBRACE,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23745] = 2,
    ACTIONS(2164), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23753] = 2,
    ACTIONS(2166), 1,
      anon_sym_STAR_PERCENT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23761] = 2,
    ACTIONS(2168), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23769] = 2,
    ACTIONS(2170), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23777] = 2,
    ACTIONS(2172), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23785] = 2,
    ACTIONS(2174), 1,
      anon_sym_COMMA2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23793] = 2,
    ACTIONS(2176), 1,
      anon_sym_COMMA2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23801] = 2,
    ACTIONS(2178), 1,
      anon_sym_RPAREN2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23809] = 2,
    ACTIONS(2180), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23817] = 2,
    ACTIONS(1431), 1,
      anon_sym_RBRACE,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23825] = 2,
    ACTIONS(2182), 1,
      sym_colon,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23833] = 2,
    ACTIONS(2184), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23841] = 2,
    ACTIONS(2186), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23849] = 2,
    ACTIONS(2188), 1,
      sym_number,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23857] = 2,
    ACTIONS(2190), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23865] = 2,
    ACTIONS(2192), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23873] = 2,
    ACTIONS(2194), 1,
      anon_sym_RBRACE2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23881] = 3,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(2196), 1,
      aux_sym_doc_comment_token1,
  [23891] = 2,
    ACTIONS(2198), 1,
      anon_sym_RBRACK,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23899] = 3,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(2200), 1,
      aux_sym_doc_comment_token1,
  [23909] = 2,
    ACTIONS(2202), 1,
      anon_sym_RBRACE2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23917] = 2,
    ACTIONS(2204), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23925] = 2,
    ACTIONS(2206), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23933] = 2,
    ACTIONS(2208), 1,
      anon_sym_DOT,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23941] = 2,
    ACTIONS(1387), 1,
      anon_sym_SLASH,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23949] = 2,
    ACTIONS(2210), 1,
      anon_sym_RPAREN2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23957] = 2,
    ACTIONS(2212), 1,
      anon_sym_RBRACE2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23965] = 2,
    ACTIONS(2214), 1,
      anon_sym_RPAREN2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23973] = 2,
    ACTIONS(2216), 1,
      anon_sym_RBRACK,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23981] = 2,
    ACTIONS(2218), 1,
      anon_sym_RBRACE2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [23989] = 3,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(2220), 1,
      aux_sym_doc_comment_token1,
  [23999] = 2,
    ACTIONS(2222), 1,
      anon_sym_RPAREN2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [24007] = 3,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(2224), 1,
      sym_code,
  [24017] = 2,
    ACTIONS(2226), 1,
      anon_sym_RPAREN2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [24025] = 2,
    ACTIONS(2228), 1,
      anon_sym_RPAREN,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [24033] = 2,
    ACTIONS(2230), 1,
      anon_sym_RBRACE2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [24041] = 2,
    ACTIONS(2232), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [24049] = 2,
    ACTIONS(2234), 1,
      anon_sym_COMMA2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [24057] = 2,
    ACTIONS(2236), 1,
      anon_sym_RBRACE2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [24065] = 2,
    ACTIONS(2238), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [24073] = 2,
    ACTIONS(2240), 1,
      anon_sym_COLON,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [24081] = 2,
    ACTIONS(1960), 1,
      anon_sym_RBRACE,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [24089] = 2,
    ACTIONS(1838), 1,
      anon_sym_RPAREN2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [24097] = 2,
    ACTIONS(2242), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [24105] = 2,
    ACTIONS(2244), 1,
      anon_sym_RPAREN2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [24113] = 2,
    ACTIONS(2246), 1,
      sym_doc_var,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [24121] = 2,
    ACTIONS(2248), 1,
      anon_sym_LBRACK,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [24129] = 2,
    ACTIONS(2250), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [24137] = 2,
    ACTIONS(2252), 1,
      anon_sym_RBRACE2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [24145] = 2,
    ACTIONS(1294), 1,
      anon_sym_RBRACE,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [24153] = 2,
    ACTIONS(2254), 1,
      anon_sym_RPAREN2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [24161] = 2,
    ACTIONS(1433), 1,
      anon_sym_SLASH,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [24169] = 2,
    ACTIONS(2256), 1,
      aux_sym_fstring_accessor_token3,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [24177] = 2,
    ACTIONS(2258), 1,
      sym_const_type,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [24185] = 2,
    ACTIONS(2260), 1,
      sym_number,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [24193] = 2,
    ACTIONS(2262), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [24201] = 2,
    ACTIONS(2264), 1,
      anon_sym_LBRACE,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [24209] = 2,
    ACTIONS(1445), 1,
      sym_colon,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [24217] = 2,
    ACTIONS(2266), 1,
      anon_sym_RBRACE2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [24225] = 2,
    ACTIONS(2268), 1,
      anon_sym_LBRACE,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [24233] = 3,
    ACTIONS(3), 1,
      sym_line_comment,
    ACTIONS(5), 1,
      sym_block_comment,
    ACTIONS(2270), 1,
      aux_sym_fstring_conversion_token2,
  [24243] = 2,
    ACTIONS(2272), 1,
      sym_identifier,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [24251] = 2,
    ACTIONS(2274), 1,
      ts_builtin_sym_end,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [24259] = 2,
    ACTIONS(2276), 1,
      anon_sym_RPAREN2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [24267] = 2,
    ACTIONS(2278), 1,
      anon_sym_COMMA2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
  [24275] = 2,
    ACTIONS(2280), 1,
      anon_sym_RBRACE2,
    ACTIONS(5), 2,
      sym_block_comment,
      sym_line_comment,
};

static const uint32_t ts_small_parse_table_map[] = {
  [SMALL_STATE(4)] = 0,
  [SMALL_STATE(5)] = 111,
  [SMALL_STATE(6)] = 222,
  [SMALL_STATE(7)] = 333,
  [SMALL_STATE(8)] = 444,
  [SMALL_STATE(9)] = 555,
  [SMALL_STATE(10)] = 666,
  [SMALL_STATE(11)] = 777,
  [SMALL_STATE(12)] = 888,
  [SMALL_STATE(13)] = 996,
  [SMALL_STATE(14)] = 1101,
  [SMALL_STATE(15)] = 1206,
  [SMALL_STATE(16)] = 1258,
  [SMALL_STATE(17)] = 1310,
  [SMALL_STATE(18)] = 1362,
  [SMALL_STATE(19)] = 1414,
  [SMALL_STATE(20)] = 1466,
  [SMALL_STATE(21)] = 1515,
  [SMALL_STATE(22)] = 1564,
  [SMALL_STATE(23)] = 1613,
  [SMALL_STATE(24)] = 1662,
  [SMALL_STATE(25)] = 1711,
  [SMALL_STATE(26)] = 1760,
  [SMALL_STATE(27)] = 1809,
  [SMALL_STATE(28)] = 1858,
  [SMALL_STATE(29)] = 1907,
  [SMALL_STATE(30)] = 1956,
  [SMALL_STATE(31)] = 2005,
  [SMALL_STATE(32)] = 2054,
  [SMALL_STATE(33)] = 2103,
  [SMALL_STATE(34)] = 2152,
  [SMALL_STATE(35)] = 2201,
  [SMALL_STATE(36)] = 2250,
  [SMALL_STATE(37)] = 2299,
  [SMALL_STATE(38)] = 2348,
  [SMALL_STATE(39)] = 2397,
  [SMALL_STATE(40)] = 2446,
  [SMALL_STATE(41)] = 2495,
  [SMALL_STATE(42)] = 2544,
  [SMALL_STATE(43)] = 2593,
  [SMALL_STATE(44)] = 2642,
  [SMALL_STATE(45)] = 2691,
  [SMALL_STATE(46)] = 2740,
  [SMALL_STATE(47)] = 2789,
  [SMALL_STATE(48)] = 2838,
  [SMALL_STATE(49)] = 2887,
  [SMALL_STATE(50)] = 2936,
  [SMALL_STATE(51)] = 2985,
  [SMALL_STATE(52)] = 3034,
  [SMALL_STATE(53)] = 3083,
  [SMALL_STATE(54)] = 3132,
  [SMALL_STATE(55)] = 3181,
  [SMALL_STATE(56)] = 3230,
  [SMALL_STATE(57)] = 3279,
  [SMALL_STATE(58)] = 3328,
  [SMALL_STATE(59)] = 3377,
  [SMALL_STATE(60)] = 3426,
  [SMALL_STATE(61)] = 3475,
  [SMALL_STATE(62)] = 3524,
  [SMALL_STATE(63)] = 3573,
  [SMALL_STATE(64)] = 3622,
  [SMALL_STATE(65)] = 3671,
  [SMALL_STATE(66)] = 3720,
  [SMALL_STATE(67)] = 3769,
  [SMALL_STATE(68)] = 3818,
  [SMALL_STATE(69)] = 3867,
  [SMALL_STATE(70)] = 3916,
  [SMALL_STATE(71)] = 3965,
  [SMALL_STATE(72)] = 4014,
  [SMALL_STATE(73)] = 4099,
  [SMALL_STATE(74)] = 4180,
  [SMALL_STATE(75)] = 4260,
  [SMALL_STATE(76)] = 4342,
  [SMALL_STATE(77)] = 4422,
  [SMALL_STATE(78)] = 4500,
  [SMALL_STATE(79)] = 4582,
  [SMALL_STATE(80)] = 4662,
  [SMALL_STATE(81)] = 4742,
  [SMALL_STATE(82)] = 4822,
  [SMALL_STATE(83)] = 4902,
  [SMALL_STATE(84)] = 4984,
  [SMALL_STATE(85)] = 5064,
  [SMALL_STATE(86)] = 5144,
  [SMALL_STATE(87)] = 5224,
  [SMALL_STATE(88)] = 5306,
  [SMALL_STATE(89)] = 5384,
  [SMALL_STATE(90)] = 5466,
  [SMALL_STATE(91)] = 5541,
  [SMALL_STATE(92)] = 5616,
  [SMALL_STATE(93)] = 5691,
  [SMALL_STATE(94)] = 5770,
  [SMALL_STATE(95)] = 5846,
  [SMALL_STATE(96)] = 5919,
  [SMALL_STATE(97)] = 5992,
  [SMALL_STATE(98)] = 6065,
  [SMALL_STATE(99)] = 6134,
  [SMALL_STATE(100)] = 6207,
  [SMALL_STATE(101)] = 6275,
  [SMALL_STATE(102)] = 6343,
  [SMALL_STATE(103)] = 6411,
  [SMALL_STATE(104)] = 6479,
  [SMALL_STATE(105)] = 6540,
  [SMALL_STATE(106)] = 6601,
  [SMALL_STATE(107)] = 6676,
  [SMALL_STATE(108)] = 6737,
  [SMALL_STATE(109)] = 6798,
  [SMALL_STATE(110)] = 6873,
  [SMALL_STATE(111)] = 6937,
  [SMALL_STATE(112)] = 6999,
  [SMALL_STATE(113)] = 7059,
  [SMALL_STATE(114)] = 7123,
  [SMALL_STATE(115)] = 7187,
  [SMALL_STATE(116)] = 7247,
  [SMALL_STATE(117)] = 7311,
  [SMALL_STATE(118)] = 7371,
  [SMALL_STATE(119)] = 7435,
  [SMALL_STATE(120)] = 7499,
  [SMALL_STATE(121)] = 7559,
  [SMALL_STATE(122)] = 7619,
  [SMALL_STATE(123)] = 7679,
  [SMALL_STATE(124)] = 7740,
  [SMALL_STATE(125)] = 7801,
  [SMALL_STATE(126)] = 7840,
  [SMALL_STATE(127)] = 7899,
  [SMALL_STATE(128)] = 7958,
  [SMALL_STATE(129)] = 8027,
  [SMALL_STATE(130)] = 8066,
  [SMALL_STATE(131)] = 8135,
  [SMALL_STATE(132)] = 8196,
  [SMALL_STATE(133)] = 8257,
  [SMALL_STATE(134)] = 8316,
  [SMALL_STATE(135)] = 8375,
  [SMALL_STATE(136)] = 8434,
  [SMALL_STATE(137)] = 8493,
  [SMALL_STATE(138)] = 8552,
  [SMALL_STATE(139)] = 8611,
  [SMALL_STATE(140)] = 8670,
  [SMALL_STATE(141)] = 8729,
  [SMALL_STATE(142)] = 8788,
  [SMALL_STATE(143)] = 8849,
  [SMALL_STATE(144)] = 8918,
  [SMALL_STATE(145)] = 8977,
  [SMALL_STATE(146)] = 9043,
  [SMALL_STATE(147)] = 9099,
  [SMALL_STATE(148)] = 9157,
  [SMALL_STATE(149)] = 9223,
  [SMALL_STATE(150)] = 9289,
  [SMALL_STATE(151)] = 9347,
  [SMALL_STATE(152)] = 9413,
  [SMALL_STATE(153)] = 9479,
  [SMALL_STATE(154)] = 9545,
  [SMALL_STATE(155)] = 9598,
  [SMALL_STATE(156)] = 9631,
  [SMALL_STATE(157)] = 9664,
  [SMALL_STATE(158)] = 9727,
  [SMALL_STATE(159)] = 9760,
  [SMALL_STATE(160)] = 9793,
  [SMALL_STATE(161)] = 9848,
  [SMALL_STATE(162)] = 9881,
  [SMALL_STATE(163)] = 9944,
  [SMALL_STATE(164)] = 9977,
  [SMALL_STATE(165)] = 10010,
  [SMALL_STATE(166)] = 10043,
  [SMALL_STATE(167)] = 10076,
  [SMALL_STATE(168)] = 10109,
  [SMALL_STATE(169)] = 10166,
  [SMALL_STATE(170)] = 10199,
  [SMALL_STATE(171)] = 10232,
  [SMALL_STATE(172)] = 10284,
  [SMALL_STATE(173)] = 10336,
  [SMALL_STATE(174)] = 10396,
  [SMALL_STATE(175)] = 10448,
  [SMALL_STATE(176)] = 10500,
  [SMALL_STATE(177)] = 10552,
  [SMALL_STATE(178)] = 10604,
  [SMALL_STATE(179)] = 10656,
  [SMALL_STATE(180)] = 10708,
  [SMALL_STATE(181)] = 10760,
  [SMALL_STATE(182)] = 10812,
  [SMALL_STATE(183)] = 10872,
  [SMALL_STATE(184)] = 10932,
  [SMALL_STATE(185)] = 10992,
  [SMALL_STATE(186)] = 11041,
  [SMALL_STATE(187)] = 11080,
  [SMALL_STATE(188)] = 11129,
  [SMALL_STATE(189)] = 11160,
  [SMALL_STATE(190)] = 11209,
  [SMALL_STATE(191)] = 11258,
  [SMALL_STATE(192)] = 11307,
  [SMALL_STATE(193)] = 11356,
  [SMALL_STATE(194)] = 11405,
  [SMALL_STATE(195)] = 11454,
  [SMALL_STATE(196)] = 11503,
  [SMALL_STATE(197)] = 11552,
  [SMALL_STATE(198)] = 11601,
  [SMALL_STATE(199)] = 11650,
  [SMALL_STATE(200)] = 11699,
  [SMALL_STATE(201)] = 11748,
  [SMALL_STATE(202)] = 11797,
  [SMALL_STATE(203)] = 11846,
  [SMALL_STATE(204)] = 11895,
  [SMALL_STATE(205)] = 11944,
  [SMALL_STATE(206)] = 11993,
  [SMALL_STATE(207)] = 12042,
  [SMALL_STATE(208)] = 12091,
  [SMALL_STATE(209)] = 12140,
  [SMALL_STATE(210)] = 12189,
  [SMALL_STATE(211)] = 12238,
  [SMALL_STATE(212)] = 12287,
  [SMALL_STATE(213)] = 12336,
  [SMALL_STATE(214)] = 12385,
  [SMALL_STATE(215)] = 12434,
  [SMALL_STATE(216)] = 12483,
  [SMALL_STATE(217)] = 12532,
  [SMALL_STATE(218)] = 12581,
  [SMALL_STATE(219)] = 12620,
  [SMALL_STATE(220)] = 12669,
  [SMALL_STATE(221)] = 12718,
  [SMALL_STATE(222)] = 12767,
  [SMALL_STATE(223)] = 12816,
  [SMALL_STATE(224)] = 12865,
  [SMALL_STATE(225)] = 12914,
  [SMALL_STATE(226)] = 12945,
  [SMALL_STATE(227)] = 12994,
  [SMALL_STATE(228)] = 13043,
  [SMALL_STATE(229)] = 13092,
  [SMALL_STATE(230)] = 13141,
  [SMALL_STATE(231)] = 13190,
  [SMALL_STATE(232)] = 13239,
  [SMALL_STATE(233)] = 13288,
  [SMALL_STATE(234)] = 13337,
  [SMALL_STATE(235)] = 13386,
  [SMALL_STATE(236)] = 13435,
  [SMALL_STATE(237)] = 13484,
  [SMALL_STATE(238)] = 13533,
  [SMALL_STATE(239)] = 13582,
  [SMALL_STATE(240)] = 13631,
  [SMALL_STATE(241)] = 13680,
  [SMALL_STATE(242)] = 13710,
  [SMALL_STATE(243)] = 13740,
  [SMALL_STATE(244)] = 13774,
  [SMALL_STATE(245)] = 13808,
  [SMALL_STATE(246)] = 13838,
  [SMALL_STATE(247)] = 13871,
  [SMALL_STATE(248)] = 13904,
  [SMALL_STATE(249)] = 13949,
  [SMALL_STATE(250)] = 13985,
  [SMALL_STATE(251)] = 14015,
  [SMALL_STATE(252)] = 14043,
  [SMALL_STATE(253)] = 14071,
  [SMALL_STATE(254)] = 14107,
  [SMALL_STATE(255)] = 14139,
  [SMALL_STATE(256)] = 14175,
  [SMALL_STATE(257)] = 14207,
  [SMALL_STATE(258)] = 14237,
  [SMALL_STATE(259)] = 14291,
  [SMALL_STATE(260)] = 14319,
  [SMALL_STATE(261)] = 14363,
  [SMALL_STATE(262)] = 14391,
  [SMALL_STATE(263)] = 14419,
  [SMALL_STATE(264)] = 14453,
  [SMALL_STATE(265)] = 14485,
  [SMALL_STATE(266)] = 14515,
  [SMALL_STATE(267)] = 14553,
  [SMALL_STATE(268)] = 14581,
  [SMALL_STATE(269)] = 14609,
  [SMALL_STATE(270)] = 14639,
  [SMALL_STATE(271)] = 14675,
  [SMALL_STATE(272)] = 14703,
  [SMALL_STATE(273)] = 14731,
  [SMALL_STATE(274)] = 14759,
  [SMALL_STATE(275)] = 14787,
  [SMALL_STATE(276)] = 14815,
  [SMALL_STATE(277)] = 14849,
  [SMALL_STATE(278)] = 14879,
  [SMALL_STATE(279)] = 14909,
  [SMALL_STATE(280)] = 14952,
  [SMALL_STATE(281)] = 14993,
  [SMALL_STATE(282)] = 15032,
  [SMALL_STATE(283)] = 15058,
  [SMALL_STATE(284)] = 15084,
  [SMALL_STATE(285)] = 15110,
  [SMALL_STATE(286)] = 15136,
  [SMALL_STATE(287)] = 15162,
  [SMALL_STATE(288)] = 15188,
  [SMALL_STATE(289)] = 15222,
  [SMALL_STATE(290)] = 15254,
  [SMALL_STATE(291)] = 15282,
  [SMALL_STATE(292)] = 15310,
  [SMALL_STATE(293)] = 15346,
  [SMALL_STATE(294)] = 15372,
  [SMALL_STATE(295)] = 15398,
  [SMALL_STATE(296)] = 15424,
  [SMALL_STATE(297)] = 15450,
  [SMALL_STATE(298)] = 15488,
  [SMALL_STATE(299)] = 15526,
  [SMALL_STATE(300)] = 15552,
  [SMALL_STATE(301)] = 15582,
  [SMALL_STATE(302)] = 15608,
  [SMALL_STATE(303)] = 15636,
  [SMALL_STATE(304)] = 15674,
  [SMALL_STATE(305)] = 15704,
  [SMALL_STATE(306)] = 15730,
  [SMALL_STATE(307)] = 15755,
  [SMALL_STATE(308)] = 15794,
  [SMALL_STATE(309)] = 15829,
  [SMALL_STATE(310)] = 15864,
  [SMALL_STATE(311)] = 15899,
  [SMALL_STATE(312)] = 15934,
  [SMALL_STATE(313)] = 15969,
  [SMALL_STATE(314)] = 15994,
  [SMALL_STATE(315)] = 16019,
  [SMALL_STATE(316)] = 16054,
  [SMALL_STATE(317)] = 16079,
  [SMALL_STATE(318)] = 16118,
  [SMALL_STATE(319)] = 16153,
  [SMALL_STATE(320)] = 16184,
  [SMALL_STATE(321)] = 16215,
  [SMALL_STATE(322)] = 16250,
  [SMALL_STATE(323)] = 16276,
  [SMALL_STATE(324)] = 16308,
  [SMALL_STATE(325)] = 16332,
  [SMALL_STATE(326)] = 16370,
  [SMALL_STATE(327)] = 16394,
  [SMALL_STATE(328)] = 16422,
  [SMALL_STATE(329)] = 16458,
  [SMALL_STATE(330)] = 16488,
  [SMALL_STATE(331)] = 16526,
  [SMALL_STATE(332)] = 16552,
  [SMALL_STATE(333)] = 16578,
  [SMALL_STATE(334)] = 16606,
  [SMALL_STATE(335)] = 16641,
  [SMALL_STATE(336)] = 16670,
  [SMALL_STATE(337)] = 16703,
  [SMALL_STATE(338)] = 16736,
  [SMALL_STATE(339)] = 16769,
  [SMALL_STATE(340)] = 16798,
  [SMALL_STATE(341)] = 16833,
  [SMALL_STATE(342)] = 16855,
  [SMALL_STATE(343)] = 16891,
  [SMALL_STATE(344)] = 16919,
  [SMALL_STATE(345)] = 16953,
  [SMALL_STATE(346)] = 16977,
  [SMALL_STATE(347)] = 16999,
  [SMALL_STATE(348)] = 17021,
  [SMALL_STATE(349)] = 17043,
  [SMALL_STATE(350)] = 17065,
  [SMALL_STATE(351)] = 17087,
  [SMALL_STATE(352)] = 17115,
  [SMALL_STATE(353)] = 17141,
  [SMALL_STATE(354)] = 17165,
  [SMALL_STATE(355)] = 17189,
  [SMALL_STATE(356)] = 17223,
  [SMALL_STATE(357)] = 17245,
  [SMALL_STATE(358)] = 17281,
  [SMALL_STATE(359)] = 17315,
  [SMALL_STATE(360)] = 17343,
  [SMALL_STATE(361)] = 17379,
  [SMALL_STATE(362)] = 17411,
  [SMALL_STATE(363)] = 17447,
  [SMALL_STATE(364)] = 17478,
  [SMALL_STATE(365)] = 17511,
  [SMALL_STATE(366)] = 17542,
  [SMALL_STATE(367)] = 17573,
  [SMALL_STATE(368)] = 17604,
  [SMALL_STATE(369)] = 17635,
  [SMALL_STATE(370)] = 17666,
  [SMALL_STATE(371)] = 17697,
  [SMALL_STATE(372)] = 17728,
  [SMALL_STATE(373)] = 17749,
  [SMALL_STATE(374)] = 17782,
  [SMALL_STATE(375)] = 17813,
  [SMALL_STATE(376)] = 17846,
  [SMALL_STATE(377)] = 17877,
  [SMALL_STATE(378)] = 17907,
  [SMALL_STATE(379)] = 17937,
  [SMALL_STATE(380)] = 17967,
  [SMALL_STATE(381)] = 17997,
  [SMALL_STATE(382)] = 18027,
  [SMALL_STATE(383)] = 18057,
  [SMALL_STATE(384)] = 18087,
  [SMALL_STATE(385)] = 18117,
  [SMALL_STATE(386)] = 18145,
  [SMALL_STATE(387)] = 18169,
  [SMALL_STATE(388)] = 18199,
  [SMALL_STATE(389)] = 18229,
  [SMALL_STATE(390)] = 18260,
  [SMALL_STATE(391)] = 18287,
  [SMALL_STATE(392)] = 18314,
  [SMALL_STATE(393)] = 18341,
  [SMALL_STATE(394)] = 18369,
  [SMALL_STATE(395)] = 18389,
  [SMALL_STATE(396)] = 18407,
  [SMALL_STATE(397)] = 18425,
  [SMALL_STATE(398)] = 18443,
  [SMALL_STATE(399)] = 18461,
  [SMALL_STATE(400)] = 18479,
  [SMALL_STATE(401)] = 18497,
  [SMALL_STATE(402)] = 18525,
  [SMALL_STATE(403)] = 18543,
  [SMALL_STATE(404)] = 18561,
  [SMALL_STATE(405)] = 18579,
  [SMALL_STATE(406)] = 18597,
  [SMALL_STATE(407)] = 18615,
  [SMALL_STATE(408)] = 18643,
  [SMALL_STATE(409)] = 18671,
  [SMALL_STATE(410)] = 18689,
  [SMALL_STATE(411)] = 18707,
  [SMALL_STATE(412)] = 18729,
  [SMALL_STATE(413)] = 18749,
  [SMALL_STATE(414)] = 18768,
  [SMALL_STATE(415)] = 18799,
  [SMALL_STATE(416)] = 18818,
  [SMALL_STATE(417)] = 18839,
  [SMALL_STATE(418)] = 18858,
  [SMALL_STATE(419)] = 18877,
  [SMALL_STATE(420)] = 18896,
  [SMALL_STATE(421)] = 18915,
  [SMALL_STATE(422)] = 18940,
  [SMALL_STATE(423)] = 18957,
  [SMALL_STATE(424)] = 18979,
  [SMALL_STATE(425)] = 18993,
  [SMALL_STATE(426)] = 19021,
  [SMALL_STATE(427)] = 19035,
  [SMALL_STATE(428)] = 19053,
  [SMALL_STATE(429)] = 19075,
  [SMALL_STATE(430)] = 19089,
  [SMALL_STATE(431)] = 19111,
  [SMALL_STATE(432)] = 19125,
  [SMALL_STATE(433)] = 19141,
  [SMALL_STATE(434)] = 19155,
  [SMALL_STATE(435)] = 19177,
  [SMALL_STATE(436)] = 19191,
  [SMALL_STATE(437)] = 19207,
  [SMALL_STATE(438)] = 19225,
  [SMALL_STATE(439)] = 19241,
  [SMALL_STATE(440)] = 19259,
  [SMALL_STATE(441)] = 19276,
  [SMALL_STATE(442)] = 19291,
  [SMALL_STATE(443)] = 19306,
  [SMALL_STATE(444)] = 19319,
  [SMALL_STATE(445)] = 19334,
  [SMALL_STATE(446)] = 19357,
  [SMALL_STATE(447)] = 19372,
  [SMALL_STATE(448)] = 19385,
  [SMALL_STATE(449)] = 19400,
  [SMALL_STATE(450)] = 19413,
  [SMALL_STATE(451)] = 19428,
  [SMALL_STATE(452)] = 19443,
  [SMALL_STATE(453)] = 19458,
  [SMALL_STATE(454)] = 19471,
  [SMALL_STATE(455)] = 19492,
  [SMALL_STATE(456)] = 19507,
  [SMALL_STATE(457)] = 19522,
  [SMALL_STATE(458)] = 19543,
  [SMALL_STATE(459)] = 19564,
  [SMALL_STATE(460)] = 19585,
  [SMALL_STATE(461)] = 19608,
  [SMALL_STATE(462)] = 19625,
  [SMALL_STATE(463)] = 19644,
  [SMALL_STATE(464)] = 19665,
  [SMALL_STATE(465)] = 19686,
  [SMALL_STATE(466)] = 19705,
  [SMALL_STATE(467)] = 19726,
  [SMALL_STATE(468)] = 19741,
  [SMALL_STATE(469)] = 19756,
  [SMALL_STATE(470)] = 19771,
  [SMALL_STATE(471)] = 19789,
  [SMALL_STATE(472)] = 19809,
  [SMALL_STATE(473)] = 19829,
  [SMALL_STATE(474)] = 19845,
  [SMALL_STATE(475)] = 19861,
  [SMALL_STATE(476)] = 19877,
  [SMALL_STATE(477)] = 19897,
  [SMALL_STATE(478)] = 19913,
  [SMALL_STATE(479)] = 19929,
  [SMALL_STATE(480)] = 19949,
  [SMALL_STATE(481)] = 19965,
  [SMALL_STATE(482)] = 19981,
  [SMALL_STATE(483)] = 19997,
  [SMALL_STATE(484)] = 20013,
  [SMALL_STATE(485)] = 20033,
  [SMALL_STATE(486)] = 20053,
  [SMALL_STATE(487)] = 20071,
  [SMALL_STATE(488)] = 20087,
  [SMALL_STATE(489)] = 20099,
  [SMALL_STATE(490)] = 20114,
  [SMALL_STATE(491)] = 20127,
  [SMALL_STATE(492)] = 20138,
  [SMALL_STATE(493)] = 20153,
  [SMALL_STATE(494)] = 20164,
  [SMALL_STATE(495)] = 20177,
  [SMALL_STATE(496)] = 20188,
  [SMALL_STATE(497)] = 20201,
  [SMALL_STATE(498)] = 20212,
  [SMALL_STATE(499)] = 20223,
  [SMALL_STATE(500)] = 20240,
  [SMALL_STATE(501)] = 20255,
  [SMALL_STATE(502)] = 20272,
  [SMALL_STATE(503)] = 20283,
  [SMALL_STATE(504)] = 20294,
  [SMALL_STATE(505)] = 20309,
  [SMALL_STATE(506)] = 20320,
  [SMALL_STATE(507)] = 20331,
  [SMALL_STATE(508)] = 20348,
  [SMALL_STATE(509)] = 20363,
  [SMALL_STATE(510)] = 20380,
  [SMALL_STATE(511)] = 20395,
  [SMALL_STATE(512)] = 20406,
  [SMALL_STATE(513)] = 20421,
  [SMALL_STATE(514)] = 20432,
  [SMALL_STATE(515)] = 20447,
  [SMALL_STATE(516)] = 20464,
  [SMALL_STATE(517)] = 20481,
  [SMALL_STATE(518)] = 20496,
  [SMALL_STATE(519)] = 20513,
  [SMALL_STATE(520)] = 20528,
  [SMALL_STATE(521)] = 20543,
  [SMALL_STATE(522)] = 20558,
  [SMALL_STATE(523)] = 20575,
  [SMALL_STATE(524)] = 20592,
  [SMALL_STATE(525)] = 20603,
  [SMALL_STATE(526)] = 20620,
  [SMALL_STATE(527)] = 20635,
  [SMALL_STATE(528)] = 20650,
  [SMALL_STATE(529)] = 20667,
  [SMALL_STATE(530)] = 20678,
  [SMALL_STATE(531)] = 20689,
  [SMALL_STATE(532)] = 20706,
  [SMALL_STATE(533)] = 20721,
  [SMALL_STATE(534)] = 20736,
  [SMALL_STATE(535)] = 20753,
  [SMALL_STATE(536)] = 20770,
  [SMALL_STATE(537)] = 20787,
  [SMALL_STATE(538)] = 20804,
  [SMALL_STATE(539)] = 20819,
  [SMALL_STATE(540)] = 20834,
  [SMALL_STATE(541)] = 20845,
  [SMALL_STATE(542)] = 20856,
  [SMALL_STATE(543)] = 20870,
  [SMALL_STATE(544)] = 20884,
  [SMALL_STATE(545)] = 20898,
  [SMALL_STATE(546)] = 20912,
  [SMALL_STATE(547)] = 20926,
  [SMALL_STATE(548)] = 20940,
  [SMALL_STATE(549)] = 20954,
  [SMALL_STATE(550)] = 20968,
  [SMALL_STATE(551)] = 20978,
  [SMALL_STATE(552)] = 20992,
  [SMALL_STATE(553)] = 21006,
  [SMALL_STATE(554)] = 21018,
  [SMALL_STATE(555)] = 21032,
  [SMALL_STATE(556)] = 21046,
  [SMALL_STATE(557)] = 21060,
  [SMALL_STATE(558)] = 21074,
  [SMALL_STATE(559)] = 21088,
  [SMALL_STATE(560)] = 21102,
  [SMALL_STATE(561)] = 21116,
  [SMALL_STATE(562)] = 21130,
  [SMALL_STATE(563)] = 21144,
  [SMALL_STATE(564)] = 21158,
  [SMALL_STATE(565)] = 21172,
  [SMALL_STATE(566)] = 21186,
  [SMALL_STATE(567)] = 21200,
  [SMALL_STATE(568)] = 21214,
  [SMALL_STATE(569)] = 21228,
  [SMALL_STATE(570)] = 21242,
  [SMALL_STATE(571)] = 21256,
  [SMALL_STATE(572)] = 21270,
  [SMALL_STATE(573)] = 21284,
  [SMALL_STATE(574)] = 21298,
  [SMALL_STATE(575)] = 21312,
  [SMALL_STATE(576)] = 21326,
  [SMALL_STATE(577)] = 21340,
  [SMALL_STATE(578)] = 21354,
  [SMALL_STATE(579)] = 21368,
  [SMALL_STATE(580)] = 21382,
  [SMALL_STATE(581)] = 21396,
  [SMALL_STATE(582)] = 21410,
  [SMALL_STATE(583)] = 21424,
  [SMALL_STATE(584)] = 21438,
  [SMALL_STATE(585)] = 21448,
  [SMALL_STATE(586)] = 21462,
  [SMALL_STATE(587)] = 21476,
  [SMALL_STATE(588)] = 21490,
  [SMALL_STATE(589)] = 21502,
  [SMALL_STATE(590)] = 21516,
  [SMALL_STATE(591)] = 21530,
  [SMALL_STATE(592)] = 21544,
  [SMALL_STATE(593)] = 21558,
  [SMALL_STATE(594)] = 21568,
  [SMALL_STATE(595)] = 21580,
  [SMALL_STATE(596)] = 21594,
  [SMALL_STATE(597)] = 21608,
  [SMALL_STATE(598)] = 21620,
  [SMALL_STATE(599)] = 21632,
  [SMALL_STATE(600)] = 21644,
  [SMALL_STATE(601)] = 21658,
  [SMALL_STATE(602)] = 21672,
  [SMALL_STATE(603)] = 21686,
  [SMALL_STATE(604)] = 21700,
  [SMALL_STATE(605)] = 21714,
  [SMALL_STATE(606)] = 21728,
  [SMALL_STATE(607)] = 21742,
  [SMALL_STATE(608)] = 21752,
  [SMALL_STATE(609)] = 21762,
  [SMALL_STATE(610)] = 21774,
  [SMALL_STATE(611)] = 21788,
  [SMALL_STATE(612)] = 21802,
  [SMALL_STATE(613)] = 21812,
  [SMALL_STATE(614)] = 21822,
  [SMALL_STATE(615)] = 21834,
  [SMALL_STATE(616)] = 21848,
  [SMALL_STATE(617)] = 21862,
  [SMALL_STATE(618)] = 21876,
  [SMALL_STATE(619)] = 21890,
  [SMALL_STATE(620)] = 21904,
  [SMALL_STATE(621)] = 21918,
  [SMALL_STATE(622)] = 21932,
  [SMALL_STATE(623)] = 21946,
  [SMALL_STATE(624)] = 21958,
  [SMALL_STATE(625)] = 21972,
  [SMALL_STATE(626)] = 21984,
  [SMALL_STATE(627)] = 21994,
  [SMALL_STATE(628)] = 22008,
  [SMALL_STATE(629)] = 22022,
  [SMALL_STATE(630)] = 22036,
  [SMALL_STATE(631)] = 22050,
  [SMALL_STATE(632)] = 22064,
  [SMALL_STATE(633)] = 22078,
  [SMALL_STATE(634)] = 22092,
  [SMALL_STATE(635)] = 22102,
  [SMALL_STATE(636)] = 22116,
  [SMALL_STATE(637)] = 22130,
  [SMALL_STATE(638)] = 22139,
  [SMALL_STATE(639)] = 22150,
  [SMALL_STATE(640)] = 22159,
  [SMALL_STATE(641)] = 22168,
  [SMALL_STATE(642)] = 22179,
  [SMALL_STATE(643)] = 22188,
  [SMALL_STATE(644)] = 22197,
  [SMALL_STATE(645)] = 22206,
  [SMALL_STATE(646)] = 22215,
  [SMALL_STATE(647)] = 22224,
  [SMALL_STATE(648)] = 22235,
  [SMALL_STATE(649)] = 22244,
  [SMALL_STATE(650)] = 22253,
  [SMALL_STATE(651)] = 22264,
  [SMALL_STATE(652)] = 22273,
  [SMALL_STATE(653)] = 22282,
  [SMALL_STATE(654)] = 22291,
  [SMALL_STATE(655)] = 22300,
  [SMALL_STATE(656)] = 22311,
  [SMALL_STATE(657)] = 22322,
  [SMALL_STATE(658)] = 22333,
  [SMALL_STATE(659)] = 22342,
  [SMALL_STATE(660)] = 22351,
  [SMALL_STATE(661)] = 22362,
  [SMALL_STATE(662)] = 22373,
  [SMALL_STATE(663)] = 22382,
  [SMALL_STATE(664)] = 22391,
  [SMALL_STATE(665)] = 22402,
  [SMALL_STATE(666)] = 22411,
  [SMALL_STATE(667)] = 22422,
  [SMALL_STATE(668)] = 22431,
  [SMALL_STATE(669)] = 22440,
  [SMALL_STATE(670)] = 22449,
  [SMALL_STATE(671)] = 22460,
  [SMALL_STATE(672)] = 22469,
  [SMALL_STATE(673)] = 22480,
  [SMALL_STATE(674)] = 22489,
  [SMALL_STATE(675)] = 22500,
  [SMALL_STATE(676)] = 22509,
  [SMALL_STATE(677)] = 22518,
  [SMALL_STATE(678)] = 22529,
  [SMALL_STATE(679)] = 22538,
  [SMALL_STATE(680)] = 22549,
  [SMALL_STATE(681)] = 22558,
  [SMALL_STATE(682)] = 22567,
  [SMALL_STATE(683)] = 22576,
  [SMALL_STATE(684)] = 22585,
  [SMALL_STATE(685)] = 22594,
  [SMALL_STATE(686)] = 22605,
  [SMALL_STATE(687)] = 22616,
  [SMALL_STATE(688)] = 22625,
  [SMALL_STATE(689)] = 22634,
  [SMALL_STATE(690)] = 22643,
  [SMALL_STATE(691)] = 22652,
  [SMALL_STATE(692)] = 22661,
  [SMALL_STATE(693)] = 22670,
  [SMALL_STATE(694)] = 22681,
  [SMALL_STATE(695)] = 22692,
  [SMALL_STATE(696)] = 22703,
  [SMALL_STATE(697)] = 22712,
  [SMALL_STATE(698)] = 22723,
  [SMALL_STATE(699)] = 22734,
  [SMALL_STATE(700)] = 22743,
  [SMALL_STATE(701)] = 22754,
  [SMALL_STATE(702)] = 22763,
  [SMALL_STATE(703)] = 22772,
  [SMALL_STATE(704)] = 22781,
  [SMALL_STATE(705)] = 22790,
  [SMALL_STATE(706)] = 22801,
  [SMALL_STATE(707)] = 22812,
  [SMALL_STATE(708)] = 22821,
  [SMALL_STATE(709)] = 22830,
  [SMALL_STATE(710)] = 22839,
  [SMALL_STATE(711)] = 22848,
  [SMALL_STATE(712)] = 22859,
  [SMALL_STATE(713)] = 22868,
  [SMALL_STATE(714)] = 22879,
  [SMALL_STATE(715)] = 22887,
  [SMALL_STATE(716)] = 22895,
  [SMALL_STATE(717)] = 22903,
  [SMALL_STATE(718)] = 22911,
  [SMALL_STATE(719)] = 22919,
  [SMALL_STATE(720)] = 22927,
  [SMALL_STATE(721)] = 22935,
  [SMALL_STATE(722)] = 22943,
  [SMALL_STATE(723)] = 22951,
  [SMALL_STATE(724)] = 22959,
  [SMALL_STATE(725)] = 22967,
  [SMALL_STATE(726)] = 22975,
  [SMALL_STATE(727)] = 22983,
  [SMALL_STATE(728)] = 22991,
  [SMALL_STATE(729)] = 22999,
  [SMALL_STATE(730)] = 23007,
  [SMALL_STATE(731)] = 23015,
  [SMALL_STATE(732)] = 23025,
  [SMALL_STATE(733)] = 23033,
  [SMALL_STATE(734)] = 23041,
  [SMALL_STATE(735)] = 23049,
  [SMALL_STATE(736)] = 23057,
  [SMALL_STATE(737)] = 23065,
  [SMALL_STATE(738)] = 23073,
  [SMALL_STATE(739)] = 23081,
  [SMALL_STATE(740)] = 23089,
  [SMALL_STATE(741)] = 23097,
  [SMALL_STATE(742)] = 23105,
  [SMALL_STATE(743)] = 23115,
  [SMALL_STATE(744)] = 23123,
  [SMALL_STATE(745)] = 23131,
  [SMALL_STATE(746)] = 23139,
  [SMALL_STATE(747)] = 23147,
  [SMALL_STATE(748)] = 23155,
  [SMALL_STATE(749)] = 23163,
  [SMALL_STATE(750)] = 23171,
  [SMALL_STATE(751)] = 23179,
  [SMALL_STATE(752)] = 23189,
  [SMALL_STATE(753)] = 23197,
  [SMALL_STATE(754)] = 23205,
  [SMALL_STATE(755)] = 23213,
  [SMALL_STATE(756)] = 23221,
  [SMALL_STATE(757)] = 23229,
  [SMALL_STATE(758)] = 23237,
  [SMALL_STATE(759)] = 23245,
  [SMALL_STATE(760)] = 23253,
  [SMALL_STATE(761)] = 23261,
  [SMALL_STATE(762)] = 23269,
  [SMALL_STATE(763)] = 23277,
  [SMALL_STATE(764)] = 23285,
  [SMALL_STATE(765)] = 23293,
  [SMALL_STATE(766)] = 23301,
  [SMALL_STATE(767)] = 23309,
  [SMALL_STATE(768)] = 23317,
  [SMALL_STATE(769)] = 23327,
  [SMALL_STATE(770)] = 23335,
  [SMALL_STATE(771)] = 23343,
  [SMALL_STATE(772)] = 23351,
  [SMALL_STATE(773)] = 23359,
  [SMALL_STATE(774)] = 23367,
  [SMALL_STATE(775)] = 23375,
  [SMALL_STATE(776)] = 23383,
  [SMALL_STATE(777)] = 23391,
  [SMALL_STATE(778)] = 23399,
  [SMALL_STATE(779)] = 23407,
  [SMALL_STATE(780)] = 23415,
  [SMALL_STATE(781)] = 23423,
  [SMALL_STATE(782)] = 23431,
  [SMALL_STATE(783)] = 23439,
  [SMALL_STATE(784)] = 23447,
  [SMALL_STATE(785)] = 23455,
  [SMALL_STATE(786)] = 23463,
  [SMALL_STATE(787)] = 23473,
  [SMALL_STATE(788)] = 23481,
  [SMALL_STATE(789)] = 23491,
  [SMALL_STATE(790)] = 23499,
  [SMALL_STATE(791)] = 23507,
  [SMALL_STATE(792)] = 23515,
  [SMALL_STATE(793)] = 23523,
  [SMALL_STATE(794)] = 23531,
  [SMALL_STATE(795)] = 23541,
  [SMALL_STATE(796)] = 23549,
  [SMALL_STATE(797)] = 23557,
  [SMALL_STATE(798)] = 23565,
  [SMALL_STATE(799)] = 23575,
  [SMALL_STATE(800)] = 23583,
  [SMALL_STATE(801)] = 23593,
  [SMALL_STATE(802)] = 23601,
  [SMALL_STATE(803)] = 23609,
  [SMALL_STATE(804)] = 23617,
  [SMALL_STATE(805)] = 23625,
  [SMALL_STATE(806)] = 23633,
  [SMALL_STATE(807)] = 23641,
  [SMALL_STATE(808)] = 23649,
  [SMALL_STATE(809)] = 23657,
  [SMALL_STATE(810)] = 23665,
  [SMALL_STATE(811)] = 23673,
  [SMALL_STATE(812)] = 23681,
  [SMALL_STATE(813)] = 23689,
  [SMALL_STATE(814)] = 23697,
  [SMALL_STATE(815)] = 23705,
  [SMALL_STATE(816)] = 23713,
  [SMALL_STATE(817)] = 23721,
  [SMALL_STATE(818)] = 23729,
  [SMALL_STATE(819)] = 23737,
  [SMALL_STATE(820)] = 23745,
  [SMALL_STATE(821)] = 23753,
  [SMALL_STATE(822)] = 23761,
  [SMALL_STATE(823)] = 23769,
  [SMALL_STATE(824)] = 23777,
  [SMALL_STATE(825)] = 23785,
  [SMALL_STATE(826)] = 23793,
  [SMALL_STATE(827)] = 23801,
  [SMALL_STATE(828)] = 23809,
  [SMALL_STATE(829)] = 23817,
  [SMALL_STATE(830)] = 23825,
  [SMALL_STATE(831)] = 23833,
  [SMALL_STATE(832)] = 23841,
  [SMALL_STATE(833)] = 23849,
  [SMALL_STATE(834)] = 23857,
  [SMALL_STATE(835)] = 23865,
  [SMALL_STATE(836)] = 23873,
  [SMALL_STATE(837)] = 23881,
  [SMALL_STATE(838)] = 23891,
  [SMALL_STATE(839)] = 23899,
  [SMALL_STATE(840)] = 23909,
  [SMALL_STATE(841)] = 23917,
  [SMALL_STATE(842)] = 23925,
  [SMALL_STATE(843)] = 23933,
  [SMALL_STATE(844)] = 23941,
  [SMALL_STATE(845)] = 23949,
  [SMALL_STATE(846)] = 23957,
  [SMALL_STATE(847)] = 23965,
  [SMALL_STATE(848)] = 23973,
  [SMALL_STATE(849)] = 23981,
  [SMALL_STATE(850)] = 23989,
  [SMALL_STATE(851)] = 23999,
  [SMALL_STATE(852)] = 24007,
  [SMALL_STATE(853)] = 24017,
  [SMALL_STATE(854)] = 24025,
  [SMALL_STATE(855)] = 24033,
  [SMALL_STATE(856)] = 24041,
  [SMALL_STATE(857)] = 24049,
  [SMALL_STATE(858)] = 24057,
  [SMALL_STATE(859)] = 24065,
  [SMALL_STATE(860)] = 24073,
  [SMALL_STATE(861)] = 24081,
  [SMALL_STATE(862)] = 24089,
  [SMALL_STATE(863)] = 24097,
  [SMALL_STATE(864)] = 24105,
  [SMALL_STATE(865)] = 24113,
  [SMALL_STATE(866)] = 24121,
  [SMALL_STATE(867)] = 24129,
  [SMALL_STATE(868)] = 24137,
  [SMALL_STATE(869)] = 24145,
  [SMALL_STATE(870)] = 24153,
  [SMALL_STATE(871)] = 24161,
  [SMALL_STATE(872)] = 24169,
  [SMALL_STATE(873)] = 24177,
  [SMALL_STATE(874)] = 24185,
  [SMALL_STATE(875)] = 24193,
  [SMALL_STATE(876)] = 24201,
  [SMALL_STATE(877)] = 24209,
  [SMALL_STATE(878)] = 24217,
  [SMALL_STATE(879)] = 24225,
  [SMALL_STATE(880)] = 24233,
  [SMALL_STATE(881)] = 24243,
  [SMALL_STATE(882)] = 24251,
  [SMALL_STATE(883)] = 24259,
  [SMALL_STATE(884)] = 24267,
  [SMALL_STATE(885)] = 24275,
};

static const TSParseActionEntry ts_parse_actions[] = {
  [0] = {.entry = {.count = 0, .reusable = false}},
  [1] = {.entry = {.count = 1, .reusable = false}}, RECOVER(),
  [3] = {.entry = {.count = 1, .reusable = false}}, SHIFT_EXTRA(),
  [5] = {.entry = {.count = 1, .reusable = true}}, SHIFT_EXTRA(),
  [7] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_source_file, 0, 0, 0),
  [9] = {.entry = {.count = 1, .reusable = true}}, SHIFT(794),
  [11] = {.entry = {.count = 1, .reusable = false}}, SHIFT(218),
  [13] = {.entry = {.count = 1, .reusable = true}}, SHIFT(515),
  [15] = {.entry = {.count = 1, .reusable = true}}, SHIFT(454),
  [17] = {.entry = {.count = 1, .reusable = true}}, SHIFT(89),
  [19] = {.entry = {.count = 1, .reusable = true}}, SHIFT(158),
  [21] = {.entry = {.count = 1, .reusable = false}}, SHIFT(158),
  [23] = {.entry = {.count = 1, .reusable = true}}, SHIFT(820),
  [25] = {.entry = {.count = 1, .reusable = true}}, SHIFT(233),
  [27] = {.entry = {.count = 1, .reusable = true}}, SHIFT(198),
  [29] = {.entry = {.count = 1, .reusable = true}}, SHIFT(199),
  [31] = {.entry = {.count = 1, .reusable = true}}, SHIFT(90),
  [33] = {.entry = {.count = 1, .reusable = true}}, SHIFT(775),
  [35] = {.entry = {.count = 1, .reusable = true}}, SHIFT(426),
  [37] = {.entry = {.count = 1, .reusable = false}}, SHIFT(305),
  [39] = {.entry = {.count = 1, .reusable = true}}, SHIFT(305),
  [41] = {.entry = {.count = 1, .reusable = false}}, SHIFT(790),
  [43] = {.entry = {.count = 1, .reusable = true}}, SHIFT(4),
  [45] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5),
  [47] = {.entry = {.count = 1, .reusable = true}}, SHIFT(807),
  [49] = {.entry = {.count = 1, .reusable = true}}, SHIFT(819),
  [51] = {.entry = {.count = 1, .reusable = true}}, SHIFT(168),
  [53] = {.entry = {.count = 1, .reusable = true}}, SHIFT(618),
  [55] = {.entry = {.count = 1, .reusable = true}}, SHIFT(523),
  [57] = {.entry = {.count = 1, .reusable = true}}, SHIFT(867),
  [59] = {.entry = {.count = 1, .reusable = true}}, SHIFT(763),
  [61] = {.entry = {.count = 1, .reusable = true}}, SHIFT(720),
  [63] = {.entry = {.count = 1, .reusable = true}}, SHIFT(757),
  [65] = {.entry = {.count = 1, .reusable = true}}, SHIFT(549),
  [67] = {.entry = {.count = 1, .reusable = true}}, SHIFT(575),
  [69] = {.entry = {.count = 1, .reusable = true}}, SHIFT(576),
  [71] = {.entry = {.count = 1, .reusable = true}}, SHIFT(822),
  [73] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_source_file, 1, 0, 0),
  [75] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0),
  [77] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(794),
  [80] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(218),
  [83] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(515),
  [86] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(454),
  [89] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(89),
  [92] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(158),
  [95] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(158),
  [98] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(820),
  [101] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(233),
  [104] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(198),
  [107] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(199),
  [110] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(90),
  [113] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(775),
  [116] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(426),
  [119] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(305),
  [122] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(305),
  [125] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(790),
  [128] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(4),
  [131] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(5),
  [134] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(807),
  [137] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(819),
  [140] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(168),
  [143] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(618),
  [146] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(523),
  [149] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(867),
  [152] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(763),
  [155] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(720),
  [158] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(757),
  [161] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(549),
  [164] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(575),
  [167] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(576),
  [170] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_source_file_repeat1, 2, 0, 0), SHIFT_REPEAT(822),
  [173] = {.entry = {.count = 1, .reusable = true}}, SHIFT(818),
  [175] = {.entry = {.count = 1, .reusable = true}}, SHIFT(30),
  [177] = {.entry = {.count = 1, .reusable = true}}, SHIFT(866),
  [179] = {.entry = {.count = 1, .reusable = true}}, SHIFT(34),
  [181] = {.entry = {.count = 1, .reusable = true}}, SHIFT(791),
  [183] = {.entry = {.count = 1, .reusable = true}}, SHIFT(20),
  [185] = {.entry = {.count = 1, .reusable = true}}, SHIFT(52),
  [187] = {.entry = {.count = 1, .reusable = true}}, SHIFT(18),
  [189] = {.entry = {.count = 1, .reusable = true}}, SHIFT(60),
  [191] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_body_repeat1, 2, 0, 0), SHIFT_REPEAT(218),
  [194] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_body_repeat1, 2, 0, 0), SHIFT_REPEAT(515),
  [197] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_body_repeat1, 2, 0, 0), SHIFT_REPEAT(454),
  [200] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_body_repeat1, 2, 0, 0), SHIFT_REPEAT(87),
  [203] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_body_repeat1, 2, 0, 0), SHIFT_REPEAT(158),
  [206] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_body_repeat1, 2, 0, 0), SHIFT_REPEAT(158),
  [209] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_body_repeat1, 2, 0, 0), SHIFT_REPEAT(820),
  [212] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_body_repeat1, 2, 0, 0), SHIFT_REPEAT(233),
  [215] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_body_repeat1, 2, 0, 0), SHIFT_REPEAT(198),
  [218] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_body_repeat1, 2, 0, 0), SHIFT_REPEAT(199),
  [221] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_body_repeat1, 2, 0, 0), SHIFT_REPEAT(90),
  [224] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_body_repeat1, 2, 0, 0), SHIFT_REPEAT(775),
  [227] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_body_repeat1, 2, 0, 0), SHIFT_REPEAT(426),
  [230] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_body_repeat1, 2, 0, 0), SHIFT_REPEAT(305),
  [233] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_body_repeat1, 2, 0, 0), SHIFT_REPEAT(305),
  [236] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_body_repeat1, 2, 0, 0), SHIFT_REPEAT(876),
  [239] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_show_signature, 3, 0, 26),
  [241] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_show_signature, 3, 0, 26),
  [243] = {.entry = {.count = 1, .reusable = true}}, SHIFT(623),
  [245] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_external, 3, 0, 27),
  [247] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_external, 3, 0, 27),
  [249] = {.entry = {.count = 1, .reusable = true}}, SHIFT(194),
  [251] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const, 5, 0, 64),
  [253] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_const, 5, 0, 64),
  [255] = {.entry = {.count = 1, .reusable = true}}, SHIFT(873),
  [257] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_external, 4, 0, 27),
  [259] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_external, 4, 0, 27),
  [261] = {.entry = {.count = 1, .reusable = true}}, SHIFT(236),
  [263] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_external, 5, 0, 63),
  [265] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_external, 5, 0, 63),
  [267] = {.entry = {.count = 1, .reusable = true}}, SHIFT(191),
  [269] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_project_atom, 4, 0, 27),
  [271] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_project_atom, 4, 0, 27),
  [273] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_statement, 1, 2, 0),
  [275] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_statement, 1, 2, 0),
  [277] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_integrity_constraint, 3, 0, 22),
  [279] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_integrity_constraint, 3, 0, 22),
  [281] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_statement, 1, 0, 0),
  [283] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_statement, 1, 0, 0),
  [285] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_show_term, 3, 0, 25),
  [287] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_show_term, 3, 0, 25),
  [289] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_defined, 3, 0, 26),
  [291] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_defined, 3, 0, 26),
  [293] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_project_atom, 3, 0, 27),
  [295] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_project_atom, 3, 0, 27),
  [297] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_project_signature, 3, 0, 26),
  [299] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_project_signature, 3, 0, 26),
  [301] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_program, 3, 0, 7),
  [303] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_program, 3, 0, 7),
  [305] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_include, 3, 0, 0),
  [307] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_include, 3, 0, 0),
  [309] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_integrity_constraint, 2, 0, 0),
  [311] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_integrity_constraint, 2, 0, 0),
  [313] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_doc_comment, 4, 0, 31),
  [315] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_doc_comment, 4, 0, 31),
  [317] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_maximize, 4, 0, 0),
  [319] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_maximize, 4, 0, 0),
  [321] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_minimize, 4, 0, 0),
  [323] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_minimize, 4, 0, 0),
  [325] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_show_term, 4, 0, 25),
  [327] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_show_term, 4, 0, 25),
  [329] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_rule, 4, 0, 47),
  [331] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_rule, 4, 0, 47),
  [333] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_doc_comment, 5, 0, 48),
  [335] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_doc_comment, 5, 0, 48),
  [337] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_doc_comment, 5, 0, 49),
  [339] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_doc_comment, 5, 0, 49),
  [341] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_weak_constraint, 5, 0, 56),
  [343] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_weak_constraint, 5, 0, 56),
  [345] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_minimize, 5, 0, 60),
  [347] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_minimize, 5, 0, 60),
  [349] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_show_term, 5, 0, 62),
  [351] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_show_term, 5, 0, 62),
  [353] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_project_atom, 5, 0, 63),
  [355] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_project_atom, 5, 0, 63),
  [357] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_program, 5, 0, 7),
  [359] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_program, 5, 0, 7),
  [361] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_show, 2, 0, 0),
  [363] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_show, 2, 0, 0),
  [365] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_edge, 5, 0, 65),
  [367] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_edge, 5, 0, 65),
  [369] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_include, 5, 0, 0),
  [371] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_include, 5, 0, 0),
  [373] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_rule, 2, 0, 14),
  [375] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_rule, 2, 0, 14),
  [377] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory, 5, 0, 7),
  [379] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_theory, 5, 0, 7),
  [381] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_doc_comment, 6, 0, 71),
  [383] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_doc_comment, 6, 0, 71),
  [385] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_weak_constraint, 6, 0, 79),
  [387] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_weak_constraint, 6, 0, 79),
  [389] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_show_signature, 6, 0, 80),
  [391] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_show_signature, 6, 0, 80),
  [393] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_program, 6, 0, 81),
  [395] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_program, 6, 0, 81),
  [397] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_edge, 6, 0, 65),
  [399] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_edge, 6, 0, 65),
  [401] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_statement, 1, 1, 0),
  [403] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_statement, 1, 1, 0),
  [405] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_edge, 6, 0, 83),
  [407] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_edge, 6, 0, 83),
  [409] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_external, 6, 0, 84),
  [411] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_external, 6, 0, 84),
  [413] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory, 6, 0, 7),
  [415] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_theory, 6, 0, 7),
  [417] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_weak_constraint, 7, 0, 88),
  [419] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_weak_constraint, 7, 0, 88),
  [421] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_script, 7, 0, 90),
  [423] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_script, 7, 0, 90),
  [425] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_edge, 7, 0, 91),
  [427] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_edge, 7, 0, 91),
  [429] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_edge, 7, 0, 83),
  [431] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_edge, 7, 0, 83),
  [433] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_external, 7, 0, 92),
  [435] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_external, 7, 0, 92),
  [437] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory, 7, 0, 7),
  [439] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_theory, 7, 0, 7),
  [441] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_weak_constraint, 8, 0, 94),
  [443] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_weak_constraint, 8, 0, 94),
  [445] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const, 8, 0, 95),
  [447] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_const, 8, 0, 95),
  [449] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_edge, 8, 0, 96),
  [451] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_edge, 8, 0, 96),
  [453] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_heuristic, 8, 0, 97),
  [455] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_heuristic, 8, 0, 97),
  [457] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_external, 8, 0, 98),
  [459] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_external, 8, 0, 98),
  [461] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory, 8, 0, 7),
  [463] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_theory, 8, 0, 7),
  [465] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_heuristic, 9, 0, 100),
  [467] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_heuristic, 9, 0, 100),
  [469] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_heuristic, 10, 0, 101),
  [471] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_heuristic, 10, 0, 101),
  [473] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_maximize, 5, 0, 60),
  [475] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_maximize, 5, 0, 60),
  [477] = {.entry = {.count = 1, .reusable = false}}, SHIFT(253),
  [479] = {.entry = {.count = 1, .reusable = true}}, SHIFT(226),
  [481] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__conditional_literal_0, 2, 0, 6),
  [483] = {.entry = {.count = 1, .reusable = true}}, SHIFT(447),
  [485] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_optimize_element, 2, 0, 24),
  [487] = {.entry = {.count = 1, .reusable = true}}, SHIFT(135),
  [489] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_set_aggregate_element, 2, 0, 6),
  [491] = {.entry = {.count = 1, .reusable = true}}, SHIFT(108),
  [493] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_element, 1, 0, 0),
  [495] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_head_aggregate_element, 4, 0, 54),
  [497] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_optimize_element, 4, 0, 59),
  [499] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_body_aggregate_element, 1, 0, 0),
  [501] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_disjunction_repeat1, 2, 0, 0), SHIFT_REPEAT(218),
  [504] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_disjunction_repeat1, 2, 0, 0), SHIFT_REPEAT(515),
  [507] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_disjunction_repeat1, 2, 0, 0), SHIFT_REPEAT(454),
  [510] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_disjunction_repeat1, 2, 0, 0), SHIFT_REPEAT(158),
  [513] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_disjunction_repeat1, 2, 0, 0), SHIFT_REPEAT(158),
  [516] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_disjunction_repeat1, 2, 0, 0), SHIFT_REPEAT(233),
  [519] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_disjunction_repeat1, 2, 0, 0), SHIFT_REPEAT(198),
  [522] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_disjunction_repeat1, 2, 0, 0), SHIFT_REPEAT(199),
  [525] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_disjunction_repeat1, 2, 0, 0), SHIFT_REPEAT(90),
  [528] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_disjunction_repeat1, 2, 0, 0), SHIFT_REPEAT(775),
  [531] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_disjunction_repeat1, 2, 0, 0), SHIFT_REPEAT(426),
  [534] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_disjunction_repeat1, 2, 0, 0), SHIFT_REPEAT(305),
  [537] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_disjunction_repeat1, 2, 0, 0), SHIFT_REPEAT(305),
  [540] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_element, 2, 0, 37),
  [542] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_head_aggregate_element, 3, 0, 39),
  [544] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_body_aggregate_element, 2, 0, 42),
  [546] = {.entry = {.count = 1, .reusable = true}}, SHIFT(133),
  [548] = {.entry = {.count = 1, .reusable = true}}, SHIFT(87),
  [550] = {.entry = {.count = 1, .reusable = true}}, SHIFT(876),
  [552] = {.entry = {.count = 1, .reusable = true}}, SHIFT(107),
  [554] = {.entry = {.count = 1, .reusable = false}}, SHIFT(129),
  [556] = {.entry = {.count = 1, .reusable = true}}, SHIFT(626),
  [558] = {.entry = {.count = 1, .reusable = true}}, SHIFT(709),
  [560] = {.entry = {.count = 1, .reusable = true}}, SHIFT(166),
  [562] = {.entry = {.count = 1, .reusable = true}}, SHIFT(697),
  [564] = {.entry = {.count = 1, .reusable = true}}, SHIFT(259),
  [566] = {.entry = {.count = 1, .reusable = true}}, SHIFT(282),
  [568] = {.entry = {.count = 1, .reusable = false}}, SHIFT(319),
  [570] = {.entry = {.count = 1, .reusable = true}}, SHIFT(239),
  [572] = {.entry = {.count = 1, .reusable = true}}, SHIFT(687),
  [574] = {.entry = {.count = 1, .reusable = true}}, SHIFT(245),
  [576] = {.entry = {.count = 1, .reusable = true}}, SHIFT(705),
  [578] = {.entry = {.count = 1, .reusable = true}}, SHIFT(294),
  [580] = {.entry = {.count = 1, .reusable = true}}, SHIFT(436),
  [582] = {.entry = {.count = 1, .reusable = true}}, SHIFT(164),
  [584] = {.entry = {.count = 1, .reusable = false}}, SHIFT(254),
  [586] = {.entry = {.count = 1, .reusable = true}}, SHIFT(464),
  [588] = {.entry = {.count = 1, .reusable = true}}, SHIFT(299),
  [590] = {.entry = {.count = 1, .reusable = false}}, SHIFT(299),
  [592] = {.entry = {.count = 1, .reusable = true}}, SHIFT(217),
  [594] = {.entry = {.count = 1, .reusable = true}}, SHIFT(240),
  [596] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_set_aggregate, 3, 0, 16),
  [598] = {.entry = {.count = 1, .reusable = true}}, SHIFT(92),
  [600] = {.entry = {.count = 1, .reusable = true}}, SHIFT(856),
  [602] = {.entry = {.count = 1, .reusable = true}}, SHIFT(189),
  [604] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_set_aggregate, 4, 0, 45),
  [606] = {.entry = {.count = 1, .reusable = false}}, SHIFT(416),
  [608] = {.entry = {.count = 1, .reusable = true}}, SHIFT(534),
  [610] = {.entry = {.count = 1, .reusable = true}}, SHIFT(145),
  [612] = {.entry = {.count = 1, .reusable = true}}, SHIFT(467),
  [614] = {.entry = {.count = 1, .reusable = false}}, SHIFT(467),
  [616] = {.entry = {.count = 1, .reusable = true}}, SHIFT(130),
  [618] = {.entry = {.count = 1, .reusable = true}}, SHIFT(359),
  [620] = {.entry = {.count = 1, .reusable = true}}, SHIFT(148),
  [622] = {.entry = {.count = 1, .reusable = true}}, SHIFT(428),
  [624] = {.entry = {.count = 1, .reusable = false}}, SHIFT(372),
  [626] = {.entry = {.count = 1, .reusable = true}}, SHIFT(79),
  [628] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_set_aggregate, 2, 0, 0),
  [630] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_set_aggregate, 3, 0, 30),
  [632] = {.entry = {.count = 1, .reusable = true}}, SHIFT(434),
  [634] = {.entry = {.count = 1, .reusable = false}}, SHIFT(243),
  [636] = {.entry = {.count = 1, .reusable = true}}, SHIFT(516),
  [638] = {.entry = {.count = 1, .reusable = true}}, SHIFT(458),
  [640] = {.entry = {.count = 1, .reusable = true}}, SHIFT(252),
  [642] = {.entry = {.count = 1, .reusable = false}}, SHIFT(252),
  [644] = {.entry = {.count = 1, .reusable = true}}, SHIFT(202),
  [646] = {.entry = {.count = 1, .reusable = true}}, SHIFT(238),
  [648] = {.entry = {.count = 1, .reusable = true}}, SHIFT(91),
  [650] = {.entry = {.count = 1, .reusable = true}}, SHIFT(809),
  [652] = {.entry = {.count = 1, .reusable = true}}, SHIFT(136),
  [654] = {.entry = {.count = 1, .reusable = true}}, SHIFT(82),
  [656] = {.entry = {.count = 1, .reusable = true}}, SHIFT(678),
  [658] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_body_aggregate, 3, 0, 21),
  [660] = {.entry = {.count = 1, .reusable = true}}, SHIFT(122),
  [662] = {.entry = {.count = 1, .reusable = true}}, SHIFT(126),
  [664] = {.entry = {.count = 1, .reusable = true}}, SHIFT(97),
  [666] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__term_comma, 2, 0, 0),
  [668] = {.entry = {.count = 1, .reusable = true}}, SHIFT(127),
  [670] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__terms_trail, 3, 0, 0),
  [672] = {.entry = {.count = 1, .reusable = true}}, SHIFT(112),
  [674] = {.entry = {.count = 1, .reusable = true}}, SHIFT(139),
  [676] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_body_aggregate, 5, 0, 70),
  [678] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_body_aggregate, 4, 0, 41),
  [680] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_body_aggregate, 4, 0, 46),
  [682] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_external_function, 2, 0, 7),
  [684] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_external_function, 2, 0, 7),
  [686] = {.entry = {.count = 1, .reusable = true}}, SHIFT(103),
  [688] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_head_aggregate, 4, 0, 46),
  [690] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_head_aggregate, 3, 0, 21),
  [692] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function, 1, 0, 1),
  [694] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function, 1, 0, 1),
  [696] = {.entry = {.count = 1, .reusable = false}}, SHIFT(386),
  [698] = {.entry = {.count = 1, .reusable = true}}, SHIFT(536),
  [700] = {.entry = {.count = 1, .reusable = true}}, SHIFT(151),
  [702] = {.entry = {.count = 1, .reusable = true}}, SHIFT(395),
  [704] = {.entry = {.count = 1, .reusable = false}}, SHIFT(395),
  [706] = {.entry = {.count = 1, .reusable = true}}, SHIFT(739),
  [708] = {.entry = {.count = 1, .reusable = true}}, SHIFT(143),
  [710] = {.entry = {.count = 1, .reusable = true}}, SHIFT(441),
  [712] = {.entry = {.count = 1, .reusable = true}}, SHIFT(153),
  [714] = {.entry = {.count = 1, .reusable = true}}, SHIFT(803),
  [716] = {.entry = {.count = 1, .reusable = true}}, SHIFT(209),
  [718] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_head_aggregate, 4, 0, 41),
  [720] = {.entry = {.count = 1, .reusable = true}}, SHIFT(824),
  [722] = {.entry = {.count = 1, .reusable = true}}, SHIFT(398),
  [724] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_head_aggregate, 5, 0, 70),
  [726] = {.entry = {.count = 1, .reusable = true}}, SHIFT(469),
  [728] = {.entry = {.count = 1, .reusable = true}}, SHIFT(442),
  [730] = {.entry = {.count = 1, .reusable = true}}, SHIFT(402),
  [732] = {.entry = {.count = 1, .reusable = true}}, SHIFT(397),
  [734] = {.entry = {.count = 1, .reusable = true}}, SHIFT(446),
  [736] = {.entry = {.count = 1, .reusable = true}}, SHIFT(399),
  [738] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_lower, 2, 0, 0),
  [740] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_abs, 3, 0, 0),
  [742] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_abs, 3, 0, 0),
  [744] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function, 2, 0, 5),
  [746] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function, 2, 0, 5),
  [748] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__theory_terms_trail, 2, 0, 0),
  [750] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_term, 1, 0, 0),
  [752] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_term, 1, 0, 0),
  [754] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple, 3, 0, 0),
  [756] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_tuple, 3, 0, 0),
  [758] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_external_function, 3, 0, 20),
  [760] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_external_function, 3, 0, 20),
  [762] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__theory_terms_trail, 3, 0, 0),
  [764] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_abs, 4, 0, 0),
  [766] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_abs, 4, 0, 0),
  [768] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__arg_pool, 2, 0, 0),
  [770] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__arg_pool, 2, 0, 0),
  [772] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__arg_pool, 3, 0, 34),
  [774] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__arg_pool, 3, 0, 34),
  [776] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple, 2, 0, 0),
  [778] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_tuple, 2, 0, 0),
  [780] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__arg_pool, 3, 0, 33),
  [782] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__arg_pool, 3, 0, 33),
  [784] = {.entry = {.count = 1, .reusable = false}}, SHIFT(339),
  [786] = {.entry = {.count = 1, .reusable = true}}, SHIFT(197),
  [788] = {.entry = {.count = 1, .reusable = true}}, SHIFT(43),
  [790] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_fstring, 3, 0, 0),
  [792] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_fstring, 3, 0, 0),
  [794] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_fstring, 2, 0, 0),
  [796] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_fstring, 2, 0, 0),
  [798] = {.entry = {.count = 1, .reusable = true}}, SHIFT(528),
  [800] = {.entry = {.count = 1, .reusable = true}}, SHIFT(227),
  [802] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_symbolic_atom, 2, 0, 8),
  [804] = {.entry = {.count = 1, .reusable = true}}, SHIFT(100),
  [806] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_symbolic_atom, 2, 0, 8),
  [808] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_string, 3, 0, 0),
  [810] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_string, 3, 0, 0),
  [812] = {.entry = {.count = 1, .reusable = false}}, SHIFT(335),
  [814] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_symbolic_atom, 1, 0, 1),
  [816] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_symbolic_atom, 1, 0, 1),
  [818] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_string, 2, 0, 0),
  [820] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_string, 2, 0, 0),
  [822] = {.entry = {.count = 1, .reusable = false}}, SHIFT(255),
  [824] = {.entry = {.count = 1, .reusable = false}}, SHIFT(186),
  [826] = {.entry = {.count = 1, .reusable = false}}, SHIFT(320),
  [828] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_symbolic_atom, 2, 0, 5),
  [830] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_symbolic_atom, 2, 0, 5),
  [832] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_symbolic_atom, 3, 0, 19),
  [834] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_symbolic_atom, 3, 0, 19),
  [836] = {.entry = {.count = 1, .reusable = true}}, SHIFT(327),
  [838] = {.entry = {.count = 1, .reusable = true}}, SHIFT(364),
  [840] = {.entry = {.count = 1, .reusable = true}}, SHIFT(308),
  [842] = {.entry = {.count = 1, .reusable = true}}, SHIFT(321),
  [844] = {.entry = {.count = 1, .reusable = true}}, SHIFT(781),
  [846] = {.entry = {.count = 1, .reusable = true}}, SHIFT(248),
  [848] = {.entry = {.count = 1, .reusable = true}}, SHIFT(346),
  [850] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_binary_operation, 3, 0, 28),
  [852] = {.entry = {.count = 1, .reusable = true}}, SHIFT(205),
  [854] = {.entry = {.count = 1, .reusable = false}}, SHIFT(206),
  [856] = {.entry = {.count = 1, .reusable = true}}, SHIFT(206),
  [858] = {.entry = {.count = 1, .reusable = true}}, SHIFT(207),
  [860] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_binary_operation, 3, 0, 28),
  [862] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_unary_operation, 2, 0, 9),
  [864] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_unary_operation, 2, 0, 9),
  [866] = {.entry = {.count = 1, .reusable = true}}, SHIFT(215),
  [868] = {.entry = {.count = 1, .reusable = true}}, SHIFT(101),
  [870] = {.entry = {.count = 1, .reusable = true}}, SHIFT(465),
  [872] = {.entry = {.count = 1, .reusable = true}}, SHIFT(872),
  [874] = {.entry = {.count = 1, .reusable = true}}, SHIFT(880),
  [876] = {.entry = {.count = 1, .reusable = true}}, SHIFT(414),
  [878] = {.entry = {.count = 1, .reusable = true}}, SHIFT(514),
  [880] = {.entry = {.count = 1, .reusable = true}}, SHIFT(228),
  [882] = {.entry = {.count = 1, .reusable = true}}, SHIFT(229),
  [884] = {.entry = {.count = 1, .reusable = false}}, SHIFT(230),
  [886] = {.entry = {.count = 1, .reusable = true}}, SHIFT(230),
  [888] = {.entry = {.count = 1, .reusable = true}}, SHIFT(231),
  [890] = {.entry = {.count = 1, .reusable = true}}, SHIFT(185),
  [892] = {.entry = {.count = 1, .reusable = true}}, SHIFT(204),
  [894] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_comparison, 3, 0, 0),
  [896] = {.entry = {.count = 1, .reusable = true}}, SHIFT(208),
  [898] = {.entry = {.count = 1, .reusable = true}}, SHIFT(196),
  [900] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_comparison, 3, 0, 0),
  [902] = {.entry = {.count = 1, .reusable = true}}, SHIFT(203),
  [904] = {.entry = {.count = 1, .reusable = false}}, SHIFT(211),
  [906] = {.entry = {.count = 1, .reusable = true}}, SHIFT(211),
  [908] = {.entry = {.count = 1, .reusable = true}}, SHIFT(200),
  [910] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_body_repeat1, 2, 0, 0),
  [912] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_body_repeat1, 2, 0, 0),
  [914] = {.entry = {.count = 1, .reusable = true}}, SHIFT(219),
  [916] = {.entry = {.count = 1, .reusable = true}}, SHIFT(220),
  [918] = {.entry = {.count = 1, .reusable = false}}, SHIFT(221),
  [920] = {.entry = {.count = 1, .reusable = true}}, SHIFT(221),
  [922] = {.entry = {.count = 1, .reusable = true}}, SHIFT(222),
  [924] = {.entry = {.count = 1, .reusable = true}}, SHIFT(223),
  [926] = {.entry = {.count = 1, .reusable = true}}, SHIFT(234),
  [928] = {.entry = {.count = 1, .reusable = true}}, SHIFT(373),
  [930] = {.entry = {.count = 1, .reusable = true}}, SHIFT(347),
  [932] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_comparison_repeat1, 2, 0, 0),
  [934] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_comparison_repeat1, 2, 0, 0),
  [936] = {.entry = {.count = 1, .reusable = true}}, SHIFT(385),
  [938] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__const_term_comma, 2, 0, 0),
  [940] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__const_terms_trail, 3, 0, 0),
  [942] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_literal_sign, 1, 0, 0),
  [944] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_literal_sign, 1, 0, 0),
  [946] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_disjunction_repeat1, 2, 0, 0),
  [948] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_disjunction_repeat1, 2, 0, 0),
  [950] = {.entry = {.count = 1, .reusable = true}}, SHIFT(416),
  [952] = {.entry = {.count = 1, .reusable = true}}, SHIFT(345),
  [954] = {.entry = {.count = 1, .reusable = true}}, SHIFT(351),
  [956] = {.entry = {.count = 1, .reusable = true}}, SHIFT(352),
  [958] = {.entry = {.count = 1, .reusable = true}}, SHIFT(353),
  [960] = {.entry = {.count = 1, .reusable = true}}, SHIFT(354),
  [962] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_disjunction_repeat1, 2, 0, 12),
  [964] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_disjunction_repeat1, 2, 0, 12),
  [966] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_disjunction_repeat1, 2, 0, 13),
  [968] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_disjunction_repeat1, 2, 0, 13),
  [970] = {.entry = {.count = 1, .reusable = true}}, SHIFT(386),
  [972] = {.entry = {.count = 1, .reusable = true}}, SHIFT(390),
  [974] = {.entry = {.count = 1, .reusable = true}}, SHIFT(391),
  [976] = {.entry = {.count = 1, .reusable = true}}, SHIFT(235),
  [978] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_terms, 1, 0, 0),
  [980] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const_function, 1, 0, 1),
  [982] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_const_function, 1, 0, 1),
  [984] = {.entry = {.count = 1, .reusable = true}}, SHIFT(280),
  [986] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_weight, 1, 0, 23),
  [988] = {.entry = {.count = 1, .reusable = true}}, SHIFT(187),
  [990] = {.entry = {.count = 1, .reusable = true}}, SHIFT(216),
  [992] = {.entry = {.count = 1, .reusable = true}}, SHIFT(214),
  [994] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_upper, 1, 0, 0),
  [996] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_upper, 1, 0, 0),
  [998] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_function, 1, 0, 1), SHIFT(833),
  [1001] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_weight, 3, 0, 57),
  [1003] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_terms_repeat1, 2, 0, 0),
  [1005] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_function, 1, 0, 1), SHIFT(874),
  [1008] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_upper, 2, 0, 0),
  [1010] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_upper, 2, 0, 0),
  [1012] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const_function, 2, 0, 5),
  [1014] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_const_function, 2, 0, 5),
  [1016] = {.entry = {.count = 1, .reusable = true}}, SHIFT(155),
  [1018] = {.entry = {.count = 1, .reusable = true}}, SHIFT(224),
  [1020] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_theory_operators_repeat1, 2, 0, 0),
  [1022] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_theory_operators_repeat1, 2, 0, 0),
  [1024] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_theory_operators_repeat1, 2, 0, 0), SHIFT_REPEAT(343),
  [1027] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_theory_operators_repeat1, 2, 0, 0), SHIFT_REPEAT(372),
  [1030] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__term_or_projection, 1, 0, 0),
  [1032] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_pool_item, 1, 0, 0),
  [1034] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const_unary_operation, 2, 0, 9),
  [1036] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_const_unary_operation, 2, 0, 9),
  [1038] = {.entry = {.count = 1, .reusable = true}}, SHIFT(312),
  [1040] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const_tuple, 2, 0, 0),
  [1042] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_const_tuple, 2, 0, 0),
  [1044] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__const_arg_pool, 2, 0, 0),
  [1046] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__const_arg_pool, 2, 0, 0),
  [1048] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const_abs, 3, 0, 0),
  [1050] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_const_abs, 3, 0, 0),
  [1052] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const_tuple, 3, 0, 82),
  [1054] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_const_tuple, 3, 0, 82),
  [1056] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const_tuple, 3, 0, 0),
  [1058] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_const_tuple, 3, 0, 0),
  [1060] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const_binary_operation, 3, 0, 28),
  [1062] = {.entry = {.count = 1, .reusable = true}}, SHIFT(310),
  [1064] = {.entry = {.count = 1, .reusable = false}}, SHIFT(311),
  [1066] = {.entry = {.count = 1, .reusable = true}}, SHIFT(311),
  [1068] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_const_binary_operation, 3, 0, 28),
  [1070] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_lower, 1, 0, 0),
  [1072] = {.entry = {.count = 1, .reusable = true}}, SHIFT(154),
  [1074] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__const_arg_pool, 3, 0, 33),
  [1076] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__const_arg_pool, 3, 0, 33),
  [1078] = {.entry = {.count = 1, .reusable = true}}, SHIFT(212),
  [1080] = {.entry = {.count = 1, .reusable = true}}, SHIFT(213),
  [1082] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_theory_operators, 1, 0, 0),
  [1084] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_operators, 1, 0, 0),
  [1086] = {.entry = {.count = 1, .reusable = true}}, SHIFT(343),
  [1088] = {.entry = {.count = 1, .reusable = true}}, SHIFT(272),
  [1090] = {.entry = {.count = 1, .reusable = true}}, SHIFT(285),
  [1092] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_edge_pair, 3, 0, 0),
  [1094] = {.entry = {.count = 1, .reusable = true}}, SHIFT(309),
  [1096] = {.entry = {.count = 1, .reusable = true}}, SHIFT(297),
  [1098] = {.entry = {.count = 1, .reusable = true}}, SHIFT(350),
  [1100] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_term, 1, 0, 0),
  [1102] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_theory_term, 1, 0, 0),
  [1104] = {.entry = {.count = 1, .reusable = true}}, SHIFT(372),
  [1106] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_unparsed_term, 1, 0, 0),
  [1108] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_theory_unparsed_term, 1, 0, 0),
  [1110] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_unparsed_term, 2, 0, 0),
  [1112] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_theory_unparsed_term, 2, 0, 0),
  [1114] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_theory_unparsed_term_repeat1, 2, 0, 0),
  [1116] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_theory_unparsed_term_repeat1, 2, 0, 0),
  [1118] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_theory_unparsed_term_repeat1, 2, 0, 0), SHIFT_REPEAT(359),
  [1121] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_theory_unparsed_term_repeat1, 2, 0, 0), SHIFT_REPEAT(372),
  [1124] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_theory_operators_repeat1, 1, 0, 36),
  [1126] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_theory_operators_repeat1, 1, 0, 36),
  [1128] = {.entry = {.count = 1, .reusable = true}}, SHIFT(315),
  [1130] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const_terms, 1, 0, 0),
  [1132] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_abs_repeat1, 2, 0, 0),
  [1134] = {.entry = {.count = 1, .reusable = false}}, SHIFT(24),
  [1136] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6),
  [1138] = {.entry = {.count = 1, .reusable = true}}, SHIFT(66),
  [1140] = {.entry = {.count = 1, .reusable = true}}, SHIFT(55),
  [1142] = {.entry = {.count = 1, .reusable = true}}, SHIFT(69),
  [1144] = {.entry = {.count = 1, .reusable = true}}, SHIFT(61),
  [1146] = {.entry = {.count = 1, .reusable = true}}, SHIFT(70),
  [1148] = {.entry = {.count = 1, .reusable = true}}, SHIFT(193),
  [1150] = {.entry = {.count = 1, .reusable = true}}, SHIFT(192),
  [1152] = {.entry = {.count = 1, .reusable = true}}, SHIFT(210),
  [1154] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_const_terms_repeat1, 2, 0, 0),
  [1156] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_function, 1, 0, 1),
  [1158] = {.entry = {.count = 1, .reusable = true}}, SHIFT(149),
  [1160] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_theory_function, 1, 0, 1),
  [1162] = {.entry = {.count = 1, .reusable = true}}, SHIFT(67),
  [1164] = {.entry = {.count = 1, .reusable = true}}, SHIFT(232),
  [1166] = {.entry = {.count = 1, .reusable = true}}, SHIFT(106),
  [1168] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_atom, 2, 0, 7),
  [1170] = {.entry = {.count = 1, .reusable = true}}, SHIFT(102),
  [1172] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_theory_atom, 2, 0, 7),
  [1174] = {.entry = {.count = 1, .reusable = true}}, SHIFT(184),
  [1176] = {.entry = {.count = 1, .reusable = true}}, SHIFT(183),
  [1178] = {.entry = {.count = 1, .reusable = true}}, SHIFT(17),
  [1180] = {.entry = {.count = 1, .reusable = true}}, SHIFT(348),
  [1182] = {.entry = {.count = 1, .reusable = true}}, SHIFT(31),
  [1184] = {.entry = {.count = 1, .reusable = true}}, SHIFT(420),
  [1186] = {.entry = {.count = 1, .reusable = true}}, SHIFT(800),
  [1188] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_comparison_repeat1, 2, 0, 0), SHIFT_REPEAT(196),
  [1191] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_root_term, 1, 0, 0),
  [1193] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_theory_root_term, 1, 0, 0),
  [1195] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_function, 2, 0, 5),
  [1197] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_theory_function, 2, 0, 5),
  [1199] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_set, 2, 0, 0),
  [1201] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_theory_set, 2, 0, 0),
  [1203] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_tuple, 2, 0, 0),
  [1205] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_theory_tuple, 2, 0, 0),
  [1207] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_list, 2, 0, 0),
  [1209] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_theory_list, 2, 0, 0),
  [1211] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__theory_arguments, 2, 0, 0),
  [1213] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__theory_arguments, 2, 0, 0),
  [1215] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_set, 3, 0, 0),
  [1217] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_theory_set, 3, 0, 0),
  [1219] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_tuple, 3, 0, 0),
  [1221] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_theory_tuple, 3, 0, 0),
  [1223] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_list, 3, 0, 0),
  [1225] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_theory_list, 3, 0, 0),
  [1227] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__theory_arguments, 3, 0, 33),
  [1229] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__theory_arguments, 3, 0, 33),
  [1231] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_doc_arg, 6, 0, 104),
  [1233] = {.entry = {.count = 1, .reusable = true}}, SHIFT(419),
  [1235] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_comparison, 4, 0, 0),
  [1237] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_doc_desc_repeat1, 2, 0, 0),
  [1239] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_doc_desc_repeat1, 2, 0, 0), SHIFT_REPEAT(413),
  [1242] = {.entry = {.count = 1, .reusable = false}}, SHIFT(608),
  [1244] = {.entry = {.count = 1, .reusable = false}}, SHIFT(425),
  [1246] = {.entry = {.count = 1, .reusable = false}}, SHIFT(460),
  [1248] = {.entry = {.count = 1, .reusable = false}}, SHIFT(472),
  [1250] = {.entry = {.count = 1, .reusable = false}}, SHIFT(700),
  [1252] = {.entry = {.count = 1, .reusable = false}}, SHIFT(869),
  [1254] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_fstring_spec, 1, 0, 0),
  [1256] = {.entry = {.count = 1, .reusable = true}}, SHIFT(152),
  [1258] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_comparison_repeat1, 2, 0, 0), SHIFT_REPEAT(234),
  [1261] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_doc_desc_repeat1, 2, 0, 0), SHIFT_REPEAT(418),
  [1264] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_doc_desc, 1, 0, 0),
  [1266] = {.entry = {.count = 1, .reusable = true}}, SHIFT(418),
  [1268] = {.entry = {.count = 1, .reusable = true}}, SHIFT(413),
  [1270] = {.entry = {.count = 1, .reusable = true}}, SHIFT(109),
  [1272] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_atom, 3, 0, 17),
  [1274] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_theory_atom, 3, 0, 17),
  [1276] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_doc_predicate, 2, 0, 1),
  [1278] = {.entry = {.count = 1, .reusable = true}}, SHIFT(742),
  [1280] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_atom, 5, 0, 53),
  [1282] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_theory_atom, 5, 0, 53),
  [1284] = {.entry = {.count = 1, .reusable = true}}, SHIFT(608),
  [1286] = {.entry = {.count = 1, .reusable = true}}, SHIFT(445),
  [1288] = {.entry = {.count = 1, .reusable = true}}, SHIFT(471),
  [1290] = {.entry = {.count = 1, .reusable = true}}, SHIFT(713),
  [1292] = {.entry = {.count = 1, .reusable = true}}, SHIFT(759),
  [1294] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_fstring_spec, 2, 0, 0),
  [1296] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_simple_atom, 1, 0, 0),
  [1298] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_condition, 2, 0, 0),
  [1300] = {.entry = {.count = 1, .reusable = true}}, SHIFT(96),
  [1302] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_atom, 4, 0, 7),
  [1304] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_theory_atom, 4, 0, 7),
  [1306] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_doc_predicate, 6, 0, 1),
  [1308] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_atom, 6, 0, 76),
  [1310] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_theory_atom, 6, 0, 76),
  [1312] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_doc_predicate, 7, 0, 99),
  [1314] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_literal, 2, 0, 11),
  [1316] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_atom, 5, 0, 17),
  [1318] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_theory_atom, 5, 0, 17),
  [1320] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_literal, 1, 0, 2),
  [1322] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_condition, 1, 0, 0),
  [1324] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_condition_repeat1, 2, 0, 0),
  [1326] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_condition_repeat1, 2, 0, 0), SHIFT_REPEAT(96),
  [1329] = {.entry = {.count = 1, .reusable = true}}, SHIFT(306),
  [1331] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_disjunction, 2, 0, 0),
  [1333] = {.entry = {.count = 1, .reusable = true}}, SHIFT(73),
  [1335] = {.entry = {.count = 1, .reusable = true}}, SHIFT(485),
  [1337] = {.entry = {.count = 1, .reusable = true}}, SHIFT(638),
  [1339] = {.entry = {.count = 1, .reusable = true}}, SHIFT(829),
  [1341] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_fstring_spec, 3, 0, 0),
  [1343] = {.entry = {.count = 1, .reusable = false}}, SHIFT(170),
  [1345] = {.entry = {.count = 1, .reusable = true}}, SHIFT(463),
  [1347] = {.entry = {.count = 1, .reusable = false}}, SHIFT(237),
  [1349] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_fstring_repeat1, 2, 0, 0),
  [1351] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_fstring_repeat1, 2, 0, 0), SHIFT_REPEAT(457),
  [1354] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_fstring_repeat1, 2, 0, 0), SHIFT_REPEAT(237),
  [1357] = {.entry = {.count = 1, .reusable = false}}, SHIFT(262),
  [1359] = {.entry = {.count = 1, .reusable = true}}, SHIFT(459),
  [1361] = {.entry = {.count = 1, .reusable = false}}, SHIFT(268),
  [1363] = {.entry = {.count = 1, .reusable = true}}, SHIFT(457),
  [1365] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_head, 1, 0, 0),
  [1367] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_fstring_accessor_repeat1, 2, 0, 0), SHIFT_REPEAT(462),
  [1370] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_fstring_accessor_repeat1, 2, 0, 0), SHIFT_REPEAT(872),
  [1373] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_fstring_accessor_repeat1, 2, 0, 0),
  [1375] = {.entry = {.count = 1, .reusable = false}}, SHIFT(169),
  [1377] = {.entry = {.count = 1, .reusable = false}}, SHIFT(301),
  [1379] = {.entry = {.count = 1, .reusable = true}}, SHIFT(466),
  [1381] = {.entry = {.count = 1, .reusable = true}}, SHIFT(462),
  [1383] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_fstring_accessor, 1, 0, 0),
  [1385] = {.entry = {.count = 1, .reusable = false}}, SHIFT(283),
  [1387] = {.entry = {.count = 1, .reusable = true}}, SHIFT(874),
  [1389] = {.entry = {.count = 1, .reusable = true}}, SHIFT(182),
  [1391] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_terms, 2, 0, 0),
  [1393] = {.entry = {.count = 1, .reusable = true}}, SHIFT(173),
  [1395] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_theory_terms_repeat1, 2, 0, 0), SHIFT_REPEAT(182),
  [1398] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_theory_terms_repeat1, 2, 0, 0),
  [1400] = {.entry = {.count = 1, .reusable = true}}, SHIFT(784),
  [1402] = {.entry = {.count = 1, .reusable = true}}, SHIFT(881),
  [1404] = {.entry = {.count = 1, .reusable = true}}, SHIFT(780),
  [1406] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_theory_terms_repeat1, 2, 0, 0), SHIFT_REPEAT(173),
  [1409] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_terms, 1, 0, 0),
  [1411] = {.entry = {.count = 1, .reusable = true}}, SHIFT(715),
  [1413] = {.entry = {.count = 1, .reusable = true}}, SHIFT(654),
  [1415] = {.entry = {.count = 1, .reusable = true}}, SHIFT(717),
  [1417] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_terms, 2, 0, 0),
  [1419] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_comparison_repeat1, 2, 0, 0), SHIFT_REPEAT(214),
  [1422] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_terms_repeat1, 2, 0, 0), SHIFT_REPEAT(235),
  [1425] = {.entry = {.count = 1, .reusable = true}}, SHIFT(512),
  [1427] = {.entry = {.count = 1, .reusable = true}}, SHIFT(693),
  [1429] = {.entry = {.count = 1, .reusable = true}}, SHIFT(797),
  [1431] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_fstring_spec, 4, 0, 0),
  [1433] = {.entry = {.count = 1, .reusable = true}}, SHIFT(833),
  [1435] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_fstring_accessor_repeat1, 3, 0, 72),
  [1437] = {.entry = {.count = 1, .reusable = true}}, SHIFT(146),
  [1439] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__terms_or_projection, 2, 0, 0),
  [1441] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_body_literal, 1, 0, 2),
  [1443] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__conditional_literal_n, 3, 0, 29),
  [1445] = {.entry = {.count = 1, .reusable = true}}, SHIFT(93),
  [1447] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_set_aggregate, 4, 0, 44),
  [1449] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_body_literal, 2, 0, 11),
  [1451] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_string_repeat1, 2, 0, 0),
  [1453] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_string_repeat1, 2, 0, 0), SHIFT_REPEAT(499),
  [1456] = {.entry = {.count = 1, .reusable = true}}, SHIFT(117),
  [1458] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__terms_trail, 2, 0, 0),
  [1460] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_atom_upper, 2, 0, 0),
  [1462] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_atom_upper, 2, 0, 36),
  [1464] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_atom, 4, 0, 38),
  [1466] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_set_aggregate, 4, 0, 35),
  [1468] = {.entry = {.count = 1, .reusable = true}}, SHIFT(501),
  [1470] = {.entry = {.count = 1, .reusable = true}}, SHIFT(769),
  [1472] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__terms_or_projection_repeat1, 2, 0, 0), SHIFT_REPEAT(146),
  [1475] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__terms_or_projection_repeat1, 2, 0, 0),
  [1477] = {.entry = {.count = 1, .reusable = true}}, SHIFT(737),
  [1479] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_fstring_field, 5, 0, 0),
  [1481] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_fstring_field, 5, 0, 0),
  [1483] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_atom, 6, 0, 74),
  [1485] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_fstring_field, 4, 0, 0),
  [1487] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_fstring_field, 4, 0, 0),
  [1489] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_set_aggregate, 3, 0, 15),
  [1491] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_fstring_field, 3, 0, 0),
  [1493] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_fstring_field, 3, 0, 0),
  [1495] = {.entry = {.count = 1, .reusable = false}}, SHIFT(225),
  [1497] = {.entry = {.count = 1, .reusable = true}}, SHIFT(522),
  [1499] = {.entry = {.count = 1, .reusable = false}}, SHIFT(261),
  [1501] = {.entry = {.count = 1, .reusable = true}}, SHIFT(518),
  [1503] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_doc_args, 2, 0, 0),
  [1505] = {.entry = {.count = 1, .reusable = true}}, SHIFT(751),
  [1507] = {.entry = {.count = 1, .reusable = false}}, SHIFT(267),
  [1509] = {.entry = {.count = 1, .reusable = true}}, SHIFT(499),
  [1511] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_doc_args_repeat1, 2, 0, 0),
  [1513] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_doc_args_repeat1, 2, 0, 0), SHIFT_REPEAT(751),
  [1516] = {.entry = {.count = 1, .reusable = true}}, SHIFT(313),
  [1518] = {.entry = {.count = 1, .reusable = true}}, SHIFT(314),
  [1520] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_conditional_literal, 1, 0, 3),
  [1522] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_fstring_field, 6, 0, 0),
  [1524] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_fstring_field, 6, 0, 0),
  [1526] = {.entry = {.count = 1, .reusable = false}}, SHIFT(188),
  [1528] = {.entry = {.count = 1, .reusable = true}}, SHIFT(470),
  [1530] = {.entry = {.count = 1, .reusable = true}}, SHIFT(770),
  [1532] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_atom, 5, 0, 52),
  [1534] = {.entry = {.count = 1, .reusable = true}}, SHIFT(570),
  [1536] = {.entry = {.count = 1, .reusable = true}}, SHIFT(857),
  [1538] = {.entry = {.count = 1, .reusable = true}}, SHIFT(571),
  [1540] = {.entry = {.count = 1, .reusable = true}}, SHIFT(99),
  [1542] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_condition_repeat1, 2, 0, 0), SHIFT_REPEAT(99),
  [1545] = {.entry = {.count = 1, .reusable = false}}, SHIFT(326),
  [1547] = {.entry = {.count = 1, .reusable = true}}, SHIFT(531),
  [1549] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_atom, 7, 0, 86),
  [1551] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_set_aggregate, 5, 0, 68),
  [1553] = {.entry = {.count = 1, .reusable = false}}, SHIFT(324),
  [1555] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_doc_args, 3, 0, 0),
  [1557] = {.entry = {.count = 1, .reusable = false}}, SHIFT(448),
  [1559] = {.entry = {.count = 1, .reusable = true}}, SHIFT(535),
  [1561] = {.entry = {.count = 1, .reusable = false}}, SHIFT(452),
  [1563] = {.entry = {.count = 1, .reusable = false}}, SHIFT(409),
  [1565] = {.entry = {.count = 1, .reusable = true}}, SHIFT(537),
  [1567] = {.entry = {.count = 1, .reusable = false}}, SHIFT(410),
  [1569] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__terms_or_projection, 1, 0, 0),
  [1571] = {.entry = {.count = 1, .reusable = true}}, SHIFT(180),
  [1573] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_optimize_element, 1, 0, 24),
  [1575] = {.entry = {.count = 1, .reusable = true}}, SHIFT(74),
  [1577] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_atom, 3, 0, 18),
  [1579] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_atom, 6, 0, 75),
  [1581] = {.entry = {.count = 1, .reusable = true}}, SHIFT(510),
  [1583] = {.entry = {.count = 1, .reusable = true}}, SHIFT(558),
  [1585] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_operator_definitions, 1, 0, 0),
  [1587] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__theory_definitions_repeat1, 2, 0, 0), SHIFT_REPEAT(670),
  [1590] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__theory_definitions_repeat1, 2, 0, 0),
  [1592] = {.entry = {.count = 1, .reusable = true}}, SHIFT(670),
  [1594] = {.entry = {.count = 1, .reusable = true}}, SHIFT(779),
  [1596] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__theory_definitions_repeat2, 2, 0, 0), SHIFT_REPEAT(686),
  [1599] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__theory_definitions_repeat2, 2, 0, 0),
  [1601] = {.entry = {.count = 1, .reusable = true}}, SHIFT(147),
  [1603] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_body_aggregate_elements, 2, 0, 0),
  [1605] = {.entry = {.count = 1, .reusable = true}}, SHIFT(504),
  [1607] = {.entry = {.count = 1, .reusable = true}}, SHIFT(863),
  [1609] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_body_aggregate, 6, 0, 85),
  [1611] = {.entry = {.count = 1, .reusable = true}}, SHIFT(159),
  [1613] = {.entry = {.count = 1, .reusable = true}}, SHIFT(98),
  [1615] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_const_terms, 2, 0, 0),
  [1617] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_optimize_element, 3, 0, 59),
  [1619] = {.entry = {.count = 1, .reusable = true}}, SHIFT(81),
  [1621] = {.entry = {.count = 1, .reusable = true}}, SHIFT(624),
  [1623] = {.entry = {.count = 1, .reusable = true}}, SHIFT(843),
  [1625] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_const_terms_repeat1, 2, 0, 0), SHIFT_REPEAT(315),
  [1628] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_optimize_elements_repeat1, 2, 0, 0), SHIFT_REPEAT(160),
  [1631] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_optimize_elements_repeat1, 2, 0, 0),
  [1633] = {.entry = {.count = 1, .reusable = true}}, SHIFT(163),
  [1635] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_operator_definitions, 2, 0, 0),
  [1637] = {.entry = {.count = 1, .reusable = true}}, SHIFT(677),
  [1639] = {.entry = {.count = 1, .reusable = true}}, SHIFT(174),
  [1641] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__tuple_pool_n, 2, 0, 0),
  [1643] = {.entry = {.count = 1, .reusable = true}}, SHIFT(850),
  [1645] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_variables, 2, 0, 0),
  [1647] = {.entry = {.count = 1, .reusable = true}}, SHIFT(706),
  [1649] = {.entry = {.count = 1, .reusable = true}}, SHIFT(861),
  [1651] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_fstring_spec, 5, 0, 0),
  [1653] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_theory_operator_definitions_repeat1, 2, 0, 0), SHIFT_REPEAT(558),
  [1656] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_theory_operator_definitions_repeat1, 2, 0, 0),
  [1658] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_variables, 3, 0, 0),
  [1660] = {.entry = {.count = 1, .reusable = true}}, SHIFT(859),
  [1662] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameters, 2, 0, 0),
  [1664] = {.entry = {.count = 1, .reusable = true}}, SHIFT(639),
  [1666] = {.entry = {.count = 1, .reusable = true}}, SHIFT(774),
  [1668] = {.entry = {.count = 1, .reusable = true}}, SHIFT(640),
  [1670] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_variables_repeat1, 2, 0, 0), SHIFT_REPEAT(850),
  [1673] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_variables_repeat1, 2, 0, 0),
  [1675] = {.entry = {.count = 1, .reusable = true}}, SHIFT(647),
  [1677] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__theory_operators_sep, 1, 0, 0),
  [1679] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__theory_operators_sep, 1, 0, 36),
  [1681] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__theory_operators_sep, 2, 0, 0),
  [1683] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__theory_operators_sep, 2, 0, 36),
  [1685] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__theory_operators_sep_repeat1, 2, 0, 0), SHIFT_REPEAT(647),
  [1688] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__theory_operators_sep_repeat1, 2, 0, 0),
  [1690] = {.entry = {.count = 1, .reusable = true}}, SHIFT(726),
  [1692] = {.entry = {.count = 1, .reusable = true}}, SHIFT(36),
  [1694] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__arg_pool_n, 2, 0, 32),
  [1696] = {.entry = {.count = 1, .reusable = true}}, SHIFT(111),
  [1698] = {.entry = {.count = 1, .reusable = true}}, SHIFT(150),
  [1700] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_head_aggregate_elements, 1, 0, 0),
  [1702] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_edge_repeat1, 2, 0, 67),
  [1704] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_edge_repeat1, 2, 0, 67), SHIFT_REPEAT(174),
  [1707] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_set_aggregate_elements_repeat1, 2, 0, 0), SHIFT_REPEAT(94),
  [1710] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_set_aggregate_elements_repeat1, 2, 0, 0),
  [1712] = {.entry = {.count = 1, .reusable = true}}, SHIFT(275),
  [1714] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_body, 2, 0, 0),
  [1716] = {.entry = {.count = 1, .reusable = true}}, SHIFT(157),
  [1718] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__theory_terms_trail, 1, 0, 0),
  [1720] = {.entry = {.count = 1, .reusable = true}}, SHIFT(664),
  [1722] = {.entry = {.count = 1, .reusable = true}}, SHIFT(798),
  [1724] = {.entry = {.count = 1, .reusable = true}}, SHIFT(837),
  [1726] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_terms_repeat1, 2, 0, 0), SHIFT_REPEAT(212),
  [1729] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_set_aggregate_element, 1, 0, 6),
  [1731] = {.entry = {.count = 1, .reusable = true}}, SHIFT(76),
  [1733] = {.entry = {.count = 1, .reusable = true}}, SHIFT(94),
  [1735] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_set_aggregate_elements, 2, 0, 0),
  [1737] = {.entry = {.count = 1, .reusable = true}}, SHIFT(316),
  [1739] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_conditional_literal, 1, 0, 4),
  [1741] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_element, 1, 0, 37),
  [1743] = {.entry = {.count = 1, .reusable = true}}, SHIFT(84),
  [1745] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_body, 1, 0, 0),
  [1747] = {.entry = {.count = 1, .reusable = true}}, SHIFT(162),
  [1749] = {.entry = {.count = 1, .reusable = true}}, SHIFT(128),
  [1751] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_elements, 1, 0, 0),
  [1753] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_elements, 2, 0, 0),
  [1755] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__arg_pool_n_repeat1, 2, 0, 50),
  [1757] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__arg_pool_n_repeat1, 2, 0, 50), SHIFT_REPEAT(111),
  [1760] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_theory_elements_repeat1, 2, 0, 0), SHIFT_REPEAT(128),
  [1763] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_theory_elements_repeat1, 2, 0, 0),
  [1765] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_abs_repeat1, 2, 0, 0), SHIFT_REPEAT(224),
  [1768] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__tuple_pool_n_repeat1, 2, 0, 0),
  [1770] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__tuple_pool_n_repeat1, 2, 0, 0), SHIFT_REPEAT(98),
  [1773] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_fstring_width, 1, 0, 0),
  [1775] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_head_aggregate_element, 2, 0, 39),
  [1777] = {.entry = {.count = 1, .reusable = true}}, SHIFT(85),
  [1779] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_body_aggregate_elements_repeat1, 2, 0, 0), SHIFT_REPEAT(147),
  [1782] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_body_aggregate_elements_repeat1, 2, 0, 0),
  [1784] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_head_aggregate_elements, 2, 0, 0),
  [1786] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_body_aggregate, 5, 0, 55),
  [1788] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_body_aggregate, 5, 0, 69),
  [1790] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_body_aggregate_element, 1, 0, 42),
  [1792] = {.entry = {.count = 1, .reusable = true}}, SHIFT(86),
  [1794] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_body_aggregate_elements, 1, 0, 0),
  [1796] = {.entry = {.count = 1, .reusable = true}}, SHIFT(160),
  [1798] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_optimize_elements, 1, 0, 0),
  [1800] = {.entry = {.count = 1, .reusable = true}}, SHIFT(273),
  [1802] = {.entry = {.count = 1, .reusable = true}}, SHIFT(844),
  [1804] = {.entry = {.count = 1, .reusable = true}}, SHIFT(875),
  [1806] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_parameters_repeat1, 2, 0, 0), SHIFT_REPEAT(859),
  [1809] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_parameters_repeat1, 2, 0, 0),
  [1811] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_optimize_elements, 2, 0, 0),
  [1813] = {.entry = {.count = 1, .reusable = true}}, SHIFT(438),
  [1815] = {.entry = {.count = 1, .reusable = true}}, SHIFT(251),
  [1817] = {.entry = {.count = 1, .reusable = true}}, SHIFT(799),
  [1819] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameters, 1, 0, 0),
  [1821] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_head_aggregate_element, 3, 0, 54),
  [1823] = {.entry = {.count = 1, .reusable = true}}, SHIFT(80),
  [1825] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_projection, 1, 0, 0),
  [1827] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_set_aggregate_elements, 1, 0, 0),
  [1829] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_head_aggregate_elements_repeat1, 2, 0, 0), SHIFT_REPEAT(150),
  [1832] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_head_aggregate_elements_repeat1, 2, 0, 0),
  [1834] = {.entry = {.count = 1, .reusable = true}}, SHIFT(303),
  [1836] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__const_terms_trail, 2, 0, 0),
  [1838] = {.entry = {.count = 1, .reusable = true}}, SHIFT(286),
  [1840] = {.entry = {.count = 1, .reusable = true}}, SHIFT(167),
  [1842] = {.entry = {.count = 1, .reusable = true}}, SHIFT(293),
  [1844] = {.entry = {.count = 1, .reusable = true}}, SHIFT(295),
  [1846] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_body_aggregate, 4, 0, 40),
  [1848] = {.entry = {.count = 1, .reusable = true}}, SHIFT(241),
  [1850] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_head_aggregate_element, 5, 0, 87),
  [1852] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_operator_definition, 5, 0, 102),
  [1854] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_operator_definition, 5, 0, 103),
  [1856] = {.entry = {.count = 1, .reusable = true}}, SHIFT(525),
  [1858] = {.entry = {.count = 1, .reusable = true}}, SHIFT(642),
  [1860] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_atom_definition, 8, 0, 105),
  [1862] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_variables_repeat1, 4, 0, 0),
  [1864] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_doc_arg, 7, 0, 106),
  [1866] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_operator_definition, 7, 0, 107),
  [1868] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_operator_definition, 7, 0, 108),
  [1870] = {.entry = {.count = 1, .reusable = true}}, SHIFT(648),
  [1872] = {.entry = {.count = 1, .reusable = true}}, SHIFT(649),
  [1874] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__theory_operators_sep_repeat1, 2, 0, 109),
  [1876] = {.entry = {.count = 1, .reusable = true}}, SHIFT(115),
  [1878] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_atom_definition, 13, 0, 110),
  [1880] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_atom_definition, 14, 0, 111),
  [1882] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_term_definition, 3, 0, 1),
  [1884] = {.entry = {.count = 1, .reusable = true}}, SHIFT(26),
  [1886] = {.entry = {.count = 1, .reusable = true}}, SHIFT(8),
  [1888] = {.entry = {.count = 1, .reusable = true}}, SHIFT(554),
  [1890] = {.entry = {.count = 1, .reusable = true}}, SHIFT(28),
  [1892] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_optimize_element, 5, 0, 89),
  [1894] = {.entry = {.count = 1, .reusable = true}}, SHIFT(172),
  [1896] = {.entry = {.count = 1, .reusable = true}}, SHIFT(49),
  [1898] = {.entry = {.count = 1, .reusable = true}}, SHIFT(805),
  [1900] = {.entry = {.count = 1, .reusable = true}}, SHIFT(7),
  [1902] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_element, 2, 0, 51),
  [1904] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_edge_repeat1, 2, 0, 66),
  [1906] = {.entry = {.count = 1, .reusable = true}}, SHIFT(54),
  [1908] = {.entry = {.count = 1, .reusable = true}}, SHIFT(11),
  [1910] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_optimize_element, 3, 0, 58),
  [1912] = {.entry = {.count = 1, .reusable = true}}, SHIFT(78),
  [1914] = {.entry = {.count = 1, .reusable = true}}, SHIFT(724),
  [1916] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_set_aggregate_element, 3, 0, 29),
  [1918] = {.entry = {.count = 1, .reusable = true}}, SHIFT(16),
  [1920] = {.entry = {.count = 1, .reusable = true}}, SHIFT(10),
  [1922] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_head_aggregate, 5, 0, 69),
  [1924] = {.entry = {.count = 1, .reusable = true}}, SHIFT(44),
  [1926] = {.entry = {.count = 1, .reusable = true}}, SHIFT(9),
  [1928] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__arg_pool_n_repeat1, 2, 0, 33),
  [1930] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_term_definition, 4, 0, 93),
  [1932] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_theory_element, 3, 0, 73),
  [1934] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_disjunction, 1, 0, 0),
  [1936] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_fstring_conversion, 2, 0, 0),
  [1938] = {.entry = {.count = 1, .reusable = true}}, SHIFT(46),
  [1940] = {.entry = {.count = 1, .reusable = true}}, SHIFT(12),
  [1942] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_head_aggregate, 6, 0, 85),
  [1944] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_head_aggregate, 4, 0, 40),
  [1946] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_head_aggregate_element, 4, 0, 77),
  [1948] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_body_aggregate_element, 3, 0, 78),
  [1950] = {.entry = {.count = 1, .reusable = true}}, SHIFT(839),
  [1952] = {.entry = {.count = 1, .reusable = true}}, SHIFT(801),
  [1954] = {.entry = {.count = 1, .reusable = true}}, SHIFT(181),
  [1956] = {.entry = {.count = 1, .reusable = true}}, SHIFT(38),
  [1958] = {.entry = {.count = 1, .reusable = true}}, SHIFT(782),
  [1960] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_fstring_spec, 6, 0, 0),
  [1962] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_head_aggregate, 5, 0, 55),
  [1964] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_body_aggregate_element, 2, 0, 51),
  [1966] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_tuple_pool_item, 1, 0, 10),
  [1968] = {.entry = {.count = 1, .reusable = true}}, SHIFT(75),
  [1970] = {.entry = {.count = 1, .reusable = true}}, SHIFT(879),
  [1972] = {.entry = {.count = 1, .reusable = true}}, SHIFT(59),
  [1974] = {.entry = {.count = 1, .reusable = true}}, SHIFT(765),
  [1976] = {.entry = {.count = 1, .reusable = true}}, SHIFT(740),
  [1978] = {.entry = {.count = 1, .reusable = true}}, SHIFT(767),
  [1980] = {.entry = {.count = 1, .reusable = true}}, SHIFT(195),
  [1982] = {.entry = {.count = 1, .reusable = true}}, SHIFT(802),
  [1984] = {.entry = {.count = 1, .reusable = true}}, SHIFT(178),
  [1986] = {.entry = {.count = 1, .reusable = true}}, SHIFT(190),
  [1988] = {.entry = {.count = 1, .reusable = true}}, SHIFT(120),
  [1990] = {.entry = {.count = 1, .reusable = true}}, SHIFT(113),
  [1992] = {.entry = {.count = 1, .reusable = true}}, SHIFT(816),
  [1994] = {.entry = {.count = 1, .reusable = true}}, SHIFT(144),
  [1996] = {.entry = {.count = 1, .reusable = true}}, SHIFT(19),
  [1998] = {.entry = {.count = 1, .reusable = true}}, SHIFT(734),
  [2000] = {.entry = {.count = 1, .reusable = true}}, SHIFT(744),
  [2002] = {.entry = {.count = 1, .reusable = true}}, SHIFT(643),
  [2004] = {.entry = {.count = 1, .reusable = true}}, SHIFT(51),
  [2006] = {.entry = {.count = 1, .reusable = true}}, SHIFT(680),
  [2008] = {.entry = {.count = 1, .reusable = true}}, SHIFT(773),
  [2010] = {.entry = {.count = 1, .reusable = true}}, SHIFT(451),
  [2012] = {.entry = {.count = 1, .reusable = true}}, SHIFT(29),
  [2014] = {.entry = {.count = 1, .reusable = true}}, SHIFT(56),
  [2016] = {.entry = {.count = 1, .reusable = true}}, SHIFT(645),
  [2018] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__theory_terms_trail, 1, 0, 10),
  [2020] = {.entry = {.count = 1, .reusable = true}}, SHIFT(745),
  [2022] = {.entry = {.count = 1, .reusable = true}}, SHIFT(64),
  [2024] = {.entry = {.count = 1, .reusable = true}}, SHIFT(591),
  [2026] = {.entry = {.count = 1, .reusable = true}}, SHIFT(455),
  [2028] = {.entry = {.count = 1, .reusable = true}}, SHIFT(747),
  [2030] = {.entry = {.count = 1, .reusable = true}}, SHIFT(651),
  [2032] = {.entry = {.count = 1, .reusable = true}}, SHIFT(852),
  [2034] = {.entry = {.count = 1, .reusable = true}}, SHIFT(750),
  [2036] = {.entry = {.count = 1, .reusable = true}}, SHIFT(646),
  [2038] = {.entry = {.count = 1, .reusable = true}}, SHIFT(65),
  [2040] = {.entry = {.count = 1, .reusable = true}}, SHIFT(652),
  [2042] = {.entry = {.count = 1, .reusable = true}}, SHIFT(865),
  [2044] = {.entry = {.count = 1, .reusable = true}}, SHIFT(476),
  [2046] = {.entry = {.count = 1, .reusable = true}}, SHIFT(456),
  [2048] = {.entry = {.count = 1, .reusable = true}}, SHIFT(201),
  [2050] = {.entry = {.count = 1, .reusable = true}}, SHIFT(834),
  [2052] = {.entry = {.count = 1, .reusable = true}}, SHIFT(48),
  [2054] = {.entry = {.count = 1, .reusable = true}}, SHIFT(175),
  [2056] = {.entry = {.count = 1, .reusable = true}}, SHIFT(25),
  [2058] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_signature, 4, 0, 61),
  [2060] = {.entry = {.count = 1, .reusable = true}}, SHIFT(521),
  [2062] = {.entry = {.count = 1, .reusable = true}}, SHIFT(785),
  [2064] = {.entry = {.count = 1, .reusable = true}}, SHIFT(450),
  [2066] = {.entry = {.count = 1, .reusable = true}}, SHIFT(825),
  [2068] = {.entry = {.count = 1, .reusable = true}}, SHIFT(45),
  [2070] = {.entry = {.count = 1, .reusable = true}}, SHIFT(826),
  [2072] = {.entry = {.count = 1, .reusable = true}}, SHIFT(411),
  [2074] = {.entry = {.count = 1, .reusable = true}}, SHIFT(62),
  [2076] = {.entry = {.count = 1, .reusable = true}}, SHIFT(486),
  [2078] = {.entry = {.count = 1, .reusable = true}}, SHIFT(721),
  [2080] = {.entry = {.count = 1, .reusable = true}}, SHIFT(423),
  [2082] = {.entry = {.count = 1, .reusable = true}}, SHIFT(842),
  [2084] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__theory_operator_arity_binary, 1, 0, 0),
  [2086] = {.entry = {.count = 1, .reusable = true}}, SHIFT(125),
  [2088] = {.entry = {.count = 1, .reusable = true}}, SHIFT(488),
  [2090] = {.entry = {.count = 1, .reusable = true}}, SHIFT(738),
  [2092] = {.entry = {.count = 1, .reusable = true}}, SHIFT(27),
  [2094] = {.entry = {.count = 1, .reusable = true}}, SHIFT(68),
  [2096] = {.entry = {.count = 1, .reusable = true}}, SHIFT(47),
  [2098] = {.entry = {.count = 1, .reusable = true}}, SHIFT(349),
  [2100] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_fstring_spec, 7, 0, 0),
  [2102] = {.entry = {.count = 1, .reusable = true}}, SHIFT(105),
  [2104] = {.entry = {.count = 1, .reusable = true}}, SHIFT(479),
  [2106] = {.entry = {.count = 1, .reusable = true}}, SHIFT(746),
  [2108] = {.entry = {.count = 1, .reusable = true}}, SHIFT(431),
  [2110] = {.entry = {.count = 1, .reusable = true}}, SHIFT(430),
  [2112] = {.entry = {.count = 1, .reusable = true}}, SHIFT(860),
  [2114] = {.entry = {.count = 1, .reusable = true}}, SHIFT(35),
  [2116] = {.entry = {.count = 1, .reusable = true}}, SHIFT(116),
  [2118] = {.entry = {.count = 1, .reusable = true}}, SHIFT(179),
  [2120] = {.entry = {.count = 1, .reusable = true}}, SHIFT(171),
  [2122] = {.entry = {.count = 1, .reusable = true}}, SHIFT(748),
  [2124] = {.entry = {.count = 1, .reusable = true}}, SHIFT(694),
  [2126] = {.entry = {.count = 1, .reusable = true}}, SHIFT(429),
  [2128] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_boolean, 1, 0, 0),
  [2130] = {.entry = {.count = 1, .reusable = true}}, SHIFT(517),
  [2132] = {.entry = {.count = 1, .reusable = true}}, SHIFT(114),
  [2134] = {.entry = {.count = 1, .reusable = true}}, SHIFT(318),
  [2136] = {.entry = {.count = 1, .reusable = true}}, SHIFT(32),
  [2138] = {.entry = {.count = 1, .reusable = true}}, SHIFT(57),
  [2140] = {.entry = {.count = 1, .reusable = true}}, SHIFT(176),
  [2142] = {.entry = {.count = 1, .reusable = true}}, SHIFT(731),
  [2144] = {.entry = {.count = 1, .reusable = true}}, SHIFT(131),
  [2146] = {.entry = {.count = 1, .reusable = true}}, SHIFT(792),
  [2148] = {.entry = {.count = 1, .reusable = true}}, SHIFT(244),
  [2150] = {.entry = {.count = 1, .reusable = true}}, SHIFT(22),
  [2152] = {.entry = {.count = 1, .reusable = true}}, SHIFT(50),
  [2154] = {.entry = {.count = 1, .reusable = true}}, SHIFT(134),
  [2156] = {.entry = {.count = 1, .reusable = true}}, SHIFT(828),
  [2158] = {.entry = {.count = 1, .reusable = true}}, SHIFT(766),
  [2160] = {.entry = {.count = 1, .reusable = true}}, SHIFT(118),
  [2162] = {.entry = {.count = 1, .reusable = true}}, SHIFT(142),
  [2164] = {.entry = {.count = 1, .reusable = true}}, SHIFT(389),
  [2166] = {.entry = {.count = 1, .reusable = true}}, SHIFT(37),
  [2168] = {.entry = {.count = 1, .reusable = true}}, SHIFT(752),
  [2170] = {.entry = {.count = 1, .reusable = true}}, SHIFT(15),
  [2172] = {.entry = {.count = 1, .reusable = true}}, SHIFT(33),
  [2174] = {.entry = {.count = 1, .reusable = true}}, SHIFT(567),
  [2176] = {.entry = {.count = 1, .reusable = true}}, SHIFT(568),
  [2178] = {.entry = {.count = 1, .reusable = true}}, SHIFT(432),
  [2180] = {.entry = {.count = 1, .reusable = true}}, SHIFT(71),
  [2182] = {.entry = {.count = 1, .reusable = true}}, SHIFT(95),
  [2184] = {.entry = {.count = 1, .reusable = true}}, SHIFT(39),
  [2186] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_signature, 3, 0, 43),
  [2188] = {.entry = {.count = 1, .reusable = true}}, SHIFT(761),
  [2190] = {.entry = {.count = 1, .reusable = true}}, SHIFT(58),
  [2192] = {.entry = {.count = 1, .reusable = true}}, SHIFT(40),
  [2194] = {.entry = {.count = 1, .reusable = true}}, SHIFT(137),
  [2196] = {.entry = {.count = 1, .reusable = true}}, SHIFT(562),
  [2198] = {.entry = {.count = 1, .reusable = true}}, SHIFT(63),
  [2200] = {.entry = {.count = 1, .reusable = true}}, SHIFT(422),
  [2202] = {.entry = {.count = 1, .reusable = true}}, SHIFT(138),
  [2204] = {.entry = {.count = 1, .reusable = true}}, SHIFT(41),
  [2206] = {.entry = {.count = 1, .reusable = true}}, SHIFT(884),
  [2208] = {.entry = {.count = 1, .reusable = true}}, SHIFT(42),
  [2210] = {.entry = {.count = 1, .reusable = true}}, SHIFT(732),
  [2212] = {.entry = {.count = 1, .reusable = true}}, SHIFT(403),
  [2214] = {.entry = {.count = 1, .reusable = true}}, SHIFT(404),
  [2216] = {.entry = {.count = 1, .reusable = true}}, SHIFT(405),
  [2218] = {.entry = {.count = 1, .reusable = true}}, SHIFT(140),
  [2220] = {.entry = {.count = 1, .reusable = true}}, SHIFT(806),
  [2222] = {.entry = {.count = 1, .reusable = true}}, SHIFT(406),
  [2224] = {.entry = {.count = 1, .reusable = false}}, SHIFT(755),
  [2226] = {.entry = {.count = 1, .reusable = true}}, SHIFT(356),
  [2228] = {.entry = {.count = 1, .reusable = true}}, SHIFT(786),
  [2230] = {.entry = {.count = 1, .reusable = true}}, SHIFT(104),
  [2232] = {.entry = {.count = 1, .reusable = true}}, SHIFT(256),
  [2234] = {.entry = {.count = 1, .reusable = true}}, SHIFT(716),
  [2236] = {.entry = {.count = 1, .reusable = true}}, SHIFT(831),
  [2238] = {.entry = {.count = 1, .reusable = true}}, SHIFT(699),
  [2240] = {.entry = {.count = 1, .reusable = true}}, SHIFT(768),
  [2242] = {.entry = {.count = 1, .reusable = true}}, SHIFT(492),
  [2244] = {.entry = {.count = 1, .reusable = true}}, SHIFT(165),
  [2246] = {.entry = {.count = 1, .reusable = true}}, SHIFT(788),
  [2248] = {.entry = {.count = 1, .reusable = true}}, SHIFT(177),
  [2250] = {.entry = {.count = 1, .reusable = true}}, SHIFT(656),
  [2252] = {.entry = {.count = 1, .reusable = true}}, SHIFT(730),
  [2254] = {.entry = {.count = 1, .reusable = true}}, SHIFT(296),
  [2256] = {.entry = {.count = 1, .reusable = true}}, SHIFT(776),
  [2258] = {.entry = {.count = 1, .reusable = true}}, SHIFT(741),
  [2260] = {.entry = {.count = 1, .reusable = true}}, SHIFT(832),
  [2262] = {.entry = {.count = 1, .reusable = true}}, SHIFT(871),
  [2264] = {.entry = {.count = 1, .reusable = true}}, SHIFT(110),
  [2266] = {.entry = {.count = 1, .reusable = true}}, SHIFT(141),
  [2268] = {.entry = {.count = 1, .reusable = true}}, SHIFT(119),
  [2270] = {.entry = {.count = 1, .reusable = true}}, SHIFT(684),
  [2272] = {.entry = {.count = 1, .reusable = true}}, SHIFT(729),
  [2274] = {.entry = {.count = 1, .reusable = true}},  ACCEPT_INPUT(),
  [2276] = {.entry = {.count = 1, .reusable = true}}, SHIFT(242),
  [2278] = {.entry = {.count = 1, .reusable = true}}, SHIFT(641),
  [2280] = {.entry = {.count = 1, .reusable = true}}, SHIFT(121),
};

enum ts_external_scanner_symbol_identifiers {
  ts_external_token_empty_pool_item_first = 0,
  ts_external_token_empty_pool_item = 1,
  ts_external_token_colon = 2,
  ts_external_token_block_comment = 3,
  ts_external_token_doc_fragment_string = 4,
  ts_external_token__doc_token_args = 5,
  ts_external_token__doc_token_paren = 6,
  ts_external_token__doc_token_minus = 7,
};

static const TSSymbol ts_external_scanner_symbol_map[EXTERNAL_TOKEN_COUNT] = {
  [ts_external_token_empty_pool_item_first] = sym_empty_pool_item_first,
  [ts_external_token_empty_pool_item] = sym_empty_pool_item,
  [ts_external_token_colon] = sym_colon,
  [ts_external_token_block_comment] = sym_block_comment,
  [ts_external_token_doc_fragment_string] = sym_doc_fragment_string,
  [ts_external_token__doc_token_args] = sym__doc_token_args,
  [ts_external_token__doc_token_paren] = sym__doc_token_paren,
  [ts_external_token__doc_token_minus] = sym__doc_token_minus,
};

static const bool ts_external_scanner_states[11][EXTERNAL_TOKEN_COUNT] = {
  [1] = {
    [ts_external_token_empty_pool_item_first] = true,
    [ts_external_token_empty_pool_item] = true,
    [ts_external_token_colon] = true,
    [ts_external_token_block_comment] = true,
    [ts_external_token_doc_fragment_string] = true,
    [ts_external_token__doc_token_args] = true,
    [ts_external_token__doc_token_paren] = true,
    [ts_external_token__doc_token_minus] = true,
  },
  [2] = {
    [ts_external_token_block_comment] = true,
  },
  [3] = {
    [ts_external_token_empty_pool_item_first] = true,
    [ts_external_token_block_comment] = true,
  },
  [4] = {
    [ts_external_token_empty_pool_item] = true,
    [ts_external_token_block_comment] = true,
  },
  [5] = {
    [ts_external_token_colon] = true,
    [ts_external_token_block_comment] = true,
  },
  [6] = {
    [ts_external_token_block_comment] = true,
    [ts_external_token_doc_fragment_string] = true,
    [ts_external_token__doc_token_args] = true,
  },
  [7] = {
    [ts_external_token_block_comment] = true,
    [ts_external_token_doc_fragment_string] = true,
    [ts_external_token__doc_token_minus] = true,
  },
  [8] = {
    [ts_external_token_block_comment] = true,
    [ts_external_token_doc_fragment_string] = true,
    [ts_external_token__doc_token_args] = true,
    [ts_external_token__doc_token_paren] = true,
  },
  [9] = {
    [ts_external_token_block_comment] = true,
    [ts_external_token__doc_token_minus] = true,
  },
  [10] = {
    [ts_external_token_block_comment] = true,
    [ts_external_token__doc_token_args] = true,
  },
};

#ifdef __cplusplus
extern "C" {
#endif
void *tree_sitter_clingo_external_scanner_create(void);
void tree_sitter_clingo_external_scanner_destroy(void *);
bool tree_sitter_clingo_external_scanner_scan(void *, TSLexer *, const bool *);
unsigned tree_sitter_clingo_external_scanner_serialize(void *, char *);
void tree_sitter_clingo_external_scanner_deserialize(void *, const char *, unsigned);

#ifdef TREE_SITTER_HIDE_SYMBOLS
#define TS_PUBLIC
#elif defined(_WIN32)
#define TS_PUBLIC __declspec(dllexport)
#else
#define TS_PUBLIC __attribute__((visibility("default")))
#endif

TS_PUBLIC const TSLanguage *tree_sitter_clingo(void) {
  static const TSLanguage language = {
    .abi_version = LANGUAGE_VERSION,
    .symbol_count = SYMBOL_COUNT,
    .alias_count = ALIAS_COUNT,
    .token_count = TOKEN_COUNT,
    .external_token_count = EXTERNAL_TOKEN_COUNT,
    .state_count = STATE_COUNT,
    .large_state_count = LARGE_STATE_COUNT,
    .production_id_count = PRODUCTION_ID_COUNT,
    .supertype_count = SUPERTYPE_COUNT,
    .field_count = FIELD_COUNT,
    .max_alias_sequence_length = MAX_ALIAS_SEQUENCE_LENGTH,
    .parse_table = &ts_parse_table[0][0],
    .small_parse_table = ts_small_parse_table,
    .small_parse_table_map = ts_small_parse_table_map,
    .parse_actions = ts_parse_actions,
    .symbol_names = ts_symbol_names,
    .field_names = ts_field_names,
    .field_map_slices = ts_field_map_slices,
    .field_map_entries = ts_field_map_entries,
    .supertype_map_slices = ts_supertype_map_slices,
    .supertype_map_entries = ts_supertype_map_entries,
    .supertype_symbols = ts_supertype_symbols,
    .symbol_metadata = ts_symbol_metadata,
    .public_symbol_map = ts_symbol_map,
    .alias_map = ts_non_terminal_alias_map,
    .alias_sequences = &ts_alias_sequences[0][0],
    .lex_modes = (const void*)ts_lex_modes,
    .lex_fn = ts_lex,
    .external_scanner = {
      &ts_external_scanner_states[0][0],
      ts_external_scanner_symbol_map,
      tree_sitter_clingo_external_scanner_create,
      tree_sitter_clingo_external_scanner_destroy,
      tree_sitter_clingo_external_scanner_scan,
      tree_sitter_clingo_external_scanner_serialize,
      tree_sitter_clingo_external_scanner_deserialize,
    },
    .primary_state_ids = ts_primary_state_ids,
    .name = "clingo",
    .max_reserved_word_set_size = 0,
    .metadata = {
      .major_version = 1,
      .minor_version = 0,
      .patch_version = 0,
    },
  };
  return &language;
}
#ifdef __cplusplus
}
#endif
